Package: relsurv
Type: Package
Title: Relative Survival
Version: 2.3-3
Date: 2025-08-08
Authors@R: c(person(c("Maja","Pohar","Perme"),role=c("aut"),email="maja.pohar@mf.uni-lj.si"), person(c("Damjan","Manevski"),role=c("aut", "cre"),email="damjan.manevski@mf.uni-lj.si"))
Author: Maja Pohar Perme [aut],
  Damjan Manevski [aut, cre]
Maintainer: Damjan Manevski <damjan.manevski@mf.uni-lj.si>
Description: Contains functions for analysing relative survival data, including nonparametric estimators of net (marginal relative) survival, relative survival ratio, crude mortality, methods for fitting and checking additive and multiplicative regression models, transformation approach, methods for dealing with population mortality tables. Work has been described in Pohar Perme, Pavlic (2018) <doi:10.18637/jss.v087.i08>.
Depends: R (>= 4.1.0), survival (>= 3.1)
Imports: splines, ggplot2, pammtools, scales, Rcpp (>= 1.0.10)
License: GPL (>= 2)
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.2
LazyData: true
NeedsCompilation: yes
Repository: CRAN
Packaged: 2025-08-18 10:43:50 UTC; dmanevski
Date/Publication: 2025-08-18 11:10:09 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 22:31:44 UTC; unix
Archs: relsurv.so.dSYM

Package: relsurv
Type: Package
Title: Relative Survival
Version: 2.3-3
Date: 2025-08-08
Authors@R: c(person(c("Maja","Pohar","Perme"),role=c("aut"),email="maja.pohar@mf.uni-lj.si"), person(c("Damjan","Manevski"),role=c("aut", "cre"),email="damjan.manevski@mf.uni-lj.si"))
Author: Maja Pohar Perme [aut],
  Damjan Manevski [aut, cre]
Maintainer: Damjan Manevski <damjan.manevski@mf.uni-lj.si>
Description: Contains functions for analysing relative survival data, including nonparametric estimators of net (marginal relative) survival, relative survival ratio, crude mortality, methods for fitting and checking additive and multiplicative regression models, transformation approach, methods for dealing with population mortality tables. Work has been described in Pohar Perme, Pavlic (2018) <doi:10.18637/jss.v087.i08>.
Depends: R (>= 4.1.0), survival (>= 3.1)
Imports: splines, ggplot2, pammtools, scales, Rcpp (>= 1.0.10)
License: GPL (>= 2)
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.2
LazyData: true
NeedsCompilation: yes
Repository: CRAN
Packaged: 2025-08-18 10:43:50 UTC; dmanevski
Date/Publication: 2025-08-18 11:10:09 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 14:03:52 UTC; unix
Archs: relsurv.so.dSYM

useDynLib(relsurv, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(survival, splines)
importFrom("graphics", "abline", "axis", "box", "legend", "lines",
             "par", "plot", "points", "polygon")
importFrom("stats", "approx", "binomial", "coef", "delete.response",
             "glm", "glm.control", "median", "model.extract",
             "model.frame", "model.matrix", "pchisq", "pnorm", "poisson",
             "printCoefmat", "qchisq", "resid", "symnum", "terms","as.formula",
           "model.offset", "predict", "aggregate")
importFrom("utils", "getFromNamespace", "read.table")
export("epa","invtime","joinrate",
	"rs.br","rs.diff","rs.surv",
	"rs.zph","rsadd","rsmul","rstrans",
	"residuals.rsadd",
	"survsplit","transrate","transrate.hld",
	"transrate.hmd","rs.surv.rsadd","survfit.rsadd","cmp.rel","nessie", 'years', 'plot_f', 'plot_years',
	'predict.rsadd', 'predict.aalen.model', 'rsaalen', 'survaalen', 'popsurv')
S3method(residuals, rsadd)
S3method(plot, rs.br)
S3method(plot, cmp.rel)
S3method(print, cmp.rel)
S3method(plot, rs.zph)
S3method(print, rsadd)
S3method(summary, rsadd)
S3method(summary, cmp.rel)
S3method(print, summary.rsadd)
S3method(print, rsdiff)
S3method(print, rs.br)
S3method(survfit, rsadd)
S3method(predict, rsadd)
S3method(predict, aalen.model)
export(expprep2)

#' Calculate the expected (population) survival
#'
#' For a given individual with sex, year, and age, calculate the expected (population) survival at the supplied time points based on the mortality tables.
#'
#' The follow-up time and age must be specified in days. The calendar year can be in any date format
#' (Date and POSIXt are allowed)
#'
#'
#' @param sex Either character ('male'/'female'), or integer (1/2).
#' @param year The year from which the individual is followed. Either a Date or POSIXt object. Default is as.Date('1970-01-01').
#' @param age The age from which the individual is followed. Must be in days.
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param times The times at which the expected (population) survival should be calculated. Must be in days.
#' @return A vector containing the survival estimate at the supplied times.
#' @seealso \code{\link{expprep2}}
#' @examples
#'
#' library(relsurv)
#' # Estimate P(T>2000 days) for a newborn:
#' popsurv(sex='male', year=as.Date('1970-01-01'), age=0, ratetable=slopop, times=2000)
#' # P(T>300 days) for a 50-year old:
#' popsurv(sex='male', year=as.Date('1970-01-01'), age=50*365.241, ratetable=slopop, times=300)
popsurv <- function(sex, year=as.Date('1970-01-01'), age=0, ratetable, times){

  # year <- as.Date(paste0(year, '-01-01'), origin=as.Date('1970-01-01'))

  data <- data.frame(sex=sex, year=year, age=age)

  data$time <- max(times)+age
  data$status <- 1

  if(age!=0) data$age <- 0

  times <- as.numeric(times)

  formula <- Surv(age, time, status)~1

  rform <- suppressWarnings(rformulate(formula, data, ratetable))

  data <- rform$data

  times_arg <- times
  if(age!=0){
    times_arg <- times_arg+age
    times_arg <- unique(c(age, times_arg))
    rform$R[,"year"] <- rform$R[,"year"]-age
  }

  temp <- exp_prep(rform$R[,,drop=FALSE], rform$Y,rform$ratetable,rform$status,times=times_arg,
                             fast=FALSE, cmp=FALSE, ys=as.numeric(age), netweiDM = TRUE)
  # if(age!=0 & length(times)>1) temp$yidli[1] <- 0

  # return(exp(-sum(temp$yidli)))
  # out <- exp(-tail(temp$yidli,1))

  out <- temp$yidli

  if(age!=0){
    out <- out[2:length(out)]
  }

  out <- exp(-cumsum(out))
  names(out) <- paste0('times=', times)

  return(out)
}

# TESTING:
# library(dplyr)
# library(survival)
# library(relsurv)
# library(ggplot2)
#
# # Define objects:
# slopop <- relsurv::slopop
# # slopop[,,] <- slopop["1","1970","male"]
# tajms <- 5*365.241
# xvred <- seq(0, 95, 1)
#
# # Calculate survival at different times since age 0:
# yvred <- sapply(xvred, function(xx) popsurv(sex='male', year=as.Date('1970-01-01'), 0, slopop, times=max(1,xx*365.241)))
# yvred[1] <- 1 # popravi prvo vrednost
# # Calculate 5-year conditional survival:
# rez <- data.frame(x=xvred[1:91], y=(yvred/lag(yvred, 5))[6:96])
#
# # Calculate 5-year conditional survival directly in popsurv:
# yvred2 <- sapply(xvred, function(xx) popsurv(sex='male', year=as.Date('1970-01-01')+xx*365.241,
#                                              slopop, times=tajms, age = xx*365.241))
#
# # Check:
# plot(rez$x, rez$y, xlab='Age', ylab='5-year survival', ylim=c(0,1), type='s')
# lines(rez$x[1:length(yvred2)], yvred2, col='red', type='s')
#
# plot(rez$x, rez$y - yvred2[1:nrow(rez)], type='s')
# # plot(rez$x, -log(rez$y) + log(yvred2[1:nrow(rez)]), type='s')
#
# ggplot(rez)+
#   geom_point(aes(x,y))+
#   theme_bw()+
#   scale_x_continuous(breaks = seq(0, 100, 10))+
#   # scale_y_continuous(breaks = seq(0, 1, 0.2), limits=c(0,1))+
#   # xlab('Age')+
#   # ylab('5-year probability of dying')+
#   # ggtitle('5-year probability of dying conditional on being alive at a given age')+
#   theme(plot.title = element_text(hjust = 0.5))

dLambdaPR <- function(data, all_times, event_times, ratetable, atts){

  # Objects for exp_prep / expc:
  fk <- (atts$factor != 1)
  nfk <- length(fk)
  cuts <- atts$cutpoints

  ltype <- length(atts$type)
  rfac <- rep(0, ltype)
  for(i in 1:ltype){
    if(atts$type[i]==1) rfac[i] <- 1
  }
  rfac <- as.integer(rfac)
  # rfac <- ifelse(atts$type == 1, 1, 0)
  adim <- atts$dim
  acuts <- unlist(cuts)

  Yt_all <- Yt(data, all_times)

  ltimes <- length(all_times)
  nr <- nrow(data)

  data_m <- as.matrix(data)

  # outcome <- vector("list", length = ltimes)
  outcome <- matrix(0, nrow=nr, ncol=ltimes)

  for(i in 1:ltimes){
    if(i==1){
      if(all_times[i]==0) next

      tstart <- 0
    } else{
      tstart <- all_times[i-1]
    }

    tstop <- all_times[i]

    # ord_id <- order(data$Y)
    # data <- data[ord_id, ]

    wh_at_risk <- (Yt_all[[i]] == 1)

    data_tmp <- data_m[wh_at_risk, 4:(nfk + 3), drop = FALSE]

    data_tmp[,fk] <- data_tmp[,fk]+tstart

    # pop_survs <- exp_prep(data_tmp,
    #                       rep(tstop-tstart, sum(wh_at_risk)),
    #                       ratetable)

    times <- rep(tstop-tstart, sum(wh_at_risk))

    temp <- .Call("expc", rfac, adim,
                  acuts, ratetable, data_tmp, times,
                  PACKAGE = "relsurv")
    pop_survs <- temp$surv

    # for(j in 1:nr){
    #   if(Yt_all[[i]][j] == 1){
    #     outcome[j,i] <- 1 # pop hazard v tem intervalu
    #     xx <- exp_prep(data[j, 4:(nfk + 3), drop = FALSE], data$Y -
    #                      data$start, ratetable)
    #   }
    # }

    outcome[wh_at_risk, i] <- -log(pop_survs)
  }

  # Cum hazards:
  outcome <- t(apply(outcome, 1, cumsum))

  # Hazards at event times only:
  # whe <- which(all_times %in% event_times)
  # outcome <- outcome[, whe]

  # Add additional zeros for time 0 (for diffs in hazard):
  outcome <- cbind(rep(0, nrow(outcome)), outcome)

  return(outcome)
}

# exp_prep(x,y,ratetable,status,times,fast=FALSE,ys,prec,cmp=F,netweiDM=FALSE)
#
# jee <- exp_prep(rform$R, rform$Y, rform$ratetable, rform$status, all_times, fast=FALSE, data$start, 1, FALSE, netweiDM=FALSE)
#
# jee <- exp_prep(rform$R, rform$Y, rform$ratetable, rform$status, all_times[1:2], fast=FALSE, data$start, 1, FALSE, netweiDM=FALSE)

rsfitterem<-function(data,b,maxiter,ratetable,tol,bwin,p,cause,Nie){
  # cause: = 2 (unknown), 0 in 1 known. Lahko preko argumenta cause v rsadd dolocis, ce kdo ima znan cause of death (ne rabijo vsi) .
  # Nie: to je lambda_0 (ti), ki se oceni v M koraku v EM algoritmu

  pr.time<-proc.time()[3]
  if (maxiter<1) stop("There must be at least one iteration run")
  n<-nrow(data)
  m <- p
  dtimes <- which(data$stat==1)			#the positions of event times in data$Y
  td <- data$Y[dtimes]				#event times
  ntd <- length(td)				#number of event times
  utimes <- which(c(1,diff(td))!=0)		#the positions of unique event times among td
  utd <- td[utimes]				#unique event times
  nutd <- length(utd)				#number of unique event times
  udtimes <- dtimes[utimes]			#the positions of unique event times among data$Y
  razteg <- function(x){
    # x is a 0/1 vector, the output is a vector of length sum(x), with the corresponding rep numbers
    n <- length(x)
    repu <- rep(1,n)
    repu[x==1] <- 0
    repu <- rev(cumsum(rev(repu)))
    repu <- repu[x==1]
    repu <- -diff(c(repu,0))+1
    if(sum(repu)!=n)repu <- c(n-sum(repu),repu)  #ce je prvi cas censoring, bo treba se kej narest??
    repu
  }
  rutd <- rep(0,ntd)
  rutd[utimes] <- 1
  rutd <- razteg(rutd)				#from  unique event times to event times
  rtd <- razteg(data$stat)			#from event times to data$Y

  a <- data$a[data$stat==1]

  if(bwin[1]!=0){
    #the vector of change points for the smoothing bandwidth
    nt4 <- c(1,ceiling(c(nutd*.25,nutd/2,nutd*.75,nutd)))
    if(missing(bwin))bwin <- rep(1,4)
    else bwin <- rep(bwin,4)
    for(it in 1:4){
      bwin[it] <- bwin[it]*max(diff(utd[nt4[it]:nt4[it+1]]))
    }
    while(utd[nt4[2]]<bwin[1]){		# ce je bwin velik, skrajsamo nt4
      nt4 <- nt4[-2]
      if(length(nt4)==1)break
    }
    #the smoothing matrix
    krn <- kernerleftch(utd,bwin,nt4)
  }



  #forming the new dataset
  if(p>0){
    whtemp <- data$stat==1&cause==2
    dataded <- data[data$stat==1&cause==2,]				#events with unknown cause
    datacens <- data[data$stat==0|cause<2,]				#censorings or known cause

    datacens$cause <- cause[data$stat==0|cause<2]*data$stat[data$stat==0|cause<2]

    databig <- lapply(dataded, rep, 2)
    databig <- do.call("data.frame", databig)
    databig$cause <- rep(2,nrow(databig))
    nded <- nrow(databig)
    databig$cens <- c(rep(1,nded/2),rep(0,nded/2))

    datacens$cens <- rep(0,nrow(datacens))
    datacens$cens[datacens$cause<2] <- datacens$cause[datacens$cause<2]
    names(datacens) <- names(databig)

    databig <- rbind(databig,datacens)

    cause <- cause[data$stat==1]

    #NEW IN 2.05 (next 4 lines)
    fk <- (attributes(ratetable)$factor != 1)
    nfk <- length(fk)
    varstart <- 3+nfk+1		#first column of covariates
    varstop <- 3+nfk+m		#last column of covariates
    #model matrix for relative survival
    xmat <- as.matrix(data[,varstart:varstop])		#NEW IN 2.05

    #ebx at initial values of b
    ebx <- as.vector(exp(xmat%*%b)) # exp(linear.predictor)

    #model matrix for coxph
    modmat <- as.matrix(databig[,varstart:varstop])		#NEW IN 2.05
    varnames <- names(data)[varstart:varstop]		#NEW IN 2.05
  }
  else{
    cause <- cause[data$stat==1]
    ebx <- rep(1,n) # exp(linear.predictor)
  }


  #for time-dependent data:
  starter <- sort(data$start)
  starter1<-c(starter[1],starter[-length(starter)])

  #the values of interest in the cumsums of the obsolete values (there is at least one value - the 1st)
  index <- c(TRUE,(starter!=starter1)[-1])

  starter <- starter[index]
  #the number of repetitions in each cumsum difference - needed for s0 calculation
  val1 <- apply(matrix(starter,ncol=1),1,function(x,Y)sum(x>=Y),data$Y)
  val1 <- c(val1[1],diff(val1),length(data$Y)-val1[length(val1)])

  eb <- ebx[data$stat==1]
  # s0 je sum_{at risk set} ebx
  s0 <- cumsum((ebx)[n:1])[n:1]

  ebx.st <- ebx[order(data$start)]
  s0.st <- ((cumsum(ebx.st[n:1]))[n:1])[index]
  s0.st <- rep(c(s0.st,0),val1)
  s0 <- s0 - s0.st


  #s0 only at times utd
  s0 <- s0[udtimes]


  #find the corresponding value of Y for each start!=0 - needed for likelihood calculation
  start <- data$start
  # if(any(start!=0)){
  #   wstart <- rep(NA,n)
  #   ustart <- unique(start[start!=0])
  #   for(its in ustart){
  #     wstart[start==its] <- min(which(data$Y==its))
  #   }
  # }


  #tale del je zelo sumljiv - kako se racuna likelihood za ties???
  difft <- c(data$Y[data$stat==1][1],diff(td))
  difft <- difftu <- difft[difft!=0]
  difft <- rep(difft,rutd)
  a0 <- a*difft



  if(sum(Nie==.5)!=0)maxit0 <- maxiter
  else maxit0<- maxiter - 3
  for(i in 1:maxit0){

    #Nie is of length ntd, should be nutd, with the values at times being the sum
    nietemp <- rep(1:nutd,rutd)
    Nies <- as.vector(by(Nie,nietemp,sum))  #shorter Nie - only at times utd

    lam0u <- lam0 <- Nies/s0
    #the smooting of lam0
    if(bwin[1]!=0){
      lam0s <- krn%*%lam0
    } else{
      lam0s <- lam0/difftu
    }

    #extended to all event times
    lam0s <- rep(lam0s,rutd)


    #compute Nie, only for those with unknown hazard
    Nie[cause==2] <- as.vector(lam0s*eb/(a+lam0s*eb))[cause==2]

  }

  if(maxit0!=maxiter & i==maxit0) i <- maxiter
  #likelihood calculation - manjka ti se likelihood za nicelni model!!!
  #the cumulative hazard
  Lam0  <- cumsum(lam0)
  #extended to all event times
  Lam0 <- rep(Lam0,rutd)
  if(data$stat[1]==0) Lam0 <- c(0,Lam0)
  #extended to all exit times
  Lam0 <- rep(Lam0,rtd)
  # save original object:
  Lam00 <- Lam0

  #for time dependent covariates and left-truncated individuals: replace by the difference
  if(any(start!=0)){
    # Calculate hazards at non-event times:
    timehaz <- data.frame(time=sort(data$Y), Lam0_2=Lam0)
    timehaz_tmp <- data.frame(time=unique(data$start), Lam0_2=NA)
    timehaz <- rbind(timehaz, timehaz_tmp)
    timehaz <- timehaz[order(timehaz$time),]
    timehaz$Lam0_2 <- mstateNAfix(timehaz$Lam0_2, 0)
    timehaz <- timehaz[!duplicated(timehaz$time),]

    # Prepare object so that you can calculate Lam0_event_time - Lam0_entry_time
    data_lt <- cbind(data, Lam0, id_0=1:nrow(data))
    data_lt <- merge(data_lt, timehaz,
                     by.x='start', by.y='time', all.x = TRUE)
    data_lt <- data_lt[order(data_lt$id_0),]

    # Check:
    # if(any(data_lt$Lam0_2[start!=0] != Lam0[wstart[start!=0]])){
    #   browser()
    # }

    # Edit Lam0:
    Lam0[start!=0] <- data_lt$Lam0[start!=0] - data_lt$Lam0_2[start!=0]

    # Old calculation:
    # Lam0[start!=0] <- Lam0[start!=0] - Lam0[wstart[start!=0]]
  }

  lam0 <- rep(lam0,rutd)

  likely0 <- sum(log(a0 + lam0*eb)) - sum(data$ds + Lam0*ebx)
  likely <- likely0
  tempind <- Nie<=0|Nie>=1
  if(any(tempind)){
    if(any(Nie<=0))Nie[Nie<=0] <- tol
    if(any(Nie>=1))Nie[Nie>=1] <- 1-tol
  }

  if(p>0)databig$wei <- c(Nie[cause==2],1-Nie[cause==2],rep(1,nrow(datacens)))


  if(maxiter>=1&p!=0){
    for(i in 1:maxiter){

      if(p>0){
        b00<-b
        if(i==1)fit <- coxph(Surv(start,Y,cens)~modmat,data=databig,weights=databig$wei,init=b00,x=TRUE,iter.max=maxiter)
        else    fit <- coxph(Surv(start,Y,cens)~modmat,data=databig,weights=databig$wei,x=TRUE,iter.max=maxiter)

        if(any(is.na(fit$coeff))) stop("X matrix deemed to be singular, variable ",which(is.na(fit$coeff)))

        b <- fit$coeff

        ebx <- as.vector(exp(xmat%*%b))
      }
      else ebx <- rep(1,n)

      eb <- ebx[data$stat==1]

      # s0 je sum_{at risk set} ebx
      s0 <- cumsum((ebx)[n:1])[n:1]

      ebx.st <- ebx[order(data$start)]
      s0.st <- ((cumsum(ebx.st[n:1]))[n:1])[index]
      s0.st <- rep(c(s0.st,0),val1)
      s0 <- s0 - s0.st

      #Nie is of length ntd, should be nutd, with the values at times being the sum
      nietemp <- rep(1:nutd,rutd)
      Nies <- as.vector(by(Nie,nietemp,sum))  #shorter Nie - only at times utd
      #s0 only at times utd
      s0 <- s0[udtimes]

      lam0u <- lam0 <- Nies/s0


      #the cumulative hazard
      Lam0  <- cumsum(lam0)
      #extended to all event times
      Lam0 <- rep(Lam0,rutd)
      if(data$stat[1]==0) Lam0 <- c(0,Lam0)
      #extended to all exit times
      Lam0 <- rep(Lam0,rtd)
      # save original object:
      Lam00 <- Lam0
      # for time dependent covariates and left-truncated individuals: replace by the difference
      if(any(start!=0)){
        timehaz <- data.frame(time=sort(data$Y), Lam0_2=Lam0)
        timehaz_tmp <- data.frame(time=unique(data$start), Lam0_2=NA)
        timehaz <- rbind(timehaz, timehaz_tmp)
        timehaz <- timehaz[order(timehaz$time),]
        timehaz$Lam0_2 <- mstateNAfix(timehaz$Lam0_2, 0)
        timehaz <- timehaz[!duplicated(timehaz$time),]

        # Prepare object so that you can calculate Lam0_event_time - Lam0_entry_time
        data_lt <- cbind(data, Lam0, id_0=1:nrow(data))
        data_lt <- merge(data_lt, timehaz,
                         by.x='start', by.y='time', all.x = TRUE)
        data_lt <- data_lt[order(data_lt$id_0),]

        # Edit Lam0:
        Lam0[start!=0] <- data_lt$Lam0[start!=0] - data_lt$Lam0_2[start!=0]
        # Lam0[start!=0] <- Lam0[start!=0] - Lam0[wstart[start!=0]]
      }

      #the smooting of lam0
      if(bwin[1]!=0){
        lam0s <- krn%*%lam0
      } else{
        lam0s <- lam0/difftu
      }

      #extended to all event times
      lam0s <- rep(lam0s,rutd)

      #compute Nie, only for those with unknown hazard
      Nie[cause==2] <- as.vector(lam0s*eb/(a+lam0s*eb))[cause==2]

      #likelihood calculation - manjka ti se likelihood za nicelni model!!!

      lam0 <- rep(lam0,rutd)

      likely <- sum(log(a0 + lam0*eb)) - sum(data$ds + Lam0*ebx)

      if(p>0){
        tempind <- Nie<=0|Nie>=1
        if(any(tempind)){
          if(any(Nie<=0))Nie[Nie<=0] <- tol
          if(any(Nie>=1))Nie[Nie>=1] <- 1-tol
          #if(which(tempind)!=nev)warning("Weights smaller than 0")
          #if(any(is.na( match(which(tempind),c(1,nev)) )))browser()
        }
        if(nded==0) break()
        databig$wei[1:nded] <- c(Nie[cause==2],1-Nie[cause==2])
        bd <- abs(b-b00)
        if(max(bd)< tol) break()
      }
      #early stopping time for no covariates???
    }
  }
  iter <- i
  #if (maxiter > 1& iter>=maxiter)
  #        warning("Ran out of iterations and did not converge")
  if(p>0){
    if(nded!=0){
      resi <- resid(fit,type="schoenfeld")
      if(!is.null(dim(resi)))resi <- resi[1:(nded/2),]
      else resi <- resi[1:(nded/2)]
      swei <- fit$weights[1:(nded/2)]

      if(is.null(dim(resi))) fishem <- sum((resi^2*swei*(1-swei)))
      else {
        fishem <- apply(resi,1,function(x)outer(x,x))
        fishem <- t(t(fishem)*swei*(1-swei))
        fishem <- matrix(apply(fishem,1,sum),ncol=m)
      }
    }

    else fishem <- 0
    fishcox <- solve(fit$var)
    fisher <- fishcox - fishem
    fit$var <- solve(fisher)
    names(fit$coefficients)<-varnames
    fit$lambda0 <- lam0s
  }
  else fit <- list(lambda0 = lam0s)

  # lam0.ns -> lam0u: editan (smoothing)
  # lambda0 -> lam0s: editan (smoothing)
  # Lambda0 -> Lam0: cumsum(lam0) - lam0 je enako lam0u

  fit$lambda0 <- fit$lambda0[utimes]
  fit$Lambda0 <- Lam00[udtimes]
  fit$times <- utd
  fit$Nie <- Nie
  fit$bwin <- bwin
  fit$iter <- i
  class(fit) <- c("rsadd",class(fit))
  fit$loglik <- c(likely0,likely)
  fit$lam0.ns <- lam0u
  fit
}



em <- function (rform, init, control, bwin)
{
  data <- rform$data
  n <- nrow(data)
  p <- rform$m
  ord_id <- order(data$Y)
  rform$cause <- rform$cause[ord_id]
  data <- data[ord_id, ]
  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)
  nev <- length(data$Y[data$stat == 1])
  data$a <- rep(NA, n)
  xx <- exp_prep(data[, 4:(nfk + 3),drop=FALSE], data$Y - data$start, rform$ratetable)
  # The cumulative population hazard of dying at time Y:
  data$ds <- -log(xx)
  data1 <- data
  data1[, 4:(nfk + 3)] <- data[, 4:(nfk + 3)] + data$Y %*% t(fk)
  xx <- exp_prep(data1[data1$stat == 1, 4:(nfk + 3),drop=FALSE], 1, rform$ratetable)
  # The population hazard of dying in the following day (for individuals that had an event):
  data$a[data$stat == 1] <- -log(xx)

  if (p > 0) {
    if (!missing(init) && !is.null(init)) {
      if (length(init) != p)
        stop("Wrong length for inital values")
    }
    else init <- rep(0, p)
    beta <- matrix(init, p, 1)
  }
  pr.time<-proc.time()[3]

  Nie <- rep(.5,sum(data$stat==1))
  Nie[rform$cause[data$stat==1]<2] <-  rform$cause[data$stat==1][rform$cause[data$stat==1]<2]

  #NEW IN 2.05
  varstart <- 3+nfk+1		#first column of covariates
  varstop <- 3+nfk+p		#last column of covariates


  if(missing(bwin))bwin <- -1
  if(bwin<0){

    if(p>0)data1 <- data[,-c(varstart:varstop)]    		#NEW IN 2.05
    else  data1 <- data
    nfk <- length(attributes(rform$ratetable)$dimid)
    names(data)[4:(3+nfk)] <- attributes(rform$ratetable)$dimid
    expe <- rs.surv(Surv(Y,stat)~1,data,ratetable=rform$ratetable,method="ederer2")
    esurv <- -log(expe$surv[expe$n.event!=0])
    if(esurv[length(esurv)]==Inf)esurv[length(esurv)] <-  esurv[length(esurv)-1]
    x <- seq(.1,3,length=5)
    dif <- rep(NA,5)
    options(warn=-1)
    diter <- max(round(max(data$Y)/356.24),3)
    for(it in 1:5){
      fit <- rsfitterem(data1,NULL,diter,rform$ratetable,control$epsilon,x[it],0,rform$cause,Nie)
      dif[it] <- sum((esurv-fit$Lambda0)^2)
    }
    wh <- which.min(dif)
    if(wh==1)x <- seq(x[wh],x[wh+1]-.1,length=5)
    else if(wh==5)x <- c(x, max(data$Y)/ max(diff(data$Y)))
    if(wh!=1)
      x <- seq(x[wh-1]+.1,x[wh+1]-.1,length=5)
    dif <- rep(NA,5)

    for(it in 1:5){
      fit <- rsfitterem(data1,NULL,diter,rform$ratetable,control$epsilon,x[it],0,rform$cause,Nie)
      dif[it] <- sum((esurv-fit$Lambda0)^2)
    }
    options(warn=0)
    Nie <- fit$Nie
    bwin <- x[which.min(dif)]
  }

  # if(any(data$start != 0)) browser()

  fit <- rsfitterem(data, beta, control$maxit, rform$ratetable,
                    control$epsilon, bwin, p, rform$cause,Nie)

  Nie <- rep(0,nrow(data))
  Nie[data$stat==1] <- fit$Nie
  fit$Nie <- Nie[order(ord_id)]
  fit$bwin <- list(bwin=fit$bwin,bwinfac=bwin)
  fit
}




#' Fit an Additive model for Relative Survival
#'
#' The function fits an additive model to the data. The methods implemented are
#' the maximum likelihood method, the semiparametric method, a glm model with a
#' \code{binomial} error and a glm model with a \code{poisson} error.
#'
#' NOTE: The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with by the \code{rmap} argument. For example, if
#' age is in years in the data set but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.
#'
#' The maximum likelihood method and both glm methods assume a fully parametric
#' model with a piecewise constant baseline excess hazard function. The
#' intervals on which the baseline is assumed constant should be passed via
#' argument \code{int}. The EM method is semiparametric, i.e. no assumptions
#' are made for the baseline hazard and therefore no intervals need to be
#' specified.
#'
#' The methods using glm are methods for grouped data. The groups are formed
#' according to the covariate values. This should be taken into account when
#' fitting a model. The glm method returns life tables for groups specified by
#' the covariates in \code{groups}.
#'
#' The EM method output includes the smoothed baseline excess hazard
#' \code{lambda0}, the cumulative baseline excess hazard \code{Lambda0} and
#' \code{times} at which they are estimated. The individual probabilites of
#' dying due to the excess risk are returned as \code{Nie}.  The EM method
#' fitting procedure requires some local smoothing of the baseline excess
#' hazard. The default \code{bwin=-1} value lets the function find an
#' appropriate value for the smoothing band width. While this ensures an
#' unbiased estimate, the procedure time is much longer. As the value found by
#' the function is independent of the covariates in the model, the value can be
#' read from the output (\code{bwinfac}) and used for refitting different
#' models to the same data to save time.
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right. \code{Surv(start,stop,event)} outcomes
#' are also possible for time-dependent covariates and left-truncation for
#' \code{method='EM'}.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param int either a single value denoting the number of follow-up years or a
#' vector specifying the intervals (in years) in which the hazard is constant
#' (the times that are bigger than \code{max(int)} are censored. If missing,
#' only one interval (from time 0 to maximum observation time) is assumed.  The
#' EM method does not need the intervals, only the maximum time can be
#' specified (all times are censored after this time point).
#' @param na.action a missing-data filter function, applied to the model.frame,
#' after any subset argument has been used.  Default is
#' \code{options()$na.action}.
#' @param method \code{glm.bin} or \code{glm.poi} for a glm model, \code{EM}
#' for the EM algorithm and \code{max.lik} for the maximum likelihood model
#' (default).
#' @param init vector of initial values of the iteration.  Default initial
#' value is zero for all variables.
#' @param bwin controls the bandwidth used for smoothing in the EM algorithm.
#' The follow-up time is divided into quartiles and \code{bwin} specifies a
#' factor by which the maximum between events time length on each interval is
#' multiplied. The default \code{bwin=-1} lets the function find an appropriate
#' value. If \code{bwin=0}, no smoothing is applied.
#' @param centered if \code{TRUE}, all the variables are centered before
#' fitting and the baseline excess hazard is calculated accordingly. Default is
#' \code{FALSE}.
#' @param cause A vector of the same length as the number of cases. \code{0}
#' for population deaths, \code{1} for disease specific deaths, \code{2}
#' (default) for unknown. Can only be used with the \code{EM} method.
#' @param control a list of parameters for controlling the fitting process.
#' See the documentation for \code{glm.control} for details.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @param ... other arguments will be passed to \code{glm.control}.
#' @return An object of class \code{rsadd}. In the case of
#' \code{method="glm.bin"} and \code{method="glm.poi"} the class also inherits
#' from \code{glm} which inherits from the class \code{lm}. Objects of this
#' class have methods for the functions \code{print} and \code{summary}. An
#' object of class \code{rsadd} is a list containing at least the following
#' components: \item{data}{the data as used in the model, along with the
#' variables defined in the rate table} \item{ratetable}{the ratetable used.}
#' \item{int}{the maximum time (in years) used. All the events at and after
#' this value are censored.} \item{method}{the fitting method that was used.}
#' \item{linear.predictors}{the vector of linear predictors, one per subject.}
#' @seealso \code{\link{rstrans}}, \code{\link{rsmul}}
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#'
#' EM algorithm: Pohar Perme M., Henderson R., Stare, J. (2009) "An approach to
#' estimation in relative survival regression." Biostatistics, \bold{10}:
#' 136--146.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #fit an additive model
#' #note that the variable year is given in days since 01.01.1960 and that
#' #age must be multiplied by 365.241 in order to be expressed in days.
#' fit <- rsadd(Surv(time,cens)~sex+as.factor(agegr)+ratetable(age=age*365.241),
#' 	    ratetable=slopop,data=rdata,int=5)
#'
#' #check the goodness of fit
#' rs.br(fit)
#'
#' #use the EM method and plot the smoothed baseline excess hazard
#' fit <- rsadd(Surv(time,cens)~sex+age,rmap=list(age=age*365.241),
#' 	 ratetable=slopop,data=rdata,int=5,method="EM")
#' sm <- epa(fit)
#' plot(sm$times,sm$lambda,type="l")
#'
rsadd <- function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,
                   int, na.action, method = "max.lik", init, bwin, centered = FALSE,
                   cause, control, rmap, ...) # rmap.subs=TRUE,
{
  call <- match.call()
  if (missing(control))
    control <- glm.control(...)

  if (!missing(rmap)) {
    rmap_tmp <- substitute(rmap)
    if(inherits(rmap_tmp, "call")){
      rmap <- rmap_tmp
    }
  }

  if(!missing(cause)){								#NEW: ce cause ne manjka, ga preverim in dodam kot spremenljivko
    if (length(cause) != nrow(data))
      stop("Length of cause does not match data dimensions")
    data$cause <- cause
    rform <- rformulate(formula, data, ratetable, na.action, rmap, int, centered, cause)
  }
  else{ #no cause
    rform <- rformulate(formula,data, ratetable, na.action, rmap, int, centered)
  }

  if (method == "EM") {
    if (!missing(int)) {
      if (length(int) > 1 | any(int <= 0))
        stop("Invalid value of 'int'")
    }
  }
  else {
    if (missing(int))
      int <- c(0,ceiling(max(rform$Y/365.241)))
    if (length(int) == 1) {
      if (int <= 0)
        stop("The value of 'int' must be positive ")
      int <- 0:int
    }
    else if (int[1] != 0)
      stop("The first interval in 'int' must start with 0")
  }
  method <-  match.arg(method,c("glm.bin","glm.poi","max.lik","EM"))

  if (method == "glm.bin" | method == "glm.poi")
    fit <- glmxp(rform = rform, interval = int, method = method,
                 control = control)
  else if (method == "max.lik")
    fit <- maxlik(rform = rform, interval = int, init = init,
                  control = control)
  else if (method == "EM")
    fit <- em(rform, init, control, bwin)
  fit$call <- call
  fit$formula <- formula
  fit$data <- rform$data
  fit$ratetable <- rform$ratetable
  fit$n <- nrow(rform$data)
  if (length(rform$na.action))
    fit$na.action <- rform$na.action
  fit$y <- rform$Y.surv
  fit$method <- method
  if (method == "EM") {
    if (!missing(int))
      fit$int <- int
    else fit$int <- ceiling(max(rform$Y[rform$status == 1])/365.241)
    fit$terms <- rform$Terms
    if(centered)fit$mvalue <- rform$mvalue
  }
  if (method == "max.lik") {
    fit$terms <- rform$Terms
  }
  if (rform$m > 0)
    fit$linear.predictors <- as.matrix(rform$X) %*% fit$coef[1:ncol(rform$X)]

  if (!missing(rmap)) {
    fit$rmap <- rmap
  }
  fit
}


maxlik <- function (rform, interval, subset, init, control)
{
  data <- rform$data
  max.time <- max(data$Y)/365.241
  if (max.time < max(interval))
    interval <- interval[1:(sum(max.time > interval) + 1)]
  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)
  data <- cbind(data, offset = rform$offset)
  data <- survsplit(data, cut = interval[-1] * 365.241, end = "Y",
                    event = "stat", start = "start", episode = "epi", interval = interval)
  del <- which(data$start==data$Y)
  if(length(del))    data <- data[-del,]
  offset <- data$offset
  data$offset <- NULL
  d.int <- diff(interval)
  data[, 4:(nfk + 3)] <- data[, 4:(nfk + 3)] + data$start %*%
    t(fk)
  data$lambda <- rep(0, nrow(data))
  nsk <- nrow(data[data$stat == 1, ])
  xx <- exp_prep(data[data$stat == 1, 4:(nfk + 3),drop=FALSE] + (data[data$stat ==
                                                                        1, ]$Y - data[data$stat == 1, ]$start) %*% t(fk), 1,  rform$ratetable)
  data$lambda[data$stat == 1] <- -log(xx) * 365.241
  xx <- exp_prep(data[, 4:(nfk + 3),drop=FALSE], data$Y - data$start, rform$ratetable)
  data$epi <- NULL
  data$ds <- -log(xx)
  data$Y <- data$Y/365.241
  data$start <- data$start/365.241
  data <- data[, -(4:(3 + nfk))]
  intn <- length(interval[-1])
  m <- rform$m
  p <-  m + intn
  if (!missing(init) && !is.null(init)) {
    if (length(init) != p)
      stop("Wrong length for inital values")
  }
  else init <- rep(0, p)
  if(m>0){
    init0 <- init[-(1:m)]
    data1 <- data[,-(4:(3+m))]
  }
  else{
    init0 <- init
    data1 <- data
  }
  fit0 <- lik.fit(data1, 0, intn, init0,  control, offset)
  if(m>0){
    init[-(1:m)] <- fit0$coef
    fit <- lik.fit(data, m, intn, init,  control, offset)
  }
  else fit <- fit0
  fit$int <- interval
  class(fit) <- "rsadd"
  fit$times <- fit$int*365.241						#dodano za potrebe rs.surv.rsadd
  fit$Lambda0 <- cumsum(c(0, exp(fit$coef[(m+1):p])*diff(fit$int)  ))
  fit
}

lik.fit <- function (data, m, intn, init, control, offset)
{
  n <- dim(data)[1]
  varpos <- 4:(3 + m + intn)
  x <- data[, varpos]
  varnames <- names(data)[varpos]
  lbs <- names(x)
  x <- as.matrix(x)
  p <- length(varpos)
  d <- data$stat
  ds <- data$ds
  h <- data$lambda
  y <- data$Y - data$start
  maxiter <- control$maxit
  if (!missing(init) && !is.null(init)) {
    if (length(init) != p)
      stop("Wrong length for inital values")
  }
  else init <- rep(0, p)
  b <- matrix(init, p, 1)
  b0 <- b
  fit <- mlfit(b, p, x, offset, d, h, ds, y, maxiter, control$epsilon)
  if (maxiter > 1 & fit$nit >= maxiter) {
    values <- apply(data[data$stat==1,varpos,drop=FALSE],2,sum)			#NEW: deluje tudi, ce je ratetable eno-dimenzionalen
    problem <- which.min(values)
    outmes <- "Ran out of iterations and did not converge"
    if(values[problem]==0)tzero <- ""
    else tzero <- "only "
    if(values[problem]<5){
      if(!is.na(strsplit(names(values)[problem],"fu")[[1]][2]))outmes <- paste(outmes, "\n This may be due to the fact that there are ",tzero, values[problem], " events on interval",strsplit(names(values)[problem],"fu")[[1]][2],"\n You can use the 'int' argument to change the follow-up intervals in which the baseline excess hazard is assumed constant",sep="")
      else outmes <- paste(outmes, "\n This may be due to the fact that there are ",tzero, values[problem], " events for covariate value ",names(values)[problem],sep="")
    }
    warning(outmes)
  }
  b <- as.vector(fit$b)
  names(b) <- varnames
  fit <- list(coefficients = b, var = -solve(fit$sd), iter = fit$nit,
              loglik = fit$loglik)
  fit
}





#' Split a Survival Data Set at Specified Times
#'
#' Given a survival data set and a set of specified cut times, the function
#' splits each record into multiple records at each cut time. The new data set
#' is be in \code{counting process} format, with a start time, stop time, and
#' event status for each record. More general than \code{survSplit} as it also
#' works with the data already in the \code{counting process} format.
#'
#'
#' @param data data frame.
#' @param cut vector of timepoints to cut at.
#' @param end character string with name of event time variable.
#' @param event character string with name of censoring indicator.
#' @param start character string with name of start variable (will be created
#' if it does not exist).
#' @param id character string with name of new id variable to create
#' (optional).
#' @param zero If \code{start} doesn't already exist, this is the time that the
#' original records start. May be a vector or single value.
#' @param episode character string with name of new episode variable
#' (optional).
#' @param interval this argument is used by \code{max.lik} function
#' @return New, longer, data frame.
#' @seealso \code{\link[survival:survSplit]{survival::survSplit}}.
#' @keywords survival
survsplit <- function (data, cut, end, event, start, id = NULL, zero = 0,
                       episode = NULL, interval = NULL)
{
  ntimes <- length(cut)
  n <- nrow(data)
  p <- ncol(data)
  if (length(interval) > 0) {
    ntimes <- ntimes - 1
    sttime <- c(rep(0, n), rep(cut[-length(cut)], each = n))
    endtime <- rep(cut, each = n)
  }
  else {
    endtime <- rep(c(cut, Inf), each = n)
    sttime <- c(rep(0, n), rep(cut, each = n))
  }
  newdata <- lapply(data, rep, ntimes + 1)
  eventtime <- newdata[[end]]
  if (start %in% names(data))
    starttime <- newdata[[start]]
  else starttime <- rep(zero, length = (ntimes + 1) * n)
  starttime <- pmax(sttime, starttime)
  epi <- rep(0:ntimes, each = n)
  if (length(interval) > 0)
    status <- ifelse(eventtime <= endtime & eventtime >=
                       starttime, newdata[[event]], 0)
  else status <- ifelse(eventtime <= endtime & eventtime >
                          starttime, newdata[[event]], 0)
  endtime <- pmin(endtime, eventtime)
  if (length(interval) > 0)
    drop <- (starttime > endtime) | (starttime == endtime &
                                       status == 0)
  else drop <- starttime >= endtime
  newdata <- do.call("data.frame", newdata)
  newdata <- newdata[!drop, ]
  newdata[, start] <- starttime[!drop]
  newdata[, end] <- endtime[!drop]
  newdata[, event] <- status[!drop]
  if (!is.null(id))
    newdata[, id] <- rep(rownames(data), ntimes + 1)[!drop]
  fu <- NULL
  if (length(interval) > 2) {
    for (it in 1:length(interval[-1])) {
      drop1 <- sum(!drop[1:(it * n - n)])
      drop2 <- sum(!drop[(it * n - n + 1):(it * n)])
      drop3 <- sum(!drop[(it * n + 1):(length(interval[-1]) *
                                         n)])
      if (it == 1)
        fu <- cbind(fu, c(rep(1, drop2), rep(0, drop3)))
      else if (it == length(interval[-1]))
        fu <- cbind(fu, c(rep(0, drop1), rep(1, drop2)))
      else fu <- cbind(fu, c(rep(0, drop1), rep(1, drop2),
                             rep(0, drop3)))
    }
    fu <- as.data.frame(fu)
    names(fu) <- c(paste("fu [", interval[-length(interval)],
                         ",", interval[-1], ")", sep = ""))
    newdata <- cbind(newdata, fu)
  }
  else if (length(interval) == 2) {
    fu <- rep(1, sum(!drop))
    newdata <- cbind(newdata, fu)
    names(newdata)[ncol(newdata)] <- paste("fu [", interval[1],
                                           ",", interval[2], "]", sep = "")
  }
  if (!is.null(episode))
    newdata[, episode] <- epi[!drop]
  newdata
}


glmxp <- function (rform, data, interval, method, control)
{
  if (rform$m == 1)
    g <- as.integer(as.factor(rform$X[[1]]))
  else if (rform$m > 1) {
    gvar <- NULL
    for (i in 1:rform$m) {
      gvar <- append(gvar, rform$X[i])
    }
    tabgr <- as.data.frame(table(gvar))
    tabgr <- tabgr[, 1:rform$m]
    n.groups <- dim(tabgr)[1]
    mat <- do.call("data.frame", gvar)
    names(mat) <- names(tabgr)
    tabgr <- cbind(tabgr, g = as.numeric(row.names(tabgr)))
    mat <- cbind(mat, id = 1:rform$n)
    c <- merge(tabgr, mat)
    g <- c[order(c$id), rform$m + 1]
  }
  else g <- rep(1, rform$n)
  vg <- function(X) {
    n <- dim(X)[1]
    w <- sum((X$event == 0) & (X$fin == 1) & (X$y != 1))
    nd <- sum((X$event == 1) & (X$fin == 1))
    ps <- exp_prep(X[, 4:(nfk + 3),drop=FALSE], t.int, rform$ratetable)
    ld <- n - w/2
    lny <- log(sum(X$y))
    k <- t.int/365.241
    dstar <- sum(-log(ps)/k * X$y)
    ps <- mean(ps)
    if (rform$m == 0)
      data.rest <- X[1, 7 + nfk + rform$m, drop = FALSE]
    else data.rest <- X[1, c((3 + nfk + 1):(3 + nfk + rform$m),
                             7 + nfk + rform$m)]
    cbind(nd = nd, ld = ld, ps = ps, lny = lny, dstar = dstar,
          k = k, data.rest)
  }
  nint <- length(interval)
  if (nint < 2)
    stop("Illegal interval value")
  meje <- interval
  my.fun <- function(x) {
    if (x > 1) {
      x.t <- rep(1, floor(x))
      if (x - floor(x) > 0)
        x.t <- c(x.t, x - floor(x))
      x.t
    }
    else x
  }
  int <- apply(matrix(diff(interval), ncol = 1), 1, my.fun)
  if (is.list(int))
    int <- c(0, cumsum(do.call("c", int)))
  else int <- c(0, cumsum(int))
  int <- int * 365.241
  nint <- length(int)
  X <- cbind(rform$data, grupa = g)
  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)
  Z <- X[X$start >= int[2], ]
  nz <- dim(Z)[1]
  Z$fin <- rep(0, nz)
  Z$event <- rep(0, nz)
  Z$fu <- rep(0, nz)
  Z$y <- rep(0, nz)
  Z$origstart <- Z$start
  Z$xind <- rep(0, nz)
  if (nrow(Z) > 0)
    Z[, 4:(nfk + 3)] <- Z[, 4:(nfk + 3)] + matrix(Z$start,
                                                  ncol = nfk, byrow = FALSE, nrow = nrow(Z)) * matrix(fk,
                                                                                                      ncol = nfk, byrow = TRUE, nrow = nrow(Z))
  X <- X[X$start < int[2], ]
  X$fin <- (X$Y <= int[2])
  X$event <- X$fin * X$stat
  ford <- eval(substitute(paste("[", a, ",", b, "]", sep = ""),
                          list(a = meje[1], b = meje[2])))
  X$fu <- rep(ford, rform$n - nz)
  t.int <- int[2] - int[1]
  X$y <- (pmin(X$Y, int[2]) - X$start)/365.241
  X$origstart <- X$start
  X$xind <- rep(1, nrow(X))
  gr1 <- by(X, X$grupa, vg)
  grm1 <- do.call("rbind", gr1)
  X <- X[X$fin == 0, ]
  X$start <- rep(int[2], dim(X)[1])
  X <- rbind(X, Z[Z$start < int[3], ])
  Z <- Z[Z$start >= int[3], ]
  temp <- 0
  if (nint > 2) {
    for (i in 3:nint) {
      ni <- dim(X)[1]
      if (ni == 0) {
        temp <- 1
        break
      }
      X$fin <- X$Y <= int[i]
      X$event <- X$fin * X$stat
      l <- sum(int[i - 1] >= meje * 365.241)
      if(l==1)
        ftemp <- eval(substitute(paste("[", a, ",", b, "]", sep = ""),
                                 list(a = meje[l], b = meje[l + 1])))

      else
        ftemp <- eval(substitute(paste("(", a, ",", b, "]", sep = ""),
                                 list(a = meje[l], b = meje[l + 1])))
      ford <- c(ford, ftemp)
      X$fu <- rep(ford[i - 1], ni)
      t.int <- int[i] - int[i - 1]
      index <- X$origstart < int[i - 1]
      index1 <- as.logical(X$xind)
      if (sum(index) > 0)
        X[index, 4:(nfk + 3)] <- X[index, 4:(nfk + 3)] +
        matrix(fk * t.int, ncol = nfk, byrow = TRUE,
               nrow = sum(index))
      X$xind <- rep(1, nrow(X))
      X$y <- (pmin(X$Y, int[i]) - X$start)/365.241
      gr1 <- by(X, X$grupa, vg)
      grm1 <- rbind(grm1, do.call("rbind", gr1))
      X <- X[X$fin == 0, ]
      X$start <- rep(int[i], dim(X)[1])
      if (i == nint)
        break
      X <- rbind(X, Z[Z$start < int[i + 1], ])
      X <- X[X$start != X$Y, ]
      Z <- Z[Z$start >= int[i + 1], ]
    }
    l <- sum(int[i - temp] > meje * 365.241)
    interval <- meje[1:(l + 1)]
  }
  else interval <- meje[1:2]
  grm1$fu <- factor(grm1$fu, levels = unique(ford))
  if (method == "glm.bin") {
    ht <- binomial(link = cloglog)
    ht$link <- "Hakulinen-Tenkanen relative survival model"
    ht$linkfun <- function(mu) log(-log((1 - mu)/ps))
    ht$linkinv <- function(eta) 1 - exp(-exp(eta)) * ps
    ht$mu.eta <- function(eta) exp(eta) * exp(-exp(eta)) *
      ps
    .ps <- ps <- grm1$ps
    #assign(".ps", grm1$ps, envir = .GlobalEnv)
    # ht$initialize <- expression({
    #     n <- y[, 1] + y[, 2]
    #     y <- ifelse(n == 0, 0, y[, 1]/n)
    #     weights <- weights * n
    #     mustart <- (n * y + 0.01)/(n + 0.02)
    #     mustart[(1 - mustart)/data$ps >= 1] <- data$ps[(1 - mustart)/data$ps >=
    #         1] * 0.9
    # })
    if (any(grm1$ld - grm1$nd > grm1$ps * grm1$ld)) {
      n <- sum(grm1$ld - grm1$nd > grm1$ps * grm1$ld)
      g <- dim(grm1)[1]
      warnme <- paste("Observed number of deaths is smaller than the expected in ",
                      n, "/", g, " groups of patients", sep = "")
    }
    else warnme <- ""

    if (length(interval) == 2 & rform$m == 0)
      stop("No groups can be formed")
    if (length(interval) == 1 | length(table(grm1$fu)) ==
        1)
      grm1$fu <- as.integer(grm1$fu)

    y <- ifelse(grm1$ld == 0, 0, grm1$nd/grm1$ld)
    #weights <- weights * grm1$ld
    mustart <- (grm1$ld * y + 0.01)/(grm1$ld + 0.02)
    mustart[(1 - mustart)/grm1$ps >= 1] <- grm1$ps[(1 - mustart)/grm1$ps >=
                                                     1] * 0.9

    if (!length(rform$X))
      local.ht <- glm(cbind(nd, ld - nd) ~ -1 + fu + offset(log(k)),
                      data = grm1, family = ht,mustart=mustart)
    else {
      xmat <- as.matrix(grm1[, 7:(ncol(grm1) - 1)])

      local.ht <- glm(cbind(nd, ld - nd) ~ -1 + xmat +
                        fu + offset(log(k)), data = grm1, family = ht,mustart=mustart)
    }
    names(local.ht[[1]]) <- c(names(rform$X), paste("fu",
                                                    levels(grm1$fu)))
  }
  else if (method == "glm.poi") {
    pot <- poisson()
    pot$link <- "glm relative survival model with Poisson error"
    pot$linkfun <- function(mu) log(mu - dstar)
    pot$linkinv <- function(eta) dstar + exp(eta)
    #assign(".dstar", grm1$dstar, envir = .GlobalEnv)
    if (any(grm1$nd - grm1$dstar < 0)) {
      pot$initialize <- expression({
        if (any(y < 0)) stop(paste("Negative values not allowed for",
                                   "the Poisson family"))
        n <- rep.int(1, nobs)
        #mustart <- pmax(y, .dstar) + 0.1
      })
    }
    if (any(grm1$nd - grm1$dstar < 0)) {
      n <- sum(grm1$nd - grm1$dstar < 0)
      g <- dim(grm1)[1]
      warnme <- paste("Observed number of deaths is smaller than the expected in ",
                      n, "/", g, " groups of patients", sep = "")
    }
    else warnme <- ""
    dstar <- grm1$dstar
    if (length(interval) == 2 & rform$m == 0)
      stop("No groups can be formed")
    if (length(interval) == 1 | length(table(grm1$fu)) ==
        1)
      grm1$fu <- as.integer(grm1$fu)

    mustart <- pmax(grm1$nd, grm1$dstar) + 0.1
    if (!length(rform$X))
      local.ht <- glm(nd ~ -1 + fu, data = grm1, family = pot,
                      offset = grm1$lny,mustart=mustart)
    else {
      xmat <- as.matrix(grm1[, 7:(ncol(grm1) - 1)])
      local.ht <- glm(nd ~ -1 + xmat + fu, data = grm1,
                      family = pot, offset = grm1$lny,mustart=mustart)
    }
    names(local.ht[[1]]) <- c(names(rform$X), paste("fu",
                                                    levels(grm1$fu)))
  }
  else stop(paste("Method '", method, "' not a valid method",
                  sep = ""))
  class(local.ht) <- c("rsadd", class(local.ht))
  local.ht$warnme <- warnme
  local.ht$int <- interval
  local.ht$groups <- local.ht$data
  return(local.ht)
}



#' Calculate Residuals for a "rsadd" Fit
#'
#' Calculates partial residuals for an additive relative survival model.
#'
#'
#' @param object an object inheriting from class \code{rsadd}, representing a
#' fitted additive relative survival model. Typically this is the output from
#' the \code{rsadd} function.
#' @param type character string indicating the type of residual desired.
#' Currently only Schoenfeld residuals are implemented.
#' @param ... other arguments.
#' @return A list of the following values is returned: \item{res}{a matrix
#' containing the residuals for each variable.} \item{varr}{the variance for
#' each residual} \item{varr1}{the sum of \code{varr}.} \item{kvarr}{the
#' derivative of each residual, to be used in \code{rs.zph} function.}
#' \item{kvarr1}{the sum of \code{kvarr}.}
#' @seealso \code{\link{rsadd}}.
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#'
#' Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005) "Goodness of
#' fit of relative survival models." Statistics in Medicine, \bold{24}:
#' 3911--3925.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#'        ratetable=slopop,data=rdata,int=5)
#' sresid <- residuals.rsadd(fit)
#'
residuals.rsadd <- function (object, type = "schoenfeld", ...)
{
  data <- object$data[order(object$data$Y), ]
  ratetable <- object$ratetable
  beta <- object$coef
  start <- data[, 1]
  stop <- data[, 2]
  event <- data[, 3]
  fk <- (attributes(ratetable)$factor != 1)
  nfk <- length(fk)
  n <- nrow(data)
  scale <- 1
  if (object$method == "EM")
    scale <- 365.241
  m <- ncol(data)
  rem <- m - nfk - 3
  interval <- object$int
  int <- ceiling(max(interval))
  R <- data[, 4:(nfk + 3)]
  lp <- matrix(-log(exp_prep(as.matrix(R), 365.241, object$ratetable))/scale, ncol = 1)
  fu <- NULL
  if (object$method == "EM") {
    death.time <- stop[event == 1]
    for (it in 1:int) {
      fu <- as.data.frame(cbind(fu, as.numeric(death.time/365.241 <
                                                 it & (death.time/365.241) >= (it - 1))))
    }
    if(length(death.time)!=length(unique(death.time))){
      utimes <- which(c(1,diff(death.time))!=0)
      razteg <- function(x){
        # x is a 0/1 vector, the output is a vector of length sum(x), with the corresponding rep numbers
        n <- length(x)
        repu <- rep(1,n)
        repu[x==1] <- 0
        repu <- rev(cumsum(rev(repu)))
        repu <- repu[x==1]
        repu <- -diff(c(repu,0))+1
        if(sum(repu)!=n)repu <- c(n-sum(repu),repu)  #ce je prvi cas censoring, bo treba se kej narest??
        repu
      }
      rutd <- rep(0,length(death.time))
      rutd[utimes] <- 1
      rutd <- razteg(rutd)				#from  unique event times to event times
    }
    else rutd <- rep(1,length(death.time))
    lambda0 <- rep(object$lambda0,rutd)
  }
  else {
    pon <- NULL
    for (i in 1:(length(interval) - 1)) {
      width <- ceiling(interval[i + 1]) - floor(interval[i])
      lo <- interval[i]
      hi <- min(interval[i + 1], floor(interval[i]) + 1)
      for (j in 1:width) {
        fu <- as.data.frame(cbind(fu, as.numeric(stop/365.241 <
                                                   hi & stop/365.241 >= lo)))
        names(fu)[ncol(fu)] <- paste("fu", lo, "-", hi,
                                     sep = "")
        if (j == width) {
          pon <- c(pon, sum(fu[event == 1, (ncol(fu) -
                                              width + 1):ncol(fu)]))
          break()
        }
        else {
          lo <- hi
          hi <- min(interval[i + 1], floor(interval[i]) +
                      1 + j)
        }
      }
    }
    m <- ncol(data)
    data <- cbind(data, fu)
    rem <- m - nfk - 3
    lambda0 <- rep(exp(beta[rem + 1:(length(interval) - 1)]),
                   pon)
    fu <- fu[event == 1, , drop = FALSE]
    beta <- beta[1:rem]
  }
  if (int >= 2) {
    for (j in 2:int) {
      R <- R + matrix(fk * 365.241, ncol = ncol(R), byrow = TRUE,
                      nrow = n)
      xx <- exp_prep(R, 365.241, object$ratetable)
      lp <- cbind(lp, -log(xx)/scale)
    }
  }
  z <- as.matrix(data[, (4 + nfk):m])
  out <- resid.com(start, stop, event, z, beta, lp, lambda0,
                   fu, n, rem, int, type)
  out
}

resid.com <- function (start, stop, event, z, beta, lp, lambda0, fup, n, rem,
                       int, type)
{
  le <- exp(z %*% beta)
  olp <- if (int > 1)
    apply(lp[n:1, ], 2, cumsum)[n:1, ]
  else matrix(cumsum(lp[n:1])[n:1], ncol = 1)
  ole <- cumsum(le[n:1])[n:1]
  lp.st <- lp[order(start), , drop = FALSE]
  le.st <- le[order(start), , drop = FALSE]
  starter <- sort(start)
  starter1 <- c(starter[1], starter[-length(starter)])
  index <- c(TRUE, (starter != starter1)[-1])
  starter <- starter[index]
  val1 <- apply(matrix(starter, ncol = 1), 1, function(x, Y) sum(x >=
                                                                   Y), stop)
  val1 <- c(val1[1], diff(val1), length(stop) - val1[length(val1)])
  olp.st <- (apply(lp.st[n:1, , drop = FALSE], 2, cumsum)[n:1,
                                                          , drop = FALSE])[index, , drop = FALSE]
  olp.st <- apply(olp.st, 2, function(x) rep(c(x, 0), val1))
  olp <- olp - olp.st
  olp <- olp[event == 1, ]
  olp <- apply(fup * olp, 1, sum)
  ole.st <- cumsum(le.st[n:1])[n:1][index]
  ole.st <- rep(c(ole.st, 0), val1)
  ole <- ole - ole.st
  ole <- ole[event == 1] * lambda0
  s0 <- ole + olp
  sc <- NULL
  zb <- NULL
  kzb <- NULL
  f1 <- function(x) rep(mean(x), length(x))
  f2 <- function(x) apply(x, 2, f1)
  f3 <- function(x) apply(x, 1:2, f1)
  ties <- length(unique(stop[event == 1])) != length(stop[event ==
                                                            1])
  for (k in 1:rem) {
    zlp <- apply((z[, k] * lp)[n:1, , drop = FALSE], 2, cumsum)[n:1,
                                                                , drop = FALSE]
    zlp.st <- (apply((z[, k] * lp.st)[n:1, , drop = FALSE],
                     2, cumsum)[n:1, , drop = FALSE])[index, , drop = FALSE]
    zlp.st <- apply(zlp.st, 2, function(x) rep(c(x, 0), val1))
    zlp <- zlp - zlp.st
    zlp <- zlp[event == 1, , drop = FALSE]
    zlp <- apply(fup * zlp, 1, sum)
    zle <- cumsum((z[, k] * le)[n:1])[n:1]
    zle.st <- cumsum((z[, k] * le.st)[n:1])[n:1][index]
    zle.st <- rep(c(zle.st, 0), val1)
    zle <- zle - zle.st
    zle <- zle[event == 1]
    zle <- zle * lambda0
    s1 <- zle + zlp
    zb <- cbind(zb, s1/s0)
    kzb <- cbind(kzb, zle/s0)
  }
  s1ties <- cbind(zb, kzb)
  if (ties) {
    s1ties <- by(s1ties, stop[event == 1], f2)
    s1ties <- do.call("rbind", s1ties)
  }
  zb <- s1ties[, 1:rem, drop = FALSE]
  kzb <- s1ties[, -(1:rem), drop = FALSE]
  sc <- z[event == 1, , drop = FALSE] - zb
  row.names(sc) <- stop[event == 1]
  out.temp <- function(x) outer(x, x, FUN = "*")
  krez <- rez <- array(matrix(NA, ncol = rem, nrow = rem),
                       dim = c(rem, rem, sum(event == 1)))
  for (a in 1:rem) {
    for (b in a:rem) {
      zzlp <- apply((z[, a] * z[, b] * lp)[n:1, , drop = FALSE],
                    2, cumsum)[n:1, , drop = FALSE]
      zzlp.st <- (apply((z[, a] * z[, b] * lp.st)[n:1,
                                                  , drop = FALSE], 2, cumsum)[n:1, , drop = FALSE])[index,
                                                                                                    , drop = FALSE]
      zzlp.st <- apply(zzlp.st, 2, function(x) rep(c(x,
                                                     0), val1))
      zzlp <- zzlp - zzlp.st
      zzlp <- zzlp[event == 1, , drop = FALSE]
      zzlp <- apply(fup * zzlp, 1, sum)
      zzle <- cumsum((z[, a] * z[, b] * le)[n:1])[n:1]
      zzle.st <- cumsum((z[, a] * z[, b] * le.st)[n:1])[n:1][index]
      zzle.st <- rep(c(zzle.st, 0), val1)
      zzle <- zzle - zzle.st
      zzle <- zzle[event == 1]
      zzle <- zzle * lambda0
      s2 <- zzlp + zzle
      s20 <- s2/s0
      ks20 <- zzle/s0
      s2ties <- cbind(s20, ks20)
      if (ties) {
        s2ties <- by(s2ties, stop[event == 1], f2)
        s2ties <- do.call("rbind", s2ties)
      }
      rez[a, b, ] <- rez[b, a, ] <- s2ties[, 1]
      krez[a, b, ] <- krez[b, a, ] <- s2ties[, 2]
    }
  }
  juhu <- apply(zb, 1, out.temp)
  if (is.null(dim(juhu)))
    juhu1 <- array(data = matrix(juhu, ncol = a), dim = c(a,
                                                          a, length(zb[, 1])))
  else juhu1 <- array(data = apply(juhu, 2, matrix, ncol = a),
                      dim = c(a, a, length(zb[, 1])))
  varr <- rez - juhu1
  kjuhu <- apply(cbind(zb, kzb), 1, function(x) outer(x[1:rem],
                                                      x[-(1:rem)], FUN = "*"))
  if (is.null(dim(kjuhu)))
    kjuhu1 <- array(data = matrix(kjuhu, ncol = rem), dim = c(rem,
                                                              rem, length(zb[, 1])))
  else kjuhu1 <- array(data = apply(kjuhu, 2, matrix, ncol = rem),
                       dim = c(rem, rem, length(zb[, 1])))
  kvarr <- krez - kjuhu1
  for (i in 1:dim(varr)[1]) varr[i, i, which(varr[i, i, ] <
                                               0)] <- 0
  for (i in 1:dim(kvarr)[1]) kvarr[i, i, which(kvarr[i, i,
  ] < 0)] <- 0
  varr1 <- apply(varr, 1:2, sum)
  kvarr1 <- apply(kvarr, 1:2, sum)
  if (type == "schoenfeld")
    out <- list(res = sc, varr1 = varr1, varr = varr, kvarr = kvarr,
                kvarr1 = kvarr1)
  out
}






#' Test the Proportional Hazards Assumption for Relative Survival Regression
#' Models
#'
#' Test the proportional hazards assumption for relative survival models
#' (\code{rsadd}, \code{rsmul} or \code{rstrans}) by forming a Brownian Bridge.
#'
#'
#' @aliases rs.br plot.rs.br print.rs.br
#' @param fit the result of fitting a relative survival model, using the
#' \code{rsadd}, \code{rsmul} or \code{rstrans} function.
#' @param sc partial residuals calculated by the \code{resid} function. This is
#' used to save time if several tests are to be calculated on these residuals
#' and can otherwise be omitted.
#' @param rho a number controlling the weigths of residuals. The weights are
#' the number of individuals at risk at each event time to the power
#' \code{rho}. The default is \code{rho=0}, which sets all weigths to 1.
#' @param test a character string specifying the test to be performed on
#' Brownian bridge. Possible values are \code{"max"} (default), which tests the
#' maximum absolute value of the bridge, and \code{cvm}, which calculates the
#' Cramer Von Mises statistic.
#' @param global should a global Brownian bridge test be performed, in addition
#' to the per-variable tests
#' @return an object of class \code{rs.br}. This function would usually be
#' followed by both a print and a plot of the result. The plot gives a Brownian
#' bridge for each of the variables. The horizontal lines are the 95% and 99%
#' confidence intervals for the maximum absolute value of the Brownian bridge
#' @seealso \code{\link{rsadd}}, \code{rsmul}, \code{rstrans},
#' \code{\link{resid}}.
#' @references Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005)
#' "Goodness of fit of relative survival models." Statistics in Medicine,
#' \bold{24}: 3911--3925.
#'
#' Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
#' Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#' 		ratetable=slopop,data=rdata,int=5)
#' rsbr <- rs.br(fit)
#' rsbr
#' plot(rsbr)
#'
rs.br <- function (fit, sc, rho = 0, test = "max", global = TRUE)
{
  test <- match.arg(test,c("max","cvm"))
  if (inherits(fit, "rsadd")) {
    if (missing(sc))
      sc <- resid(fit, "schoenfeld")
    sresid <- sc$res
    varr <- sc$varr
    sresid <- as.matrix(sresid)
  }
  else {
    coef <- fit$coef
    options(warn = -1)
    sc <- coxph.detail(fit)
    options(warn = 0)
    sresid <- sc$score
    varr <- sc$imat
    if (is.null(dim(varr)))
      varr <- array(varr, dim = c(1, 1, length(varr)))
    sresid <- as.matrix(sresid)
  }
  if (inherits(fit, "coxph")) {
    if(is.null(fit$data)){
      temp <- fit$y
      class(temp) <- "matrix"
      if(ncol(fit$y)==2)temp <- data.frame(rep(0,nrow(fit$y)),temp)
      if(is.null(fit$x))stop("The coxph model should be called with x=TRUE argument")
      fit$data <- data.frame(temp,fit$x)
      names(fit$data)[1:3] <- c("start","Y","stat")
    }
  }
  data <- fit$data[order(fit$data$Y), ]
  time <- data$Y[data$stat == 1]
  ties <- (length(unique(time)) != length(time))
  keep <- 1:(ncol(sresid))
  options(warn = -1)
  scaled <- NULL
  varnova <- NULL
  if (ncol(sresid) == 1) {
    varr <- varr[1, 1, ]
    scaled <- sresid/sqrt(varr)
  }
  else { for (i in 1:ncol(sresid)) varnova <- cbind(varnova,varr[i,i,])
  scaled <- sresid/sqrt(varnova)
  }

  options(warn = 0)
  nvar <- ncol(sresid)
  survfit <- getFromNamespace("survfit", "survival")
  temp <- survfit(fit$y~1, type = "kaplan-meier")
  n.risk <- temp$n.risk
  n.time <- temp$time
  if (temp$type == "right") {
    cji <- matrix(fit$y, ncol = 2)
    n.risk <- n.risk[match(cji[cji[, 2] == 1, 1], n.time)]
  }
  else {
    cji <- matrix(fit$y, ncol = 3)
    n.risk <- n.risk[match(cji[cji[, 3] == 1, 2], n.time)]
  }
  n.risk <- sort(n.risk, decreasing = TRUE)
  varnames <- names(fit$coef)[keep]
  u2 <- function(bb) {
    n <- length(bb)
    1/n * (sum(bb^2) - sum(bb)^2/n)
  }
  wc <- function(x, k = 1000) {
    a <- 1
    for (i in 1:k) a <- a + 2 * (-1)^i * exp(-2 * i^2 * pi^2 *
                                               x)
    a
  }
  brp <- function(x, n = 1000) {
    a <- 1
    for (i in 1:n) a <- a - 2 * (-1)^(i - 1) * exp(-2 * i^2 *
                                                     x^2)
    a
  }
  global <- as.numeric(global & ncol(sresid) > 1)
  table <- NULL
  bbt <- as.list(1:(nvar + global))
  for (i in 1:nvar) {
    if (nvar != 1)
      usable <- which(varr[i, i, ] > 1e-12)
    else usable <- which(varr > 1e-12)
    w <- (n.risk[usable])^rho
    w <- w/sum(w)
    if (nvar != 1) {
      sci <- scaled[usable, i]
    }
    else sci <- scaled[usable]
    if (ties) {
      if (inherits(fit, "rsadd")) {
        sci <- as.vector(by(sci, time[usable], function(x) sum(x)/sqrt(length(x))))
        w <- as.vector(by(w, time[usable], sum))
      }
      else {
        w <- w * as.vector(table(time))[usable]
        w <- w/sum(w)
      }
    }
    sci <- sci * sqrt(w)
    timescale <- cumsum(w)
    bm <- cumsum(sci)
    bb <- bm - timescale * bm[length(bm)]
    if (test == "max")
      table <- rbind(table, c(max(abs(bb)), 1 - brp(max(abs(bb)))))
    else if (test == "cvm")
      table <- rbind(table, c(u2(bb), 1 - wc(u2(bb))))
    bbt[[i]] <- cbind(timescale, bb)
  }
  if (inherits(fit, "rsadd")) {
    beta <- fit$coef[1:(length(fit$coef) - length(fit$int) +  1)]
  }
  else beta <- fit$coef
  if (global) {
    qform <- function(matrix, vector) t(vector) %*% matrix %*%
      vector
    diagonal <- apply(varr, 3, diag)
    sumdiag <- apply(diagonal, 2, sum)
    usable <- which(sumdiag > 1e-12)
    score <- t(beta) %*% t(sresid[usable, ])
    varr <- varr[, , usable]
    qf <- apply(varr, 3, qform, vector = beta)
    w <- (n.risk[usable])^rho
    w <- w/sum(w)
    sci <- score/(qf)^0.5
    if (ties) {
      if (inherits(fit, "rsadd")) {
        sci <- as.vector(by(t(sci), time[usable], function(x) sum(x)/sqrt(length(x))))
        w <- as.vector(by(w, time[usable], sum))
      }
      else {
        w <- w * as.vector(table(time))
        w <- w/sum(w)
      }
    }
    sci <- sci * sqrt(w)
    timescale <- cumsum(w)
    bm <- cumsum(sci)
    bb <- bm - timescale * bm[length(bm)]
    if (test == "max")
      table <- rbind(table, c(max(abs(bb)), 1 - brp(max(abs(bb)))))
    else if (test == "cvm")
      table <- rbind(table, c(u2(bb), 1 - wc(u2(bb))))
    bbt[[nvar + 1]] <- cbind(timescale, bb)
    varnames <- c(varnames, "GLOBAL")
  }
  dimnames(table) <- list(varnames, c(test, "p"))
  out <- list(table = table, bbt = bbt, rho = rho)
  class(out) <- "rs.br"
  out
}



#' Behaviour of Covariates in Time for Relative Survival Regression Models
#'
#' Calculates the scaled partial residuals of a relative survival model
#' (\code{rsadd}, \code{rsmul} or \code{rstrans})
#'
#'
#' @param fit the result of fitting an additive relative survival model, using
#' the \code{rsadd}, \code{rsmul} or \code{rstrans} function.
#'
#' In the case of multiplicative and transformation models the output is
#' identical to \code{cox.zph} function, except no test is performed.
#' @param sc partial residuals calculated by the \code{resid} function. This is
#' used to save time if several tests are to be calculated on these residuals
#' and can otherwise be omitted.
#' @param transform a character string specifying how the survival times should
#' be transformed. Possible values are \code{"km"}, \code{"rank"},
#' \code{"identity"} and \code{log}. The default is \code{"identity"}.
#' @param var.type a character string specifying the variance used to scale the
#' residuals. Possible values are \code{"each"}, which estimates the variance
#' for each residual separately, and \code{sum}(default), which assumes the
#' same variance for all the residuals.
#' @return an object of class \code{rs.zph}. This function would usually be
#' followed by a plot of the result. The plot gives an estimate of the
#' time-dependent coefficient \code{beta(t)}. If the proportional hazards
#' assumption is true, \code{beta(t)} will be a horizontal line.
#' @seealso \code{\link{rsadd}}, \code{rsmul}, \code{rstrans},
#' \code{\link{resid}}, \code{\link[survival:cox.zph]{survival::cox.zph}}.
#' @references Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005)
#' "Goodness of fit of relative survival models." Statistics in Medicine,
#' \bold{24}: 3911--3925.
#'
#' Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
#' Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#' 	ratetable=slopop,data=rdata,int=5)
#' rszph <- rs.zph(fit)
#' plot(rszph)
#'
rs.zph <- function (fit, sc, transform = "identity", var.type = "sum")
{
  if (inherits(fit, "rsadd")) {
    if (missing(sc))
      sc <- resid(fit, "schoenfeld")
    sresid <- sc$res
    varr <- sc$kvarr
    fvar <- solve(sc$kvarr1)
    sresid <- as.matrix(sresid)
  }
  else {
    coef <- fit$coef
    options(warn = -1)
    sc <- coxph.detail(fit)
    options(warn = 0)
    sresid <- as.matrix(resid(fit, "schoenfeld"))
    varr <- sc$imat
    fvar <- fit$var
  }
  data <- fit$data[order(fit$data$Y), ]
  time <- data$Y
  stat <- data$stat

  if (!inherits(fit, "rsadd")) {
    ties <- as.vector(table(time[stat==1]))
    if(is.null(dim(varr))) varr <- rep(varr/ties,ties)
    else{
      varr <- apply(varr,1:2,function(x)rep(x/ties,ties))
      varr <- aperm(varr,c(2,3,1))
    }
  }
  keep <- 1:(length(fit$coef) - length(fit$int) + 1)
  varnames <- names(fit$coef)[keep]
  nvar <- length(varnames)
  ndead <- length(sresid)/nvar
  if (inherits(fit, "rsadd"))
    times <- time[stat == 1]
  else times <- sc$time
  if (is.character(transform)) {
    tname <- transform
    ttimes <- switch(transform, identity = times, rank = rank(times),
                     log = log(times), km = {
                       fity <- Surv(time, stat)
                       temp <- survfit(fity~1)
                       t1 <- temp$surv[temp$n.event > 0]
                       t2 <- temp$n.event[temp$n.event > 0]
                       km <- rep(c(1, t1), c(t2, 0))
                       if (is.null(attr(sresid, "strata")))
                         1 - km
                       else (1 - km[sort.list(sort.list(times))])
                     }, stop("Unrecognized transform"))
  }
  else {
    tname <- deparse(substitute(transform))
    ttimes <- transform(times)
  }
  if (var.type == "each") {
    invV <- apply(varr, 3, function(x) try(solve(x), silent = TRUE))
    if (length(invV) == length(varr)){
      if(!is.numeric(invV)){
        usable <- rep(FALSE, dim(varr)[3])
        options(warn=-1)
        invV <- as.numeric(invV)
        usable[1:(min(which(is.na(invV)))-1)] <- TRUE
        invV <- invV[usable]
        sresid <- sresid[usable,,drop=FALSE]
        options(warn=0)
      }
      else usable <- rep(TRUE, dim(varr)[3])
    }
    else {
      usable <- unlist(lapply(invV, is.matrix))
      if (!any(usable))
        stop("All the matrices are singular")
      invV <- invV[usable]
      sresid <- sresid[usable, , drop = FALSE]
    }
    di1 <- dim(varr)[1]
    di3 <- sum(usable)
    u <- array(data = matrix(unlist(invV), ncol = di1), dim = c(di1,
                                                                di1, di3))
    uv <- cbind(matrix(u, ncol = di1, byrow = TRUE), as.vector(t(sresid)))
    uv <- array(as.vector(t(uv)), dim = c(di1 + 1, di1, di3))
    r2 <- t(apply(uv, 3, function(x) x[1:di1, ] %*% x[di1 +
                                                        1, ]))
    r2 <- matrix(r2, ncol = di1)
    whr2 <-  apply(r2<100,1,function(x)!any(x==FALSE))
    usable <- as.logical(usable*whr2)
    r2 <- r2[usable,,drop=FALSE]
    u <- u[,,usable]
    dimnames(r2) <- list(times[usable], varnames)
    temp <- list(x = ttimes[usable], y = r2 + outer(rep(1,
                                                        sum(usable)), fit$coef[keep]), var = u, call = call,
                 transform = tname)
  }
  else if (var.type == "sum") {
    xx <- ttimes - mean(ttimes)
    r2 <- t(fvar %*% t(sresid) * ndead)
    r2 <- as.matrix(r2)
    dimnames(r2) <- list(times, varnames)
    temp <- list(x = ttimes, y = r2 + outer(rep(1, ndead),
                                            fit$coef[keep]), var = fvar, transform = tname)
  }
  else stop("Unknown 'var.type'")
  class(temp) <- "rs.zph"
  temp
}



#' Graphical Inspection of Proportional Hazards Assumption in Relative Survival
#' Models
#'
#' Displays a graph of the scaled partial residuals, along with a smooth curve.
#'
#'
#' @param x result of the \code{rs.zph} function.
#' @param resid a logical value, if \code{TRUE} the residuals are included on
#' the plot, as well as the smooth fit.
#' @param df the degrees of freedom for the fitted natural spline, \code{df=2}
#' leads to a linear fit.
#' @param nsmo number of points used to plot the fitted spline.
#' @param var the set of variables for which plots are desired.  By default,
#' plots are produced in turn for each variable of a model.  Selection of a
#' single variable allows other features to be added to the plot, e.g., a
#' horizontal line at zero or a main title.
#' @param cex a numerical value giving the amount by which plotting text and
#' symbols should be scaled relative to the default.
#' @param add logical, if \code{TRUE} the plot is added to an existing plot
#' @param col a specification for the default plotting color.
#' @param lty the line type.
#' @param xlab x axis label.
#' @param ylab y axis label.
#' @param xscale units for x axis, default is 1, i.e. days.
#' @param ... Additional arguments passed to the \code{plot} function.
#' @seealso \code{\link{rs.zph}}, \code{\link[survival:plot.cox.zph]{survival::plot.cox.zph}}
#' @references Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005)
#' "Goodness of fit of relative survival models." Statistics in Medicine,
#' \bold{24}: 3911-3925.
#'
#' Package: Pohar M., Stare J. (2006) "Relative survival analysis in R."
#' Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741-1749, 2007.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' fit <- rsadd(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241),
#'              ratetable=slopop,data=rdata,int=5)
#' rszph <- rs.zph(fit)
#' plot(rszph)
#'
plot.rs.zph <- function (x,resid = TRUE, df = 4, nsmo = 40, var, cex = 1,  add = FALSE, col = 1,
                         lty = 1, xlab, ylab, xscale = 1, ...)
{
  #require(splines)
  xx <- x$x
  if(x$transform=="identity")xx <- xx/xscale
  yy <- x$y
  d <- nrow(yy)
  df <- max(df)
  nvar <- ncol(yy)
  pred.x <- seq(from = min(xx), to = max(xx), length = nsmo)
  temp <- c(pred.x, xx)
  lmat <- splines::ns(temp, df = df, intercept = TRUE)
  pmat <- lmat[1:nsmo, ]
  xmat <- lmat[-(1:nsmo), ]
  qmat <- qr(xmat)
  if (missing(ylab))
    ylab <- paste("Beta(t) for", dimnames(yy)[[2]])
  if (missing(xlab))
    xlab <- "Time"
  if (missing(var))
    var <- 1:nvar
  else {
    if (is.character(var))
      var <- match(var, dimnames(yy)[[2]])
    if (any(is.na(var)) || max(var) > nvar || min(var) <
        1)
      stop("Invalid variable requested")
  }
  if (x$transform == "log") {
    xx <- exp(xx)
    pred.x <- exp(pred.x)
  }
  else if (x$transform != "identity") {
    xtime <- as.numeric(dimnames(yy)[[1]])/xscale
    apr1 <- approx(xx, xtime, seq(min(xx), max(xx), length = 17)[2 *
                                                                   (1:8)])
    temp <- signif(apr1$y, 2)
    apr2 <- approx(xtime, xx, temp)
    xaxisval <- apr2$y
    xaxislab <- rep("", 8)
    for (i in 1:8) xaxislab[i] <- format(temp[i])
  }
  for (i in var) {
    y <- yy[, i]
    yhat <- pmat %*% qr.coef(qmat, y)
    yr <- range(yhat, y)
    if (!add) {
      if (x$transform == "identity")
        plot(range(xx), yr, type = "n", xlab = xlab, ylab = ylab[i],...)
      else if (x$transform == "log")
        plot(range(xx), yr, type = "n", xlab = xlab, ylab = ylab[i],log = "x", ...)
      else {
        plot(range(xx), yr, type = "n", xlab = xlab, ylab = ylab[i],axes = FALSE, ...)
        axis(1, xaxisval, xaxislab)
        axis(2)
        box()
      }
    }
    if (resid)
      points(xx, y, cex = cex, col = col)
    lines(pred.x, yhat, col = col, lty = lty)
  }
}

plot.rs.br <- function (x, var, ylim = c(-2, 2), xlab, ylab, ...)
{
  bbt <- x$bbt
  par(ask = TRUE)
  if (missing(var))
    var <- 1:nrow(x$table)
  ychange <- FALSE
  if (missing(ylab))
    ylab <- paste("Brownian bridge for", row.names(x$table))
  else {
    if (length(ylab) == 1 & nrow(x$table) > 1)
      ylab <- rep(ylab, nrow(x$table))
  }
  if (missing(xlab))
    xlab <- "Time"
  for (i in var) {
    timescale <- bbt[[i]][, 1]
    bb <- bbt[[i]][, 2]
    plot(c(0, timescale), c(0, bb), type = "l", ylim = ylim,
         xlab = xlab, ylab = ylab[i], ...)
    abline(h = 1.36, col = 2)
    abline(h = 1.63, col = 2)
    abline(h = -1.36, col = 2)
    abline(h = -1.63, col = 2)
  }
  par(ask = FALSE)
}


Kernmatch <- function (t, tv, b, tD, nt4)
{
  kmat <- NULL
  for (it in 1:(length(nt4) - 1)) {
    kmat1 <- (outer(t[(nt4[it] + 1):nt4[it + 1]], tv, "-")/b[it])
    kmat1 <- kmat1^(kmat1 >= 0)
    kmat <- rbind(kmat, pmax(1 - kmat1^2, 0) * (1.5/b[it]))
  }
  kmat
}

kernerleftch <- function (td, b, nt4)
{
  n <- length(td)
  ttemp <- td[td >= b[1]]
  ntemp <- length(ttemp)
  if (ntemp == n)
    nt4 <- c(0, nt4[-1])
  else {
    nfirst <- n - ntemp
    nt4 <- c(0, 1:nfirst, nt4[-1])
    b <- c(td[1:nfirst], b)
  }
  krn <- Kernmatch(td, td, b, max(td), nt4)
  krn
}




#' Inverse transforming of time in Relative Survival
#'
#' This function can be used when predicting in Relative Survival using the
#' transformed time regression model (using \code{rstrans} function). It
#' inverses the time from Y to T in relative survival using the given
#' ratetable. The times Y can be produced with the \code{rstrans} function, in
#' which case, this is the reverse function. This function does the
#' transformation for one person at a time.
#'
#' Works only with ratetables that are split by age, sex and year. Transforming
#' can be computationally intensive, use lower and/or upper to guess the
#' interval of the result and thus speed up the function.
#'
#' @param y time in Y.
#' @param age age of the individual.  Must be in days.
#' @param sex sex of the individual. Must be coded in the same way as in the
#' \code{ratetable}.
#' @param year date of diagnosis. Must be in a Date or POSIXt format.
#' @param scale numeric value to scale the results. If \code{ratetable} is in
#' units/day, \code{scale = 365.241} causes the output to be reported in years.
#' @param ratetable a table of event rates, such as \code{survexp.us}.
#' @param lower the lower bound of interval where the result is expected. This
#' argument is optional, but, if given, can shorten the time the function needs
#' to calculate the result.
#' @param upper the upper bound of interval where the result is expected. See
#' \code{lower}.
#' @return A list of values \item{T}{the original time} \item{Y}{the
#' transformed time}.
#' @seealso \code{\link{rstrans}}
#' @references Package: Pohar M., Stare J. (2006) "Relative survival analysis
#' in R."  Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741-1749.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' invtime(y = 0.1, age = 23011, sex = 1, year = as.Date('1986-01-01'), ratetable = slopop)
#'
invtime <- function (y = 0.1, age = 23011, sex = "male", year = as.Date('1986-01-01'), scale = 1,
                     ratetable = relsurv::slopop, lower, upper)
{
  if (!is.numeric(age))
    stop("\"age\" must be numeric", call. = FALSE)
  if (!is.numeric(y))
    stop("\"y\" must be numeric", call. = FALSE)
  if (!is.numeric(scale))
    stop("\"scale\" must be numeric", call. = FALSE)
  temp <- data.frame(age = age, sex = I(sex), year = year)
  if (missing(lower)) {
    if (!missing(upper))
      stop("Argument \"lower\" is missing, with no default",
           call. = FALSE)
    nyears <- round((110 - age/365.241))
    tab <- data.frame(age = rep(age, nyears), sex = I(rep(sex,
                                                          nyears)), year = rep(year, nyears))
    vred <- 1 - survexp(c(0, 1:(nyears - 1)) * 365.241 ~ ratetable(age = age,
                                                                   sex = sex, year = year), ratetable = ratetable, data = tab,
                        cohort = FALSE)
    place <- sum(vred <= y)
    if (place == 0)
      lower <- 0
    else lower <- floor((place - 1) * 365.241 - place)
    upper <- ceiling(place * 365.241 + place)
  }
  else {
    if (missing(upper))
      stop("Argument \"upper\" is missing, with no default",
           call. = FALSE)
    if (!is.integer(lower))
      lower <- floor(lower)
    if (!is.integer(upper))
      upper <- ceiling(upper)
    if (upper <= lower)
      stop("'upper' must be higher than 'lower'", call. = FALSE)
  }
  lower <- max(0, lower)
  tab <- data.frame(age = rep(age, upper - lower + 1), sex = I(rep(sex,
                                                                   upper - lower + 1)), year = rep(year, upper - lower +
                                                                                                     1))
  vred <- 1 - survexp((lower:upper) ~ ratetable(age = age,
                                                sex = sex, year = year), ratetable = ratetable, data = tab,
                      cohort = FALSE)
  place <- sum(vred <= y)
  if (place == 0)
    warning(paste("The event happened on or before day",
                  lower), call. = FALSE)
  if (place == length(vred))
    warning(paste("The event happened on or after day", upper),
            call. = FALSE)
  t <- (place + lower - 1)/scale
  age <- round(age/365.241, 0.01)
  return(list(age, sex, year, Y = y, T = t))
}






#' Fit Andersen et al Multiplicative Regression Model for Relative Survival
#'
#' Fits the Andersen et al multiplicative regression model in relative
#' survival. An extension of the coxph function using relative survival.
#'
#' NOTE: The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with by the \code{rmap} argument. For example, if
#' age is in years in the data set but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, such as \code{slopop}.
#' @param int the number of follow-up years used for calculating survival(the
#' data are censored after this time-point). If missing, it is set the the
#' maximum observed follow-up time.
#' @param na.action a missing-data filter function, applied to the model.frame,
#' after any subset argument has been used.  Default is
#' \code{options()$na.action}.
#' @param init vector of initial values of the iteration.  Default initial
#' value is zero for all variables.
#' @param method the default method \code{mul} assumes hazard to be constant on
#' yearly intervals. Method \code{mul1} uses the ratetable to determine the
#' time points when hazard changes. The \code{mul1} method is therefore more
#' accurate, but at the same time can be more computationally intensive.
#' @param control a list of parameters for controlling the fitting process.
#' See the documentation for \code{coxph.control} for details.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @param ... Other arguments will be passed to \code{coxph.control}.
#' @return an object of class \code{coxph} with an additional item:
#' \item{basehaz}{Cumulative baseline hazard (population values are seen as
#' offset) at centered values of covariates.}
#' @seealso \code{\link{rsadd}}, \code{\link{rstrans}}.
#' @references Method: Andersen, P.K., Borch-Johnsen, K., Deckert, T., Green,
#' A., Hougaard, P., Keiding, N. and Kreiner, S. (1985) "A Cox regression model
#' for relative mortality and its application to diabetes mellitus survival
#' data.", Biometrics, \bold{41}: 921--932.
#'
#' Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
#' Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #fit a multiplicative model
#' #note that the variable year is given in days since 01.01.1960 and that
#' #age must be multiplied by 365.241 in order to be expressed in days.
#' fit <- rsmul(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241),
#'             ratetable=slopop,data=rdata)
#'
#'
#' #check the goodness of fit
#' rs.br(fit)
#'
#'
rsmul <- function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,
                   int, na.action, init, method = "mul", control,rmap, ...)
{
  #require(survival)

  if (!missing(rmap)) {
    rmap <- substitute(rmap)
  }
  rform <- rformulate(formula,data, ratetable, na.action,rmap,int)


  U <- rform$data
  if (missing(int))
    int <- ceiling(max(rform$Y/365.241))
  if(length(int)!=1)int <- max(int)
  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)
  if (method == "mul") {
    U <- survsplit(U, cut = (1:int) * 365.241, end = "Y",
                   event = "stat", start = "start", episode = "epi")
    fk <- (attributes(rform$ratetable)$factor != 1)
    nfk <- length(fk)
    U[, 4:(nfk + 3)] <- U[, 4:(nfk + 3)] + 365.241 * (U$epi) %*%
      t(fk)
    nsk <- dim(U)[1]
    xx <- exp_prep(U[, 4:(nfk + 3),drop=FALSE], 365.241, rform$ratetable)
    lambda <- -log(xx)/365.241
  }
  else if (method == "mul1") {
    U$id <- 1:dim(U)[1]
    my.fun <- function(x, attcut, nfk, fk) {
      intr <- NULL
      for (i in 1:nfk) {
        if (fk[i]) {
          n1 <- max(findInterval(as.numeric(x[3 + i]) +
                                   as.numeric(x[1]), attcut[[i]]) + 1, 2)
          n2 <- findInterval(as.numeric(x[3 + i]) + as.numeric(x[2]),
                             attcut[[i]])
          if (n2 > n1 & length(attcut[[i]] > 1)) {
            if (n2 > length(attcut[[i]]))
              n2 <- length(attcut[[i]])
            intr <- c(intr, as.numeric(attcut[[i]][n1:n2]) -
                        as.numeric(x[3 + i]))
          }
        }
      }
      intr <- sort(unique(c(intr, as.numeric(x[2]))))
      intr
    }
    attcut <- attributes(rform$ratetable)$cutpoints
    intr <- apply(U[, 1:(3 + nfk)], 1, my.fun, attcut, nfk,
                  fk)
    dolg <- unlist(lapply(intr, length))
    newdata <- lapply(U, rep, dolg)
    stoptime <- unlist(intr)
    starttime <- c(-1, stoptime[-length(stoptime)])
    first <- newdata$id != c(-1, newdata$id[-length(newdata$id)])
    starttime[first] <- newdata$start[first]
    last <- newdata$id != c(newdata$id[-1], -1)
    event <- rep(0, length(newdata$id))
    event[last] <- newdata$stat[last]
    U <- do.call("data.frame", newdata)
    U$start <- starttime
    U$Y <- stoptime
    U$stat <- event
    U[, 4:(nfk + 3)] <- U[, 4:(nfk + 3)] + (U$start) %*%
      t(fk)
    nsk <- dim(U)[1]
    xx <- exp_prep(U[, 4:(nfk + 3),drop=FALSE], 1, rform$ratetable)
    lambda <- -log(xx)/1
  }
  else stop("'method' must be one of 'mul' or 'mul1'")
  U$lambda <- log(lambda)
  if (rform$m == 0)
    fit <- coxph(Surv(start, Y, stat) ~ 1 + offset(lambda),
                 data = U, init = init, control = control, x = TRUE,
                 ...)
  else {
    xmat <- as.matrix(U[, (3 + nfk + 1):(ncol(U) - 2)])
    fit <- coxph(Surv(start, Y, stat) ~ xmat + offset(lambda),
                 data = U, init = init, control = control, x = TRUE,
                 ...)
    names(fit[[1]]) <- names(U)[(3 + nfk + 1):(ncol(U) -
                                                 2)]
  }
  class(fit) <- c("rsmul",class(fit))
  fit$basehaz <- basehaz(fit)			#NEW 2.05
  fit$data <- rform$data
  fit$call <- match.call()
  fit$int <- int
  if (length(rform$na.action))
    fit$na.action <- rform$na.action
  fit
}



#' Fit Cox Proportional Hazards Model in Transformed Time
#'
#' The function transforms each person's time to his/her probability of dying
#' at that time according to the ratetable. It then fits the Cox proportional
#' hazards model with the transformed times as a response. It can also be used
#' for calculatin the transformed times (no covariates are needed in the
#' formula for that purpose).
#'
#' NOTE: The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with by the \code{rmap} argument. For example, if
#' age is in years in the data set but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.  A side product of this
#' function are the transformed times - stored in teh \code{y} object of the
#' output. To get these times, covariates are of course irrelevant.
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, such as \code{slopop}.
#' @param int the number of follow-up years used for calculating survival(the
#' rest is censored). If missing, it is set the the maximum observed follow-up
#' time.
#' @param na.action a missing-data filter function, applied to the model.frame,
#' after any subset argument has been used.  Default is
#' \code{options()$na.action}.
#' @param init vector of initial values of the iteration.  Default initial
#' value is zero for all variables.
#' @param control a list of parameters for controlling the fitting process.
#' See the documentation for \code{coxph.control} for details.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @param ... other arguments will be passed to \code{coxph.control}.
#' @return an object of class \code{coxph}. See \code{coxph.object} and
#' \code{coxph.detail} for details.  \item{y}{ an object of class \code{Surv}
#' containing the transformed times (these times do not depend on covariates).
#' }
#' @seealso \code{\link{rsmul}}, \code{\link{invtime}}, \code{\link{rsadd}},
#' \code{\link[survival:survexp]{survival::survexp}}.
#' @references Method: Stare J., Henderson R., Pohar M. (2005) "An individual
#' measure for relative survival." Journal of the Royal Statistical Society:
#' Series C, \bold{54} 115--126.
#'
#' Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
#' Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#'
#' #fit a Cox model using the transformed times
#' #note that the variable year is given in days since 01.01.1960 and that
#' #age must be multiplied by 365.241 in order to be expressed in days.
#' fit <- rstrans(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241,
#'         sex=sex,year=year),ratetable=slopop,data=rdata)
#'
#'
#' #check the goodness of fit
#' rs.br(fit)
#'
rstrans <- function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,
                     int, na.action, init, control,rmap, ...)
{
  if (!missing(rmap)) {
    rmap <- substitute(rmap)
  }
  rform <- rformulate(formula, data, ratetable, na.action, rmap, int)


  if (missing(int))
    int <- ceiling(max(rform$Y/365.241))
  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)
  if (rform$type == "counting") {
    start <- 1 - exp_prep(rform$R, rform$start, rform$ratetable)
  }
  else start <- rep(0, rform$n)
  stop <- 1 - exp_prep(rform$R, rform$Y, rform$ratetable)
  if(any(stop==0&rform$Y!=0))stop[stop==0&rform$Y!=0] <- .Machine$double.eps
  if(length(int)!=1)int <- max(int)
  data <- rform$data
  stat <- rform$status
  if (rform$m == 0) {
    if (rform$type == "counting")
      fit <- coxph(Surv(start, stop, stat) ~ 1,
                   init = init, control = control, x = TRUE, ...)
    else fit <- coxph(Surv(stop, stat) ~ 1,
                      init = init, control = control, x = TRUE, ...)
  }
  else {
    xmat <- as.matrix(data[, (4 + nfk):ncol(data)])
    fit <- coxph(Surv(start, stop, stat) ~ xmat,
                 init = init, control = control, x = TRUE, ...)
    names(fit[[1]]) <- names(rform$X)
  }
  fit$call <- match.call()
  if (length(rform$na.action))
    fit$na.action <- rform$na.action
  data$start <- start
  data$Y <- stop
  fit$data <- data
  fit$int <- int
  return(fit)
}


#' Reorganize Data into a Ratetable Object
#'
#' The function assists in reorganizing certain types of data into a ratetable
#' object.
#'
#' This function only applies for ratetables that are organized by age, sex and
#' year.
#'
#' @param men a matrix containing the yearly (conditional) probabilities of one
#' year survival for men. Rows represent age (increasing 1 year per
#' line,starting with 0), the columns represent cohort years (the limits are in
#' \code{yearlim}, the increase is in \code{int.length}.
#' @param women a matrix containing the yearly (conditional) probabilities of
#' one year survival for women.
#' @param yearlim the first and last cohort year given in the tables.
#' @param int.length the length of intervals in which cohort years are given.
#' @return An object of class \code{ratetable}.
#' @seealso \code{\link[survival:ratetable]{survival::ratetable}}.
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' men <- cbind(exp(-365.241*exp(-14.5+.08*(0:100))),exp(-365*exp(-14.7+.085*(0:100))))
#' women <- cbind(exp(-365.241*exp(-15.5+.085*(0:100))),exp(-365*exp(-15.7+.09*(0:100))))
#' table <- transrate(men,women,yearlim=c(1980,1990),int.length=10)
#'
transrate <- function (men, women, yearlim, int.length = 1)
{
  if (any(dim(men) != dim(women)))
    stop("The men and women matrices must be of the same size. \n In case of missing values at the end carry the last value forward")
  if ((yearlim[2] - yearlim[1])/int.length + 1 != dim(men)[2])
    stop("'yearlim' cannot be divided into intervals of equal length")
  if (!is.matrix(men) | !is.matrix(women))
    stop("input tables must be of class matrix")
  dimi <- dim(men)
  temp <- array(c(men, women), dim = c(dimi, 2))
  temp <- -log(temp)/365.241
  temp <- aperm(temp, c(1, 3, 2))
  cp <- as.Date(apply(matrix(yearlim[1] + int.length * (0:(dimi[2] -
                                                             1)), ncol = 1), 1, function(x) {
                                                               paste(x, "-01-01", sep = "")
                                                             }))
  attributes(temp) <- list(dim = c(dimi[1], 2, dimi[2]), dimnames = list(age=as.character(0:(dimi[1] -
                                                                                               1)), sex=c("male", "female"), year=as.character(yearlim[1] + int.length *
                                                                                                                                                 (0:(dimi[2] - 1)))), dimid = c("age", "sex", "year"),
                           factor = c(0, 1, 0),type=c(2,1,3), cutpoints = list((0:(dimi[1] - 1)) *
                                                                                 (365.241), NULL, cp), class = "ratetable")
  attributes(temp)$summary <- function (R)
  {
    x <- c(format(round(min(R[, 1])/365.241, 1)), format(round(max(R[,
                                                                     1])/365.241, 1)), sum(R[, 2] == 1), sum(R[, 2] == 2))
    x2 <- as.character(as.Date(c(min(R[, 3]), max(R[, 3])), origin=as.Date('1970-01-01')))
    paste("  age ranges from", x[1], "to", x[2], "years\n", " male:",
          x[3], " female:", x[4], "\n", " date of entry from",
          x2[1], "to", x2[2], "\n")
  }
  temp
}



#' Reorganize Data obtained from Human Life-Table Database into a Ratetable
#' Object
#'
#' The function assists in reorganizing the .txt files obtained from Human
#' Life-Table Database (http://www.lifetable.de -> Data by Country) into a
#' ratetable object.
#'
#' This function works with any table organised in the format provided by the
#' Human Life-Table Database, but currently only works with TypeLT 1 (i.e. age
#' intervals of length 1). The age must always start with value 0, but can end
#' at different values (when that happens, the last value is carried forward).
#' The rates between the cutpoints are taken to be constant.
#'
#' @param file a vector of file names which the data are to be read from. Must
#' be in .tex format and in the same format as the files in Human Life-Table
#' Database.
#' @param cut.year a vector of cutpoints for years. Must be specified when the
#' year spans in the files are not consecutive.
#' @param race a vector of race names for the input files.
#' @return An object of class \code{ratetable}.
#' @seealso \code{\link[survival:ratetable]{survival::ratetable}}, \code{\link{transrate.hmd}},
#' \code{\link{joinrate}}, \code{\link{transrate}}.
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' \dontrun{
#' finpop <- transrate.hld(c("FIN_1981-85.txt","FIN_1986-90.txt","FIN_1991-95.txt"))
#' }
#' \dontrun{
#' nzpop <- transrate.hld(c("NZL_1980-82_Non-maori.txt","NZL_1985-87_Non-maori.txt",
#' 	 "NZL_1980-82_Maori.txt","NZL_1985-87_Maori.txt"),
#' 	 cut.year=c(1980,1985),race=rep(c("nonmaori","maori"),each=2))
#' }
#'
transrate.hld <- function(file, cut.year,race){
  nfiles <- length(file)
  data <- NULL
  for(it in 1:nfiles){
    tdata <- read.table(file[it],sep=",",header=TRUE)
    if(!any(tdata$TypeLT==1)) stop("Currently only TypeLT 1 is implemented")
    names(tdata) <- gsub(".","",names(tdata),fixed=TRUE)
    tdata <- tdata[,c("Country","Year1","Year2","TypeLT","Sex","Age","AgeInt","qx")]
    tdata <- tdata[tdata$TypeLT==1,]		#NEW - prej sem gledala tudi AgeInt, izkaze se, da ni treba. pri q(x) bi bilo vseeno tudi, ce bi gledala TypeLT=3.
    tdata <- tdata[!is.na(tdata$AgeInt),]		#NEW - vrzem ven zadnji interval, ki gre v neskoncnost in vsi umrejo (inf hazard)
    if(!missing(race))tdata$race <- rep(race[it],nrow(tdata))
    data <- rbind(data,tdata)
  }
  if(length(unique(data$Country))>1)warning("The data belongs to different countries")
  data <- data[order(data$Year1,data$Age),]
  data$qx <- as.character(data$qx)
  options(warn = -1)
  data$qx[data$qx=="."] <- NA
  data$qx <- as.numeric(data$qx)
  options(warn = 0)
  if(missing(cut.year)){
    y1 <-  unique(data$Year1)
    y2 <-  unique(data$Year2)
    if(any(apply(cbind(y1[-1],y2[-length(y2)]),1,diff)!=-1))warning("Data is not given for all the cut.year between the minimum and the maximum, use argument 'cut.year'")
  }
  else
    y1 <- cut.year
  if(length(y1)!=length(unique(data$Year1)))stop("Length 'cut.year' must match the number of unique values of Year1")
  cp <- as.Date(apply(matrix(y1,ncol=1),1,function(x){paste(x,"-01-01",sep="")}))
  dn2 <- as.character(y1)
  amax <- max(data$Age)
  a.fun <- function(data,amax){
    mdata <- data[data$Sex==1,]
    wdata <- data[data$Sex==2,]
    men <-NULL
    women <- NULL
    k <- sum(mdata$Age==0)
    mind <- c(which(mdata$Age[-nrow(mdata)] != mdata$Age[-1]-1),nrow(mdata))
    wind <- c(which(wdata$Age[-nrow(wdata)] != wdata$Age[-1]-1),nrow(wdata))
    mst <- wst <- 1
    for(it in 1:k){
      qx <- mdata[mst:mind[it],]$qx
      lqx <- length(qx)
      if(lqx!=amax+1){
        nmiss <- amax + 1 - lqx
        qx <- c(qx,rep(qx[lqx],nmiss))
      }
      naqx <- max(which(!is.na(qx)))
      if(naqx!=amax+1) qx[(naqx+1):(amax+1)] <- qx[naqx]
      men <- cbind(men,qx)
      mst <- mind[it]+1
      qx <- wdata[wst:wind[it],]$qx
      lqx <- length(qx)
      if(lqx!=amax+1){
        nmiss <- amax + 1 - lqx
        qx <- c(qx,rep(qx[lqx],nmiss))
      }
      naqx <- max(which(!is.na(qx)))
      if(naqx!=amax+1) qx[(naqx+1):(amax+1)] <- qx[naqx]
      women <- cbind(women,qx)
      wst <- wind[it]+1
    }
    men<- -log(1-men)/365.241
    women<- -log(1-women)/365.241
    dims <- c(dim(men),2)
    array(c(men,women),dim=dims)
  }
  if(missing(race)){
    out <- a.fun(data,amax)
    dims <- dim(out)
    attributes(out)<-list(
      dim=dims,
      dimnames=list(as.character(0:amax),as.character(y1),c("male","female")),
      dimid=c("age","year","sex"),
      factor=c(0,0,1),type=c(2,3,1),
      cutpoints=list((0:amax)*(365.241),cp,NULL),
      class="ratetable"
    )


  }
  else{
    race.val <- unique(race)
    if(length(race)!=length(file))stop("Length of 'race' must match the number of files")
    for(it in 1:length(race.val)){
      if(it==1){
        out <- a.fun(data[data$race==race.val[it],],amax)
        dims <- dim(out)
        out <- array(out,dim=c(dims,1))
      }
      else{
        out1 <- array(a.fun(data[data$race==race.val[it],],amax),dim=c(dims,1))
        out <- array(c(out,out1),dim=c(dims,it))
      }
    }
    attributes(out)<-list(
      dim=c(dims,it),
      dimnames=list(age=as.character(0:amax),year=as.character(y1),sex=c("male","female"),race=race.val),
      dimid=c("age","year","sex","race"),
      factor=c(0,0,1,1),type=c(2,3,1,1),
      cutpoints=list((0:amax)*(365.241),cp,NULL,NULL),
      class="ratetable"
    )
  }
  attributes(out)$summary <- function (R)
  {
    x <- c(format(round(min(R[, 1])/365.241, 1)), format(round(max(R[,
                                                                     1])/365.241, 1)), sum(R[, 3] == 1), sum(R[, 3] == 2))
    x2 <- as.character(as.Date(c(min(R[, 2]), max(R[, 2])), origin=as.Date('1970-01-01')))
    paste("  age ranges from", x[1], "to", x[2], "years\n", " male:",
          x[3], " female:", x[4], "\n", " date of entry from",
          x2[1], "to", x2[2], "\n")
  }
  out
}



#' Reorganize Data obtained from Human Mortality Database into a Ratetable
#' Object
#'
#' The function assists in reorganizing the .txt files obtained from Human
#' Mortality Database (http://www.mortality.org) into a ratetable object.
#'
#' This function works automatically with tables organised in the format
#' provided by the Human Mortality Database. Download Life Tables for Males and
#' Females separately from the column named 1x1 (period life tables, organized
#' by date of death, yearly cutpoints for age as well as calendar year).
#'
#' If you wish to provide the data in the required format by yourself, note
#' that the only two columns needed are calendar year (Year) and probability of
#' death (qx). Death probabilities must be calculated up to age 110 (in yearly
#' intervals).
#'
#' @param male a .txt file, containing the data on males.
#' @param female a .txt file, containing the data on females.
#' @return An object of class \code{ratetable}.
#' @seealso \code{\link[survival:ratetable]{survival::ratetable}}, \code{\link{transrate.hld}},
#' \code{\link{joinrate}}, \code{\link{transrate}}.
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#'
#' \dontrun{
#' auspop <- transrate.hmd("mltper_1x1.txt","fltper_1x1.txt")
#' }
#'
transrate.hmd <- function(male,female){
  nfiles <- 2
  men <- try(read.table(male,sep="",header=TRUE),silent=TRUE)
  if(inherits(men, "try-error")){ men <- read.table(male,sep="",header=TRUE,skip=1)}
  men <- men[,c("Year","Age","qx")]
  y1 <- sort(unique(men$Year))
  ndata <- nrow(men)/111
  if(round(ndata)!=ndata)stop("Each year must contain ages from 0 to 110")
  men <- matrix(men$qx, ncol=ndata)
  men <- matrix(as.numeric(men),ncol=ndata)
  women <- try(read.table(female,sep="",header=TRUE),silent=TRUE)
  if(inherits(women, "try-error")) {women <- read.table(female,sep="",header=TRUE,skip=1)}
  women <- women[,"qx"]
  if(length(women)!=length(men))stop("Number of rows in the table must be equal for both sexes")
  women <- matrix(women, ncol=ndata)
  women <- matrix(as.numeric(women),ncol=ndata)
  cp <- as.Date(apply(matrix(y1, ncol = 1), 1, function(x) {
    paste(x, '-01-01', sep = "")
  }))
  dn2 <- as.character(y1)
  tfun <- function(vec){
    ind <- which(vec == 1 | is.na(vec))
    if(length(ind)>0)vec[min(ind):length(vec)] <- 0.999
    vec
  }
  men <- apply(men,2,tfun)
  women <- apply(women,2,tfun)
  men<- -log(1-men)/365.241
  women<- -log(1-women)/365.241
  nr <- nrow(men)-1
  dims <- c(dim(men),2)
  out <- array(c(men,women),dim=dims)
  attributes(out)<-list(
    dim=dims,
    dimnames=list(age=as.character(0:nr),year=as.character(y1),sex=c("male","female")),
    dimid=c("age","year","sex"),
    factor=c(0,0,1),type=c(2,3,1),
    cutpoints=list((0:nr)*(365.241),cp,NULL),
    class="ratetable"
  )
  attributes(out)$summary <- function (R)
  {
    x <- c(format(round(min(R[, 1])/365.241, 1)), format(round(max(R[,
                                                                     1])/365.241, 1)), sum(R[, 3] == 1), sum(R[, 3] == 2))
    x2 <- as.character(as.Date(c(min(R[, 2]), max(R[, 2])), origin=as.Date('1970-01-01')))
    paste("  age ranges from", x[1], "to", x[2], "years\n", " male:",
          x[3], " female:", x[4], "\n", " date of entry from",
          x2[1], "to", x2[2], "\n")
  }
  out
}






#' Join ratetables
#'
#' The function joins two or more objects organized as \code{ratetable} by
#' adding a new dimension.
#'
#' This function joins two or more \code{ratetable} objects by adding a new
#' dimension. The cutpoints of all the rate tables are compared; if merge=FALSE
#' (default) only the common intervals are kept, otherwise if merge=TRUE all
#' intervals are added (and hazards are extrapolated).
#' If the intervals defined by the cutpoints are not of
#' the same length, a warning message is displayed.  Each rate table must have
#' the same dimensions.
#'
#' @param tables a list of ratetables. If names are given, they are included as
#' \code{dimnames}.
#' @param dim.name the name of the added dimension.
#' @param merge if FALSE (default) only the intersect of all years/ages is taken.
#' If TRUE all possible years/ages are included (NOTE: in this case hazards are extrapolated
#' from earlier or later years/ages). This option only works for ratetables with dimensions
#  in the following order: age, year, sex.
#' @return An object of class \code{ratetable}.
#' @seealso \code{\link[survival:ratetable]{survival::ratetable}}, \code{\link{transrate.hld}},
#' \code{\link{transrate.hmd}}, \code{\link{transrate}}.
#' @references Package: Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741-1749.
#' @keywords survival
#' @examples
#'
#' #newpop <- joinrate(list(Arizona=survexp.az,Florida=survexp.fl,
#' #                   Minnesota=survexp.mn),dim.name="state")
#'
joinrate <- function(tables,dim.name="country", merge=FALSE){
  nfiles <- length(tables)
  if(is.null(names(tables))) names(tables) <- paste("D",1:nfiles,sep="")
  if(any(!unlist(lapply(tables,is.ratetable))))stop("Tables must be in ratetable format")
  if(length(attributes(tables[[1]])$dim)<3)stop("Currently implemented for ratetables with 3 or more dimensions")

  if(is.null(attr(tables[[1]],"dimid")))attr(tables[[1]],"dimid") <- names((attr(tables[[1]],"dimnames")))

  if(merge){
    # Find all possible years and ages:
    add.years <- sort(unique(unlist(lapply(1:length(tables),
                                           function(x) attributes(tables[[x]])$dimnames$year))))
    add.ages <- as.character(sort(as.integer(unique(unlist(lapply(1:length(tables),
                                                                  function(x) attributes(tables[[x]])$dimnames$age))))))

    # Add the extra years/ages:
    for(tb in 1:length(tables)){
      tables[[tb]] <- ratetable_add_vals(tables[[tb]], add.years, add.ages)
    }
  }

  mc_length <- length(attributes(tables[[1]])$dimid)

  for(it in 2:nfiles){
    if(is.null(attr(tables[[it]],"dimid")))attr(tables[[it]],"dimid") <- names((attr(tables[[it]],"dimnames")))

    if(length(attributes(tables[[it]])$dimid)!=mc_length)stop("Each ratetable must have the same number of dimensions")
    mc <- match(attributes(tables[[it]])$dimid,attributes(tables[[1]])$dimid,nomatch=0)
    if(any(mc)==0) stop("Each ratetable must have the same number of dimensions")
    if(any(mc!=1:mc_length)){
      atts <- attributes(tables[[it]])
      tables[[it]] <- aperm(tables[[it]],mc)
      atts$dimid <- atts$dimid[mc]
      atts$dimnames <- atts$dimnames[mc]
      atts$cutpoints <- atts$cutpoints[mc]
      atts$factor <- atts$factor[mc]
      atts$type <- atts$type[mc]
      atts$dim <- atts$dim[mc]
      attributes(tables[[it]]) <- atts
    }
  }

  # Check if lists are equal:
  list.eq <- function(l1,l2){
    n <- length(l1)
    rez <- rep(TRUE,n)
    for(it in 1:n){
      if(length(l1[[it]])!=length(l2[[it]]))rez[it] <- FALSE
      else if(any(l1[[it]]!=l2[[it]]))rez[it] <- FALSE
    }
    rez
  }


  equal <- rep(TRUE,mc_length)
  for(it in 2:nfiles){
    equal <- equal*list.eq(attributes(tables[[1]])$cutpoints,attributes(tables[[it]])$cutpoints)
  }


  kir <-  which(!equal)

  newat <- attributes(tables[[1]])
  # imena <- list(d1=NULL,d2=NULL,d3=NULL,d4=NULL)
  imena <- eval(parse(text=paste0('list(', paste0('d', 1:mc_length, '=NULL', collapse = ', '), ')')))

  for(jt in kir){
    listy <- NULL
    for(it in 1:nfiles){
      listy <- c(listy,attributes(tables[[it]])$cutpoints[[jt]])
    }

    imena[[jt]] <- names(table(listy)[table(listy) == nfiles])

    # if(inherits(attributes(tables[[it]])$cutpoints[[jt]], 'date')){
    #   imena[[jt]] <- as.date(as.integer(imena[[jt]]))
    # }

    if(!length(imena[[jt]]))stop(paste("There are no common cutpoints for dimension", attributes(tables[[1]])$dimid[jt]))
  }

  # Merging:
  for(it in 1:nfiles){
    keep <- lapply(dim(tables[[it]]),function(x)1:x)
    for(jt in kir){
      meci <- which(match(attributes(tables[[it]])$cutpoints[[jt]],imena[[jt]],nomatch=0)!=0)

      if(it==1){
        newat$dimnames[[jt]] <- attributes(tables[[it]])$dimnames[[jt]][meci]
        newat$dim[[jt]] <- length(imena[[jt]])
        newat$cutpoints[[jt]] <- attributes(tables[[it]])$cutpoints[[jt]][meci]
      }
      if(length(meci)>1){if(max(diff(meci)!=1))warning(paste("The cutpoints for ",attributes(tables[[1]])$dimid[jt] ," are not equally spaced",sep=""))}
      keep[[jt]] <- meci
    }
    if(mc_length==3){
      tables[[it]] <- tables[[it]][keep[[1]],keep[[2]],keep[[3]]]
    } else if(mc_length==4){
      tables[[it]] <- tables[[it]][keep[[1]],keep[[2]],keep[[3]],keep[[4]]]
    } else if(mc_length==5){
      tables[[it]] <- tables[[it]][keep[[1]],keep[[2]],keep[[3]],keep[[4]],keep[[5]]]
    } else if(mc_length==6){
      tables[[it]] <- tables[[it]][keep[[1]],keep[[2]],keep[[3]],keep[[4]],keep[[5]],keep[[6]]]
    }
  }

  # Finish final object:
  dims <- newat$dim
  out <- array(tables[[1]],dim=c(dims,1))
  for(it in 2:nfiles){
    out1 <- array(tables[[it]],dim=c(dims,1))
    out <- array(c(out,out1),dim=c(dims,it))
  }
  mc <- 1:(mc_length+1)
  if(any(newat$factor>1)){
    wh <- which(newat$factor>1)
    mc <- c(mc[-wh],wh)
    out <- aperm(out,mc)
  }
  newat$dim <- c(dims,nfiles)[mc]
  newat$dimid <- c(newat$dimid,dim.name)[mc]
  if(mc_length==3){
    newat$cutpoints <- list(newat$cutpoints[[1]],newat$cutpoints[[2]],newat$cutpoints[[3]],NULL)[mc]
  } else if(mc_length==4){
    newat$cutpoints <- list(newat$cutpoints[[1]],newat$cutpoints[[2]],newat$cutpoints[[3]],newat$cutpoints[[4]],NULL)[mc]
  } else if(mc_length==5){
    newat$cutpoints <- list(newat$cutpoints[[1]],newat$cutpoints[[2]],newat$cutpoints[[3]],newat$cutpoints[[4]],newat$cutpoints[[5]],NULL)[mc]
  } else if(mc_length==6){
    newat$cutpoints <- list(newat$cutpoints[[1]],newat$cutpoints[[2]],newat$cutpoints[[3]],newat$cutpoints[[4]],newat$cutpoints[[5]],newat$cutpoints[[6]],NULL)[mc]
  }
  newat$factor <- c(newat$factor,1)[mc]
  newat$type <- c(newat$type,1)[mc]
  if(mc_length==3){
    newat$dimnames <- list(newat$dimnames[[1]],newat$dimnames[[2]],newat$dimnames[[3]],names(tables))[mc]
  } else if(mc_length==4){
    newat$dimnames <- list(newat$dimnames[[1]],newat$dimnames[[2]],newat$dimnames[[3]],newat$dimnames[[4]],names(tables))[mc]
  } else if(mc_length==5){
    newat$dimnames <- list(newat$dimnames[[1]],newat$dimnames[[2]],newat$dimnames[[3]],newat$dimnames[[4]],newat$dimnames[[5]],names(tables))[mc]
  } else if(mc_length==6){
    newat$dimnames <- list(newat$dimnames[[1]],newat$dimnames[[2]],newat$dimnames[[3]],newat$dimnames[[4]],newat$dimnames[[5]],newat$dimnames[[6]],names(tables))[mc]
  }
  names(newat$dimnames) <- newat$dimid
  attributes(out) <- newat
  out
}
# joinrate <- function(tables,dim.name="country", merge=FALSE){
#   nfiles <- length(tables)
#   if(is.null(names(tables))) names(tables) <- paste("D",1:nfiles,sep="")
#   if(any(!unlist(lapply(tables,is.ratetable))))stop("Tables must be in ratetable format")
#   if(length(attributes(tables[[1]])$dim)!=3)stop("Currently implemented only for ratetables with 3 dimensions")
#
#   if(is.null(attr(tables[[1]],"dimid")))attr(tables[[1]],"dimid") <- names((attr(tables[[1]],"dimnames")))
#
#   if(merge){
#     # Find all possible years and ages:
#     add.years <- sort(unique(unlist(lapply(1:length(tables),
#                                            function(x) attributes(tables[[x]])$dimnames$year))))
#     add.ages <- as.character(sort(as.integer(unique(unlist(lapply(1:length(tables),
#                                                                   function(x) attributes(tables[[x]])$dimnames$age))))))
#
#     # Add the extra years/ages:
#     for(tb in 1:length(tables)){
#       tables[[tb]] <- ratetable_add_vals(tables[[tb]], add.years, add.ages)
#     }
#   }
#
#   for(it in 2:nfiles){
#     if(is.null(attr(tables[[it]],"dimid")))attr(tables[[it]],"dimid") <- names((attr(tables[[it]],"dimnames")))
#     if(length(attributes(tables[[it]])$dimid)!=3)stop("Each ratetable must have 3 dimensions: age, year and sex")
#     mc <- match(attributes(tables[[it]])$dimid,attributes(tables[[1]])$dimid,nomatch=0)
#     if(any(mc)==0) stop("Each ratetable must have 3 dimensions: age, year and sex")
#     if(any(mc!=1:3)){
#       atts <- attributes(tables[[it]])
#       tables[[it]] <- aperm(tables[[it]],mc)
#       atts$dimid <- atts$dimid[mc]
#       atts$dimnames <- atts$dimnames[mc]
#       atts$cutpoints <- atts$cutpoints[mc]
#       atts$factor <- atts$factor[mc]
#       atts$type <- atts$type[mc]
#       atts$dim <- atts$dim[mc]
#       attributes(tables[[it]]) <- atts
#     }
#   }
#
#   # Check if lists are equal:
#   list.eq <- function(l1,l2){
#     n <- length(l1)
#     rez <- rep(TRUE,n)
#     for(it in 1:n){
#       if(length(l1[[it]])!=length(l2[[it]]))rez[it] <- FALSE
#       else if(any(l1[[it]]!=l2[[it]]))rez[it] <- FALSE
#     }
#     rez
#   }
#
#
#   equal <- rep(TRUE,3)
#   for(it in 2:nfiles){
#     equal <- equal*list.eq(attributes(tables[[1]])$cutpoints,attributes(tables[[it]])$cutpoints)
#   }
#
#
#   kir <-  which(!equal)
#
#   newat <- attributes(tables[[1]])
#   imena <- list(d1=NULL,d2=NULL,d3=NULL)
#
#   for(jt in kir){
#     listy <- NULL
#     for(it in 1:nfiles){
#       listy <- c(listy,attributes(tables[[it]])$cutpoints[[jt]])
#     }
#
#     imena[[jt]] <- names(table(listy)[table(listy) == nfiles])
#
#     if(inherits(attributes(tables[[it]])$cutpoints[[jt]], 'date')){
#       imena[[jt]] <- as.date(as.integer(imena[[jt]]))
#     }
#
#     if(!length(imena[[jt]]))stop(paste("There are no common cutpoints for dimension", attributes(tables[[1]])$dimid[jt]))
#   }
#
#   # Merging:
#   for(it in 1:nfiles){
#     keep <- lapply(dim(tables[[it]]),function(x)1:x)
#     for(jt in kir){
#       meci <- which(match(attributes(tables[[it]])$cutpoints[[jt]],imena[[jt]],nomatch=0)!=0)
#
#       if(it==1){
#         newat$dimnames[[jt]] <- attributes(tables[[it]])$dimnames[[jt]][meci]
#         newat$dim[[jt]] <- length(imena[[jt]])
#         newat$cutpoints[[jt]] <- attributes(tables[[it]])$cutpoints[[jt]][meci]
#       }
#       if(length(meci)>1){if(max(diff(meci)!=1))warning(paste("The cutpoints for ",attributes(tables[[1]])$dimid[jt] ," are not equally spaced",sep=""))}
#       keep[[jt]] <- meci
#     }
#     tables[[it]] <- tables[[it]][keep[[1]],keep[[2]],keep[[3]]]
#   }
#
#   # Finish final object:
#   dims <- newat$dim
#   out <- array(tables[[1]],dim=c(dims,1))
#   for(it in 2:nfiles){
#     out1 <- array(tables[[it]],dim=c(dims,1))
#     out <- array(c(out,out1),dim=c(dims,it))
#   }
#   mc <- 1:4
#   if(any(newat$factor>1)){
#     wh <- which(newat$factor>1)
#     mc <- c(mc[-wh],wh)
#     out <- aperm(out,mc)
#   }
#   newat$dim <- c(dims,nfiles)[mc]
#   newat$dimid <- c(newat$dimid,dim.name)[mc]
#   newat$cutpoints <- list(newat$cutpoints[[1]],newat$cutpoints[[2]],newat$cutpoints[[3]],NULL)[mc]
#   newat$factor <- c(newat$factor,1)[mc]
#   newat$type <- c(newat$type,1)[mc]
#   newat$dimnames <- list(newat$dimnames[[1]],newat$dimnames[[2]],newat$dimnames[[3]],names(tables))[mc]
#   names(newat$dimnames) <- newat$dimid
#   attributes(out) <- newat
#   out
# }



mlfit <- function (b, p, x, offset, d, h, ds, y, maxiter, tol)
{
  for (nit in 1:maxiter) {
    b0 <- b
    fd <- matrix(0, p, 1)
    sd <- matrix(0, p, p)
    if (nit == 1) {
      ebx <- exp(x %*% b) * exp(offset)
      l0 <- sum(d * log(h + ebx) - ds - y * ebx)
    }
    for (it in 1:p) {
      fd[it, 1] <- sum((d/(h + ebx) - y) * x[, it] * ebx)
      for (jt in 1:p) sd[it, jt] = sum((d/(h + ebx) - d *
                                          ebx/(h + ebx)^2 - y) * x[, it] * x[, jt] * ebx)
    }
    b <- b - solve(sd) %*% fd
    ebx <- exp(x %*% b) * exp(offset)
    l <- sum(d * log(h + ebx) - ds - y * ebx)
    bd <- abs(b - b0)
    if (max(bd) < tol)
      break()
  }
  out <- list(b = b, sd = sd, nit = nit, loglik = c(l0, l))
  out
}

print.rs.br <- function (x, digits = max(options()$digits - 4, 3), ...)
{
  invisible(print(x$table, digits = digits))
  if (x$rho != 0)
    invisible(cat("Weighted Brownian bridge with rho=", x$rho,
                  "\n"))
}

print.rsadd <- function (x, digits = max(3, getOption("digits") - 3), ...)
{
  cat("\nCall: ", paste(deparse(x$call), sep = "\n", collapse = "\n"),
      "\n\n", sep = "", "\n")
  if (length(coef(x))) {
    cat("Coefficients")
    cat(":\n")
    print.default(format(x$coefficients, digits = digits),
                  print.gap = 2, quote = FALSE)
  }
  else cat("No coefficients\n\n")
  if(x$method=="EM")
    cat("\n", "Expected number of disease specific deaths: ",format(round(sum(x$Nie),2))," = ",format(round(100*sum(x$Nie)/sum(x$data$stat),1)),"% \n" ,sep="")
  if(x$method=="EM"|x$method=="max.lik"){
    chi <- 2*max((x$loglik[2]-x$loglik[1]),0)
    if(x$method=="EM")df <- length(x$coef)
    else df <- length(x$coef)-length(x$int)+1
    if(df>0){
      p.val <- 1- pchisq(chi,df)
      if(x$method=="max.lik")cat("\n")
      cat("Likelihood ratio test=",format(round(chi,2)),", on ",df," df, p=",format(p.val),"\n",sep="")
    }
    else cat("\n")
  }
  cat("n=",nrow(x$data),sep="")
  if(length(x$na.action))cat("  (",length(x$na.action)," observations deleted due to missing)",sep="")
  cat("\n")
  if (length(x$warnme))
    cat("\n", x$warnme, "\n\n")
  else cat("\n")
  invisible(x)
}

summary.rsadd <- function (object, correlation = FALSE, symbolic.cor = FALSE,
                           ...)
{
  if (inherits(object, "glm")) {
    p <- object$rank
    if (p > 0) {
      p1 <- 1:p
      Qr <- object$qr
      aliased <- is.na(coef(object))
      coef.p <- object$coefficients[Qr$pivot[p1]]
      covmat <- chol2inv(Qr$qr[p1, p1, drop = FALSE])
      dimnames(covmat) <- list(names(coef.p), names(coef.p))
      var.cf <- diag(covmat)
      s.err <- sqrt(var.cf)
      tvalue <- coef.p/s.err
      dn <- c("Estimate", "Std. Error")
      pvalue <- 2 * pnorm(-abs(tvalue))
      coef.table <- cbind(coef.p, s.err, tvalue, pvalue)
      dimnames(coef.table) <- list(names(coef.p), c(dn,
                                                    "z value", "Pr(>|z|)"))
      df.f <- NCOL(Qr$qr)
    }
    else {
      coef.table <- matrix(, 0, 4)
      dimnames(coef.table) <- list(NULL, c("Estimate",
                                           "Std. Error", "t value", "Pr(>|t|)"))
      covmat.unscaled <- covmat <- matrix(, 0, 0)
      aliased <- is.na(coef(object))
      df.f <- length(aliased)
    }
    ans <- c(object[c("call", "terms", "family", "iter",
                      "warnme")], list(coefficients = coef.table, var = covmat,
                                       aliased = aliased))
    if (correlation && p > 0) {
      dd <- s.err
      ans$correlation <- covmat/outer(dd, dd)
      ans$symbolic.cor <- symbolic.cor
    }
    class(ans) <- "summary.rsadd"
  }
  else if (inherits(object, "rsadd")) {
    aliased <- is.na(coef(object))
    coef.p <- object$coef
    var.cf <- diag(object$var)
    s.err <- sqrt(var.cf)
    tvalue <- coef.p/s.err
    dn <- c("Estimate", "Std. Error")
    pvalue <- 2 * pnorm(-abs(tvalue))
    coef.table <- cbind(coef.p, s.err, tvalue, pvalue)
    dimnames(coef.table) <- list(names(coef.p), c(dn, "z value",
                                                  "Pr(>|z|)"))
    ans <- c(object[c("call", "terms", "iter", "var")], list(coefficients = coef.table,
                                                             aliased = aliased))
    if (correlation && sum(aliased) != length(aliased)) {
      dd <- s.err
      ans$correlation <- object$var/outer(dd, dd)
      ans$symbolic.cor <- symbolic.cor
    }
    class(ans) <- "summary.rsadd"
  }
  else ans <- object
  return(ans)
}

print.summary.rsadd <- function (x, digits = max(3, getOption("digits") - 3), symbolic.cor = x$symbolic.cor,
                                 signif.stars = getOption("show.signif.stars"), ...)
{
  cat("\nCall:\n")
  cat(paste(deparse(x$call), sep = "\n", collapse = "\n"),
      "\n\n", sep = "")
  if (length(x$aliased) == 0) {
    cat("\nNo Coefficients\n")
  }
  else {
    cat("\nCoefficients:\n")
    coefs <- x$coefficients
    if (!is.null(aliased <- x$aliased) && any(aliased)) {
      cn <- names(aliased)
      coefs <- matrix(NA, length(aliased), 4, dimnames = list(cn,
                                                              colnames(coefs)))
      coefs[!aliased, ] <- x$coefficients
    }
    printCoefmat(coefs, digits = digits, signif.stars = signif.stars,
                 na.print = "NA", ...)
  }
  if (length(x$warnme))
    cat("\n", x$warnme, "\n")
  correl <- x$correlation
  if (!is.null(correl)) {
    p <- NCOL(correl)
    if (p > 1) {
      cat("\nCorrelation of Coefficients:\n")
      if (is.logical(symbolic.cor) && symbolic.cor) {
        print(symnum(correl, abbr.colnames = NULL))
      }
      else {
        correl <- format(round(correl, 2), nsmall = 2,
                         digits = digits)
        correl[!lower.tri(correl)] <- ""
        print(correl[-1, -p, drop = FALSE], quote = FALSE)
      }
    }
  }
  cat("\n")
  invisible(x)
}



#' Excess hazard function smoothing
#'
#' An Epanechnikov kernel function based smoother for smoothing the baseline
#' excess hazard calculated by the \code{rsadd} function with the \code{EM}
#' method.
#'
#' The function performs Epanechnikov kernel smoothing. The follow up time is
#' divided (according to percentiles of event times) into several intervals
#' (number of intervals defined by \code{n.bwin}) in which the width is
#' calculated as a factor of the maximum span between event times.  Boundary
#' effects are also taken into account on both sides.
#'
#' @param fit Fit from the additive relative survival model using the \code{EM}
#' method.
#' @param bwin The relative width of the smoothing window (default is 1).
#' @param times The times at which the smoother is to be evaluated. If missing,
#' it is evaluated at all event times.
#' @param n.bwin Number of times that the window width may change.
#' @param left If \code{FALSE} (default) smoothing is performed symmetrically,
#' if \code{TRUE} only leftside neighbours are considered.
#' @return A list with two components: \item{lambda}{the smoothed excess
#' baseline hazard function} \item{times}{the times at which the smoothed
#' excess baseline hazard is evaluated.}
#' @seealso \code{\link{rsadd}},
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#'
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#'
#' EM algorithm: Pohar Perme M., Henderson R., Stare, J. (2009) "An approach to
#' estimation in relative survival regression." Biostatistics, \bold{10}:
#' 136--146.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #fit an additive model with the EM method
#' fit <- rsadd(Surv(time,cens)~sex+age,rmap=list(age=age*365.241),
#' 		ratetable=slopop,data=rdata,int=5,method="EM")
#' sm <- epa(fit)
#' plot(sm$times,sm$lambda)
#'
epa <- function(fit,bwin,times,n.bwin=16,left=FALSE){
  #bwin ... width of the window, relative to the default (1)
  #fit ... EM fit
  #times... times at which the smoothed plot is calculated
  #n.bwin ... number of different windows
  #left ... only predictable smoothing
  utd <- fit$times
  if(missing(times))times <- seq(1,max(utd),length=100)
  if(max(times)>max(utd)){
    warning("Cannot extrapolate beyond max event time")
    times <- pmax(times,max(utd))
  }
  nutd <- length(utd)
  nt4 <- c(1,ceiling(nutd*(1:n.bwin)/n.bwin))
  if(missing(bwin))bwin <- rep(length(fit$times)/100,n.bwin)
  else bwin <- rep(bwin*length(fit$times)/100,n.bwin)
  for(it in 1:n.bwin){
    bwin[it] <- bwin[it]*max(diff(utd[nt4[it]:nt4[it+1]]))
  }
  while(utd[nt4[2]]<bwin[1]){		# ce je bwin velik, skrajsamo nt4
    nt4 <- nt4[-2]
    if(length(nt4)==1)break
  }
  #the smoothing matrix
  if(left) krn <- kernerleftch(utd,bwin,nt4)
  else krn <- kern(times,utd,bwin,nt4)
  lams <- pmax(krn%*%fit$lam0.ns,0)
  list(lambda=lams,times=times)				#	, weights=c(fit$times[1],diff(fit$times)))
}

Kern <- function (t, tv, b, tD, nt4)
{
  Rb <- max(tv)					#Right border
  kmat <- NULL
  tvs <- tv
  tv <- tv[-1]
  kt <- function(q,t)12*(t+1)/(1+q)^4*( (1-2*q)*t + (3*q^2-2*q+1)/2  )
  totcajti <- NULL
  for (it in 1:(length(nt4) - 1)) {
    cajti <- t[t>tvs[nt4[it]] & t<=tvs[nt4[it + 1]]]
    if(length(cajti)){
      q <- min( cajti/b[it],1,(Rb-cajti)/b[it])
      if(q<1 & length(cajti)>1){
        jc <- 1
        while(jc <=length(cajti)){
          qd <- pmin( cajti[jc:length(cajti)]/b[it],1,(Rb-cajti[jc:length(cajti)])/b[it])
          q <- qd[1]
          if(q==1){
            casi <- cajti[jc:length(cajti)][qd==1]
            q <- 1
            jc <- sum(qd==1)+jc
          }
          else{
            casi <- cajti[jc]
            jc <- jc+1
          }
          kmat1 <- outer(casi, tv, "-")/b[it]		#z - to je ok
          if(q<1){
            if(casi>b[it]) kmt1 <- -kmat1
            vr <- kt(q,kmat1)*(kmat1>=-1 & kmat1 <= q)
          }
          else vr <-  pmax((1 - kmat1^2) * .75,0)
          kmat <- rbind(kmat, vr/b[it])
          totcajti <- c(totcajti,casi)
        }
      }
      else{
        kmat1 <- outer(cajti, tv, "-")/b[it]		#z - to je ok
        q <- min( cajti/b[it],1)
        if(q<1)vr <- kt(q,kmat1)*(kmat1>=-1 & kmat1 <= q)
        else vr <-  pmax((1 - kmat1^2) * .75,0)
        kmat <- rbind(kmat, vr/b[it])
        totcajti <- c(totcajti,cajti)
      }#else
    }#if

  }#for
  kmat
}

kern <- function (times,td, b, nt4)
{
  n <- length(td)
  ttemp <- td[td >= b[1]]
  ntemp <- length(ttemp)
  if (ntemp == n)
    nt4 <- c(0, nt4[-1])
  td <- c(0,td)
  nt4 <- c(1,nt4+1)
  b <- c(b[1],b)
  krn <- Kern(times, td, b, max(td), nt4)
  krn
}

exp_prep <- function (x, y,ratetable,status,times,fast=FALSE,ys,prec,cmp=F,netweiDM=FALSE) {			#function that prepares the data for C function call

  #x= matrix of demographic covariates - each individual has one line
  #y= follow-up time for each individual (same length as nrow(x)!)
  #ratetable= rate table used for calculation
  #status= status for each individual (same length as nrow(x)!), not needed if we only need Spi, status needed for rs.surv
  #times= times at which we wish to evaluate the quantities, not needed if we only need Spi, times needed for rs.surv
  #fast=for mpp method only
  #netweiDM=should new netwei script be used

  x <- as.matrix(x)
  if (ncol(x) != length(dim(ratetable)))
    stop("x matrix does not match the rate table")
  atts <- attributes(ratetable)

  cuts <- atts$cutpoints

  if (is.null(atts$type)) {
    rfac <- atts$factor
    us.special <- (rfac > 1)
  }
  else {
    rfac <- 1 * (atts$type == 1)
    us.special <- (atts$type == 4)
  }
  if (length(rfac) != ncol(x))
    stop("Wrong length for rfac")


  if (any(us.special)) {
    if (sum(us.special) > 1)
      stop("Two columns marked for special handling as a US rate table")
    cols <- match(c("age", "year"), atts$dimid)
    if (any(is.na(cols)))
      stop("Ratetable does not have expected shape")
    if (exists("as.Date")) {
      bdate <- as.Date("1960/1/1") + (x[, cols[2]] - x[,
                                                       cols[1]])
      byear <- format(bdate, "%Y")
      offset <- as.numeric(bdate - as.Date(paste(byear,
                                                 "01/01", sep = "/")))
    } else stop("Can't find an appropriate date class\n")
    # Prej je bilo tole:
    # else if (exists("date.mdy")) {
    #   bdate <- as.date(x[, cols[2]] - x[, cols[1]])
    #   byear <- date.mdy(bdate)$year
    #   offset <- bdate - mdy.date(1, 1, byear)
    # }

    x[, cols[2]] <- x[, cols[2]] - offset
    if (any(rfac > 1)) {
      temp <- which(us.special)
      nyear <- length(cuts[[temp]])
      nint <- rfac[temp]
      cuts[[temp]] <- round(approx(nint * (1:nyear), cuts[[temp]],
                                   nint:(nint * nyear))$y - 1e-04)
    }
  }

  if(!missing(status)){		#the function was called from rs.surv
    if(length(status)!=nrow(x))    stop("Wrong length for status")

    if(missing(times))    times <- sort(unique(y))

    if (any(times < 0))
      stop("Negative time point requested")
    ntime <- length(times)
    if(missing(ys)) ys <- rep(0,length(y))
    #    times2 <- times
    #    times2[1] <- preci

    # It may be an integer...check and make it numeric. Otherwise problems in C functions:
    if(is.integer(x[1,1])){
      x <- apply(x, 2, as.numeric)
    }

    if(cmp)   temp <- .Call("cmpfast",  as.integer(rfac), 		#fast=pohar-perme or ederer2 - data from pop. tables only while under follow-up
                            as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                            x, y, ys,as.integer(status), times,PACKAGE="relsurv")
    else if(fast&!missing(prec))    temp <- .Call("netfastpinter2",  as.integer(rfac), 		#fast=pohar-perme or ederer2 - data from pop. tables only while under follow-up
                                                  as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                                                  x, y, ys,as.integer(status), times,prec,PACKAGE="relsurv")
    else if(fast&missing(prec))    temp <- .Call("netfastpinter",  as.integer(rfac), 		#fast=pohar-perme or ederer2 - data from pop. tables only while under follow-up
                                                 as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                                                 x, y, ys,as.integer(status), times,PACKAGE="relsurv")
    else if(netweiDM==TRUE)    temp <- .Call("netweiDM",  as.integer(rfac),
                                             as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                                             x, y, ys,as.integer(status), times,PACKAGE="relsurv")
    else    temp <- .Call("netwei",  as.integer(rfac),
                          as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                          x, y, as.integer(status), times,PACKAGE="relsurv")
  }
  else{				#only expected survival at time y is needed for each individual
    if(length(y)==1)y <- rep(y,nrow(x))
    if(length(y)!=nrow(x)) stop("Wrong length for status")
    temp <- .Call("expc",  as.integer(rfac),
                  as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                  x, y,PACKAGE="relsurv")
    temp  <- temp$surv
  }
  temp
}



#' Compute a Relative Survival Curve
#'
#' Computes an estimate of the relative survival curve using the Ederer I,
#' Ederer II method, Pohar-Perme method or the Hakulinen method
#'
#' NOTE: The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with by the \code{rmap} argument. For example, if
#' age is in years in the data set but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.
#'
#' The potential censoring times needed for the calculation of the expected
#' survival by the Hakulinen method are calculated automatically. The times of
#' censoring are left as they are, the times of events are replaced with
#' \code{fin.date - year}.
#'
#' The calculation of the Pohar-Perme estimate is more time consuming since
#' more data are needed from the population tables.  The old version of the
#' function, now named \code{rs.survo} can be used as a faster version for the
#' Hakulinen and Ederer II estimate.
#'
#' Numerical integration is required for Pohar-Perme estimate. The integration
#' precision is set with argument \code{precision}, which defaults to daily
#' intervals, a default that should give enough precision for any practical
#' purpose.
#'
#' Note that even though the estimate is always calculated using numerical
#' integration, only the values at event and censoring times are reported.
#' Hence, the function \code{plot} draws a step function in between and the
#' function \code{summary} reports the value at the last event or censoring
#' time before the specified time. If the output of the estimated values at
#' other points is required, this should be specified with argument
#' \code{add.times}.
#'
#' Standardized net survival can be also calculated:
#'
#' \eqn{SNS (t) = \sum_{j} w_j {NS}_j (t)}
#'
#' where \eqn{NS_j} is the net survival in the j-th group, \eqn{w_j}
#' is the weight for the j-th group and \eqn{SNS} is the standardized net survival.
#' \eqn{SNS} can be calculated by using the \code{weight.table} and \code{weight.names} arguments. The function
#' also returns the corresponding standard error and confidence interval.
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right. If no strata are used, \code{~1} should be
#' specified.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param na.action a missing-data filter function, applied to the model.frame,
#' after any subset argument has been used.  Default is
#' \code{options()$na.action}.
#' @param fin.date the date of the study ending, used for calculating the
#' potential follow-up times in the Hakulinen method. If missing, it is
#' calculated as \code{max(year+time)}.
#' @param method the method for calculating the relative survival. The options
#' are \code{pohar-perme}(default), \code{ederer1}, \code{ederer2} and
#' \code{hakulinen}.
#' @param conf.type one of \code{plain}, \code{log} (the default), or
#' \code{log-log}. The first option causes the standard intervals curve +- k
#' *se(curve), where k is determined from conf.int. The log option calculates
#' intervals based on the cumulative hazard or log(survival). The last option
#' bases intervals on the log hazard or log(-log(survival)).
#' @param conf.int the level for a two-sided confidence interval on the
#' survival curve(s). Default is 0.95.
#' @param type defines how survival estimates are to be calculated given the
#' hazards.  The default (\code{kaplan-meier}) calculates the product integral,
#' whereas the option \code{fleming-harrington} exponentiates the negative
#' cumulative hazard. Analogous to the usage in \code{survfit}.
#' @param add.times specific times at which the curve should be evaluated.
#' @param precision Precision for numerical integration. Default is 1, which
#' means that daily intervals are taken, the value may be decreased to get a
#' higher precision or increased to achieve a faster calculation. The
#' calculation intervals always include at least all times of event and
#' censoring as border points.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @param weight.table Default is NULL. If defined it is used for calculating
#' standardized net survival. Supply a data.frame
#' that contains the weights and group names for calculating the standardized net survival.
#' The group column also has to be included in the data argument.
#' For the theory see the details below.
#' @param weight.names Default is NULL. If defined it is used for calculating
#' standardized net survival. Supply a character vector of length two with the
#' names of the group and weight columns in \code{weight.table}.
#' @return a \code{survfit} object; see the help on \code{survfit.object} for
#' details.  The \code{survfit} methods are used for \code{print},
#' \code{summary}, \code{plot}, \code{lines}, and \code{points}.
#' @seealso \code{survfit}, \code{survexp}
#' @references Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric
#' Relative Survival Analysis with the R Package relsurv". Journal of
#' Statistical Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08" Theory:
#' Pohar Perme, M., Esteve, J., Rachet, B. (2016) "Analysing Population-Based
#' Cancer Survival - Settling the Controversies." BMC Cancer, 16 (933), 1-8.
#' doi:10.1186/s12885-016-2967-9. Theory: Pohar Perme, M., Stare, J., Esteve,
#' J. (2012) "On Estimation in Relative Survival", Biometrics, 68(1), 113-120.
#' doi:10.1111/j.1541-0420.2011.01640.x.
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #calculate the relative survival curve
#' #note that the variable year must be given in a date format and that
#' #age must be multiplied by 365.241 in order to be expressed in days.
#' rs.surv(Surv(time,cens)~sex,rmap=list(age=age*365.241), ratetable=slopop,data=rdata)
#'
#' # SNS:
#' wei <- data.frame(agegr=c('<54', '54-61', '62-70', '71-95'),
#'                   weight=c(0.2, 0.2, 0.3, 0.3))
#' rs.surv(Surv(time,cens)~1, rmap=list(age=age*365.241), ratetable=slopop, data=rdata,
#'    weight.table=wei, weight.names = c('agegr', 'weight'))
#'
rs.surv <- function(formula = formula(data), data = parent.frame(),ratetable = relsurv::slopop,
                    na.action, fin.date, method = "pohar-perme", conf.type = "log",
                    conf.int = 0.95,type="kaplan-meier",add.times,precision=1,rmap,
                    weight.table=NULL, weight.names=NULL){

  # SNS part:
  if(!is.null(weight.table) & (!is.numeric(weight.table))){
    call <- match.call()

    # MAKE ALL CHECKS:
    if(!(inherits(weight.table, 'data.frame')
       # & all(colnames(weight.table) %in% c('weight', 'group'))
       # & ncol(weight.table) == 2
       )){
      stop("In the argument weight.table you should supply a data.frame.")
    }

    if(!(inherits(weight.names, 'character'))){
      stop("In the argument weight.names you should supply a character vector of length 2 containing the names of the group and weight columns in weight.table.")
    }

    if(!all(weight.names %in% colnames(weight.table))){
      stop('The table in weight.table does not contain the columns given in weight.names.')
    }

    if(!(weight.names[1] %in% colnames(data))){
      stop('The data does not contain the grouping column (first value) given in weight.names.')
    }

    group_val <- weight.table[,weight.names[1]]
    weight_val <- weight.table[,weight.names[2]]

    if(!all(group_val %in% unique(data[,weight.names[1]]))){
      stop("Not all weight categories from weight.table are present in the data.")
    }

    if(!all(unique(data[,weight.names[1]]) %in% group_val)){
      stop("Not all weight categories from the data are present in weight.table.")
    }

    if(abs(sum(weight_val) - 1) > 1e-16){
      # weight.table$weight <- weight.table$weight/sum(weight.table$weight)
      stop('The sum of the weights in the weight.table argument is not equal to 1.')
    }

    # Find groups for SNS:
    groups_u <- unique(group_val)
    groups_l <- length(groups_u)

    # Fix rmap:
    if (!missing(rmap)) {
      rmap <- substitute(rmap)
    }

    # Get full model:
    mod0 <- rs.surv(formula, data, ratetable,
                    na.action, fin.date, method, conf.type,
                    conf.int, type, add.times, precision=1, rmap,
                    weight.table=-999) # add weights arg so that it is detected later for rmap

    # Prepare object for output:
    out <- mod0

    # Save separate models:
    mod_l <- list()

    # Save maximum follow-up times among groups:
    if('strata' %in% names(mod0)){
      max.times <- data.frame(matrix(NA, nrow=length(groups_u), ncol=length(mod0$strata)))
    } else{
      max.times <- data.frame(X1=rep(NA, length(groups_u)))
    }

    # Find max times:
    for(group in 1:length(groups_u)){
      data_tmp <- data[data[,weight.names[1]]==groups_u[group],]

      mod_tmp <- rs.surv(formula, data_tmp, ratetable,
                         na.action, fin.date, method, conf.type,
                         conf.int, type, add.times, precision=1, rmap,
                         weight.table=-999) # add weights arg so that it is detected later for rmap

      mod_tmp <- summary(mod_tmp, times=sort(unique(mod0$time)))

      mod_l[[group]] <- mod_tmp

      if('strata' %in% names(mod0)){
        tajms_groupby <- aggregate(time~strata,
                                   data=data.frame(time=mod_tmp$time, strata=mod_tmp$strata),
                                   FUN = \(x) max(x))
        max.times[group,] <- tajms_groupby$time

      } else{
        max.times[group,] <- mod_tmp$time[length(mod_tmp$time)]
      }
    }

    # Find minimum among maximum follow-up times:
    find.max.time <- sapply(max.times, min)

    # Define data.frame:
    if('strata' %in% names(mod0)){
      find.max.time.df <- data.frame(final.time=find.max.time, strata=names(mod0$strata))
    } else{
      find.max.time.df <- data.frame(final.time=find.max.time)
    }

    # Find true length:
    if('strata' %in% names(mod0)){
      df.length <- data.frame(time=mod0$time, strata=rep(names(mod0$strata), mod0$strata))
      df.length <- merge(x=df.length, y=find.max.time.df, by='strata', all.x = TRUE)
    } else{
      df.length <- data.frame(time=mod0$time)
      df.length$final.time <- find.max.time.df$final.time
    }
    df.length <- df.length[df.length$time <= df.length$final.time,] # subset(df.length, time<=final.time)

    # Merge data sets:
    for(group in 1:length(groups_u)){
      mod_tmp <- mod_l[[group]]

      # Prepare data set for this group:
      if('strata' %in% names(mod0)){
        df_g <- data.frame(time=mod_tmp$time, surv=mod_tmp$surv, std.err=mod_tmp$std.err, strata=mod_tmp$strata)
        df_g <- merge(x=df_g, y=find.max.time.df, by='strata', all.x = TRUE)
      } else{
        df_g <- data.frame(time=mod_tmp$time, surv=mod_tmp$surv, std.err=mod_tmp$std.err)
        df_g$final.time <- find.max.time.df$final.time
      }

      df_g <- df_g[df_g$time <= df_g$final.time,] # subset(df_g, time<=final.time)

      # Take only relevant values:
      if('strata' %in% names(mod0)){
        df_g$st <- paste0(df_g$strata, df_g$time)
        df.length$st <- paste0(df.length$strata, df.length$time)
        df_g <- df_g[df_g$st %in% df.length$st,]
      } else{
        df_g <- df_g[df_g$time %in% df.length$time,]
      }

      if(group == 1){
        # Define objects:
        srvs <- matrix(NA, nrow=nrow(df.length), ncol=length(groups_u))
        std.errs <- matrix(NA, nrow=nrow(df.length), ncol=length(groups_u))
        # Save values:
        srvs[,group] <- df_g$surv*weight_val[group_val == groups_u[group]]
        std.errs[,group] <- df_g$std.err^2*weight_val[group_val == groups_u[group]]^2 # mod_tmp$surv[whi]^2*

      } else{
        # Save values:
        srvs[,group] <- df_g$surv*weight_val[group_val == groups_u[group]]
        std.errs[,group] <- df_g$std.err^2*weight_val[group_val == groups_u[group]]^2 # mod_tmp$surv[whi]^2*
      }
    }

    # Calculate the survival and std.err (this is sd(log(SNS))):
    out$surv <- rowSums(srvs)
    out$std.err <- sqrt(rowSums(std.errs)) / out$surv

    # Find which will stay in n.risk, n.event, n.censor:
    wh.stay <- which(out$time %in% df.length$time)

    # Take the correct times:
    out$time <- df.length$time

    # Calculate CI:
    se.fac <- sqrt(qchisq(conf.int, 1))						#factor needed for confidence interval
    if (conf.type == "plain") {
      out$lower <- as.vector(out$surv - out$std.err * se.fac * 		#surv + fac*se
                               out$surv)
      out$upper <- as.vector(out$surv + out$std.err * se.fac *
                               out$surv)
    }
    else if (conf.type == "log") {						#on log scale and back
      out$lower <- exp(as.vector(log(out$surv) - out$std.err *
                                   se.fac))
      out$upper <- exp(as.vector(log(out$surv) + out$std.err *
                                   se.fac))
    }
    else if (conf.type == "log-log") {						#on log-log scale and back
      out$lower <- exp(-exp(as.vector(log(-log(out$surv)) -
                                        out$std.err * se.fac/log(out$surv))))
      out$upper <- exp(-exp(as.vector(log(-log(out$surv)) +
                                        out$std.err * se.fac/log(out$surv))))
    }

    out$call <- call

    if('strata' %in% names(mod0)){
      out$strata <- table(df.length$strata)
    }
    # Remove unneeded objects:
    out$n.risk <- out$n.risk[wh.stay]
    out$n.event <- out$n.event[wh.stay]
    out$n.censor <- out$n.censor[wh.stay]

    return(out)

    # Regular part:
  } else{
    call <- match.call()
    if (!missing(rmap)) {
      if(is.null(weight.table)){
        rmap <- substitute(rmap)
      } else{
        if(weight.table != -999){
          rmap <- substitute(rmap)
        }
      }
    }
    rform <- rformulate(formula,data, ratetable, na.action,rmap)
    data <- rform$data								#the data set
    type <- match.arg(type, c("kaplan-meier", "fleming-harrington"))		#method of hazard -> survival scale transformation
    type <- match(type, c("kaplan-meier", "fleming-harrington"))
    method <- match.arg(method,c("pohar-perme", "ederer2", "hakulinen","ederer1"))	#method of relative surv. curve estimation
    method <- match(method,c("pohar-perme", "ederer2", "hakulinen","ederer1"))
    conf.type <- match.arg(conf.type,c("plain","log","log-log"))		#conf. interval type

    if (method == 3) {						#need potential follow-up time for Hak. method
      R <- rform$R
      coll <- match("year", attributes(ratetable)$dimid)
      year <- R[, coll]							#calendar year in the data
      if (missing(fin.date))
        fin.date <- max(rform$Y + year)					#final date for everybody set to the last day observed
      Y2 <- rform$Y							#change into potential follow-up time
      if (length(fin.date) == 1) 						#if final date equal for everyone
        Y2[rform$status == 1] <- fin.date - year[rform$status == 1]#set pot.time for those that died (equal to censoring time for others)
      else if (length(fin.date) == nrow(rform$R))
        Y2[rform$status == 1] <- fin.date[rform$status ==
                                            1] - year[rform$status == 1]
      else stop("fin.date must be either one value or a vector of the same length as the data")
      status2 <- rep(0, nrow(rform$X))						#stat2=0 for everyone
    }
    p <- rform$m								#number of covariates

    # strata - use same approach as in survival::survfit.formula:
    indx <- match(c("formula", "data"), names(call),
                  nomatch = 0)
    temp2 <- call[c(1, indx)]
    temp2[[1L]] <- quote(stats::model.frame)
    mf <- eval.parent(temp2)
    n <- nrow(mf)
    Terms2 <- terms(formula, c("strata", "cluster"))
    ll <- attr(Terms2, "term.labels")
    # Check if ratetable in linear predictor, if yes remove it:
    if(any(grepl(pattern = 'ratetable', x = ll))){
      ll <- ll[!grepl(pattern = 'ratetable', x = ll)]
    }
    if (length(ll) == 0)
      data$Xs <- factor(rep(1, n))
    else data$Xs <- strata(mf[ll])

    # OLD
    # if (p > 0) 									#if covariates
    #   data$Xs <- strata(rform$X[, ,drop=FALSE ])				#make strata according to covariates
    # else data$Xs <- rep(1, nrow(data))						#if no covariates, just put 1

    se.fac <- sqrt(qchisq(conf.int, 1))						#factor needed for confidence interval
    out <- NULL
    out$n <- table(data$Xs)							#table of strata
    out$time <- out$n.risk <- out$n.event <- out$n.censor <- out$surv <- out$std.err <- out$strata <-  NULL
    #out$index <- out$strata0 <- NULL
    # out$index = indices of the original times from the data among the times used for calculations
    # out$strata0 = the same as out$strata but only on the original times from the data


    for (kt in 1:length(out$n)) {						#for each stratum
      inx <- which(data$Xs == names(out$n)[kt])				#individuals within this stratum
      tis <- sort(unique(rform$Y[inx])) #unique times

      #if (method == 1 & all.times == TRUE) tis <- sort(union(rform$Y[inx],as.numeric(1:max(floor(rform$Y[inx])))))	#1-day long intervals used - to take into the account the continuity of the pop. part
      if (method == 1 & !missing(add.times)){
        #tis <- sort(union(rform$Y[inx],as.numeric(1:max(floor(rform$Y[inx])))))	#1-day long intervals used - to take into the account the continuity of the pop. part
        add.times <- pmin(as.numeric(add.times),max(rform$Y[inx]))
        tis <- sort(union(rform$Y[inx],as.numeric(add.times)))	#1-day long intervals used - to take into the account the continuity of the pop. part
      }
      if(method==3)tis <- sort(unique(pmin(max(tis),c(tis,Y2[inx]))))				#add potential times in case of Hakulinen
      #out$index <- c(out$index, which(tis %in% rform$Y[inx])+length(out$time))

      temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=(method<3),prec=precision)	#calculate the values for each interval of time

      out$time <- c(out$time, tis)						#add times
      out$n.risk <- c(out$n.risk, temp$yi)					#add number at risk for each time
      out$n.event <- c(out$n.event, temp$dni)					#add number of events for each time
      out$n.censor <- c(out$n.censor,  c(-diff(temp$yi),temp$yi[length(temp$yi)]) - temp$dni) 	#add number of censored for each time

      if(method==1){ 								#pohar perme method
        #approximate1 <- (temp$yidlisi/temp$yisi +temp$yidlisitt/temp$yisitt)/2
        #approximate <- (temp$yidlisiw/temp$yisi +temp$yidlisiw/temp$yisitt)/2		#approximation for integration
        approximate <- temp$yidlisiw
        #haz <- temp$dnisi/temp$yisi - temp$yidlisi/temp$yisi		#cumulative hazard increment on each interval
        haz <- temp$dnisi/temp$yisi - approximate			#cumulative hazard increment on each interval
        out$std.err <- c(out$std.err, sqrt(cumsum(temp$dnisisq/(temp$yisi)^2)))  #standard error on each interval
      }
      else if(method==2){							#ederer2 method
        haz <- temp$dni/temp$yi - temp$yidli/temp$yi			#cumulative hazard increment on each interval
        out$std.err <- c(out$std.err, sqrt(cumsum(temp$dni/(temp$yi)^2)))  #standard error on each interval
      }
      else if(method==3){							#Hakulinen method
        temp2 <- exp_prep(rform$R[inx,,drop=FALSE],Y2[inx],ratetable,status2[inx],times=tis)	#calculate the values for each interval of time
        popsur <- exp(-cumsum(temp2$yisidli/temp2$yisis))			#population survival
        haz <- temp$dni/temp$yi						#observed hazard on each interval
        out$std.err <- c(out$std.err, sqrt(cumsum(temp$dni/(temp$yi)^2)))  #standard error on each interval
      }
      else if(method==4){							#Ederer I
        popsur <- temp$sis/length(inx)					#population survival
        haz <- temp$dni/temp$yi						#observed hazard on each interval
        out$std.err <- c(out$std.err, sqrt(cumsum(temp$dni/(temp$yi)^2)))  #standard error on each interval
      }
      if(type==2)survtemp <- exp(-cumsum(haz))
      else survtemp <-  cumprod(1-haz)
      if(method>2){
        survtemp <- survtemp/popsur
      }
      out$surv <- c(out$surv,survtemp)
      out$strata <- c(out$strata, length(tis))				#number of times in this strata
      #out$strata0 <- c(out$strata0, length(unique(rform$Y[inx])))
    }
    if (conf.type == "plain") {
      out$lower <- as.vector(out$surv - out$std.err * se.fac * 		#surv + fac*se
                               out$surv)
      out$upper <- as.vector(out$surv + out$std.err * se.fac *
                               out$surv)
    }
    else if (conf.type == "log") {						#on log scale and back
      out$lower <- exp(as.vector(log(out$surv) - out$std.err *
                                   se.fac))
      out$upper <- exp(as.vector(log(out$surv) + out$std.err *
                                   se.fac))
    }
    else if (conf.type == "log-log") {						#on log-log scale and back
      out$lower <- exp(-exp(as.vector(log(-log(out$surv)) -
                                        out$std.err * se.fac/log(out$surv))))
      out$upper <- exp(-exp(as.vector(log(-log(out$surv)) +
                                        out$std.err * se.fac/log(out$surv))))
    }

    names(out$strata) <-  names(out$n)
    out$n <- as.vector(out$n)

    #names(out$strata0) <- names(out$n)

    if (p == 0){
      out$strata <-  NULL						#if no covariates
    }

    #if (method != 1) out$index <- out$strata0 <- NULL # if method != pohar-perme
    out$conf.type <- conf.type
    out$conf.int <- conf.int
    out$method <- method
    out$call <- call
    out$type <- "right"
    class(out) <- c("survfit", "rs.surv")
    out
  }
}


#' Net Expected Sample Size Is Estimated
#'
#' Calculates how the sample size decreases in time due to population mortality
#'
#' The function calculates the sample size we can expect at a certain time
#' point if the patients die only due to population causes (population survival
#' * initial sample size in a certain category), i.e. the number of individuals
#' that remains at risk at given timepoints after the individuals who die due
#' to population causes are removed. The result should be used as a guideline
#' for the sensible length of follow-up interval when calculating the net
#' survival.
#'
#' The first column of the output reports the number of individuals at time 0.
#' The last column of the output reports the conditional expected (population)
#' survival time for each subgroup.
#'
#' @param formula a formula object, same as in \code{rs.surv}. The right-hand
#' side of the formula object includes the variable that defines the subgroups
#' (a variable of type \code{factor}) by which the expected sample size is to
#' be calculated.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param times Times at which the calculation should be evaluated - in years!
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details of
#' the \code{rs.surv} function.
#' @return A list of values.
#' @seealso \code{rs.surv}
#' @references Pohar Perme, M., Pavlic, K. (2018) "Nonparametric Relative
#' Survival Analysis with the R Package relsurv". Journal of Statistical
#' Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08"
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' rdata$agegr <-cut(rdata$age,seq(40,95,by=5))
#' nessie(Surv(time,cens)~agegr,rmap=list(age=age*365.241),
#' 	ratetable=slopop,data=rdata,times=c(1,3,5,10,15))
#'
nessie <- function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,times,rmap)

  #formula: for example Surv(time,cens)~sex
  #data: the observed data set
  #ratetable: the population mortality tables
  #times: the times at which to report NESS, if no default, then all unique times

{

  call <- match.call()
  if (!missing(rmap)) {
    rmap <- substitute(rmap)
  }
  na.action <- NA		#set the object just to be able to execute the rformulate call
  rform <- rformulate(formula, data, ratetable,na.action, rmap)			#get the data ready

  templab <- attr(rform$Terms,"term.labels")
  if(!is.null(attr(rform$Terms,"specials")$ratetable))templab <- templab[-length(templab)]	#delete the last term in the formula if the ratetable argument is there
  nameslist <- vector("list",length(templab))
  for(it in 1:length(nameslist)){
    valuetab <- table(data[,match(templab[it],names(data))])
    nameslist[[it]] <- paste(templab[it],names(valuetab),sep="")
  }
  names(nameslist) <- templab


  data <- rform$data								#the data set


  p <- rform$m								#number of covariates
  if (p > 0) 	{	#if covariates

    data$Xs <- my.strata(rform$X[,,drop=F],nameslist=nameslist)				#make strata according to covariates
    #data$Xs <- factor(data$Xs,levels=nameslist)				#order them in the same way as namelist
  }
  else data$Xs <- rep(1, nrow(data))						#if no covariates, just put 1

  if(!missing(times)) tis <- times
  else tis <- unique(sort(floor(rform$Y/365.241)))			#unique years of follow-up
  tis <- unique(c(0,tis))
  tisd <- tis*365.241


  out <- NULL
  out$n <- table(data$Xs)							#table of strata
  out$sp <- out$strata <- NULL
  # for (kt in 1:length(out$n)) {						#for each stratum
  for (kt in order(names(table(data$Xs)))) {						#for each stratum
    inx <- which(data$Xs == names(out$n)[kt])				#individuals within this stratum

    temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tisd,fast=FALSE)	#calculate the values for each interval of time

    out$time <- c(out$time, tisd)						#add times
    out$sp <- c(out$sp, temp$sis)						#add expected number of individuals alive
    out$strata <- c(out$strata, length(tis))				#number of times in this strata

    temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=(seq(0,100,by=.5)*365.241)[-1],fast=FALSE)	#calculate the values for each interval of time
    out$povp <- c(out$povp,mean(temp$sit/365.241))
  }

  names(out$strata) <- names(out$n)[order(names(table(data$Xs)))]
  if (p == 0) out$strata <- NULL						#if no covariates

  mata <- matrix(out$sp,ncol=length(tis),byrow=TRUE)
  mata <- data.frame(mata)
  mata <- cbind(mata,out$povp)
  row.names(mata) <- names(out$n)[order(names(table(data$Xs)))]
  names(mata) <- c(tis,"c.exp.surv")


  cat("\n")
  print(round(mata,1))
  cat("\n")

  out$mata <- mata
  out$n <- as.vector(out$n)
  class(out) <- "nessie"
  invisible(out)
}







rs.period <- function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,
                       na.action, fin.date, method = "pohar-perme", conf.type = "log",
                       conf.int = 0.95,type="kaplan-meier",winst,winfin,diag.date,rmap)

  #formula: for example Surv(time,cens)~sex
  #data: the observed data set
  #ratetable: the population mortality tables
  #conf.type: confidence interval calculation (plain, log or log-log)
  #conf.int: confidence interval
  #winst: start of the period window (inclusive)
  #winfin: end of the period window (inclusive)

{

  call <- match.call()
  if (!missing(rmap)) {
    rmap <- substitute(rmap)
  }
  rform <- rformulate(formula, data, ratetable, na.action,rmap)			#get the data ready
  data <- rform$data								#the data set
  type <- match.arg(type, c("kaplan-meier", "fleming-harrington"))		#method of hazard -> survival scale transformation
  type <- match(type, c("kaplan-meier", "fleming-harrington"))
  method <- match.arg(method,c("pohar-perme", "ederer2", "hakulinen","ederer1"))	#method of relative surv. curve estimation
  method <- match(method,c("pohar-perme", "ederer2", "hakulinen","ederer1"))
  conf.type <- match.arg(conf.type,c("plain","log","log-log"))		#conf. interval type

  #machinations needed for period survival:
  R <- rform$R
  coll <- match("year", attributes(ratetable)$dimid)
  year <- R[, coll]							#calendar year in the data

  ys <- as.numeric(winst - year)
  yf <- as.numeric(winfin - year)

  relv <- which(ys <= rform$Y & yf>0)					#relevant individuals -> live up to the period window and were diagnosed before window end
  centhem <- which(yf < rform$Y)					#censor these - their event happens outside of the period window

  rform$status[centhem] <- 0
  rform$Y[centhem] <- yf[centhem]

  rform$Y <- rform$Y[relv]
  rform$X <- rform$X[relv,,drop=F]
  rform$R <- rform$R[relv,,drop=F]
  rform$status <- rform$status[relv]
  data <- data[relv,,drop=F]
  ys <- ys[relv]
  yf <- yf[relv]
  year <- year[relv]

  if (method == 3) {								#need potential follow-up time for Hak. method
    if (missing(fin.date))
      fin.date <- max(rform$Y + year)					#final date for everybody set to the last day observed
    Y2 <- rform$Y							#change into potential follow-up time
    if (length(fin.date) == 1) 						#if final date equal for everyone
      Y2[rform$status == 1] <- fin.date - year[rform$status == 1]#set pot.time for those that died (equal to censoring time for others)
    else if (length(fin.date[relv]) == nrow(rform$R)) 	{
      fin.date <- fin.date[relv]
      Y2[rform$status == 1] <- fin.date[rform$status ==
                                          1] - year[rform$status == 1]
    }
    else stop("fin.date must be either one value of a vector of the same length as the data")
    status2 <- rep(0, nrow(rform$X))						#stat2=0 for everyone
  }
  p <- rform$m								#number of covariates
  if (p > 0) 									#if covariates
    data$Xs <- strata(rform$X[, ,drop=FALSE ])				#make strata according to covariates
  else data$Xs <- rep(1, nrow(data))						#if no covariates, just put 1

  se.fac <- sqrt(qchisq(conf.int, 1))						#factor needed for confidence interval
  out <- NULL
  out$n <- table(data$Xs)							#table of strata
  out$time <- out$n.risk <- out$n.event <- out$n.censor <- out$surv <- out$std.err <- out$strata <- NULL
  for (kt in 1:length(out$n)) {						#for each stratum
    inx <- which(data$Xs == names(out$n)[kt])				#individuals within this stratum

    tis <- sort(unique(rform$Y[inx]))					#unique times
    if(method==3)tis <- sort(unique(pmin(max(tis),c(tis,Y2[inx]))))				#add potential times in case of Hakulinen

    ys <- pmax(ys,0)
    #tis <- sort(unique(c(tis,ys[ys>0]-1,ys[ys>0])))
    tis <- sort(unique(c(tis,ys[ys>0])))
    tis <- sort(unique(c(tis,tis-1,tis+1)))					#the day after exiting, the day before entering
    tis <- tis[-length(tis)]							#exclude the largest since it is beyond observation time (1 day later)

    temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=(method<3),ys=ys)	#calculate the values for each interval of time

    out$time <- c(out$time, tis)						#add times
    out$n.risk <- c(out$n.risk, temp$yi)					#add number at risk for each time
    out$n.event <- c(out$n.event, temp$dni)					#add number of events for each time
    out$n.censor <- c(out$n.censor,  c(-diff(temp$yi),temp$yi[length(temp$yi)]) - temp$dni) 	#add number of censored for each time

    if(method==1){ 								#pohar perme method
      haz <- temp$dnisi/temp$yisi - temp$yidlisi/temp$yisi			#cumulative hazard increment on each interval
      out$std.err <- c(out$std.err, sqrt(cumsum(temp$dnisisq/(temp$yisi)^2)))  #standard error on each interval
    }
    else if(method==2){							#ederer2 method
      haz <- temp$dni/temp$yi - temp$yidli/temp$yi			#cumulative hazard increment on each interval
      out$std.err <- c(out$std.err, sqrt(cumsum(temp$dni/(temp$yi)^2)))  #standard error on each interval
    }
    else if(method==3){							#Hakulinen method
      temp2 <- exp_prep(rform$R[inx,,drop=FALSE],Y2[inx],rform$ratetable,status2[inx],times=tis,ys=ys)	#calculate the values for each interval of time
      popsur <- exp(-cumsum(temp2$yisidli/temp2$yisis))			#population survival
      haz <- temp$dni/temp$yi						#observed hazard on each interval
      out$std.err <- c(out$std.err, sqrt(cumsum(temp$dni/(temp$yi)^2)))  #standard error on each interval
    }
    else if(method==4){							#Ederer I
      popsur <- temp$sis/length(inx)					#population survival
      haz <- temp$dni/temp$yi						#observed hazard on each interval
      out$std.err <- c(out$std.err, sqrt(cumsum(temp$dni/(temp$yi)^2)))  #standard error on each interval
    }
    if(type==2)survtemp <- exp(-cumsum(haz))
    else survtemp <-  cumprod(1-haz)
    if(method>2){
      survtemp <- survtemp/popsur
    }
    out$surv <- c(out$surv,survtemp)
    out$strata <- c(out$strata, length(tis))				#number of times in this strata
  }
  if (conf.type == "plain") {
    out$lower <- as.vector(out$surv - out$std.err * se.fac * 		#surv + fac*se
                             out$surv)
    out$upper <- as.vector(out$surv + out$std.err * se.fac *
                             out$surv)
  }
  else if (conf.type == "log") {						#on log scale and back
    out$lower <- exp(as.vector(log(out$surv) - out$std.err *
                                 se.fac))
    out$upper <- exp(as.vector(log(out$surv) + out$std.err *
                                 se.fac))
  }
  else if (conf.type == "log-log") {						#on log-log scale and back
    out$lower <- exp(-exp(as.vector(log(-log(out$surv)) -
                                      out$std.err * se.fac/log(out$surv))))
    out$upper <- exp(-exp(as.vector(log(-log(out$surv)) +
                                      out$std.err * se.fac/log(out$surv))))
  }
  names(out$strata) <- names(out$n)
  if (p == 0) out$strata <- NULL						#if no covariates
  out$n <- as.vector(out$n)
  out$conf.type <- conf.type
  out$conf.int <- conf.int
  out$method <- method
  out$call <- call
  out$type <- "right"
  class(out) <- c("survfit", "rs.surv")
  out
}



#' expprep2 function
#'
#' Helper calculation function using C code. Saved also as exp_prep (unexported
#' function).
#'
#' Helper function used in rs.surv and other relsurv functions.
#'
#' @param x matrix of demographic covariates - each individual has one line
#' @param y follow-up time for each individual (same length as nrow(x))
#' @param ratetable rate table used for calculation
#' @param status status for each individual (same length as nrow(x)!), not
#' needed if we only need Spi, status needed for rs.surv
#' @param times times at which we wish to evaluate the quantities, not needed
#' if we only need Spi, times needed for rs.surv
#' @param fast for mpp method only
#' @param ys entry times (if empty, individuals are followed from time 0)
#' @param prec deprecated
#' @param cmp should cmpfast.C be used
#' @param netweiDM should new netwei script be used
#' @return List containing the calculated hazards and probabilities using the
#' population mortality tables.
#' @seealso rs.surv
#' @keywords survival
#' @export expprep2
expprep2 <- function (x, y,ratetable,status,times,fast=FALSE,ys,prec,cmp=F,netweiDM=FALSE) {			#function that prepares the data for C function call

  #x= matrix of demographic covariates - each individual has one line
  #y= follow-up time for each individual (same length as nrow(x)!)
  #ratetable= rate table used for calculation
  #status= status for each individual (same length as nrow(x)!), not needed if we only need Spi, status needed for rs.surv
  #times= times at which we wish to evaluate the quantities, not needed if we only need Spi, times needed for rs.surv
  #fast=for mpp method only
  #netweiDM=should new netwei script be used

  x <- as.matrix(x)
  if (ncol(x) != length(dim(ratetable)))
    stop("x matrix does not match the rate table")
  atts <- attributes(ratetable)

  cuts <- atts$cutpoints

  if (is.null(atts$type)) {
    rfac <- atts$factor
    us.special <- (rfac > 1)
  }
  else {
    rfac <- 1 * (atts$type == 1)
    us.special <- (atts$type == 4)
  }
  if (length(rfac) != ncol(x))
    stop("Wrong length for rfac")


  if (any(us.special)) {
    if (sum(us.special) > 1)
      stop("Two columns marked for special handling as a US rate table")
    cols <- match(c("age", "year"), atts$dimid)
    if (any(is.na(cols)))
      stop("Ratetable does not have expected shape")
    if (exists("as.Date")) {
      bdate <- as.Date("1960/1/1") + (x[, cols[2]] - x[,
                                                       cols[1]])
      byear <- format(bdate, "%Y")
      offset <- as.numeric(bdate - as.Date(paste(byear,
                                                 "01/01", sep = "/")))
    } else stop("Can't find an appropriate date class\n")
    # Tole je bilo prej:
    # else if (exists("date.mdy")) {
    #   bdate <- as.date(x[, cols[2]] - x[, cols[1]])
    #   byear <- date.mdy(bdate)$year
    #   offset <- bdate - mdy.date(1, 1, byear)
    # }
    x[, cols[2]] <- x[, cols[2]] - offset
    if (any(rfac > 1)) {
      temp <- which(us.special)
      nyear <- length(cuts[[temp]])
      nint <- rfac[temp]
      cuts[[temp]] <- round(approx(nint * (1:nyear), cuts[[temp]],
                                   nint:(nint * nyear))$y - 1e-04)
    }
  }

  if(!missing(status)){		#the function was called from rs.surv
    if(length(status)!=nrow(x))    stop("Wrong length for status")

    if(missing(times))    times <- sort(unique(y))

    if (any(times < 0))
      stop("Negative time point requested")
    ntime <- length(times)
    if(missing(ys)) ys <- rep(0,length(y))
    #    times2 <- times
    #    times2[1] <- preci

    # It may be an integer...check and make it numeric. Otherwise problems in C functions:
    if(is.integer(x[1,1])){
      x <- apply(x, 2, as.numeric)
    }

    if(cmp)   temp <- .Call("cmpfast",  as.integer(rfac), 		#fast=pohar-perme or ederer2 - data from pop. tables only while under follow-up
                            as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                            x, y, ys,as.integer(status), times,PACKAGE="relsurv")
    else if(fast&!missing(prec))    temp <- .Call("netfastpinter2",  as.integer(rfac), 		#fast=pohar-perme or ederer2 - data from pop. tables only while under follow-up
                                                  as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                                                  x, y, ys,as.integer(status), times,prec,PACKAGE="relsurv")
    else if(fast&missing(prec))    temp <- .Call("netfastpinter",  as.integer(rfac), 		#fast=pohar-perme or ederer2 - data from pop. tables only while under follow-up
                                                 as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                                                 x, y, ys,as.integer(status), times,PACKAGE="relsurv")
    else if(netweiDM==TRUE)    temp <- .Call("netweiDM",  as.integer(rfac),
                                             as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                                             x, y, ys,as.integer(status), times,PACKAGE="relsurv")
    else    temp <- .Call("netwei",  as.integer(rfac),
                          as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                          x, y, as.integer(status), times,PACKAGE="relsurv")
  }
  else{				#only expected survival at time y is needed for each individual
    if(length(y)==1)y <- rep(y,nrow(x))
    if(length(y)!=nrow(x)) stop("Wrong length for status")
    temp <- .Call("expc",  as.integer(rfac),
                  as.integer(atts$dim), as.double(unlist(cuts)), ratetable,
                  x, y,PACKAGE="relsurv")
    temp  <- temp$surv
  }
  temp
}

#' Subject-specific prediction for an excess hazards model
#'
#' A function that returns subject-specific predictions from an rsadd object.
#' @param object An rsadd object
#' @param newdata A data.frame with one row (add covariate values in columns)
#' @param type By default ('Haz') returns cumulative hazards. If 'prob' it calculates CIFs. If 'both' a list is returned with Haz and prob.
#' @param add.times specific times at which predictions should be obtained.
#' @param ... Not used for now
#' @return A data.frame containing the times, excess and population components.
#'
#' @author Damjan Manevski \email{damjan.manevski@@mf.uni-lj.si}
#' @examples
#' data(rdata)
#' fit <- rsadd(Surv(time,cens)~sex+age,rmap=list(age=age*365.241),
#'              ratetable=slopop,data=rdata,int=5,method="EM")
#' predict(fit, data.frame(sex=2,age=68, year=as.Date('1982-06-24')))
predict.rsadd <- function(object, newdata, type='Haz', add.times,
                          ...){
  # Zaenkrat delamo z object$times. Treba bo dodati se add.times. Ta add.times bo verjetno v rsadd

  type <- match.arg(type, c("Haz", "prob", "both"))

  # lin.pred <- 0
  # for(cov in names(object$coefficients)){
  #   lin.pred <- lin.pred + sum(newdata[1,cov]*object$coefficients[cov])
  # }
  lin.pred <- sum(model.matrix(object, newdata[1,])*object$coefficients)

  # haz_function(object$formula, rdata, object$ratetable, rmap=list(age=age*365.241), add.times=0, include.all.times = FALSE)

  kar <- deparse(object$formula[[2]])
  autkam <- gsub(' ', '', strsplit(substr(kar, start = 6, stop=nchar(kar)-1), ',')[[1]])
  newdata_2 <- data.frame(matrix(1, nrow=1, ncol=length(autkam)))
  if(length(autkam)==3) newdata_2[,1] <- 0
  colnames(newdata_2) <- autkam
  newdata_2 <- cbind(newdata, newdata_2)

  if(is.null(object$rmap)){
    rform <- suppressWarnings(rformulate(object$formula,
                                         newdata_2,object$ratetable))
  } else{
    rform <- suppressWarnings(rformulate(object$formula,
                                         newdata_2,object$ratetable, rmap = object$rmap))
  }

  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)

  # Include add.times:
  all_times <- object$times
  if(!missing(add.times)){
    add.times <- add.times[add.times<=all_times[length(all_times)]]
    add.times <- add.times[!(add.times %in% all_times)]
    all_times <- sort(unique(c(add.times, all_times)))
  }

  pop.surv <- sapply(1:length(all_times), function(i) exp_prep(rform$data[1, 4:(nfk + 3), drop = FALSE], all_times[i], object$ratetable))
  Haz.p <- -log(pop.surv)

  # Expand Lambda0:
  if (!missing(add.times)) {
    df_tmp <- data.frame(time = object$times, Lambda0 = object$Lambda0)
    df_novo <- data.frame(time = add.times, Lambda0 = NA)
    df_add <- rbind(df_tmp, df_novo)
    df_add <- df_add[order(df_add$time), ]
    df_add$Lambda0 <- mstateNAfix(df_add$Lambda0,
                                            0)
    rownames(df_add) <- as.character(1:nrow(df_add))

    Haz.e <- df_add$Lambda0 * exp(lin.pred)
  } else{
    Haz.e <- object$Lambda0 * exp(lin.pred)
  }

  if(type=='Haz'){
    data.frame(time=all_times, Haz.e=Haz.e, Haz.p=Haz.p)
  } else if(type=='prob'){
    S_O <- exp(-(Haz.e + Haz.p))

    prob.e <- cumsum(S_O*diff(c(0, Haz.e)))
    prob.p <- cumsum(S_O*diff(c(0, Haz.p)))

    data.frame(time=all_times, prob.e=prob.e, prob.p=prob.p)
  } else{
    out <- list()
    out$Haz <- data.frame(time=all_times, Haz.e=Haz.e, Haz.p=Haz.p)

    S_O <- exp(-(Haz.e + Haz.p))

    prob.e <- cumsum(S_O*diff(c(0, Haz.e)))
    prob.p <- cumsum(S_O*diff(c(0, Haz.p)))
    out$prob <- data.frame(time=all_times, prob.e=prob.e, prob.p=prob.p)
    out
  }
}

#.First.lib <- function(lib, pkg) library.dynam("runproba", pkg, lib)

# use .onLoad instead of .First.lib for use with NAMESPACE and R(>= 1.7.0)
.onLoad <- function(lib, pkg) {
  # library.dynam <- function (chname, package, lib.loc, verbose = getOption("verbose"),
  #                            file.ext = .Platform$dynlib.ext, ...)
  # {
  #   dll_list <- .dynLibs()
  #   if (missing(chname) || !nzchar(chname))
  #     return(dll_list)
  #   package
  #   lib.loc
  #   r_arch <- .Platform$r_arch
  #   chname1 <- paste0(chname, file.ext)
  #   # browser()
  #   for (pkg in "C:/Users/dame_/Dropbox (MF Uni LJ)/Damjan Manevski/Research/relsurv/relsurv_2.2-5"){
  #        #find.package('relsurv_2.2-5', lib.loc, verbose = verbose)) {
  #     DLLpath <- if (nzchar(r_arch))
  #       # file.path(pkg, "libs", r_arch)
  #       "C:/Users/dame_/Dropbox (MF Uni LJ)/Damjan Manevski/Research/relsurv/relsurv_2.2-5/src"
  #     else file.path(pkg, "libs")
  #     # file <- file.path(DLLpath, chname1)
  #     file <- "C:/Users/dame_/Dropbox (MF Uni LJ)/Damjan Manevski/Research/relsurv/relsurv_2.2-5/src/relsurv.dll"
  #     # browser()
  #     if (file.exists(file))
  #       break
  #     else file <- ""
  #   }
  #   if (file == "")
  #     if (.Platform$OS.type == "windows")
  #       stop(gettextf("DLL %s not found: maybe not installed for this architecture?",
  #                     sQuote(chname)), domain = NA)
  #   else stop(gettextf("shared object %s not found", sQuote(chname1)),
  #             domain = NA)
  #   # browser()
  #   file <- file.path(normalizePath(DLLpath, "/", TRUE), chname1)
  #   ind <- vapply(dll_list, function(x) x[["path"]] == file,
  #                 NA)
  #   if (length(ind) && any(ind)) {
  #     if (verbose)
  #       if (.Platform$OS.type == "windows")
  #         message(gettextf("DLL %s already loaded", sQuote(chname1)),
  #                 domain = NA)
  #     else message(gettextf("shared object '%s' already loaded",
  #                           sQuote(chname1)), domain = NA)
  #     return(invisible(dll_list[[seq_along(dll_list)[ind]]]))
  #   }
  #   if (.Platform$OS.type == "windows") {
  #     PATH <- Sys.getenv("PATH")
  #     Sys.setenv(PATH = paste(gsub("/", "\\\\", DLLpath),
  #                             PATH, sep = ";"))
  #     on.exit(Sys.setenv(PATH = PATH))
  #   }
  #   if (verbose)
  #     message(gettextf("now dyn.load(\"%s\") ...", file),
  #             domain = NA)
  #   dll <- if ("DLLpath" %in% names(list(...)))
  #     dyn.load(file, ...)
  #   else dyn.load(file, DLLpath = DLLpath, ...)
  #   .dynLibs(c(dll_list, list(dll)))
  #   invisible(dll)
  # }
  library.dynam("relsurv", pkg, lib)

}#end of .onLoad

predict.aalen.relsurv <- function(object, newdata, ...){
  # object: an aalen.model object
  # newdata: data.frame with one row, in columns add covariate values.

  # subject-specific prediction.

  # Zaenkrat delamo z object$times. Treba bo dodati se add.times. Ta add.times bo verjetno v rsadd

  lin.pred <- object$coefficients[,2]

  if(ncol(object$coefficients) >= 3){
    for(cov in colnames(object$coefficients)[3:length(colnames(object$coefficients))]){
      lin.pred <- lin.pred + newdata[1,cov]*object$coefficients[,cov]
    }
  }

  # haz_function(object$formula, rdata, object$ratetable, rmap=list(age=age*365.241), add.times=0, include.all.times = FALSE)

  kar <- deparse(object$formula[[2]])
  autkam <- gsub(' ', '', strsplit(substr(kar, start = 6, stop=nchar(kar)-1), ',')[[1]])
  newdata_2 <- data.frame(matrix(1, nrow=1, ncol=length(autkam)))
  if(length(autkam)==3) newdata_2[,1] <- 0
  colnames(newdata_2) <- autkam
  newdata_2 <- cbind(newdata, newdata_2)

  rform <- suppressWarnings(rformulate(object$formula,
                                       newdata_2, object$ratetable, stats::na.omit(), rmap = object$rmap))


  fk <- (attributes(rform$ratetable)$factor != 1)
  nfk <- length(fk)
  pop.surv <- sapply(1:nrow(object$coefficients), function(i) exp_prep(rform$data[1, 4:(nfk + 3), drop = FALSE], object$coefficients[i,1], object$ratetable))
  Haz.p <- -log(pop.surv)

  data.frame(time=object$coefficients[,1], Haz.e=lin.pred, Haz.p=Haz.p)
}

# Like rformulate, just does not use ratetable/rmap and the remaining relsurv components
rformulate2 <- function (formula, data = parent.frame(), ratetable, na.action, rmap,
                        int, centered, cause)
{
  call <- match.call()
  m <- match.call(expand.dots = FALSE)

  # keep the parts of the call that we want, toss others
  m <- m[c(1, match(c("formula", "data", "cause"), names(m), nomatch=0))]
  m[[1L]] <- quote(stats::model.frame)  # per CRAN, the formal way to set it

  Terms <- if (missing(data))
    terms(formula, specials= c("strata","ratetable"))
  else terms(formula, specials=c("strata", "ratetable"), data = data)
  Term2 <- Terms

  # #sorting out the ratetable argument - matching demographic variables
  # rate <- attr(Terms, "specials")$ratetable
  # if (length(rate) > 1)
  #   stop("Can have only 1 ratetable() call in a formula")

  # #matching demographic variables via rmap
  # if (!missing(rmap)) {  # use this by preference
  #   if (length(rate) >0)
  #     stop("cannot have both ratetable() in the formula and a rmap argument")
  #   rcall <- rmap
  #   if (!is.call(rcall) || rcall[[1]] != as.name('list'))
  #     stop ("Invalid rcall argument")
  # } #done with rmap
  # else if (length(rate) >0) { #sorting out ratetable
  #   stemp <- untangle.specials(Terms, 'ratetable')
  #   rcall <- as.call(parse(text=stemp$var)[[1]])   # as a call object
  #   rcall[[1]] <- as.name('list')                  # make it a call to list
  #   Term2 <- Term2[-stemp$terms]                   # remove from the formula
  # }
  # else rcall <- NULL  # A ratetable, but no rcall or ratetable()

  # # Check that there are no illegal names in rcall, then expand it
  # #  to include all the names in the ratetable
  # if (is.ratetable(ratetable))   {
  #   israte <- TRUE
  #   dimid <- names(dimnames(ratetable))
  #   if (is.null(dimid))
  #     dimid <- attr(ratetable, "dimid") # older style
  #   else  attr(ratetable, "dimid") <- dimid		#put all tables into the old style
  #
  #   temp <- match(names(rcall)[-1], dimid) # 2,3,... are the argument names
  #   if (any(is.na(temp)))
  #     stop("Variable not found in the ratetable:", (names(rcall))[is.na(temp)])
  #
  #   if (any(!(dimid %in% names(rcall)))) {
  #     to.add <- dimid[!(dimid %in% names(rcall))]
  #     temp1 <- paste(text=paste(to.add, to.add, sep='='), collapse=',')
  #     if (is.null(rcall)) rcall <- parse(text=paste("list(", temp1, ")"))[[1]]
  #     else {
  #       temp2 <- deparse(rcall)
  #       rcall <- parse(text=paste("c(", temp2, ",list(", temp1, "))"))[[1]]
  #     }
  #   }
  # }
  # else stop("invalid ratetable")

  # Create a temporary formula, used only in the call to model.frame,
  #  that has extra variables
  # newvar <- all.vars(rcall)
  # if (length(newvar) > 0) {
  #   tform <- paste(paste(deparse(Term2), collapse=""),
  #                  paste(newvar, collapse='+'), sep='+')
  #   m$formula <- as.formula(tform, environment(Terms))
  # }

  m <- eval(m, parent.frame())
  n <- nrow(m)
  if (n==0) stop("data set has 0 rows")

  Y <- model.extract(m, "response")
  offset <- model.offset(m)
  if (length(offset)==0) offset <- rep(0., n)

  if (!is.Surv(Y))
    stop("Response must be a survival object")
  Y.surv <- Y
  if (attr(Y, "type") == "right") {
    type <- attr(Y, "type")
    status <- Y[, 2]
    Y <- Y[, 1]
    start <- rep(0, n)
    ncol0 <- 2
  }
  else if (attr(Y, "type") == "counting") {
    type <- attr(Y, "type")
    status <- Y[, 3]
    start <- Y[, 1]
    Y <- Y[, 2]
    ncol0 <- 3
  }
  else stop("Illegal response value")
  if (any(c(Y, start) < 0))
    stop("Negative follow up time")
  # if(max(Y)<30)
  #   warning("The event times must be expressed in days! (Your max time in the data is less than 30 days) \n")

  # # rdata contains the variables matching the ratetable
  # rdata <- data.frame(eval(rcall, m), stringsAsFactors=TRUE)
  # rtemp <- match.ratetable(rdata, ratetable)		#this function puts the dates in R and in cutpoints in rtabledate
  # R <- rtemp$R
  # cutpoints <- rtemp$cutpoints

  # if(is.null(attr(ratetable, "factor")))
  #   attr(ratetable, "factor") <- (attr(ratetable, "type") ==1)
  # attr(ratetable, "dimid") <- dimid
  # rtorig <- attributes(ratetable)
  # nrt <- length(rtorig$dimid)

  #checking if the ratetable variables are given in days
  # wh.age <- which(dimid=="age")
  # wh.year <- which(dimid=="year")
  # if(length(wh.age)>0){
  #   if (max(R[,wh.age])<150 & median(diff(cutpoints[[wh.age]]))>12)
  #     warning("Age in the ratetable part of the formula must be expressed in days! \n (Your max age is less than 150 days) \n")
  # }
  # TMT -- note the new class
  # if(length(wh.year)>0){
  #   if(min(R[,wh.year])>1850 & max(R[,wh.year])<2020&
  #      inherits(cutpoints[[wh.year]], "rtdate"))
  #     warning("The calendar year must be one of the date classes (Date, date, POSIXt)\n (Your variable seems to be expressed in years) \n")
  # }
  #checking if one of the continuous variables is fixed:
  # if(nrt!=ncol(R)){
  #   nonex <- which(is.na(match(rtorig$dimid,attributes(ratetable)$dimid)))
  #   for(it in nonex){
  #     if(rtorig$type[it]!=1)warning(paste("Variable ",rtorig$dimid[it]," is held fixed even though it changes in time in the population tables. \n (You may wish to set a value for each individual and not just one value for all)",sep=""))
  #   }
  # }

  #NEW in 2.05 (strata)
  # Now create the X matrix and strata
  strats <- attr(Term2, "specials")$strata
  if (length(strats)) {
    temp_str <- untangle.specials(Term2,"strata",1)
    if (length(temp_str$vars) == 1)
      strata.keep <- m[[temp_str$vars]]
    else strata.keep <- strata(m[,temp_str$vars],shortlabel=TRUE,sep=",")
    Term2 <- Term2[-temp_str$terms]
  }
  else strata.keep <- factor(rep(1,n)) # zgoraj ze definirano n = nrow(m)
  if (!missing(cause)) strata.keep <- factor(rep(1,n))

  attr(Term2, "intercept") <- 1   # ignore a "-1" in the formula
  X <- model.matrix(Term2, m)[,-1, drop=FALSE]
  mm <- ncol(X)

  if (mm > 0 && !missing(centered) && centered) {
    mvalue <- colMeans(X)
    X <- X - rep(mvalue, each=nrow(X))
  }
  else mvalue <- double(mm)

  cause <- model.extract(m, "cause")
  if(is.null(cause)) cause <- rep(2,nrow(m))

  #NEW: ce cause manjka
  #status[cause==0] <- 0
  keep <- Y > start
  if (!missing(int)) {
    int <- max(int)
    status[Y > int * 365.241] <- 0
    Y <- pmin(Y, int * 365.241)
    keep <- keep & (start < int * 365.241)
  }
  if (any(start > Y) | any(Y < 0))
    stop("Negative follow-up times")

  if (!all(keep)) {
    X <- X[keep, , drop = FALSE]
    Y <- Y[keep]
    start <- start[keep]
    status <- status[keep]
    # R <- R[keep, ,drop=FALSE]
    strata.keep <- strata.keep[keep] # dodano za strato  #NEW in 2.05
    offset <- offset[keep]
    Y.surv <- Y.surv[keep, , drop = FALSE]
    cause <- cause[keep]
    n <- sum(keep)
    # rdata <- rdata[keep,]
  }

  # I do not want to preserve variable class here - so paste R onto here, give it names
  # temp <- R
  # names(temp) <- paste0("X", 1:ncol(temp))    # with the right names

  #if variable class needs to be preserved, use this instead
  #  variable class.  So paste on rdata, but with the right order and names
  #temp <- rdata[,match(dimid, names(rdata))]  # in the right order
  #names(temp) <- paste0("X", 1:ncol(temp))    # with the right names

  data <- data.frame(start = start, Y = Y, stat = status)
  if (mm != 0) data <- cbind(data, X)

  # # we pass the altered cutpoints forward, keep them in the date format (could be changed eventually to get rid of the date package dependence)
  # attr(ratetable, "cutpoints") <- lapply(cutpoints, function(x) {
  #   if(inherits(x, 'rtabledate')) class(x) <- 'date'
  #   x})

  out <- list(data = data, X = as.data.frame(X))

  out
}

# This is a version with suggested updates by T Therneau
#  All updates are stolen from survexp in the survival package, with comments.
# Most changes are used, some further corrections were required.
rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rmap,
                        int, centered, cause)
{
  call <- match.call()
  m <- match.call(expand.dots = FALSE)

  # keep the parts of the call that we want, toss others
  m <- m[c(1, match(c("formula", "data", "cause"), names(m), nomatch=0))]
  m[[1L]] <- quote(stats::model.frame)  # per CRAN, the formal way to set it

  Terms <- if (missing(data))
               terms(formula, specials= c("strata","ratetable"))
           else terms(formula, specials=c("strata", "ratetable"), data = data)
  Term2 <- Terms

  #sorting out the ratetable argument - matching demographic variables
  rate <- attr(Terms, "specials")$ratetable
  if (length(rate) > 1)
    stop("Can have only 1 ratetable() call in a formula")

  #matching demographic variables via rmap
  if (!missing(rmap)) {  # use this by preference
      if (length(rate) >0)
          stop("cannot have both ratetable() in the formula and a rmap argument")
          rcall <- rmap
          if (!is.call(rcall) || rcall[[1]] != as.name('list'))
          stop ("Invalid rcall argument")
  } #done with rmap
  else if (length(rate) >0) { #sorting out ratetable
      stemp <- untangle.specials(Terms, 'ratetable')
      rcall <- as.call(parse(text=stemp$var)[[1]])   # as a call object
      rcall[[1]] <- as.name('list')                  # make it a call to list
      Term2 <- Term2[-stemp$terms]                   # remove from the formula
  }
  else rcall <- NULL  # A ratetable, but no rcall or ratetable()

  # Check that there are no illegal names in rcall, then expand it
  #  to include all the names in the ratetable
  if (is.ratetable(ratetable))   {
      israte <- TRUE
      dimid <- names(dimnames(ratetable))
      if (is.null(dimid))
        dimid <- attr(ratetable, "dimid") # older style
     else  attr(ratetable, "dimid") <- dimid		#put all tables into the old style

      temp <- match(names(rcall)[-1], dimid) # 2,3,... are the argument names
      if (any(is.na(temp)))
        stop("Variable not found in the ratetable:", (names(rcall))[is.na(temp)])

      if (any(!(dimid %in% names(rcall)))) {
        to.add <- dimid[!(dimid %in% names(rcall))]
        temp1 <- paste(text=paste(to.add, to.add, sep='='), collapse=',')
        if (is.null(rcall)) rcall <- parse(text=paste("list(", temp1, ")"))[[1]]
        else {
            temp2 <- deparse(rcall)
            rcall <- parse(text=paste("c(", temp2, ",list(", temp1, "))"))[[1]]
            }
        }
  }
  else stop("invalid ratetable")

  # Create a temporary formula, used only in the call to model.frame,
  #  that has extra variables
  newvar <- all.vars(rcall)
  if (length(newvar) > 0) {
      tform <- paste(paste(deparse(Term2), collapse=""),
                     paste(newvar, collapse='+'), sep='+')
      m$formula <- as.formula(tform, environment(Terms))
  }
  m <- eval(m, parent.frame())
  n <- nrow(m)
  if (n==0) stop("data set has 0 rows")

  Y <- model.extract(m, "response")
  offset <- model.offset(m)
  if (length(offset)==0) offset <- rep(0., n)

  if (!is.Surv(Y))
    stop("Response must be a survival object")
  Y.surv <- Y
  if (attr(Y, "type") == "right") {
    type <- attr(Y, "type")
    status <- Y[, 2]
    Y <- Y[, 1]
    start <- rep(0, n)
    ncol0 <- 2
  }
  else if (attr(Y, "type") == "counting") {
    type <- attr(Y, "type")
    status <- Y[, 3]
    start <- Y[, 1]
    Y <- Y[, 2]
    ncol0 <- 3
  }
  else stop("Illegal response value")
  if (any(c(Y, start) < 0))
    stop("Negative follow up time")
  if(max(Y)<30)
    warning("The event times must be expressed in days! (Your max time in the data is less than 30 days) \n")

  # rdata contains the variables matching the ratetable
  rdata <- data.frame(eval(rcall, m), stringsAsFactors=TRUE)
  rtemp <- match.ratetable(rdata, ratetable)		#this function puts the dates in R and in cutpoints in rtabledate
  R <- rtemp$R
  cutpoints <- rtemp$cutpoints

  if(is.null(attr(ratetable, "factor")))
      attr(ratetable, "factor") <- (attr(ratetable, "type") ==1)
  attr(ratetable, "dimid") <- dimid
  rtorig <- attributes(ratetable)
  nrt <- length(rtorig$dimid)

  #checking if the ratetable variables are given in days
  wh.age <- which(dimid=="age")
  wh.year <- which(dimid=="year")
  if(length(wh.age)>0){
      if (max(R[,wh.age])<150 & median(diff(cutpoints[[wh.age]]))>12)
        warning("Age in the ratetable part of the formula must be expressed in days! \n (Your max age is less than 150 days) \n")
  }
  # TMT -- note the new class
  if(length(wh.year)>0){
      if(min(R[,wh.year])>1850 & max(R[,wh.year])<2020&
         inherits(cutpoints[[wh.year]], "rtdate"))
        warning("The calendar year must be one of the date classes (Date, POSIXt)\n (Your variable seems to be expressed in years) \n")
  }
  #checking if one of the continuous variables is fixed:
  if(nrt!=ncol(R)){
    nonex <- which(is.na(match(rtorig$dimid,attributes(ratetable)$dimid)))
    for(it in nonex){
        if(rtorig$type[it]!=1)warning(paste("Variable ",rtorig$dimid[it]," is held fixed even though it changes in time in the population tables. \n (You may wish to set a value for each individual and not just one value for all)",sep=""))
    }
  }

  #NEW in 2.05 (strata)
  # Now create the X matrix and strata
  strats <- attr(Term2, "specials")$strata
  if (length(strats)) {
      temp_str <- untangle.specials(Term2,"strata",1)
      if (length(temp_str$vars) == 1)
          strata.keep <- m[[temp_str$vars]]
      else strata.keep <- strata(m[,temp_str$vars],shortlabel=TRUE,sep=",")
      Term2 <- Term2[-temp_str$terms]
  }
  else strata.keep <- factor(rep(1,n)) # zgoraj ze definirano n = nrow(m)
  if (!missing(cause)) strata.keep <- factor(rep(1,n))

  attr(Term2, "intercept") <- 1   # ignore a "-1" in the formula
  X <- model.matrix(Term2, m)[,-1, drop=FALSE]
  mm <- ncol(X)

  if (mm > 0 && !missing(centered) && centered) {
      mvalue <- colMeans(X)
      X <- X - rep(mvalue, each=nrow(X))
  }
  else mvalue <- double(mm)

  cause <- model.extract(m, "cause")
  if(is.null(cause)) cause <- rep(2,nrow(m))

  #NEW: ce cause manjka
  #status[cause==0] <- 0
  keep <- Y > start
  if (!missing(int)) {
    int <- max(int)
    status[Y > int * 365.241] <- 0
    Y <- pmin(Y, int * 365.241)
    keep <- keep & (start < int * 365.241)
  }
  if (any(start > Y) | any(Y < 0))
    stop("Negative follow-up times")

  if (!all(keep)) {
      X <- X[keep, , drop = FALSE]
      Y <- Y[keep]
      start <- start[keep]
      status <- status[keep]
      R <- R[keep, ,drop=FALSE]
      strata.keep <- strata.keep[keep] # dodano za strato  #NEW in 2.05
      offset <- offset[keep]
      Y.surv <- Y.surv[keep, , drop = FALSE]
      cause <- cause[keep]
      n <- sum(keep)
      rdata <- rdata[keep,]
  }

  # I do not want to preserve variable class here - so paste R onto here, give it names
temp <- R
  names(temp) <- paste0("X", 1:ncol(temp))    # with the right names

  #if variable class needs to be preserved, use this instead
  #  variable class.  So paste on rdata, but with the right order and names
  #temp <- rdata[,match(dimid, names(rdata))]  # in the right order
  #names(temp) <- paste0("X", 1:ncol(temp))    # with the right names

 data <- data.frame(start = start, Y = Y, stat = status, temp)
  if (mm != 0) data <- cbind(data, X)

  # we pass the altered cutpoints forward, keep them in the date format (could be changed eventually to get rid of the date package dependence)
  attr(ratetable, "cutpoints") <- lapply(cutpoints, function(x) {
      if(inherits(x, 'rtabledate')) class(x) <- 'Date'
      x})

  out <- list(data = data, R = R, status = status, start = start,
            Y = Y, X = as.data.frame(X), m = mm, n = n, type = type,
            Y.surv = Y.surv,
            Terms = Terms, ratetable = ratetable, offset = offset,
            formula=formula,
            cause = cause, mvalue=mvalue, strata.keep=strata.keep) # dodano za strato  #NEW in 2.05

  na.action <- attr(m, "na.action")
  if (length(na.action))
    out$na.action <- na.action
  out
}

#' Compute a Predicited Survival Curve
#' 
#' Computes a predicted survival curve based on the additive model estimated by
#' rsadd function.
#' 
#' When predicting the survival curve, the ratetable values for future years
#' will be equal to those of the last given year. The same ratetables will be
#' used for fitting and predicting. To predict a relative survival curve, use
#' \code{rs.surv.rsadd}.
#' 
#' @param formula a rsadd object
#' @param newdata a data frame with the same variable names as those that
#' appear in the rsadd formula. The curve(s) produced will be representative of
#' a cohort who's covariates correspond to the values in newdata.
#' @param se.fit a logical value indicating whether standard errors should be
#' computed. Default is \code{TRUE}.
#' @param conf.int the level for a two-sided confidence interval on the
#' survival curve(s). Default is 0.95.
#' @param individual a logical value indicating whether the data frame
#' represents different time epochs for only one individual (T), or whether
#' multiple rows indicate multiple individuals (F, the default). If the former
#' only one curve will be produced; if the latter there will be one curve per
#' row in newdata.
#' @param conf.type One of \code{none}, \code{plain}, \code{log} (the default),
#' or \code{log-log}. The first option causes confidence intervals not to be
#' generated. The second causes the standard intervals curve +- k *se(curve),
#' where k is determined from conf.int. The log option calculates intervals
#' based on the cumulative hazard or log(survival). The last option bases
#' intervals on the log hazard or log(-log(survival)).
#' @param ... Currently not implemented
#' @return a \code{survfit} object; see the help on \code{survfit.object} for
#' details.  The \code{survfit} methods are used for \code{print}, \code{plot},
#' \code{lines}, and \code{points}.
#' @seealso \code{survfit}, \code{survexp}, \code{\link{rs.surv}}
#' @references Package: Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine,\bold{81}: 272--278.
#' 
#' Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
#' analysis relatively easy." Computers in biology and medicine, \bold{37}:
#' 1741--1749.
#' @keywords survival
#' @examples
#' 
#' data(slopop)
#' data(rdata)
#' #BTW: work on a smaller dataset here to run the example faster
#' fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#' 	ratetable=slopop,data=rdata[1:500,],method="EM")
#' survfit.rsadd(fit,newdata=data.frame(sex=1,age=60,year=17000))
#' 
#' 
survfit.rsadd <- 
function (formula, newdata, se.fit = TRUE, conf.int = 0.95, individual = FALSE, 
      conf.type = c("log", "log-log", "plain", "none"),...) 
{
   
    call <- match.call()
    Terms <- terms(formula)  #to rabis, ce je model mal bl smotan - as.factor ali splines ali svasta
    Terms <- delete.response(Terms)
    popdata <- newdata
    newdata <- model.frame(Terms,newdata)
    resp <- list(y=formula$y,x=newdata)
    n <- formula$n
    nvar <- length(formula$coef)
    nx <- nrow(newdata)
    nt <- length(formula$times)
    temp <- list(n=formula$n,time=formula$times,call=call,type="right")
    Lambda0 <- formula$Lambda0
    Lambda0 <- matrix(Lambda0,ncol=nt,nrow=nrow(newdata),byrow=TRUE)
    
    rate <- attr(Terms, "specials")$ratetable
    
    #rat <- attributes(formula$ratetable)$dimid
    rat <- names(attributes(formula$ratetable)$dimnames)
    #mein <- attributes(newdata[,rate])$dimnames[[2]]
    mein <- names(popdata)
    x <- match(rat,mein)
    #R <- as.matrix(newdata[, rate, drop = FALSE])
    R <- as.matrix(popdata)
    R <- R[,x,drop=FALSE]
    R <- data.frame(R)
    names(R) <- rat
   
    
    #newdata <- newdata[,1:(rate-1),drop=FALSE]
    labeli <- attr(attr(newdata,"terms"),"term.labels")
    colnami <- colnames(newdata)
    if(length(rate>0)){
      labeli <- labeli[-rate]
      colnami <- colnami[-rate] 
    }
    newdata <- newdata[,match(colnami,labeli),drop=F]
    if(any(formula$mvalue)>0)newdata <- newdata - matrix(formula$mvalue,nrow=nrow(newdata),byrow=TRUE)
    nx <- ncol(newdata)
    #getl <- function(times,data=R,ratetable=formula$ratetable){
    #	-log(srvxp.fit(data,times,ratetable))
    #}
    #Lambdap <- sapply(formula$times, getl)
   # Lambdap <- NULL
   # for(it in 1:nt){
   # 	Lambdap <- cbind(Lambdap,-log(srvxp.fit(R,formula$times[it],formula$ratetable)))
   # }
   
   Lambdap <- NULL
   for(it in 1:nrow(newdata)){
    Lambdap <- rbind(Lambdap,-log(survexp(~1,data=R[it,,drop=FALSE],times=formula$times,ratetable=formula$ratetable)$surv))
	}
    ebx <- exp(as.matrix(formula$coef %*%as.numeric(newdata)))
    ebx <- matrix(ebx,ncol=nt,nrow=length(ebx))
    Lambda <- Lambdap + Lambda0*ebx
    temp$surv <- t(exp(-Lambda))
    temp$n.event <- rep(1,nt)
    temp$n.risk <- n+1 - cumsum(temp$n.event)
    class(temp) <- c("rs.surv.rsadd", "rs.surv","survfit")
    temp
}

# Calculating betas:
calculateBetasRelsurv22R <- function(data, xt, event_times, all_times,
                                     ratetable, atts, data_mat, variance, var_estimator='dN'){

  ncol <- length(all_times)

  Yt_val <- Yt(data, all_times)
  dNt_val <- dNt(data, all_times)
  dLambdaP_val <- dLambdaPR(data_mat, all_times, event_times, ratetable, atts)
  # dLambdaP_val <- c(0, dLambdaP_val)

  sample_size = nrow(xt)
  number_covs = ncol(xt)

  diag_dNt = matrix(0, nrow=sample_size, ncol=sample_size)
  beta_var = matrix(0, nrow=number_covs, ncol=number_covs)

  betas_var_list <- vector("list", ncol)

  if(variance){
    betas_list0 <- lapply(1:ncol, function(i) fitOLS2(prepareX(Yt_val[[i]], xt), dNt_val[[i]] - (dLambdaP_val[,i+1] - dLambdaP_val[,i]), Yt_val[[i]]))

    if(var_estimator=='dN'){
      betas_var_list <- lapply(1:ncol, function(i) betas_list0[[i]][[2]] %*% diag(dNt_val[[i]] - (dLambdaP_val[,i+1] - dLambdaP_val[,i])) %*% t(betas_list0[[i]][[2]]))
    } else if(var_estimator=='XdB'){
      betas_var_list <- lapply(1:ncol, function(i) betas_list0[[i]][[2]] %*% diag(as.vector(prepareX(Yt_val[[i]], xt) %*% betas_list0[[i]][[1]])) %*% t(betas_list0[[i]][[2]]))
    }
    betas_list <- lapply(1:ncol, function(i) betas_list0[[i]][[1]])
  } else{
    betas_list <- lapply(1:ncol, function(i) fitOLS2(prepareX(Yt_val[[i]], xt), dNt_val[[i]] - (dLambdaP_val[,i+1] - dLambdaP_val[,i]), Yt_val[[i]])[[1]])
  }
  # for(i in 1:ncol){
  #   xx2 = prepareX(Yt_val[[i]], xt)
  #   betas = fitOLS2(xx2, dNt_val[[i]], Yt_val[[i]])
  #
  #   betas_list[[i]] = betas[[1]]
  #
  #   if(var_estimator==1){
  #     diag(diag_dNt) <- dNt_val[[i]]
  #   } else if(var_estimator == 2){
  #     betas0_vec <- betas[[1]];
  #     diag(diag_dNt) <- xx2 %*% betas0 # mogoce je treba dati na betas0 t()
  #   }
  #
  #   betas1 = betas[[2]]
  #   # betas_var_list[[i]] <- betas1 %*% diag_dNt %*% t(betas1)
  # }

  out <- list(betas_list, betas_var_list)

  return(out)
}

# Calculating betas and gamma:
calculateBetasGammasRelsurv22R <- function(data, xt, zt, event_times, all_times,
                                     ratetable, atts, data_mat, var_estimator=1){

  ncol <- length(all_times)

  Yt_val <- Yt(data, all_times)
  dNt_val <- dNt(data, all_times)
  dLambdaP_val <- dLambdaPR(data_mat, all_times, event_times, ratetable, atts)
  # dLambdaP_val <- c(0, dLambdaP_val)

  sample_size = nrow(xt)
  number_covs = ncol(xt)

  diag_dNt = matrix(0, nrow=sample_size, ncol=sample_size)
  beta_var = matrix(0, nrow=number_covs, ncol=number_covs)

  diff_t <- diff(c(min(data$start), all_times))

  betas_var_list <- vector("list", ncol)

  mod_list <- lapply(1:ncol, function(i) fitOLSconst(prepareX(Yt_val[[i]], xt),
                                                                 prepareX(Yt_val[[i]], zt)[,2:(ncol(zt)+1), drop=FALSE],
                                                                   dNt_val[[i]] - (dLambdaP_val[,i+1] - dLambdaP_val[,i]),
                                                                 Yt_val[[i]])
  )

  # Prvi integral:
  int_ztHz <- Reduce('+', lapply(1:ncol, function(i) mod_list[[i]][[1]]*diff_t[i]))
  int_ztHz_inv <- solve(int_ztHz)

  # Drugi integral:
  int_ztHdN <- Reduce('+', lapply(1:ncol, function(i) mod_list[[i]][[2]]))

  gamma_coef <- int_ztHz_inv%*%int_ztHdN

  Xminus <- lapply(1:ncol, function(i) mod_list[[i]][[3]])

  betas_list <- lapply(1:ncol, function(i) Xminus[[i]]%*%(matrix(dNt_val[[i]], ncol=1) - zt %*% gamma_coef * diff_t[i]))

  out <- list(betas_list=betas_list, betas_var_list=betas_var_list, gamma_coef=gamma_coef)

  return(out)
}

#' Fit an extended additive hazards model using relative survival
#'
#' Fits the Aalen additive hazard model using relative survival. The function can be used for multi-state model
#' data (as in the package mstate; class msdata) by supplying the start and stop times in the
#' Surv object and adding a strata(trans) object in the formula (where trans denotes the
#' transition in the multi-state model).
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param variance a logical value indicating whether the variances of the hazards should be computed.
#' Default is FALSE.
#' @param var_estimator Choose variance estimator, in the same way as in survaalen. The default option 'dN' uses dN(t)-dLambda_P(t) in the variance estimator, equivalent to formula 4.63 in Aalen et al. (2008). Option 'XdB' uses X*dB(t), see formula 4.64 in Aalen et al. (2008).
#' @param ratetable a table of event rates, organized as a \code{ratetable} object, such as \code{slopop}.
#' @param rmap an optional list to be used if the variables are not organized and named in the same way as in the \code{ratetable} object.
#' @param split.transitions only relevant if a multi-state model is fitted. An integer vector containing the numbered transitions that should be split. Use same numbering as in the given transition matrix.
#' @return An object of class \code{aalen.model}.
#' @seealso \code{survaalen}
#' @author Damjan Manevski
#' @keywords survival
#' @examples
#'
#' # Survival:
#' data(rdata)
#' mod <- rsaalen(Surv(time, cens)~sex+age, data=rdata, ratetable=slopop,
#'      rmap=list(age=age*365.241))
#' head(mod$coefficients)
#' tail(mod$coefficients)
#'
#' # Multi-state model:
#' data(ebmt1wide)
#' # Generate sex and year data (for illustrative purposes since it is not given in the data):
#' ebmt1wide$sex <- sample(1:2, size = nrow(ebmt1wide), replace = TRUE)
#' ebmt1wide$year <- as.Date('2010-01-01')
#'
#' mod <- rsaalen(Surv(Tstart, Tstop, status)~age.1+age.2+age.3+strata(trans), data=ebmt1wide,
#'                  ratetable = slopop, rmap = list(age=age*365.241), split.transitions = 2:3)
#' head(mod$coefficients$trans1)
#' head(mod$coefficients$trans2)
#' head(mod$coefficients$trans3)
rsaalen <- function(formula, data, variance=FALSE, var_estimator='dN',
                    ratetable=relsurv::slopop, rmap, split.transitions){

  # Find covariates and strata:
  covs <- attr(terms(formula), 'term.labels')
  strata_obj <- grep("strata\\(", covs, value=TRUE)

  # Prepare objects in case of const():
  formula_new <- formula
  covs_new <- covs
  constTRUE <- grepl('const', deparse(formula[[3]]))
  if(constTRUE){
    covs_const <- grepl('const\\(', covs)
    rnames_gamma <- covs[covs_const]

    covs_wconst <- covs
    covs_wconst[covs_const] <- gsub('const\\(', '', covs_wconst[covs_const])
    covs_wconst[covs_const] <- gsub('\\)', '', covs_wconst[covs_const])

    covs_new <- covs_wconst

    if(length(covs_wconst)>1){
      covs_wconst <- paste0(covs_wconst, collapse = ' + ')
    }
    formula_new <- as.formula(paste0(deparse(formula[[2]]), '~', covs_wconst))
  }

  # Run a multi-state model:
  if(length(strata_obj)>0){

    relsurv.mods <- vector("list", length(split.transitions))

    # Check:
    if(length(strata_obj)>1) stop('You have supplied multiple strata() objects in the formula. Please supply only one.')
    if(missing('split.transitions')) stop('Please define which transitions should be split in the split.transitions argument. If relative survival is not needed, use the survaalen function.')

    # Take care of rmap:
    if (!missing(rmap)) {
      rmap_tmp <- substitute(rmap)
      if(inherits(rmap_tmp, "call")){
        rmap <- rmap_tmp
      }
    }

    # Save original data:
    data_orig <- data
    data <- as.data.frame(data)

    # Find strata object:
    strata_obj <- gsub("strata\\(|\\)", "", strata_obj)
    strata_levels <- unique(data[,strata_obj])

    coefficients <- list()
    coefficients.var <- list()

    gamma <- vector("list", length = length(strata_levels))
    # gamma.var <- vector("list", length = length(strata_levels))

    all_times <- c()

    # Find max time:
    max_time <- max(data[,as.character(formula_new[[2]][3])])

    stevec <- 1

    # For every strata:
    for(i in 1:length(strata_levels)){
      # Subset data and covs:
      data_tmp <- data[data[,strata_obj]==strata_levels[i], ]
      covs_tmp <- covs[endsWith(covs, paste0(".", strata_levels[i])) | endsWith(covs, paste0(".", strata_levels[i], ')'))] # grep(paste0('.', strata_levels[i]), covs_new, value=TRUE)

      # Prepare formula:
      covs_tmp2 <- paste0(covs_tmp, collapse=' + ')
      formula_tmp <- as.formula(paste0(deparse(formula_new[[2]]), '~', covs_tmp2))

      # Run model:
      if(i %in% split.transitions){
        mod_tmp <- rsaalen(formula_tmp, data_tmp, variance, var_estimator, ratetable, rmap)

        relsurv.mods[[stevec]] <- mod_tmp
        stevec <- stevec+1
      } else{
        mod_tmp <- survaalen(formula_tmp, data_tmp, variance)
      }

      # Remove time 0:
      # if(mod_tmp$coefficients[1,1]==0){
      #   mod_tmp$coefficients <- mod_tmp$coefficients[2:nrow(mod_tmp$coefficients),]
      # }

      # Save coefficients:
      coefficients[[i]] <- mod_tmp$coefficients
      coefficients.var[[i]] <- mod_tmp$coefficients.var

      all_times <- c(all_times, mod_tmp$coefficients[,1])

      if(constTRUE){
        if('gamma' %in% names(mod_tmp)){
          gamma[[i]] <- mod_tmp$gamma
          # if(variance){
          #   gamma.var[[i]] <- mod_tmp$gamma.var
          # }
        }
      }
    }
    names(coefficients) <- paste0('trans', strata_levels)
    if(variance) names(coefficients.var) <- paste0('trans', strata_levels)
    names(gamma) <- paste0('trans', strata_levels)
    # names(gamma.var) <- paste0('trans', strata_levels)

    # Add all times in the mstate model:
    all_times <- sort(unique(c(all_times, max_time)))
    for(i in 1:length(strata_levels)){
      all_times_tmp <- all_times[!(all_times %in% coefficients[[i]][,1])]
      tmp_df <- matrix(NA, nrow=length(all_times_tmp), ncol= ncol(coefficients[[i]]))

      tmp_df[,1] <- all_times_tmp
      colnames(tmp_df) <- colnames(coefficients[[i]])

      coefficients[[i]] <- rbind(coefficients[[i]], tmp_df)
      coefficients[[i]] <- coefficients[[i]][order(coefficients[[i]][,1]),]

      if(variance){
        if(nrow(coefficients.var[[i]]) != 0){
          coefficients.var[[i]] <- rbind(coefficients.var[[i]], tmp_df)
          coefficients.var[[i]] <- coefficients.var[[i]][order(coefficients.var[[i]][,1]),]
        }
      }

      for(j in 2:ncol(coefficients[[i]])){
        coefficients[[i]][,j] <- mstateNAfix(coefficients[[i]][,j], 0)
        if(variance){
          if((nrow(coefficients.var[[i]]) != 0)){
            coefficients.var[[i]][,j] <- mstateNAfix(coefficients.var[[i]][,j], 0)
          }
        }
      }
    }

    names(relsurv.mods) <- split.transitions

    if(constTRUE){
      if(variance){
        out <- list(coefficients=coefficients, coefficients.var=coefficients.var,
                    gamma=gamma, #gamma.var=gamma.var,
                    split.transitions=split.transitions,
                    formula=formula, ratetable=ratetable, rmap=rmap,
                    relsurv.mods=relsurv.mods)
      } else{
        out <- list(coefficients=coefficients,
                    gamma=gamma,
                    split.transitions=split.transitions,
                    formula=formula, ratetable=ratetable, rmap=rmap,
                    relsurv.mods=relsurv.mods)
      }
    } else{
      if(variance){
        out <- list(coefficients=coefficients, coefficients.var=coefficients.var,
                    split.transitions=split.transitions,
                    formula=formula, ratetable=ratetable, rmap=rmap,
                    relsurv.mods=relsurv.mods)
      } else{
        out <- list(coefficients=coefficients,
                    split.transitions=split.transitions,
                    formula=formula, ratetable=ratetable, rmap=rmap,
                    relsurv.mods=relsurv.mods)
      }
    }

    class(out) <- 'aalen.model'

    return(out)

  } else{
    if(!missing('split.transitions')) warning('The split.transitions argument is ignored since you are not using a multi-state model.')

    # Save original data:
    data_orig <- data

    # Take care of rmap:
    if (!missing(rmap)) {
      rmap_tmp <- substitute(rmap)
      if(inherits(rmap_tmp, "call")){
        rmap <- rmap_tmp
      }
    }

    # Standard relsurv format:
    rform <- rformulate(formula_new, data, ratetable, stats::na.omit(),
                        rmap)

    # Objects:
    data <- rform$data
    xt <- rform$X
    ratetable <- rform$ratetable

    # Times:
    all_times <- unique(c(data$start, data$Y))
    event_times <- unique(data$Y[data$stat==1])

    all_times <- sort(all_times)
    event_times <- sort(event_times)

    # Take only times until last event time:
    all_times <- all_times[all_times <= event_times[length(event_times)]]

    # Find Yt / dNt:
    # dNt <- dNt(data, all_times)
    # Yt <- Yt(data, all_times)
    # dLambdaPs_0 <- dLambdaP0(data, all_times, event_times, ratetable, atts)

    atts <- attributes(ratetable)
    atts$cutpoints <- atts$cutpoints[lapply(atts$cutpoints,length)>0]
    # dLambdaPs <- dLambdaP(as.matrix(data), all_times, event_times, as.vector(ratetable), atts)

    # xx1 <- prepareX(Yt[[1]], as.matrix(xt))
    # fuu <- fitOLS2(xx1, dNt[[1]], Yt[[1]])

    # Find betas:
    # betas <- calculateBetas(data, as.matrix(xt), event_times)
    # betas <- calculateBetasRelsurv(data, as.matrix(xt), event_times, all_times, as.vector(ratetable), atts, as.matrix(data))
    # betas <- calculateBetasRelsurv2(data, as.matrix(xt), event_times, all_times, as.vector(ratetable), atts, as.matrix(data))

    # betas <- calculateBetasRelsurv22(data, as.matrix(xt), event_times, all_times, as.vector(ratetable), atts, as.matrix(data), 1)
    if(constTRUE){
      zt <- xt[,covs_const, drop=FALSE]
      xt <- xt[,!covs_const, drop=FALSE]

      # all_times_w0 <- all_times[2:length(all_times)]
      # betas0 <- calculateBetasGammasRelsurv22R(data, as.matrix(xt), as.matrix(zt), event_times, all_times_w0, ratetable, atts, as.matrix(data), 1)
      betas0 <- calculateBetasGammasRelsurv22R(data, as.matrix(xt), as.matrix(zt), event_times, all_times, ratetable, atts, as.matrix(data), 1)
    } else{
      betas0 <- calculateBetasRelsurv22R(data, as.matrix(xt), event_times, all_times, ratetable, atts, as.matrix(data), variance, var_estimator)
    }

    # Variance:
    betas_var <- betas0[[2]]
    betas_var <- lapply(betas_var, function(x) diag(x))
    betas_var2 <- do.call(rbind, betas_var)

    # Point estimates:
    betas <- betas0[[1]]
    # Take care of format:
    betas2 <- t(do.call(cbind, betas))
    # betas2 <- rbind(matrix(0, nrow=1, ncol=(ncol(xt)+1)),
    #                 betas2)

    # Cumulative:
    for(iu in 1:ncol(betas2)){
      betas2[,iu] <- cumsum(betas2[,iu])
      if(ncol(betas_var2)!=0){
        betas_var2[,iu] <- cumsum(betas_var2[,iu])
      }
    }

    # Add times:
    if(constTRUE){
      # betas2 <- cbind(all_times_w0, betas2)
      # betas2 <- betas2[all_times_w0 %in% c(0, event_times),]
      betas2 <- cbind(all_times, betas2)
    } else{
      betas2 <- cbind(all_times, betas2)
    }

    # Add times:
    if(ncol(betas_var2)!=0){
      betas_var2 <- cbind(all_times, betas_var2)
    }

    # Add naming:
    colnames(betas2) <- c('time', '(Intercept)', colnames(xt))
    if(ncol(betas_var2)!=0){
      colnames(betas_var2) <- c('time', '(Intercept)', colnames(xt))
    }

    # Don't have double zeros:
    # if(betas2[1,1] == 0 & betas2[2,1] == 0){
    #   betas2 <- betas2[-1,]
    # }

    betas3 <- betas2[betas2[,1] %in% c(0, event_times), ]

    if(!(0 %in% betas3[,1])){
      betas3 <- rbind(matrix(0, nrow=1,ncol=ncol(betas3)), betas3)
    }

    if(ncol(betas_var2)!=0){
      betas_var2 <- betas_var2[betas_var2[,1] %in% c(0, event_times), ]
    }

    # Prepare final object:
    if(variance){
      var.obj <- betas_var2
      # var.obj[,2:ncol(var.obj)] <- abs(var.obj[,2:ncol(var.obj)])/4

      out <- list(coefficients=betas3, coefficients.var=var.obj, # prej je betas_var2
                  formula=formula, ratetable=ratetable, rmap=rmap)
    } else{
      out <- list(coefficients=betas3,
                  formula=formula, ratetable=ratetable, rmap=rmap)
    }


    if(constTRUE){
      rownames(betas0[[3]]) <- rnames_gamma
      colnames(betas0[[3]]) <- 'gamma'

      out[[length(out)+1]] <- betas0[[3]]
      names(out)[length(out)] <- 'gamma'

      # if(variance){
      #   out[[length(out)+1]] <- betas0[[3]] # tmp
      #   colnames(out[[length(out)]]) <- 'gamma.var'
      #   names(out)[length(out)] <- 'gamma.var'
      # }
    }

    class(out) <- 'aalen.model'

    return(out)
  }
}

# TO DO:
# implement variances
# other left-truncation solutions? for now you put 0s
# time dependent covariates?

# multi-state:
# Popravi racunanje vo multi-state - za sea site vreminja gi vlecis, ne zemas vo predvid promeni vo L_P

#' Subject-specific prediction for an Aalen model
#'
#' A function that returns subject-specific predictions from a survaalen or rsaalen object.
#' @param object A survaalen or rsaalen object
#' @param newdata A data.frame with one row (add covariate values in columns)
#' @param type By default ('Haz') returns cumulative hazards. If 'prob' it calculates CDF/CIFs. If 'both' a list is returned with Haz and prob.
#' @param ... Not used for now
#' @return A data.frame containing the times and survival (or excess/population) estimates.
#'
#' @author Damjan Manevski \email{damjan.manevski@@mf.uni-lj.si}
#' @examples
#' data(rdata)
#' mod <- survaalen(Surv(time, cens)~sex+age, data=rdata)
#' predict(mod, data.frame(age=68, sex=2))
#'
#' mod <- rsaalen(Surv(time, cens)~sex+age, data=rdata, ratetable=slopop,
#'                 rmap=list(age=age*365.241))
#' predict(mod, data.frame(age=68, sex=2, year=as.Date('1982-06-24')))
predict.aalen.model <- function(object, newdata, type='Haz', ...){
  # Zaenkrat delamo z object$times. Treba bo dodati se add.times.

  type <- match.arg(type, c("Haz", "prob", "both"))

  # lin.pred <- object$coefficients[,2]
  # if(ncol(object$coefficients) >= 3){
  #   for(cov in colnames(object$coefficients)[3:length(colnames(object$coefficients))]){
  #     lin.pred <- lin.pred + newdata[1,cov]*object$coefficients[,cov]
  #   }
  # }

  kar <- deparse(object$formula[[2]])
  autkam <- gsub(' ', '', strsplit(substr(kar, start = 6, stop=nchar(kar)-1), ',')[[1]])
  newdata[autkam] <- 1

  # Calculate linear predictor:
  lin.pred <- as.vector(model.matrix(object, newdata[1,])%*%t(object$coefficients[,2:ncol(object$coefficients)]))


  # rsaalen:
  if('ratetable' %in% names(object)){
    newdata_2 <- data.frame(matrix(1, nrow=1, ncol=length(autkam)))
    if(length(autkam)==3) newdata_2[,1] <- 0
    colnames(newdata_2) <- autkam
    newdata_2 <- cbind(newdata, newdata_2)

    rform <- suppressWarnings(rformulate(object$formula,
                                         newdata_2, object$ratetable, stats::na.omit(), rmap = object$rmap))


    fk <- (attributes(rform$ratetable)$factor != 1)
    nfk <- length(fk)
    pop.surv <- sapply(1:nrow(object$coefficients), function(i) exp_prep(rform$data[1, 4:(nfk + 3), drop = FALSE], object$coefficients[i,1], object$ratetable))
    Haz.p <- -log(pop.surv)

    if(type=='Haz'){
      data.frame(time=object$coefficients[,1], Haz.e=lin.pred, Haz.p=Haz.p)
    } else if(type=='prob'){
      S_O <- exp(-(lin.pred + Haz.p))

      prob.e <- cumsum(S_O*diff(c(0, lin.pred)))
      prob.p <- cumsum(S_O*diff(c(0, Haz.p)))

      data.frame(time=object$coefficients[,1], prob.e=prob.e, prob.p=prob.p)
    } else{
      out <- list()
      out$Haz <- data.frame(time=object$coefficients[,1], Haz.e=lin.pred, Haz.p=Haz.p)

      # copied from up:
      S_O <- exp(-(lin.pred + Haz.p))

      prob.e <- cumsum(S_O*diff(c(0, lin.pred)))
      prob.p <- cumsum(S_O*diff(c(0, Haz.p)))
      out$prob <- data.frame(time=object$coefficients[,1], prob.e=prob.e, prob.p=prob.p)
      out

    }
  } else{
    # survaalen:
    if(type=='Haz'){
      data.frame(time=object$coefficients[,1], Haz=lin.pred)
    } else if(type=='prob'){
      data.frame(time=object$coefficients[,1], prob=1-exp(-lin.pred))
    } else{
      out <- list()
      out$Haz <- data.frame(time=object$coefficients[,1], Haz=lin.pred)
      out$prob <- data.frame(time=object$coefficients[,1], prob=1-exp(-lin.pred))
      out
    }

  }
}

ratetable_add_vals <- function(ratetable, add.years=c(), add.ages=c()){
  # The function takes a ratetable and adds years and ages to the object.

  # New attribute object:
  newat <- attributes(ratetable)
  # Find dates (3 or 4 values):
  wh_date <- newat$type>=3
  # Find ages:
  wh_age <- newat$type==2

  # Find unique years/ages not present in the ratetable:
  add.years <- add.years[!(add.years %in% newat$dimnames$year)]
  add.ages <- add.ages[!(add.ages %in% newat$dimnames$age)]
  # All years:
  nova_leta <- sort(c(add.years, newat$dimnames$year))
  nove_starosti <- as.character(sort(as.integer(c(add.ages, newat$dimnames$age))))
  # Find where the new years appear:
  m_nova_leta <- match(nova_leta, add.years, nomatch = 0)
  m_nove_starosti <- match(nove_starosti, add.ages, nomatch = 0)

  # Adjust newat object:
  newat$dimnames$year <- nova_leta
  newat$dim[wh_date] <- length(nova_leta)
  if(length(add.years)>0) newat$cutpoints[[which(wh_date)]] <- sort(c(newat$cutpoints[[which(newat$dimid=='year')]], as.Date(paste0(add.years, '-01-01'))))
  # newat$cutpoints[[which(wh_date)]] <- as.date(newat$cutpoints[[which(wh_date)]])

  newat$dimnames$age <- nove_starosti
  newat$dim[wh_age] <- length(nove_starosti)
  newat$cutpoints[[which(wh_age)]] <- sort(c(newat$cutpoints[[which(newat$dimid=='age')]], as.integer(add.ages)*365.241))

  # prepare vecs:
  vecs <- vector("list", length=length(newat$type))
  for(i in 1:length(newat$type)){
    vecs[[i]] <- 1:newat$dim[i]
  }

  # Define new ratetable:
  out <- array(NA, dim=newat$dim)

  # Add values for existing columns:
  out[m_nove_starosti==0,m_nova_leta==0,] <- ratetable

  # YEAR: For remaining columns either take earlier or later values:
  wh_diff0 <- which(m_nova_leta!=0)
  wh_eq0 <- which(m_nova_leta==0)
  for(ay in wh_diff0){
    # If first year:
    if(ay==1){
      possible_val <- wh_eq0[ay<wh_eq0]
      choose_year <- min(possible_val)
    } else{
      # If later year take last year before:
      possible_val <- wh_eq0[ay>wh_eq0]
      if(length(possible_val)>0){
        choose_year <- max(possible_val)
        # Except if there is no last year, then take next year:
      } else{
        possible_val <- wh_eq0[ay<wh_eq0]
        choose_year <- min(possible_val)
      }
    }
    # Add values to out:
    vecs_tmp <- vecs
    vecs_tmp[[which(wh_date)]] <- ay
    out[vecs_tmp[[1]],vecs_tmp[[2]],vecs_tmp[[3]]] <- out[vecs_tmp[[1]],choose_year,vecs_tmp[[3]]]
  }
  # AGE: For remaining columns either take earlier or later values:
  wh_diff0 <- which(m_nove_starosti!=0)
  wh_eq0 <- which(m_nove_starosti==0)
  for(ay in wh_diff0){
    # If later age take last age before:
    possible_val <- wh_eq0[ay>wh_eq0]
    if(length(possible_val)>0){
      choose_age <- max(possible_val)
      # Except if there is no last age, then take next age:
    } else{
      possible_val <- wh_eq0[ay<wh_eq0]
      choose_age <- min(possible_val)
    }

    # Add values to out:
    vecs_tmp <- vecs
    vecs_tmp[[which(wh_age)]] <- ay
    out[vecs_tmp[[1]],vecs_tmp[[2]],vecs_tmp[[3]]] <- out[choose_age, vecs_tmp[[2]],vecs_tmp[[3]]]
  }

  # Return:
  attributes(out) <- newat
  out
}

# s2 <- slopop
# add.years <- c('1920', "1950", "2024")
# add.ages <- c(110)
#
# newrt <- ratetable_add_vals(s2, add.years, add.ages)
# # newrt[,"1920",] %>% head()
# # newrt[,"1930",] %>% head()
# identical(newrt[,"1920",], newrt[,"1930",])
# identical(newrt[,"1950",], newrt[,"1948",])
# identical(newrt[,"2024",], newrt[,"2021",])

colVars <- function(x, na.rm = FALSE){
  f <- function(v, na.rm = na.rm) {
    if(is.numeric(v) || is.logical(v) || is.complex(v))
      stats::var(v, na.rm = na.rm)
    else NA
  }
  return(unlist(lapply(x, f, na.rm = na.rm)))
}

# Copied function from mstate:::NAfix.
mstateNAfix <- function (x, subst = -Inf)
{
  spec <- max(x[!is.na(x)]) + 1
  x <- c(spec, x)
  while (any(is.na(x))) x[is.na(x)] <- x[(1:length(x))[is.na(x)] -
                                           1]
  x[x == spec] <- subst
  x <- x[-1]
  x
}

# Helper function:
nessie_spi <- function(formula = formula(data), data, ratetable = relsurv::slopop,
                       tis, starting.time, include.censoring=FALSE,
                       arg.example=FALSE, rmap){
  data_orig <- data
  call <- match.call()
  if (!missing(rmap)) {
    rmap <- substitute(rmap)
  }
  na.action <- NA
  rform <- rformulate(formula, data, ratetable, na.action,
                      rmap)

  data <- rform$data
  data$Xs <- rep(1, nrow(data))
  n_rows <- nrow(data)

  # Fix demographic covariates:
  if(starting.time == "left.truncated"){
    rform$R[,"year"] <- rform$R[,"year"] - rform$R[,"age"]
    rform$R[,"age"] <- 0
  }
  if(include.censoring){
    # browser()
    wh <- which(rform$status==1)
    rform$Y[wh] <- max(rform$Y)

    if(arg.example){
      wh2 <- which(rform$status==1 & data$age==18262)
      rform$Y[wh2] <- 1826
    }
  }
  else{
    rform$Y <- rep(max(rform$Y), length(rform$Y))
    # status is not relevant in this case
  }

  out <- NULL
  out$yi <- NULL
  out$yidli <- NULL
  l_tis <- length(tis)
  temps <- lapply(1:n_rows, function(inx) {
    temp <- exp_prep(rform$R[inx, , drop = FALSE], rform$Y[inx], rform$ratetable,
                     rform$status[inx], times = tis, fast = TRUE, cmp=FALSE,ys=data$start[inx])
    s_pi <- exp(-cumsum(temp$yidli))

    s_pi_helper <- which.min(temp$yidli==0)-1
    if(s_pi_helper>1){ s_pi[1:s_pi_helper] <- 0}

    if(include.censoring){ s_pi[(s_pi_helper+1):l_tis] <- pmin(s_pi[(s_pi_helper+1):l_tis],
                                                               temp$yi[(s_pi_helper+1):l_tis])}

    c(s_pi, # s_pi
      temp$yidli*s_pi) # l_pi * s_pi
  })

  temps2 <- do.call("cbind", temps)

  temps2 <- rowSums(temps2)

  out$yi <- temps2[1:(length(temps2)/2)]
  out$yidli <- temps2[(length(temps2)/2+1):length(temps2)]
  return(out)
}

# Copied scales::trans_new:
# scales_trans_new <- function (name, transform, inverse, breaks = extended_breaks(),
#           minor_breaks = regular_minor_breaks(), format = format_format(),
#           domain = c(-Inf, Inf))
# {
#   if (is.character(transform))
#     transform <- match.fun(transform)
#   if (is.character(inverse))
#     inverse <- match.fun(inverse)
#   structure(list(name = name, transform = transform, inverse = inverse,
#                  breaks = breaks, minor_breaks = minor_breaks, format = format,
#                  domain = domain), class = "trans")
# }


#' Compute one of the life years measures
#'
#' Provides an estimate for one of the following measures: years lost (Andersen, 2013), years lost/saved (Andersen, 2017), or
#' life years difference (Manevski, Ruzic Gorenjec, Andersen, Pohar Perme, 2022).
#'
#' The life years difference (\code{measure='yd'}) is taken by default. If other
#' measures are of interest, use the \code{measure} argument.
#'
#' The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with the \code{rmap} argument. For example, if
#' age is in years in the data but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.
#'
#' Numerical integration is performed, argument
#' precision is set with argument \code{precision}, which defaults to 30-day
#' intervals for intergration. For higher accuracy take a smaller value (e.g. precision=1 makes
#' the integration on a daily basis).
#'
#' The observed curves are reported at event and censoring times. The
#' population curves are reported at all times used for the numerical integration.
#' Note that for the years lost (Andersen, 2013) measure, only the excess absolute risk is reported.
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, \code{~1} specified on the right.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param measure choose which measure is used: 'yd' (life years difference; Manevski, Ruzic Gorenjec, Andersen, Pohar Perme, 2022), 'yl2017' (years lost/saved; Andersen 2017),
#' 'yl2013' (years lost/saved; Andersen 2013).
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @param var.estimator Choose the estimator for the variance ('none', 'bootstrap', 'greenwood'). Default is 'none'.
#' The 'greenwood' option is possible only for \code{measure='yd'}.
#' @param B if \code{var.estimator} is 'bootstrap'. The number of bootstrap replications. Default is 100.
#' @param precision precision for numerical integration of the population curve. Default is 30 (days).
#' The value may be decreased to get a
#' higher precision or increased to achieve a faster calculation.
#' @param add.times specific times at which the curves should be reported.
#' @param na.action a missing-data filter function. Default is \code{na.omit}.
#' @param conf.int the confidence level for a two-sided confidence interval. Default is 0.95.
#' @param timefix the timefix argument in survival::survfit.formula. Default is FALSE.
#' @param is.boot if TRUE, the function \code{years} has been called during a bootstrap replication.
#' @param first.boot if TRUE, this is the first bootstrap replication.
#' @return A list containing the years measure, the observed and population curves (or the excess curve for Andersen 2013).
#' The values are given as separate data.frames through time. Times are given in days, all areas are given in years.
#' For \code{measure='yl2017'} values are reported only at the last time point.
#' Functions \code{plot_f} and \code{plot_years} can be then used for plotting.
#' @seealso \code{\link{plot_f}}, \code{\link{plot_years}}
#' @examples
#'
#' library(relsurv)
#' # Estimate the life years difference for the rdata dataset.
#' mod <- years(Surv(time, cens)~1, data=rdata, measure='yd', ratetable=slopop,
#'              rmap=list(age=age*365.241), var.estimator = 'none')
#' # Plot the absolute risk (observed and population curve):
#' plot_f(mod)
#' # Plot the life years difference estimate:
#' plot_years(mod, conf.int=FALSE)
years <- function(
    formula=formula(data),
    data,
    measure=c('yd', 'yl2017', 'yl2013'),
    # estimator=c("F_P_final"),#, "F_P_Spi", "F_P_Spi2", "F_P", "F_P2", "all"),
    ratetable=relsurv::slopop,
    rmap,
    var.estimator=c('none', 'bootstrap', 'greenwood'),
    B=100,
    precision=30,
    add.times,
    na.action=stats::na.omit,
    conf.int=0.95,
    timefix=FALSE,
    # admin.cens,
    # cause.val,
    is.boot=FALSE,
    first.boot=FALSE
    # ,estimator.observed='Kaplan-Meier'
){

  # OLD ARGUMENTS:
  # F_P_Spi: Tako kot F_P_final, ignorira censoring. Ali pa vzame samo admin cens
  # F_P_Spi2: Vzame ves censoring
  # @param cause.val for competing risks, to be added.
  # @param admin.cens if a Date is supplied, administrative censoring is taken into account at that time
  # in the population curve. Works only if there's late entry, e.g. if the formula is \code{Surv(start,stop,event)~1}.

  ############ #
  # PREPARE OBJECTS:
  ############ #

  estimator=c("F_P_final") #  #' @param estimator which estimator should be used for calculating
  # estimator <- match.arg(estimator)
  arg.example <- FALSE # @param arg.example temporary argument, used for checking additionalities.

  Call <- match.call()
  if(!missing(rmap) & !is.boot & !first.boot)  rmap <- substitute(rmap)
  measure <- match.arg(measure)
  var.estimator <- match.arg(var.estimator)

  if(var.estimator=='bootstrap'){
    bootstrap <- TRUE
  } else if(var.estimator %in% c('none', 'greenwood')){
    bootstrap <- FALSE
  } else{
    stop('Incorrect value provided in argument var.estimator.')
  }

  if(!is.data.frame(data)) stop('Argument data is not a data.frame object.')
  data <- as.data.frame(data)

  out <- NULL
  late.values <- FALSE

  # These were arguments. To be deleted?
  exact.hazards <- FALSE # calculate hazards on a daily basis (to be checked)
  find.cond.time <- FALSE # if TRUE, return time at which there are at least 5 individuals in the at-risk set.

  # if(!missing(cause.val)){
  #   data$status <- ifelse(data$cause == cause.val, 1, 0)
  #   # Remove NAs:
  #   eniNAs <- which(is.na(data$status))
  #   if(length(eniNAs)>0) data <- data[-eniNAs,]
  # }

  # data$age <- round(data$age*365.241)
  # data$stop <- round(data$stop*365.241)

  # If Surv(start,stop, event) (possibly + mstate)
  if_start_stop <- length(as.character(formula[[2]])) %in% c(4,5)

  if(if_start_stop){
    start_col <- as.character(formula[[2]])[2]
    stop_col <- as.character(formula[[2]])[3]
    status_col <- as.character(formula[[2]])[4]
    starting_age <- as.vector(as.matrix(data[, start_col]))
  } else{
    stop_col <- as.character(formula[[2]])[2]
    if(!(stop_col %in% colnames(data))){
      stop(paste0('Instead of \'', stop_col, '\', please use a column from the data in the formula.'))
    }
  }

  # Check if no. at risk falls to zero at some point:
  if(if_start_stop){
    # Prepare at-risk matrix:
    find_tajms <- unique(sort(c(data[,start_col], data[,stop_col])))
    mat <- lapply(1:nrow(data), function(x) ifelse((data[x, start_col] < find_tajms) & (find_tajms <= data[x, stop_col]), 1, 0))
    mat2 <- matrix(unlist(mat), nrow = nrow(data), byrow = TRUE)
    # The sum of the individual at-risk processes:
    yi_left <- colSums(mat2)
    # If there's an empty at-risk at a later timepoint, censor the data:
    wh_yi <- which(yi_left==0)
    if(length(wh_yi)>1){
      if((!is.boot) & (!first.boot)){
        warning(paste0('In the time interval ', find_tajms[wh_yi[2]-1], '-', find_tajms[wh_yi[2]],
                       ' the at-risk sample is empty (nobody is followed). Survival cannot be estimated in this time interval.',
                       ' The data is censored at time ', find_tajms[wh_yi[2]-1], '.'))
      }
      # Censor data:
      data <- data[data[,start_col] <= find_tajms[wh_yi[2]-1], ]
      wh_cen <- which(data[, stop_col] > find_tajms[wh_yi[2]-1])
      data[wh_cen, stop_col] <- find_tajms[wh_yi[2]-1]
      data[wh_cen, status_col] <- 0
      if(!missing(add.times)){
        if(any(add.times > find_tajms[wh_yi[2]-1])) add.times <- add.times[add.times<=find_tajms[wh_yi[2]-1]]
      }
    }
    rm(mat,mat2)
  }
  data_orig <- data

  # if(starting.time=="left.truncated"){
  # if(!missing(admin.cens)){
  #   if(!inherits(admin.cens, 'Date')) warning('Object of class Date should be supplied to admin.cens.')
  #   end_date <- data$year+(data$stop-data$age)
  #   if(any(end_date > admin.cens)) warning('There are events that occur after the date of administrative censoring. Please check the values in arguments data and admin.cens.')
  #   id_admin_cens <- which(admin.cens==end_date)
  # }
  # }

  if(if_start_stop){
    starting.time <- 'left.truncated'
  } else{
    starting.time <- 'zero'
  }

  # Starting age
  starting_age <- rep(0,nrow(data))
  if(if_start_stop){
    starting_age <- as.vector(as.matrix(data[, start_col]))
  }
  starting_age <- as.numeric(starting_age)

  ############ #
  # YEARS ON DATA - GENERAL:
  ############ #

  surv_obj <- as.character(formula[[2]])

  if(missing(formula)){
    stop('Missing formula argument value.')
  } else{
    if('mstate' %in% surv_obj){
      juh <- 1:nrow(data)
      mod <- survival::survfit.formula(as.formula(Reduce(paste, deparse(formula))), data=data, timefix=timefix, id = juh, na.action=na.action)
    } else{
      mod <- survival::survfit.formula(formula, data=data, timefix=timefix, na.action=na.action)
    }
  }

  if('mstate' %in% surv_obj){
    surv_obj_new <- paste0(surv_obj[1], '(', surv_obj[2], ',', surv_obj[3])
    if(length(surv_obj)==5){
      surv_obj_new <- paste0(surv_obj_new, ',', surv_obj[4], ')')
    } else{
      surv_obj_new <- paste0(surv_obj_new, ')')
    }
    formula <- paste0(surv_obj_new, '~1')
  }
  status_obj <- surv_obj[length(surv_obj)]

  # if(!missing(cause.val)){
  #   mod$n.risk <- mod$n.risk[,1]
  #   mod$n.event <- mod$n.event[,cause.val+1]
  #   mod$surv <- 1-mod$pstate[,cause.val+1]
  #   mod$std.err <- mod$std.err[,cause.val+1]
  #   mod$cumhaz <- mod$cumhaz[,cause.val]
  # }

  if(!missing(add.times)){
    mod_sum <- summary(mod, times = sort(unique(c(mod$time, add.times))))
    if(any(!(add.times %in% mod_sum$time))){
      if(!is.boot){
        if(!first.boot){
          warning('Some values in add.times are after the last follow-up time. All measures are extrapolated up to these times. Please consider removing them.')
        }
        late.values <- TRUE

        miss_tajms <- add.times[!(add.times %in% mod_sum$time)]
        mod_sum$time <- c(mod_sum$time, miss_tajms)
        mod_sum$n.risk <- c(mod_sum$n.risk, rep(mod_sum$n.risk[length(mod_sum$n.risk)], length(miss_tajms)))
        mod_sum$n.event <- c(mod_sum$n.event, rep(0, length(miss_tajms)))
        mod_sum$surv <- c(mod_sum$surv, rep(mod_sum$surv[length(mod_sum$surv)], length(miss_tajms)))
        mod_sum$cumhaz <- c(mod_sum$cumhaz, rep(mod_sum$cumhaz[length(mod_sum$cumhaz)], length(miss_tajms)))

        # First fix std.err:
        if(is.nan(mod_sum$std.err[length(mod_sum$std.err)])){
          mod_sum$std.err[length(mod_sum$std.err)] <- mod_sum$std.err[length(mod_sum$std.err) - 1]
        }
        mod_sum$std.err <- c(mod_sum$std.err, rep(mod_sum$std.err[length(mod_sum$std.err)], length(miss_tajms)))
      }
    }
    mod$time <- mod_sum$time
    mod$n.risk <- mod_sum$n.risk
    mod$n.event <- mod_sum$n.event
    mod$surv <- mod_sum$surv
    mod$std.err <- mod_sum$std.err
    mod$cumhaz <- mod_sum$cumhaz
  }

  if(find.cond.time) return(mod$time[which.min(mod$n.risk<5)])

  # Calculate AUC:
  if(length(mod$time)>1){
    if(if_start_stop){
      survs <- c(1, mod$surv[1:(length(mod$surv)-1)])
      t_diff <- diff(c(mod$time[1], mod$time))
    } else{
      survs <- mod$surv
      t_diff <- diff(c(0, mod$time))
    }
    auc_data <- sum(t_diff*(1 - survs))
    auc_data_vec <- cumsum(t_diff*(1 - survs))
  } else{
    auc_data <- mod$time*mod$surv
    auc_data_vec <- auc_data
  }

  out$F_data <- 1-mod$surv
  out$auc_data <- auc_data/365.241
  out$auc_data_vec <- auc_data_vec/365.241

  # Exact hazards:
  if(exact.hazards){
    mod$time <- seq(min(mod$time), max(mod$time), by=1)
    mod$surv <- exp(-cumsum(rep(ratetable[1,1,1], max(mod$time)-min(mod$time)+1)))

    out$F_data <- 1-exp(-cumsum(c(0, rep(ratetable[1,1,1], max(mod$time)-min(mod$time)))))
    out$auc_data <- sum(out$F_data)/365.241
  }

  ############ #
  # SEPARATE YEARS FOR EVERY MEASURE:
  ############ #

  if(measure %in% c('yl2017', 'yl2013')){
    # YL_P preparation:
    data_yi <- data
    rform <- rformulate(formula, data, ratetable, na.action=na.action, rmap = rmap)
    data <- rform$data
    if(if_start_stop){
      if(!(start_col %in% colnames(data))){
        data[,start_col] <- data_orig[, start_col]
      }
    }

    # Check covariates:
    p <- rform$m
    if (p > 0) stop("There shouldn't be any covariates in the formula. This function gives non-parametric estimates of the hazards.")
    else data$Xs <- rep(1, nrow(data)) #if no covariates, just put 1

    out_n <- table(data$Xs) #table of strata
    out$time <- out$haz.excess <- out$haz.pop <- out$std.err <- out$strata <-  NULL

    kt <- 1 # the only stratum
    inx <- which(data$Xs == names(out_n)[kt]) #individuals within this stratum
    # tis <- sort(unique(rform$Y[inx])) #unique times
    if(!if_start_stop){
      tis <- rform$Y[inx] #unique times
      tis_seq <- seq(0, max(rform$Y[inx]), precision)
    } else{
      tis <- sort(unique(c(rform$Y[inx], data[, start_col]))) #unique times
      tis_seq <- seq(min(data[, start_col]), max(rform$Y[inx], data[, start_col]), precision)
    }

    if(!is.boot){
      tis <- sort(unique(c(tis, tis_seq)))
    }

    if(!missing(add.times)){
      tis <- sort(unique(c(tis, add.times)))
    }
    ltis <- length(tis)

    # Fix demographic covariates:
    if(if_start_stop){
      rform$R[,"year"] <- rform$R[,"year"] - rform$R[,"age"]
      rform$R[,"age"] <- 0
    }

    if(measure == 'yl2017'){
      # YL_O (used only for yl2017):
      if(if_start_stop){
        it_auc <- rep(NA, nrow(data_orig))
        mod_sum <- summary(mod, times=tis) # unique(sort(c(data_orig[,start_col], data_orig[,stop_col])))
        lsurv <- length(mod_sum$surv)
        val_mat <- matrix(0, nrow=nrow(data_orig), ncol=lsurv)
        for(it in 1:nrow(data_orig)){
          it_wh <- which(data_orig[it, start_col] == mod_sum$time)
          it_surv <- mod_sum$surv[it_wh:lsurv]/mod_sum$surv[it_wh]
          it_auc[it] <- sum(c(0, diff(mod_sum$time[it_wh:lsurv]))*(1 - it_surv))/365.241
          val_mat[it, it_wh:lsurv] <- cumsum(c(0, diff(mod_sum$time[it_wh:lsurv]))*(1 - it_surv))/365.241
        }
        # spodaj <- mod_sum$n.risk + cumsum(mod_sum$n.event) + cumsum(mod_sum$n.censor)
        YL_O_vec <- colMeans(val_mat) # colSums(val_mat)/spodaj
        YL_O <- mean(it_auc)
        F_O_time <- mod_sum$time
        F_O_ext <- data.frame(time=F_O_time, area=YL_O_vec)
        # Subset:
        F_O_ext2 <- subset(F_O_ext, time %in% mod$time)
        F_O_time <- F_O_ext2$time
        YL_O_vec <- F_O_ext2$area
      } else{
        YL_O_vec <- out$auc_data_vec
        YL_O <- out$auc_data
        F_O_time <- mod$time
        if(!(0 %in% F_O_time)){
          F_O_time <- c(0, F_O_time)
          YL_O_vec <- c(0, YL_O_vec)
        }
        # Prepare extended F_O object:
        if(0 %in% mod$time){
          F_O_temp <- data.frame(time=mod$time, surv=mod$surv)
        } else{
          F_O_temp <- data.frame(time=c(0, mod$time), surv=c(1, mod$surv))
        }
        F_O_ext <- data.frame(time=tis)
        F_O_ext <- merge(F_O_ext, F_O_temp, by='time', all.x=TRUE)
        F_O_ext$surv <- mstateNAfix(F_O_ext$surv, 0)

        tis_diff <- diff(c(0, F_O_ext$time))
        F_O_ext$area <- cumsum(tis_diff*(1 - F_O_ext$surv))/365.241
        F_O_ext <- F_O_ext[,c('time', 'area')]
      }

      F_O <- data.frame(time=F_O_time, area=YL_O_vec)

      ###
      # YL_P continue:
      it_auc_P <- rep(NA, nrow(data))
      it_auc_P_mat <- matrix(0, nrow=nrow(data), ncol=ltis)

      for(it in 1:nrow(data)){
        temp <- exp_prep(rform$R[it,,drop=FALSE],max(rform$Y),rform$ratetable,rform$status[it],times=tis,fast=FALSE, cmp=FALSE, ys=starting_age[it], netweiDM = FALSE)
        if(if_start_stop){
          it_wh <- which(data[it, start_col] == tis)
          hazs <- temp$yidli[it_wh:ltis]
          hazs[1] <- 0
          cumhazs <- cumsum(hazs)
          F_P <- 1 - exp(-cumhazs)
          it_auc_P[it] <- sum(c(tis[it_wh], diff(tis[it_wh:ltis]))*c(0, F_P[1:(length(F_P)-1)]))/365.241
          it_auc_P_mat[it,it_wh:ltis] <- cumsum(c(0, diff(tis[it_wh:ltis]))*c(0, F_P[1:(length(F_P)-1)]))/365.241
        } else{
          # it_wh <- which(data$age[it] == tis)
          hazs <- temp$yidli[1:ltis]
          hazs[1] <- 0
          cumhazs <- cumsum(hazs)
          F_P <- 1 - exp(-cumhazs)
          it_auc_P[it] <- sum(c(0, diff(tis))*c(0, F_P[1:(length(F_P)-1)]))/365.241
          it_auc_P_mat[it,] <- cumsum(c(0, diff(tis))*c(0, F_P[1:(length(F_P)-1)]))/365.241
        }

      }
      YL_P <- mean(it_auc_P)
      F_P <- data.frame(time=tis, area=colMeans(it_auc_P_mat))

      yd_curve <- data.frame(time=tis, est=F_O_ext$area - F_P$area)

      # Bootstrap:
      if(bootstrap){
        data_b <- data_orig
        data_b$id <- 1:nrow(data_b)
        yl_boot <- ylboot(theta=ylboot.iter, data=data_b, id="id",
                          B=B, verbose=0, #all_times = all_times,
                          ratetable=ratetable#, add.times=add.times
                          , starting.time, estimator, precision,
                          add.times = add.times,
                          formula = formula,
                          rmap = rmap, measure=measure
        )
        if(ncol(yl_boot[[2]])>nrow(F_O)){
          varsincol <- colVars(yl_boot[[2]], na.rm=TRUE)^(1/2)
          varsincol_df <- data.frame(time=yl_boot[[4]], area.se=varsincol)
          varsincol_df <- varsincol_df[varsincol_df$time %in% F_O$time,]
          F_O$area.se <- varsincol_df$area.se
        } else{
          F_O$area.se <- colVars(yl_boot[[2]], na.rm=TRUE)^(1/2)
        }
        F_P$area.se <- colVars(yl_boot[[3]], na.rm=TRUE)^(1/2)
        yl_boot <- as.data.frame(t(yl_boot[[1]]))
        yd_curve$est.se <- (colVars(yl_boot, na.rm=TRUE))^(1/2)
      }

      # Add CI:
      if((!is.boot) & (!first.boot)){
        if(!is.null(yd_curve$est.se)){
          yd_curve$lower <- yd_curve$est - yd_curve$est.se*stats::qnorm(0.5+conf.int/2)
          yd_curve$upper <- yd_curve$est + yd_curve$est.se*stats::qnorm(0.5+conf.int/2)
        }
      }
      # Values to be reported:
      if((!is.boot) & (!first.boot)){
        if(if_start_stop){
          # Report only at last time point - the values until this time are not suitable to report:
          out <- list(years=utils::tail(yd_curve,1), F_O=utils::tail(F_O,1), F_P=utils::tail(F_P,1), measure=measure)
        } else{
          # Report full measures:
          out <- list(years=yd_curve, F_O=F_O, F_P=F_P, measure=measure)
        }
      } else{
        out <- list(years=yd_curve, F_O=F_O, F_P=F_P, measure=measure)
      }
      return(out)

    } else{ # measure == 'yl2013'
      temp <- exp_prep(rform$R[,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status,times=tis, fast=TRUE, cmp=FALSE, ys=starting_age)
      temp$yi[temp$yi==0] <- Inf

      # Calculate measures:
      haz.pop <- temp$yidli/temp$yi
      mod_tis <- summary(mod, times = tis)
      F_E <- cumsum(mod_tis$surv*(mod_tis$n.event/mod_tis$n.risk - haz.pop))
      ltis <- length(tis)

      # To be checked, doesn't work ok
      # # Var as in Pavlic2018:
      # F_E_st <- sapply(1:ltis, function(s){
      #   (sum(mod_tis$surv[s:ltis]*(mod_tis$n.event[s:ltis]/mod_tis$n.risk[s:ltis] - haz.pop[s:ltis]))/mod_tis$surv[s]) # *c(0, diff(tis[s:ltis]))  /365.241
      # })
      # # Klemnova:
      # F_Ese <- (cumsum((mod_tis$surv)^2*(1 - F_E_st)^2*((mod_tis$n.event)/(mod_tis$n.risk^2))*c(0, diff(tis)))/365.241)^(1/2)
      # surv_int <- rev(cumsum(rev(c(0, diff(tis))*c(1, mod_tis$surv[1:(length(mod_tis$surv)-1)])))/365.241)
      #
      # # Moja:
      # F_E_int <- rev(cumsum(rev(c(0, diff(tis))*c(0, F_E[1:(length(F_E)-1)])))/365.241)
      # F_Ese <- (cumsum((surv_int)^2*(1 - F_E_st)^2*((mod_tis$n.event)/(mod_tis$n.risk^2))*c(0, diff(tis)))/365.241)^(1/2)
      #
      # # Observed:
      # F_Ese <- (cumsum(surv_int^2*((mod_tis$n.event)/(mod_tis$n.risk^2))*c(0, diff(tis)))/365.241)^(1/2)
      #
      # # Predlog glede na Andersen 2013:
      # F_Ese <- (cumsum((surv_int^2*(mod_tis$n.event - temp$yidli) + F_E_int^2*temp$yidli)/(mod_tis$n.risk^2)*c(0, diff(tis)))/365.241)^(1/2)

      # Calculate measures:
      YL <- cumsum(F_E*c(0, diff(tis)))/365.241
      F_E_area <- cumsum(c(0, diff(tis))*c(0, F_E[1:(length(F_E)-1)]))/365.241
      F_E_df <- data.frame(time=tis, prob=F_E, area=F_E_area) # , prob.se=F_Ese
      yd_curve <- data.frame(time=tis, est=YL)

      # Bootstrap:
      if(bootstrap){
        data_b <- data_orig
        data_b$id <- 1:nrow(data_b)
        yl_boot <- ylboot(theta=ylboot.iter, data=data_b, id="id",
                          B=B, verbose=0, #all_times = all_times,
                          ratetable=ratetable#, add.times=add.times
                          , starting.time, estimator, precision,
                          add.times = add.times,
                          formula = formula,
                          rmap = rmap, measure=measure
        )

        # Calculate area.se:
        area.se <- yl_boot[[2]]
        for(itar in 1:nrow(yl_boot[[2]])){
          prob_tmp <- as.vector(as.matrix(yl_boot[[2]][itar,]))
          area_tmp <- cumsum(c(0, diff(tis))*c(0, prob_tmp[1:(length(prob_tmp)-1)]))/365.241
          area.se[itar,] <- area_tmp
        }
        area.se <- as.vector(colVars(area.se, na.rm=TRUE))

        F_E_df$prob.se <- (colVars(yl_boot[[2]], na.rm=TRUE))^(1/2)
        F_E_df$area.se <- area.se
        yl_boot <- as.data.frame(t(yl_boot[[1]]))
        yd_curve$est.se <- (colVars(yl_boot, na.rm=TRUE))^(1/2)
      }

      if((!is.boot) & (!first.boot)){
        if(!is.null(yd_curve$est.se)){
          yd_curve$lower <- yd_curve$est - yd_curve$est.se*stats::qnorm(0.5+conf.int/2)
          yd_curve$upper <- yd_curve$est + yd_curve$est.se*stats::qnorm(0.5+conf.int/2)
        }
      }

      out <- list(years=yd_curve, F_E=F_E_df, measure=measure)
      return(out)
    }
  } else{ # measure == 'yd'

    ################################################### #
    # CIF on population:

    data_yi <- data
    rform <- rformulate(formula, data, ratetable, na.action=na.action, rmap = rmap)
    data <- rform$data
    if(if_start_stop){
      if(!(start_col %in% colnames(data))){
        data[,start_col] <- data_orig[, start_col]
      }
    }

    # Check covariates:
    p <- rform$m
    if (p > 0) stop("There shouldn't be any covariates in the formula. This function gives non-parametric estimates of the hazards.")
    else data$Xs <- rep(1, nrow(data)) #if no covariates, just put 1

    out_n <- table(data$Xs) #table of strata
    out$time <- out$haz.excess <- out$haz.pop <- out$std.err <- out$strata <-  NULL

    kt <- 1 # the only stratum
    inx <- which(data$Xs == names(out_n)[kt]) #individuals within this stratum
    if(!if_start_stop) tis <- sort(unique(c(rform$Y[inx], seq(0, max(rform$Y[inx]), precision)))) #unique times
    else tis <- sort(unique(c(rform$Y[inx], data[, start_col], seq(min(data[, start_col]), max(rform$Y[inx], data[, start_col]), precision)))) #unique times

    if(!missing(add.times)){
      tis <- sort(unique(c(tis, add.times)))
    }

    # Fix demographic covariates:
    if(if_start_stop){
      rform$R[,"year"] <- rform$R[,"year"] - rform$R[,"age"]
      rform$R[,"age"] <- 0
    }

    ### #
    # Greenwood Variance of area (not F):
    # First prepare objects:
    mod_gw <- summary(mod, times = tis)
    gw_df <- data.frame(time=mod_gw$time, surv=mod_gw$surv, n.risk=mod_gw$n.risk, n.event=mod_gw$n.event)
    # Then calculate:
    times_all2 <- c(0, diff(gw_df$time))/365.241
    surv_all <- c(1, gw_df$surv[1:(length(gw_df$surv)-1)])
    auc_all <- cumsum(times_all2*surv_all)
    area_var <- sapply(1:length(auc_all), function(x) {
      numer <- gw_df$n.risk[1:x]*(gw_df$n.risk[1:x] - gw_df$n.event[1:x])
      numer[numer==0] <- Inf
      sum(((auc_all[x] - auc_all[1:x])^2*gw_df$n.event[1:x])/numer)
    })
    if(is.nan(area_var[length(area_var)])){
      area_var[length(area_var)] <- area_var[length(area_var)-1]
    }
    ### #

    if(estimator=='F_P' | estimator=="all"){
      # Prepare at-risk matrix:
      # browser()
      # mat <- lapply(1:nrow(data), function(x) ifelse((data$start[x] < tis) & (tis <= data$Y[x]), 1, NA))
      # mat2 <- matrix(unlist(mat), nrow = nrow(data_yi), byrow = TRUE)
      # # The sum of the individual at-risk processes:
      # yi_left <- colSums(mat2)
      # yi_left[yi_left == 0] <- Inf
      #
      # mat3 <- lapply(1:nrow(data), function(x) data$age[x] + c(0, diff(tis)))

      if(any(rform$Y[inx]<=starting_age)) browser()

      temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=TRUE, cmp=FALSE, ys=starting_age)
      # Fix at-risk process, if needed:
      temp$yi[temp$yi==0] <- Inf

      out$time <- c(out$time, tis)						#add times

      # Calculate hazards:
      haz.pop <- temp$yidli/temp$yi
      out$haz.pop <- c(out$haz.pop,haz.pop)
      out$cum.haz.pop <- cumsum(out$haz.pop)
      out$F_P <- 1-exp(-out$cum.haz.pop)
      out$auc_pop <- sum(c(tis[1], diff(tis))*c(0, out$F_P[1:(length(out$F_P)-1)]))/365.241

    }

    data_spi2 <- data
    if(estimator=='F_P_Spi2' | estimator=="all"){

      if(any(data_spi2$start>=data_spi2$Y)) browser()
      # Take into account censoring:
      exp.surv2 <- nessie_spi(Surv(start, Y, stat)~1, data=data_spi2, ratetable=ratetable,
                              tis=tis, starting.time=starting.time, include.censoring = TRUE,
                              arg.example)
      out$haz.pop.spi2 <- exp.surv2$yidli/exp.surv2$yi
      out$cum.haz.pop.spi2 <- cumsum(out$haz.pop.spi2)
      out$F_P_Spi2 <- 1-exp(-out$cum.haz.pop.spi2)
      out$auc_pop_Spi2 <- sum(c(tis[1], diff(tis))*c(0, out$F_P_Spi2[1:(length(out$F_P_Spi2)-1)]))/365.241
    }
    if(estimator=='F_P_Spi' | estimator=="all"){
      if(TRUE){ # (!missing(admin.cens))    - tega nimamo vec
        data_spi2$stat <- 1
        # data_spi2$stat[id_admin_cens] <- 0  # - tole ni bilo zakomentirano, ko smo imeli admin.cens
        exp.surv <- nessie_spi(Surv(start, Y, stat)~1, data=data_spi2, ratetable=ratetable,
                               tis=tis, starting.time=starting.time, include.censoring = TRUE,
                               arg.example)
      } else{
        # Don't take into account censoring:
        exp.surv <- nessie_spi(Surv(start, Y, stat)~1, data=data_spi2, ratetable=ratetable,
                               tis=tis, starting.time=starting.time, include.censoring = FALSE,
                               arg.example)
      }

      out$haz.pop.spi <- exp.surv$yidli/exp.surv$yi
      out$cum.haz.pop.spi <- cumsum(out$haz.pop.spi)
      out$F_P_Spi <- 1-exp(-out$cum.haz.pop.spi)
      out$auc_pop_Spi <- sum(c(tis[1], diff(tis))*c(0, out$F_P_Spi[1:(length(out$F_P_Spi)-1)]))/365.241
    }
    if(estimator=='F_P_final'){
      # Shift all to the end:
      if(if_start_stop) data_yi[,stop_col] <- max(data_yi[,stop_col])

      rform2 <- rform
      rform <- rformulate(formula, data_yi, ratetable, na.action=na.action, rmap = rmap)

      # Shift all to the end:
      if(!if_start_stop){
        rform$Y <- rep(max(rform$Y), length(rform$Y))
        rform$data[,"Y"] <- rform$Y
      }
      data <- rform$data
      if(if_start_stop){
        if(!(start_col %in% colnames(data))){
          data[,start_col] <- data_orig[, start_col]
        }
      }

      # Check covariates:
      p <- rform$m
      if (p > 0) stop("There shouldn't be any covariates in the formula. This function gives non-parametric estimates of the hazards.")
      else data$Xs <- rep(1, nrow(data)) #if no covariates, just put 1

      out$haz.pop2 <- NULL

      kt <- 1 # the only stratum
      inx <- which(data$Xs == names(out_n)[kt]) #individuals within this stratum

      # Fix demographic covariates:
      if(if_start_stop){
        rform$R[,"year"] <- rform$R[,"year"] - rform$R[,"age"]
        rform$R[,"age"] <- 0
      }

      if(any(starting_age>=rform$Y[inx])) browser()
      temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=FALSE, cmp=FALSE, ys=starting_age, netweiDM = TRUE)
      temp$sidliD[1] <- 0
      # temp$sisD[1] <- 1
      temp$sisD[temp$sisD==0] <- Inf
      haz.pop2 <- temp$sidliD/temp$sisD

      out$haz.pop2 <- c(out$haz.pop2, haz.pop2)
      out$cum.haz.pop2 <- cumsum(out$haz.pop2)
      out$F_P2 <- 1-exp(-out$cum.haz.pop2)
      out$auc_pop2 <- sum(c(tis[1], diff(tis))*c(0, out$F_P2[1:(length(out$F_P2)-1)]))/365.241

      out$sidli <- temp$sidli
      out$sis <- temp$sis

      # DODATEK:
      haz.pop.ves.cas <- temp$sidli
      haz.pop.ves.cas[1] <- 0
      haz.pop.ves.cas <- haz.pop.ves.cas/temp$sis
      out$cum.haz.pop.ves.cas <- cumsum(haz.pop.ves.cas)
      out$F_P_ves_cas <- 1 - exp(-out$cum.haz.pop.ves.cas)
      out$auc_pop_ves_cas <- sum(c(tis[1], diff(tis))*c(0, out$F_P_ves_cas[1:(length(out$F_P_ves_cas)-1)]))/365.241
    }
    if(estimator=='F_P2' | estimator=="all"){
      # Shift all to the end:
      if(if_start_stop) data_yi[,stop_col] <- max(data_yi[,stop_col])

      rform2 <- rform

      rform <- rformulate(formula, data_yi, ratetable, na.action=na.action, rmap = rmap)

      # Shift all to the end:
      if(!if_start_stop){
        rform$Y <- rep(max(rform$Y), length(rform$Y))
        rform$data[,"Y"] <- rform$Y
      }
      data <- rform$data
      if(if_start_stop){
        if(!(start_col %in% colnames(data))){
          data[,start_col] <- data_orig[, start_col]
        }
      }

      # Check covariates:
      p <- rform$m
      if (p > 0) stop("There shouldn't be any covariates in the formula. This function gives non-parametric estimates of the hazards.")
      else data$Xs <- rep(1, nrow(data)) #if no covariates, just put 1

      out$haz.pop2 <- NULL

      kt <- 1 # the only stratum
      inx <- which(data$Xs == names(out_n)[kt]) #individuals within this stratum

      # Fix demographic covariates:
      if(if_start_stop){
        rform$R[,"year"] <- rform$R[,"year"] - rform$R[,"age"]
        rform$R[,"age"] <- 0
      }

      if(any(starting_age>=rform$Y[inx])) browser()

      # temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=TRUE, cmp=FALSE, ys=0)
      temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=TRUE, cmp=FALSE, ys=starting_age)

      # Fix at-risk process, if needed:
      temp$yi[temp$yi==0] <- Inf

      # Calculate hazards:
      haz.pop2 <- temp$yidli/temp$yi

      out$haz.pop2 <- c(out$haz.pop2, haz.pop2)
      out$cum.haz.pop2 <- cumsum(out$haz.pop2)
      out$F_P2 <- 1-exp(-out$cum.haz.pop2)
      # out$auc_pop2 <- sum(c(tis[1], diff(tis))*out$F_P2)/365.241
      out$auc_pop2 <- sum(c(tis[1], diff(tis))*c(0, out$F_P2[1:(length(out$F_P2)-1)]))/365.241
    }

    ###
    # Bootstrap:
    if(bootstrap){
      # browser()
      data_b <- data_orig
      data_b$id <- 1:nrow(data_b)
      yl_boot <- ylboot(theta=ylboot.iter, data=data_b, id="id",
                        B=B, verbose=0, #all_times = all_times,
                        ratetable=ratetable#, add.times=add.times
                        , starting.time, estimator, precision,
                        add.times = add.times,
                        formula = formula,
                        rmap = rmap, measure=measure
      )
      L_OP <- yl_boot[[3]]
      F_boot <- yl_boot[[2]]
      yl_boot <- as.data.frame(t(yl_boot[[1]]))
    }
    ###
    estimator.orig <- estimator
    if(estimator=='F_P_final') estimator = 'F_P2'

    out$strata <- c(out$strata, length(tis))				#number of times in this strata
    names(out$strata) <-  names(out_n)
    out$strata <-  NULL
    out$auc <- c(auc_data=out$auc_data, auc_pop=out$auc_pop, auc_pop2=out$auc_pop2, auc_pop_Spi=out$auc_pop_Spi, auc_pop_Spi2=out$auc_pop_Spi2)

    if(estimator=='all'){
      F_P_final <- data.frame(time=out$time,F_P=out$F_P, F_P2=out$F_P2, F_P_Spi=out$F_P_Spi, F_P_Spi2=out$F_P_Spi2)
    } else if(estimator=='F_P'){
      F_P_final <- data.frame(time=tis,prob=out$F_P)
    } else if(estimator=='F_P2'){
      F_P_final <- data.frame(time=tis,prob=out$F_P2)
    } else if(estimator=='F_P_Spi'){
      F_P_final <- data.frame(time=tis,prob=out$F_P_Spi)
    } else if(estimator=='F_P_Spi2'){
      F_P_final <- data.frame(time=tis,prob=out$F_P_Spi2)
    }

    # YD through time:
    F_data_yd <- data.frame(time=mod$time, F_data=out$F_data)
    pop.times <- F_P_final$time[!(F_P_final$time %in% mod$time)]
    if(length(pop.times) > 0){
      F_data_yd_tmp <- data.frame(time=pop.times, F_data=NA)
      F_data_yd <- rbind(F_data_yd, F_data_yd_tmp)
      F_data_yd <- F_data_yd[order(F_data_yd$time),]
      F_data_yd$F_data <- mstateNAfix(F_data_yd$F_data, 0)
    }
    F_data_yd$var <- area_var

    yd_data <- cumsum(c(F_data_yd$time[1], diff(F_data_yd$time))*c(0, F_data_yd$F_data[1:(nrow(F_data_yd)-1)]))/365.241

    # Population part:
    F_P_yd <- F_P_final
    yd_pop <- cumsum(c(F_P_yd$time[1], diff(F_P_yd$time))*c(0, F_P_yd$prob[1:(nrow(F_P_yd)-1)]))/365.241
    yd_curve <- data.frame(time=F_data_yd$time, yd=yd_data - yd_pop,
                           obs_var=F_data_yd$var,
                           # obs_var22=obs_var_time22,
                           yd_data=yd_data,
                           yd_pop=yd_pop
    )
    ###
    # Greenwood for prob:
    greenwood_est <- (mod$surv^2*cumsum(mod$n.event/((mod$n.risk - mod$n.event)*mod$n.risk)))^(1/2)
    # If Surv(t)=0 in the end, take the last var estimate:
    if(any(rev(mod$surv)==0)){
      greenwood_wh <- which(mod$surv==0)
      greenwood_est[greenwood_wh] <- greenwood_est[greenwood_wh[1]-1]
    }

    F_data_tmp <- data.frame(time=mod$time,
                             prob=out$F_data,
                             prob.se=greenwood_est,
                             area=NA,
                             area.se=NA)
    # Add values at time zero:
    F_tmp <- F_data_tmp[1,]
    F_tmp$time <- min(starting_age)
    F_tmp$prob <- 0
    F_tmp$prob.se <- 0
    if(!(F_tmp$time %in% F_data_tmp$time)) F_data_tmp <- rbind(F_tmp, F_data_tmp)

    if(!if_start_stop){
      F_P_final_tmp <- F_P_final[1,]
      F_P_final_tmp$time <- min(starting_age)
      F_P_final_tmp$prob <- 0
      if(!(F_P_final_tmp$time %in% F_P_final$time)) F_P_final <- rbind(F_P_final_tmp, F_P_final)
    }

    yd_curve_tmp <- yd_curve[1,]
    yd_curve_tmp$time <- min(starting_age)
    yd_curve_tmp[,2:ncol(yd_curve_tmp)] <- 0
    if(!(yd_curve_tmp$time %in% yd_curve$time)) yd_curve <- rbind(yd_curve_tmp, yd_curve)

    # Bootstrap:
    if(bootstrap){
      yd_curve$boot_var <- colVars(yl_boot, na.rm=TRUE)
      if(late.values){
        last_val <- utils::tail(yd_curve$boot_var[!is.na(yd_curve$boot_var)],1)
        yd_curve$boot_var[is.na(yd_curve$boot_var)] <- last_val
      }
      yl_sd_boot <- stats::sd(yl_boot[, ncol(yl_boot)], na.rm=TRUE)
    }

    # Add areas:
    F_data_tmp$area <- yd_curve$yd_data[yd_curve$time %in% F_data_tmp$time]
    F_P_final$area <- yd_curve$yd_pop#[yd_curve$time %in% F_P_final$time]
    F_data_tmp$area.se <- yd_curve$obs_var[yd_curve$time %in% F_data_tmp$time]^(1/2)

    # If, add boot variance:
    if(bootstrap & (!is.boot)){
      F_data_tmp$prob.se <- (F_boot$F_data[F_boot$time %in% F_data_tmp$time])^(1/2)
      F_P_final$prob.se <- (F_boot$F_P#[F_boot$time %in% F_P_final$time]
      )^(1/2)
      F_data_tmp$area.se <- L_OP$L_O[L_OP$time %in% F_data_tmp$time]^(1/2)
      F_P_final$area.se <- L_OP$L_P^(1/2)
    }

    # Column order:
    F_data_tmp <- F_data_tmp[, c('time', 'prob', 'area', 'prob.se', 'area.se')]

    # Choose relevant columns:
    if(bootstrap){
      yd_curve <- yd_curve[,c('time', 'yd', 'boot_var')]
    } else{
      yd_curve <- yd_curve[,c('time', 'yd', 'obs_var')]
    }
    yd_curve[,3] <- yd_curve[,3]^(1/2)
    colnames(yd_curve)[2:3] <- c('est', 'est.se')
    yd_curve$lower <- yd_curve$est - yd_curve$est.se*stats::qnorm(0.5+conf.int/2)
    yd_curve$upper <- yd_curve$est + yd_curve$est.se*stats::qnorm(0.5+conf.int/2)

    return_obj <- list(F_data=F_data_tmp,
                       F_P=F_P_final,
                       auc=out$auc,
                       yd_curve=yd_curve,
                       starting.time=starting.time,
                       estimator=estimator.orig,
                       out=out
    )

    if(bootstrap){
      return_obj[[length(return_obj)+1]] <- F_boot
      names(return_obj)[length(return_obj)] <- 'F_boot'
      return_obj[[length(return_obj)+1]] <- L_OP
      names(return_obj)[length(return_obj)] <- 'L_OP'
      return_obj <- append(return_obj, yl_sd_boot)
      names(return_obj)[length(return_obj)] <- 'yl_sd_boot'
    }

    return_short <- list(years=return_obj$yd_curve, F_O=return_obj$F_data, F_P=return_obj$F_P, measure=measure)

    if((bootstrap & (!is.boot)) #| ((!bootstrap) & (!is.boot))
    ){
      return_obj <- return_short
    }
    if((!bootstrap) & (!is.boot)){
      return_obj <- return_short
    }
    if(is.boot){
      return_obj <- return_short
    }

    if(var.estimator=='none'){
      return_obj$years <- return_obj$years[,1:2]
      find_cols <- (!grepl('.se', colnames(return_obj[[2]])))
      return_obj[[2]] <- return_obj[[2]][,find_cols]
      if(length(return_obj)==4){
        find_cols <- (!grepl('.se', colnames(return_obj[[3]])))
        return_obj[[3]] <- return_obj[[3]][,find_cols]
      }
    }

    return(return_obj)
  }
}

utils::globalVariables(c("time", "prob", "Curve", "est", "lower", "upper"))

# Bootstrap function:
ylboot <- function(theta, data, B = 5, id = "id", verbose = 0,
                   #all_times,
                   ratetable=relsurv::slopop, #add.times,
                   starting.time, estimator, precision,
                   add.times,
                   formula,
                   rmap, measure,
                   ...){
  ids <- unique(data[, id])
  n <- length(ids)
  if(!missing(add.times)){
    th <- ylboot.iter(formula, data, starting.time = starting.time, estimator = estimator, precision = precision,
                      ratetable=ratetable, first=TRUE, add.times = add.times, rmap = rmap, measure=measure, ...)
  } else{
    th <- ylboot.iter(formula, data, starting.time = starting.time, estimator = estimator, precision = precision,
                      ratetable=ratetable, first=TRUE, rmap = rmap, measure=measure, ...)
  }

  simple_par <- TRUE
  if(missing(add.times)) simple_par <- FALSE

  # Prepare objects:
  res <- data.frame(matrix(NA, nrow=B, ncol=nrow(th[[1]])))
  if(!missing(add.times)){
    add.times <- sort(unique(c(th[[1]]$time, add.times)))
  } else{
    add.times <- th[[1]]$time
  }

  Fdata <- data.frame(matrix(NA, nrow=B, ncol=length(add.times)))
  Fo <- data.frame(matrix(NA, nrow=B, ncol=nrow(th[[2]])))
  Fp <- data.frame(matrix(NA, nrow=B, ncol=length(add.times)))
  L_O <- data.frame(matrix(NA, nrow=B, ncol=length(add.times)))
  L_P <- data.frame(matrix(NA, nrow=B, ncol=length(add.times)))
  F_E <- data.frame(matrix(NA, nrow=B, ncol=length(add.times)))

  # Iteration:
  for (b in 1:B) {
    nek_obj <- ylboot.apply(formula, b, verbose, ids, data, id, add.times, starting.time, estimator, precision, ratetable, th, simple_par, rmap, measure, ...)
    res[b,1:length(nek_obj[[1]])] <- nek_obj[[1]]
    if(measure=='yl2013'){
      F_E[b,1:length(nek_obj[[2]])] <- nek_obj[[2]]
    }
    if(measure=='yl2017'){
      Fo[b,1:length(nek_obj[[2]])] <- nek_obj[[2]]
      Fp[b,1:length(nek_obj[[3]])] <- nek_obj[[3]]
    }
    if(measure=='yd'){
      subnek <- subset(nek_obj[[2]], time %in% add.times)

      sub_vec <- 1:nrow(subnek)
      Fdata[b,sub_vec] <- subnek$F_data
      Fp[b,sub_vec] <- subnek$F_P

      subnek2 <- subset(nek_obj[[3]], time %in% add.times)

      sub2_vec <- 1:nrow(subnek2)
      L_O[b,sub2_vec] <- subnek2$yd_data
      L_P[b,sub2_vec] <- subnek2$yd_pop
    }
  }
  res <- as.data.frame(t(res))
  if(measure == 'yl2013'){
    return(list(res, F_E))
  }
  if(measure == 'yl2017'){
    return(list(res, Fo, Fp, add.times))
  }
  else{
    if (verbose)
      cat("\n")

    F_obj <- data.frame(time=add.times,
                        F_data=colVars(Fdata, na.rm = TRUE),
                        F_P=colVars(Fp, na.rm = TRUE))
    L_OP <- data.frame(time=add.times,
                       L_O=colVars(L_O, na.rm = TRUE),
                       L_P=colVars(L_P, na.rm = TRUE))

    return(list(res, F_obj, L_OP))
  }
}

ylboot.apply <- function(formula, b, verbose, ids, data, id, add.times, starting.time, estimator, precision, ratetable, th, simple_par,
                         rmap, measure,
                         ...){

  if(starting.time=='left.truncated'){
    start_col <- as.character(formula[[2]])[2]
    stop_col <- as.character(formula[[2]])[3]
  } else{
    stop_col <- as.character(formula[[2]])[2]
  }

  if (verbose > 0) {
    cat("\nBootstrap replication", b, "\n")
  }
  bootdata <- NULL
  bids <- sample(ids, replace = TRUE)
  bidxs <- unlist(sapply(bids, function(x) which(x ==
                                                   data[, id])))
  bootdata <- data[bidxs, ]
  if (verbose > 0) {
    cat("applying theta ...")
  }

  if(length(unique(bootdata[,id]))==1){
    next
  }

  if(!missing(add.times) & simple_par){
    add.times.arg <- sort(unique(c(th[[1]]$time, add.times)))
  } else{
    add.times.arg <- th[[1]]$time
  }
  add.times.arg2 <- add.times.arg
  # Remove unnecessary times
  if(starting.time == 'left.truncated'){
    add.times.arg <- add.times.arg[add.times.arg<=max(bootdata[,stop_col])]
  } else{
    add.times.arg <- add.times.arg[add.times.arg<=max(bootdata[,stop_col])]# - bootdata[,start_col])]
  }

  thstar <- ylboot.iter(formula, bootdata, starting.time = starting.time, estimator = estimator, precision = precision,
                        ratetable=ratetable, add.times=add.times.arg, rmap=rmap, measure=measure, ...)
  if(measure == 'yl2013'){
    return(list(thstar[[1]]$est, thstar[[2]]$prob))
  }
  if(measure == 'yl2017'){
    FoO <- thstar[[2]]
    FpP <- thstar[[3]]
    thstar <- thstar[[1]]
    # if(nrow(th[[1]]) != nrow(thstar)) browser()

    if(nrow(FoO) < nrow(th[[2]])){
      mis.tajms <- th[[2]]$time[!(th[[2]]$time %in% FoO$time)]
      mis.tajms <- mis.tajms[mis.tajms <= max(FoO$time)]
      temp_df <- data.frame(time=mis.tajms, area=NA)
      FoO <- rbind(FoO, temp_df)
      FoO <- FoO[order(FoO$time),]
      FoO$area <- mstateNAfix(FoO$area, 0)
    }

    if(nrow(th[[1]]) < nrow(thstar)){
      thstar <- thstar[thstar$time %in% th[[1]]$time, ]
      FpP <- FpP[FpP$time %in% th[[1]]$time, ]
      foO <- foO[foO$time %in% th[[1]]$time, ]
    }

    if(length(th[[1]]$time[th[[1]]$time <= max(thstar$time)]) != length(thstar$time)) browser()
    pogoj <- any(th[[1]]$time[th[[1]]$time <= max(thstar$time)] != thstar$time)
    if(pogoj){
      missing_times <- th[[1]]$time[which(!(th[[1]]$time %in% thstar$time))]

      if(length(missing_times)>0){
        # There are times missing in thstar, add them:
        add_df <- thstar[1:length(missing_times),]
        add_df$time <- missing_times
        add_df$yd <- NA
        add_df$obs_var <- NA
        add_df$yd_data <- NA

        thstar <- rbind(thstar, add_df)
        thstar <- thstar[order(thstar$time),] # redundantno

        thstar$yd <- mstateNAfix(thstar$yd, 0)
        thstar$obs_var <- mstateNAfix(thstar$obs_var, 0)
        thstar$yd_data <- mstateNAfix(thstar$yd_data, 0)

        if(nrow(th[[1]]) < nrow(thstar)){
          thstar <- thstar[thstar$time %in% th[[1]]$time, ]
        }

        if(nrow(th[[1]]) != nrow(thstar)) browser()
      } else{
        # This means there's more times in thstar than needed. Remove unnecessary times:
        thstar <- thstar[-which(!(thstar$time %in% th[[1]]$time)),]
        FpP <- FpP[-which(!(FpP$time %in% th[[1]]$time)),]
        foO <- foO[-which(!(foO$time %in% th[[1]]$time)),]

        if(nrow(th[[1]]) != nrow(thstar)) browser()
      }
    }

    return(list(thstar$est, FoO$area, FpP$area))
  }

  L_OP <- thstar[[3]]
  Fobj <- thstar[[2]]
  thstar <- thstar[[1]]

  if(nrow(th[[1]]) < nrow(thstar)){
    thstar <- thstar[thstar$time %in% th[[1]]$time, ]
    L_OP <- L_OP[L_OP$time %in% th[[1]]$time, ]
    Fobj <- Fobj[Fobj$time %in% th[[1]]$time, ]
  }

  # Ali kaksne vrednosti manjkajo:
  if(length(th[[1]]$time[th[[1]]$time <= max(thstar$time)]) != length(thstar$time)) browser()
  pogoj <- any(th[[1]]$time[th[[1]]$time <= max(thstar$time)] != thstar$time)
  if(pogoj){

    missing_times <- th[[1]]$time[which(!(th[[1]]$time %in% thstar$time))]

    if(length(missing_times)>0){
      # There are times missing in thstar, add them:
      add_df <- thstar[1:length(missing_times),]
      add_df$time <- missing_times
      add_df$yd <- NA
      add_df$obs_var <- NA
      add_df$yd_data <- NA

      thstar <- rbind(thstar, add_df)
      thstar <- thstar[order(thstar$time),] # redundantno

      thstar$yd <- mstateNAfix(thstar$yd, 0)
      thstar$obs_var <- mstateNAfix(thstar$obs_var, 0)
      thstar$yd_data <- mstateNAfix(thstar$yd_data, 0)

      if(nrow(th[[1]]) < nrow(thstar)){
        thstar <- thstar[thstar$time %in% th[[1]]$time, ]
      }

      if(nrow(th[[1]]) != nrow(thstar)) browser()
    } else{
      # This means there's more times in thstar than needed. Remove unnecessary times:
      thstar <- thstar[-which(!(thstar$time %in% th[[1]]$time)),]
      L_OP <- L_OP[-which(!(L_OP$time %in% th[[1]]$time)),]
      Fobj <- Fobj[-which(!(Fobj$time %in% th[[1]]$time)),]

      if(nrow(th[[1]]) != nrow(thstar)) browser()
    }
  }

  # thstar$b <- b
  # Save result:
  # res[b,] <-

  list(thstar$est, Fobj, L_OP)
}

ylboot.iter <- function(formula, data, #all_times,
                        starting.time, estimator, precision,
                        ratetable=relsurv::slopop,
                        first=FALSE, add.times,
                        rmap, measure
){
  if(!missing(rmap))  rmap <- as.call(rmap)
  if(first){
    is.boot <- FALSE
    first.boot <- TRUE
  } else{
    is.boot <- TRUE
    first.boot <- FALSE
  }

  # Round, if needed:
  tolerance <- 1e-15

  if(missing(add.times)){
    object <- years(formula = formula, data = data, ratetable = ratetable,
                    precision=precision, var.estimator='greenwood', is.boot=is.boot, first.boot = first.boot, rmap = rmap, measure=measure)
    # estimator = estimator,
  } else{
    object <- years(formula = formula, data = data, ratetable = ratetable,
                    precision=precision, var.estimator='greenwood', add.times=add.times, is.boot=is.boot, first.boot = first.boot, rmap = rmap, measure=measure)
    # estimator = estimator,
  }
  if(measure=='yd'){
    if(first) return(list(object$years, object$F_O))
    else{
      # return(object$yd_curve)
      Fobj <- merge(object$F_P[,c('time','prob')], object$F_O[,c('time','prob')], by='time', all.x=TRUE)
      Fobj <- Fobj[,c(1,3,2)]
      colnames(Fobj)[2:3] <- c('F_data','F_P')

      L_OP <- merge(object$F_P[,c('time','area')], object$F_O[,c('time','area')], by='time', all.x = TRUE)
      L_OP <- L_OP[,c(1,3,2)]
      colnames(L_OP)[2:3] <- c('yd_data', 'yd_pop')

      return(list(object$years,
                  Fobj,
                  L_OP))
    }
  } else if(measure=='yl2013'){
    return(list(object$years, object$F_E))
  } else{
    return(list(object$years, object$F_O, object$F_P))
  }

}

plot_helper_years <- function(years, obj){

  df_poly <- data.frame(time=years[[obj]]$time/365.241,
                        prob=years[[obj]]$prob)
  df_st <- df_poly[1,]
  df_st$prob <- 0
  df_end <- df_poly[nrow(df_poly),]
  df_end$prob <- 0
  df_poly <- rbind(df_st, df_poly, df_end)

  df_poly
}

gg_color_hue <- function(n) {
  hues = seq(15, 375, length = n + 1)
  grDevices::hcl(h = hues, l = 65, c = 100)[1:n]
}

#' Plot the absolute risk (observed and population curve)
#'
#' Plots the estimated observed and population curve for the
#' life years difference (Manevski, Ruzic Gorenjec, Andersen, Pohar Perme, 2022).
#'
#' A ggplot2 implementation for plotting the observed and population curves. The type of curves is
#' dependent upon the measure calculated using \code{years} function (argument \code{measure}).

#' @param years the object obtained using function \code{years}.
#' @param xlab a title for the x axis.
#' @param ylab a title for the y axis.
#' @param xbreak the breaks on the x axis (this is supplied to \code{scale_x_continuous}).
#' @param ybreak the breaks on the y axis (this is supplied to \code{scale_y_continuous}).
#' @param xlimits define the limits on the x axis (this is supplied to \code{scale_x_continuous}).
#' @param ylimits define the limits on the y axis (this is supplied to \code{scale_y_continuous}).
#' @param show.legend if TRUE, the legend is shown on the graph.
#' @return A ggplot object
#' @seealso \code{\link{years}}, \code{\link{plot_years}}
#'
plot_f <- function(years, xlab='Time interval', ylab='Absolute risk', xbreak, ybreak, xlimits, ylimits, show.legend=TRUE){
  # years: object given from the years() function
  # xlab: define xlab
  # ylab: define ylab
  # xbreak: The breaks on x axis
  # ybreak: The breaks on y axis
  # xlimits: Define the limits on the x axis
  # ylimits: Define the limits on the y axis
  # show.legend: TRUE by default (shows the legend)

  # Checks:
  if(years$measure != 'yd'){
    stop("The plot_f function is available only for the YD measure (argument measure='yd' in the years function).")
  }

  out <- rbind(
    cbind(years$F_O[,c('time', 'prob')], Curve='Observed'),
    cbind(years$F_P[,c('time', 'prob')], Curve='Population')
  )

  if(missing(xlimits)){
    xlimits <- c(min(out$time), max(out$time))/365.241
  }
  if(missing(ylimits)){
    ylimits <- c(0,max(out$prob))*1.1
  }

  colorji <- gg_color_hue(3)
  colorji <- colorji[c(1,3)]

  g <- ggplot2::ggplot(out)+
    ggplot2::geom_step(ggplot2::aes(time/365.241, prob, color=Curve)#, size=1.001
    )+
    ggplot2::scale_color_manual(values=colorji)+
    ggplot2::xlab(xlab)+
    ggplot2::ylab(ylab)

  poly_data <- plot_helper_years(years, 'F_O')
  poly_P <- plot_helper_years(years, 'F_P')

  g <- g+
    pammtools::geom_stepribbon(ggplot2::aes(x=time/365.241, ymin=0, ymax=prob, fill=Curve), alpha=0.3, linetype='dashed')+
    ggplot2::scale_fill_manual(values = colorji)

  if(!missing(xbreak)){
    g <- g +
      ggplot2::scale_x_continuous(expand = c(0, 0), limits=xlimits, breaks = xbreak)
  } else{
    g <- g +
      ggplot2::scale_x_continuous(expand = c(0, 0), limits=xlimits)
  }

  if(!missing(ybreak)){
    g <- g +
      ggplot2::scale_y_continuous(expand = c(0, 0), limits=ylimits, breaks = ybreak)
  } else{
    g <- g +
      ggplot2::scale_y_continuous(expand = c(0, 0), limits=ylimits)
  }

  g <- g +
    ggplot2::theme_bw()+
    ggplot2::theme(legend.position = 'bottom',
                   legend.title = ggplot2::element_blank())+
    ggplot2::theme(text = ggplot2::element_text(size=14))+
    ggplot2::theme(
      panel.grid.major.x = ggplot2::element_line(linetype='dashed', colour = 'grey85'),
      panel.grid.minor.x = ggplot2::element_line(linetype='dashed', colour = 'grey85'),
      panel.grid.major.y = ggplot2::element_line(linetype='dashed', colour = 'grey85'),
      panel.grid.minor.y = ggplot2::element_line(linetype='dashed', colour = 'grey85'))

  if(!show.legend){
    g <- g +
      ggplot2::theme(legend.position = 'none')
  }

  g
}

#' Plot the years measure
#'
#' Plot the years measure obtained from the \code{years} function.
#'
#' A ggplot2 implementation for plotting the years measure. The type of curve is
#' dependent upon the measure calculated using the \code{years} function (argument \code{measure}).

#' @param years the object obtained using function \code{years}.
#' @param xlab a title for the x axis.
#' @param ylab a title for the y axis.
#' @param xbreak the breaks on the x axis (this is supplied to \code{scale_x_continuous}).
#' @param ybreak the breaks on the y axis (this is supplied to \code{scale_y_continuous}).
#' @param xlimits define the limits on the x axis (this is supplied to \code{scale_x_continuous}).
#' @param ylimits define the limits on the y axis (this is supplied to \code{scale_y_continuous}).
#' @param conf.int if TRUE, the confidence interval is plotted.
#' @param ymirror mirror the y values (w.r.t. the x axis).
#' @param yminus use function y -> -y when plotting.
#' @return A ggplot object
#' @seealso \code{\link{years}}, \code{\link{plot_f}}
#'
plot_years <- function(years, xlab='Time interval', ylab='Years', xbreak, ybreak, xlimits, ylimits, conf.int=FALSE, ymirror=FALSE, yminus=FALSE){

  out <- years$years

  if(conf.int){
    if(is.null(out$lower)){
      stop('Confidence intervals not present in the years object. Please set conf.int=FALSE or use the var.estimator argument in the years function.')
    }
  }

  if(years$measure=='yl2017' & nrow(out)==1){
    stop('The years measure is reported at the end of follow-up thus it is not plotted.')
  }

  if(yminus){
    out$est <- -out$est
    if(!is.null(out$lower)){
      tmp_lower <- out$lower
      out$lower <- -out$upper
      out$upper <- -tmp_lower
    }
  }

  if(missing(xlimits)){
    xlimits <- c(min(out$time[1]), max(out$time))/365.241
  }
  if(missing(ylimits)){
    tmp_vec <- out$est
    if(!is.null(out$lower)) tmp_vec <- c(out$est, out$lower, out$upper)
    ymax <- max(tmp_vec)
    ymin <- min(tmp_vec)
    ylimits <- c(ymin,ymax)*1.1
  }

  g <- ggplot2::ggplot(out)+
    ggplot2::geom_step(ggplot2::aes(time/365.241, est)#, size=1.001
                       )

  if(conf.int){
    g <- g+
      ggplot2::geom_step(ggplot2::aes(time/365.241, lower), linetype='dashed')+
      ggplot2::geom_step(ggplot2::aes(time/365.241, upper), linetype='dashed')
  }

  g <- g+
    ggplot2::xlab(xlab)+
    ggplot2::ylab(ylab)

  if(!missing(xbreak)){
    g <- g+
      ggplot2::scale_x_continuous(expand = c(0, 0), limits=xlimits, breaks = xbreak)
  } else{
    g <- g+
      ggplot2::scale_x_continuous(expand = c(0, 0), limits=xlimits)
  }

  # Helper:
  trans <- function(x) -x
  inv <- function(x) -x
  reverse_fun <- scales::trans_new(name = "reverse_new",
                                   transform = trans,
                                   inverse = inv
  )

  if(!missing(ybreak)){
    g <- g +
      ggplot2::scale_y_continuous(expand = c(0, 0), limits = ylimits, breaks = ybreak)
  } else{
    g <- g +
      ggplot2::scale_y_continuous(expand = c(0, 0), limits = ylimits)
  }
  if(ymirror){
    g <- g +
      ggplot2::coord_trans(y=reverse_fun)
  }

  g <- g +
    ggplot2::theme_bw()+
    ggplot2::theme(text = ggplot2::element_text(size=14))+
    ggplot2::expand_limits(y = 0)+
    ggplot2::theme(
      panel.grid.major.x = ggplot2::element_line(linetype='dashed', colour = 'grey85'),
      panel.grid.minor.x = ggplot2::element_line(linetype='dashed', colour = 'grey85'),
      panel.grid.major.y = ggplot2::element_line(linetype='dashed', colour = 'grey85'),
      panel.grid.minor.y = ggplot2::element_line(linetype='dashed', colour = 'grey85'))

  g

}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

Yt <- function(data, times) {
    .Call(`_relsurv_Yt`, data, times)
}

dNt <- function(data, times) {
    .Call(`_relsurv_dNt`, data, times)
}

prepareX <- function(Yt, xt) {
    .Call(`_relsurv_prepareX`, Yt, xt)
}

fitOLS <- function(mX, dNt, Yt) {
    .Call(`_relsurv_fitOLS`, mX, dNt, Yt)
}

fitOLS2 <- function(mX, dNt, Yt) {
    .Call(`_relsurv_fitOLS2`, mX, dNt, Yt)
}

fitOLSconst <- function(mX, mZ, dNt, Yt) {
    .Call(`_relsurv_fitOLSconst`, mX, mZ, dNt, Yt)
}

rcpp_unlist <- function(listObject) {
    .Call(`_relsurv_rcpp_unlist`, listObject)
}

build_array3 <- function(x, dimensions) {
    .Call(`_relsurv_build_array3`, x, dimensions)
}

#' Test Net Survival Curve Differences
#'
#' Tests if there is a difference between two or more net survival curves using
#' a log-rank type test.
#'
#' NOTE: The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with by the \code{rmap} argument. For example, if
#' age is in years in the data set but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.
#'
#' @aliases rs.diff print.rsdiff
#' @param formula A formula expression as for other survival models, of the
#' form \code{Surv(time, status) ~ predictors}. Each combination of predictor
#' values defines a subgroup. A \code{strata} term may be used to produce a
#' stratified test.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param na.action a missing-data filter function, applied to the model.frame,
#' after any subset argument has been used.  Default is
#' \code{options()$na.action}.
#' @param precision Precision for numerical integration. Default is 1, which
#' means that daily intervals are taken, the value may be decreased to get a
#' higher precision or increased to achieve a faster calculation. The
#' calculation intervals always include at least all times of event and
#' censoring as border points.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @return a \code{rsdiff} object; can be printed with \code{print}.
#' @seealso \code{rs.surv}, \code{survdiff}
#' @references Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric
#' Relative Survival Analysis with the R Package relsurv". Journal of
#' Statistical Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08" Theory:
#' Graffeo, N., Castell, F., Belot, A. and Giorgi, R. (2016) "A log-rank-type
#' test to compare net survival distributions. Biometrics. doi:
#' 10.1111/biom.12477" Theory: Pavlic, K., Pohar Perme, M. (2017) "On
#' comparison of net survival curves. BMC Med Res Meth. doi:
#' 10.1186/s12874-017-0351-3"
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #calculate the relative survival curve
#' #note that the variable year is given in days since 01.01.1960 and that
#' #age must be multiplied by 365.241 in order to be expressed in days.
#' rs.diff(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#' 		ratetable=slopop,data=rdata)
#'
rs.diff <- function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,
                     na.action,precision=1,rmap)

  #formula: for example Surv(time,cens)~sex
  #data: the observed data set
  #ratetable: the population mortality tables
{

  call <- match.call()
  if (!missing(rmap)) {
        rmap <- substitute(rmap)
      }
    rform <- rformulate(formula, data, ratetable, na.action,rmap)  		#get the data ready
  data <- rform$data								#the data set
  p <- rform$m								#number of covariates
  if (p > 0) 									#if covariates
    data$Xs <- strata(rform$X[, ,drop=FALSE ])	#make groups according to covariates
  else data$Xs <- rep(1, nrow(data))        					#if no covariates, just put 1
  # Xs is a  vector of factors determining the groups we wish to compare
  strats <- rform$strata.keep # added for strata
  str_num <- length(levels(strats)) # number of strata

  out <- NULL
  out$n <- table(data$Xs)	#table of groups
  out$time <- out$n.risk <- out$n.event <- out$n.censor <- out$surv <- out$std.err <- out$groups <- NULL

  #TIMES ARE EQUAL FOR ALL GROUPS
  if(!precision)tis <- sort(unique(rform$Y))					#unique times
  else{
  extra <- as.numeric(seq(1,max(rform$Y),by=precision))
  tis <- sort(union(extra,rform$Y))					#1-day long intervals used - to take into the account the continuity of the pop. part
  }
  # start working
  kgroups <- length(out$n)  						#number of groups

  if (kgroups == 1) stop("There is only one group in your data. You should choose another variable.")

  w.risk <- w.event <- dnisisq <- array(NA,dim=c(length(tis),length(out$n),str_num))			#MATRIX - COLUMNS ARE GROUPS, ROWS ARE TIMES,levels are strata
  #numOfSmallGrps <- 0
  numOfFewEvents <- 0
  for (s in 1:str_num){ # added for strata
    for (kt in 1:kgroups) {						#for each group
      inx <- which(data$Xs == names(out$n)[kt] & strats == levels(strats)[s])				#individuals within this group
      #if (length(inx)<10)numOfSmallGrps <- numOfSmallGrps + 1
      temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=TRUE)	#calculate the values for each interval of time
      out$time <- c(out$time, tis)						#add times
      out$n.risk <- c(out$n.risk, temp$yi)					#add number at risk for each time
      out$n.event <- c(out$n.event, temp$dni)					#add number of events for each time
      if (sum(temp$dni) < 10) numOfFewEvents <- numOfFewEvents + 1
      out$n.censor <- c(out$n.censor,  c(-diff(temp$yi),temp$yi[length(temp$yi)]) - temp$dni) 	#add number of censored for each time

      w.risk[,kt,s] <- temp$yisi						#Y_h^w
      w.event[,kt,s] <- temp$dnisi - temp$yidlisi				#dN_eh^w
      dnisisq[,kt,s] <- temp$dnisisq						#dN/S_p^2
      out$groups <- c(out$groups, length(tis))				#number of times in this group
    }
  }
  #if (numOfSmallGrps > 0) warning(numOfSmallGrps, " out of ", kgroups*str_num, " groups is/are smaller than 10.")
  if (numOfFewEvents > 0) warning("In ", numOfFewEvents, " out of ", kgroups*str_num, " groups there are less than 10 events.")

  w.risk.total <- apply(w.risk,c(1,3),sum)					#sum over all individuals at each time point ## Y_{.,s}^w
  w.event.total <- apply(w.event,c(1,3),sum)       #sum over all individuals at each time point ## dN_{E,.,s}^w

  zs <- rep(0,kgroups) # added for strata
  for (s in 1:str_num){
    # znotraj danega stratuma
    inx_str <- which(w.risk.total[,s] > 0)
    zhst <- w.event[inx_str,,s,drop=FALSE] - w.risk[inx_str,,s,drop=FALSE]/w.risk.total[inx_str,s]*w.event.total[inx_str,s]   #value under the integral of zh
    # integriramo po casu - sestejemo po casih dogodkov

    zhs <- apply(zhst,2,sum)    # the vector of test statistics
    zs <- zs + zhs
  }
  # cat("vektor testnih statistik je = \n")
  # print(zs)

  #covariance matrix:
  covmats <- matrix(0,nrow=kgroups,ncol=kgroups)
  d <- diag(kgroups)  					#identity matrix of groups size (for the kronecker deltas)
  for (s in 1:str_num){
    underint <- 0
    inx_str <- which(w.risk.total[,s] > 0)
    for(kt in 1:kgroups){						#matrix calculation through the groups
      ys <- matrix(d[kt,],nrow=length(inx_str),ncol=kgroups,byrow=T) - w.risk[inx_str,,s]/w.risk.total[inx_str,s]	#preparing the matrix for the first two terms
      #yslist <- apply(apply(ys,1,list),unlist)			#a list, each row of ys (each time point) represents one item
      yslist <- as.list(data.frame(t(ys)))				#a list, each row of ys (each time point) represents one item
      yprod <- lapply(yslist,function(x)outer(x,x))			#a list of matrices with y products through all the time points,
      yproda <- array(unlist(yprod),dim=c(kgroups,kgroups,length(inx_str)))#y terms transformed to an array
      dnisisqa <- array(rep(dnisisq[,kt,s],each=kgroups^2),dim=c(kgroups,kgroups,length(inx_str)))	#dnisisq terms transformed into an array of equal size
      underint <- underint +  yproda * dnisisqa			#the terms under the integral
    }
    covmat <- apply(underint,1:2,sum)						#summing down the array
    covmats <- covmats + covmat
  }
  # cat("kovariancna matrika je = \n")
  # print(covmats)
  # del za testiranje

  zs <-    zs[-kgroups]							# the last one is deleted
  zs <- matrix(zs,nrow=1)
  # print(covmats)
  covmats <- covmats[-kgroups,-kgroups,drop=F]
  # print(covmats)
  test.stat <- zs %*% solve(covmats) %*% t(zs)
  p.value <- 1-pchisq(test.stat,df=kgroups-1)


  names(out$groups) <- names(out$n)
  if (p == 0) out$groups <- NULL						#if no covariates
  out$n <- as.vector(out$n)
  out$call <- call
  #class(out) <- c("survdiff", "rs.surv")
  #cat(zh)
  out$zh <- zs
  out$covmat <- covmats
  out$test.stat <- test.stat
  out$p.value <- p.value
  out$df <- kgroups-1
  class(out) <- "rsdiff"
  out
}
 print.rsdiff <- function(x,...){
 invisible(cat("Value of test statistic:", x$test.stat, "\n"))
 invisible(cat("Degrees of freedom:", x$df, "\n"))
 invisible(cat("P value:", x$p.value, "\n"))
 }

plot.rs.surv <- function (x, conf.int, mark.time = TRUE, mark = 3, col = 1, 
    lty = 1, lwd = 1, cex = 1, log = FALSE, xscale = 1, yscale = 1, firstx = 0, 
    firsty = 1, xmax, ymin = 0, fun, xlab = "", ylab = "", xaxs = "S", 
    ...) 
{
    dotnames <- names(list(...))
    if (any(dotnames == "type")) 
        stop("The graphical argument 'type' is not allowed")
    if (is.logical(log)) {
        logy <- log
        logx <- FALSE
        if (logy) 
            logax <- "y"
        else logax <- ""
    }
    else {
        logy <- (log == "y" || log == "xy")
        logx <- (log == "x" || log == "xy")
        logax <- log
    }
    if (missing(firstx)) {
        if (!is.null(x$start.time)) 
            firstx <- x$start.time
        else {
            if (logx || (!missing(fun) && is.character(fun) && 
                fun == "cloglog")) 
                firstx <- min(x$time[x$time > 0])
            else firstx <- min(0, x$time)
        }
    }
    firstx <- firstx/xscale
    if (missing(xaxs) && firstx != 0) 
        xaxs <- par("xaxs")
    if (!inherits(x, "survfit")) 
        stop("First arg must be the result of survfit")
    if (missing(conf.int)) {
        if (is.null(x$strata) && !is.matrix(x$surv)) 
            conf.int <- TRUE
        else conf.int <- FALSE
    }
    #if (all.times == FALSE & x$method == 1){
    #if (is.null(x$strata0)){
    #    nstrat <- 1
    #    stemp <- rep(1, length(x$index)) # length(x$time[x$index]) == length(x$index)
    #  }
    #  else {
    #    nstrat <- length(x$strata0)
    #    stemp <- rep(1:nstrat,x$strata0)
    #  }
    #}
    #else {
      if (is.null(x$strata)) {
        nstrat <- 1
        stemp <- rep(1, length(x$time))
      }
      else {
        nstrat <- length(x$strata)
        stemp <- rep(1:nstrat, x$strata)
      }
    #}
    ssurv <- x$surv
    stime <- x$time
    supper <- x$upper
    slower <- x$lower
    #if (all.times == FALSE & x$method == 1){
    #  ssurv <- ssurv[x$index]; stime <- stime[x$index]; supper <- supper[x$index]; slower <- slower[x$index]
    #}
    if (!missing(xmax) && any(x$time > xmax)) {
        keepx <- keepy <- NULL
        yzero <- NULL
        tempn <- table(stemp)
        offset <- cumsum(c(0, tempn))
        for (i in 1:nstrat) {
            ttime <- stime[stemp == i]
            if (all(ttime <= xmax)) {
                keepx <- c(keepx, 1:tempn[i] + offset[i])
                keepy <- c(keepy, 1:tempn[i] + offset[i])
            }
            else {
                bad <- min((1:tempn[i])[ttime > xmax])
                if (bad == 1) {
                  keepy <- c(keepy, 1 + offset[i])
                  yzero <- c(yzero, 1 + offset[i])
                }
                else keepy <- c(keepy, c(1:(bad - 1), bad - 1) + 
                  offset[i])
                keepx <- c(keepx, (1:bad) + offset[i])
                stime[bad + offset[i]] <- xmax
                x$n.event[bad + offset[i]] <- 1
            }
        }
        stime <- stime[keepx]
        stemp <- stemp[keepx]
        x$n.event <- x$n.event[keepx]
        if (is.matrix(ssurv)) {
            if (length(yzero)) 
                ssurv[yzero, ] <- firsty
            ssurv <- ssurv[keepy, , drop = FALSE]
            if (!is.null(supper)) {
                if (length(yzero)) 
                  supper[yzero, ] <- slower[yzero, ] <- firsty
                supper <- supper[keepy, , drop = FALSE]
                slower <- slower[keepy, , drop = FALSE]
            }
        }
        else {
            if (length(yzero)) 
                ssurv[yzero] <- firsty
            ssurv <- ssurv[keepy]
            if (!is.null(supper)) {
                if (length(yzero)) 
                  supper[yzero] <- slower[yzero] <- firsty
                supper <- supper[keepy]
                slower <- slower[keepy]
            }
        }
    }
    stime <- stime/xscale
    if (!missing(fun)) {
        if (is.character(fun)) {
            tfun <- switch(fun, log = function(x) x, event = function(x) 1 - 
                x, cumhaz = function(x) -log(x), cloglog = function(x) log(-log(x)), 
                pct = function(x) x * 100, logpct = function(x) 100 * 
                  x, stop("Unrecognized function argument"))
            if (fun == "log" || fun == "logpct") 
                logy <- TRUE
            if (fun == "cloglog") {
                logx <- TRUE
                if (logy) 
                  logax <- "xy"
                else logax <- "x"
            }
        }
        else if (is.function(fun)) 
            tfun <- fun
        else stop("Invalid 'fun' argument")
        ssurv <- tfun(ssurv)
        if (!is.null(supper)) {
            supper <- tfun(supper)
            slower <- tfun(slower)
        }
        firsty <- tfun(firsty)
        ymin <- tfun(ymin)
    }
    if (is.null(x$n.event)) 
        mark.time <- FALSE
    if (is.matrix(ssurv)) 
        ncurve <- nstrat * ncol(ssurv)
    else ncurve <- nstrat
    mark <- rep(mark, length.out = ncurve)
    col <- rep(col, length.out = ncurve)
    lty <- rep(lty, length.out = ncurve)
    lwd <- rep(lwd, length.out = ncurve)
    if (is.numeric(mark.time)) 
        mark.time <- sort(mark.time)
    if (xaxs == "S") {
        xaxs <- "i"
        tempx <- max(stime) * 1.04
    }
    else tempx <- max(stime)
    tempx <- c(firstx, tempx, firstx)
    if (logy) {
        tempy <- range(ssurv[is.finite(ssurv) & ssurv > 0])
        if (tempy[2] == 1) 
            tempy[2] <- 0.99
        if (any(ssurv == 0)) {
            tempy[1] <- tempy[1] * 0.8
            ssurv[ssurv == 0] <- tempy[1]
            if (!is.null(supper)) {
                supper[supper == 0] <- tempy[1]
                slower[slower == 0] <- tempy[1]
            }
        }
        tempy <- c(tempy, firsty)
    }
    else tempy <- c(range(ssurv[is.finite(ssurv)]), firsty)
    if (missing(fun)) {
        tempx <- c(tempx, firstx)
        tempy <- c(tempy, ymin)
    }
    plot(tempx, tempy * yscale, type = "n", log = logax, xlab = xlab, 
        ylab = ylab, xaxs = xaxs, ...)
    if (yscale != 1) {
        if (logy) 
            par(usr = par("usr") - c(0, 0, log10(yscale), log10(yscale)))
        else par(usr = par("usr")/c(1, 1, yscale, yscale))
    }
    dostep <- function(x, y) {
        if (is.na(x[1] + y[1])) {
            x <- x[-1]
            y <- y[-1]
        }
        n <- length(x)
        if (n > 2) {
            dupy <- c(!duplicated(y)[-n], TRUE)
            n2 <- sum(dupy)
            xrep <- rep(x[dupy], c(1, rep(2, n2 - 1)))
            yrep <- rep(y[dupy], c(rep(2, n2 - 1), 1))
            list(x = xrep, y = yrep)
        }
        else if (n == 1) 
            list(x = x, y = y)
        else list(x = x[c(1, 2, 2)], y = y[c(1, 1, 2)])
    }
    i <- 0
    xend <- NULL
    yend <- NULL
    for (j in unique(stemp)) {
        who <- (stemp == j)
        xx <- c(firstx, stime[who])
        nn <- length(xx)
        if (x$type == "counting") {
            #if (all.times == FALSE & x$method == 1){deaths <- c(-1,x$n.censor[x$index][who])}
            #else {
            deaths <- c(-1, x$n.censor[who])
            #}
            zero.one <- 1
        }
        else if (x$type == "right") {
            #if (all.times == FALSE & x$method == 1){deaths <- c(-1,x$n.censor[x$index][who])}
            #else {
            deaths <- c(-1, x$n.censor[who])
            #}
            zero.one <- 1        
        }
        if (is.matrix(ssurv)) {
            for (k in 1:ncol(ssurv)) {
                i <- i + 1
                yy <- c(firsty, ssurv[who, k])
                lines(dostep(xx, yy), lty = lty[i], col = col[i], 
                  lwd = lwd[i])
                if (is.numeric(mark.time)) {
                  indx <- mark.time
                  for (k in seq(along.with = mark.time)) indx[k] <- sum(mark.time[k] > 
                    xx)
                  points(mark.time[indx < nn], yy[indx[indx < 
                    nn]], pch = mark[i], col = col[i], cex = cex)
                }
                else if (mark.time && any(deaths >= zero.one)) {
                  points(xx[deaths >= zero.one], yy[deaths >= 
                    zero.one], pch = mark[i], col = col[i], cex = cex)
                }
                xend <- c(xend, max(xx))
                yend <- c(yend, min(yy))
                if (conf.int && !is.null(supper)) {
                  if (ncurve == 1) 
                    lty[i] <- lty[i] + 1
                  yy <- c(firsty, supper[who, k])
                  lines(dostep(xx, yy), lty = lty[i], col = col[i], 
                    lwd = lwd[i])
                  yy <- c(firsty, slower[who, k])
                  lines(dostep(xx, yy), lty = lty[i], col = col[i], 
                    lwd = lwd[i])
                }
            }
        }
        else {
            i <- i + 1
            yy <- c(firsty, ssurv[who])
            lines(dostep(xx, yy), lty = lty[i], col = col[i], 
                lwd = lwd[i])
            if (is.numeric(mark.time)) {
                indx <- mark.time
                for (k in seq(along = mark.time)) indx[k] <- sum(mark.time[k] > 
                  xx)
                points(mark.time[indx < nn], yy[indx[indx < nn]], 
                  pch = mark[i], col = col[i], cex = cex)
            }
            else if (mark.time == TRUE && any(deaths >= zero.one)) {
                points(xx[deaths >= zero.one], yy[deaths >= zero.one], 
                  pch = mark[i], col = col[i], cex = cex)
            }
            xend <- c(xend, max(xx))
            yend <- c(yend, min(yy))
            if (conf.int == TRUE && !is.null(supper)) {
                if (ncurve == 1) 
                  lty[i] <- lty[i] + 1
                yy <- c(firsty, supper[who])
                lines(dostep(xx, yy), lty = lty[i], col = col[i], 
                  lwd = lwd[i])
                yy <- c(firsty, slower[who])
                lines(dostep(xx, yy), lty = lty[i], col = col[i], 
                  lwd = lwd[i])
            }
        }
    }
    invisible(list(x = xend, y = yend))
}

# Calculating betas:
calculateBetasR <- function(data, xt, event_times, variance=FALSE, var_estimator='dN'){

  ncol <- length(event_times)

  Yt_val <- Yt(data, event_times)
  dNt_val <- dNt(data, event_times)

  sample_size = nrow(xt)
  number_covs = ncol(xt)

  # diag_dNt = matrix(0, nrow=sample_size, ncol=sample_size)
  # beta_var = matrix(0, nrow=number_covs, ncol=number_covs)

  betas_var_list <- vector("list", ncol)

  if(variance){
    betas_list0 <- lapply(1:ncol, function(i) fitOLS2(prepareX(Yt_val[[i]], xt), dNt_val[[i]], Yt_val[[i]]))

    if(var_estimator=='dN'){
      betas_var_list <- lapply(1:ncol, function(i) betas_list0[[i]][[2]] %*% diag(dNt_val[[i]]) %*% t(betas_list0[[i]][[2]]))
    } else if(var_estimator=='XdB'){
      betas_var_list <- lapply(1:ncol, function(i) betas_list0[[i]][[2]] %*% diag(as.vector(prepareX(Yt_val[[i]], xt) %*% betas_list0[[i]][[1]])) %*% t(betas_list0[[i]][[2]]))
    }
    betas_list <- lapply(1:ncol, function(i) betas_list0[[i]][[1]])
  } else{
    betas_list <- lapply(1:ncol, function(i) fitOLS2(prepareX(Yt_val[[i]], xt), dNt_val[[i]], Yt_val[[i]])[[1]]
    )
  }


  # for(i in 1:ncol){
  #   xx2 = relsurv:::prepareX(Yt_val[[i]], xt)
  #   betas = relsurv:::fitOLS2(xx2, dNt_val[[i]], Yt_val[[i]])
  #
  #   betas_list[[i]] = betas[[1]]
  #
  #   if(var_estimator==1){
  #     diag(diag_dNt) <- dNt_val[[i]]
  #   } else if(var_estimator == 2){
  #     betas0_vec <- betas[[1]];
  #     diag(diag_dNt) <- xx2 %*% betas0 # mogoce je treba dati na betas0 t()
  #   }
  #
  #   betas1 = betas[[2]]
  #   # betas_var_list[[i]] <- betas1 %*% diag_dNt %*% t(betas1)
  # }

  out <- list(betas_list, betas_var_list)

  return(out)
}

# An R version of the fitOLSconst C function:
fitOLSconstR <- function(mX, mZ, dNt, Yt) {

  no_cov = ncol(mX)
  no_cov_Z = ncol(mZ)
  sample_size = nrow(mX)
  no_at_risk = sum(Yt)

  Xminus = matrix(0, nrow = sample_size, ncol = no_cov)
  H = matrix(0, ncol=no_cov, nrow=no_cov)
  Identity = matrix(0, ncol=sample_size, nrow=sample_size)
  diag(Identity) <- 1

  prvaKomponenta = matrix(0, ncol=no_cov_Z, nrow=no_cov_Z)
  drugaKomponenta = matrix(0, ncol=no_cov_Z, nrow=1)

  out <- list()
  out[[1]] = prvaKomponenta
  out[[2]] = drugaKomponenta
  out[[3]] = Xminus

  if(no_at_risk >= no_cov){
    mXtX = t(mX)%*%mX

    rcf = rcond(mXtX)

    if(rcf != 0){
      Xminus = solve(mXtX)%*%t(mX)

      H = Identity-mX%*%Xminus
      prvaKomponenta = t(mZ)%*%H%*%mZ
      drugaKomponenta = t(mZ)%*%H%*%dNt

      out[[1]] = prvaKomponenta
      out[[2]] = drugaKomponenta
      out[[3]] = Xminus
    }
  }
  return(out)
}

# Calculating betas and gamma:
calculateBetasGammasR <- function(data, xt, zt, event_times, var_estimator='dN'){

  ncol <- length(event_times)

  Yt_val <- Yt(data, event_times)
  dNt_val <- dNt(data, event_times)

  sample_size = nrow(xt)
  number_covs = ncol(xt)

  diff_t <- diff(c(min(data$start), event_times))

  # diag_dNt = matrix(0, nrow=sample_size, ncol=sample_size)
  # beta_var = matrix(0, nrow=number_covs, ncol=number_covs)

  betas_var_list <- vector("list", ncol)

  mod_list <- lapply(1:ncol, function(i) fitOLSconst(prepareX(Yt_val[[i]], xt),
                                                               prepareX(Yt_val[[i]], zt)[,2:(ncol(zt)+1), drop=FALSE],
                                                               dNt_val[[i]],
                                                               Yt_val[[i]]))

  # Prvi integral:
  int_ztHz <- Reduce('+', lapply(1:ncol, function(i) mod_list[[i]][[1]]*diff_t[i]))
  int_ztHz_inv <- solve(int_ztHz)

  # Drugi integral:
  int_ztHdN <- Reduce('+', lapply(1:ncol, function(i) mod_list[[i]][[2]]))

  gamma_coef <- int_ztHz_inv%*%int_ztHdN

  Xminus <- lapply(1:ncol, function(i) mod_list[[i]][[3]])

  betas_list <- lapply(1:ncol, function(i) Xminus[[i]]%*%(matrix(dNt_val[[i]], ncol=1) - zt %*% gamma_coef * diff_t[i]))

  out <- list(betas_list=betas_list, betas_var_list=betas_var_list, gamma_coef=gamma_coef)

  # Reduce('+', lapply(1:49, function(i) solve(mod_list[[i]][[1]]) %*% mod_list[[i]][[2]]))/max(data$Y)
  # sum(lapply(1:ncol, function(i) solve(mod_list[[i]][[1]]) %*% mod_list[[i]][[2]]))/max(data$Y)

  return(out)
}

#' Fit an additive hazards model
#'
#' Fits the Aalen additive hazard model. The function can be used for multi-state model
#' data (as in the package mstate; class msdata) by supplying the start and stop times in the
#' Surv object and adding a strata(trans) object in the formula (where trans denotes the
#' transition in the multi-state model).
#'
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param variance a logical value indicating whether the variances of the hazards should be computed.
#' Default is FALSE.
#' @param var_estimator Choose variance estimator. The default option 'dN' uses dN(t) in the variance formula, see formula 4.63 in Aalen et al. (2008). Option 'XdB' uses X*dB(t), see formula 4.64 in Aalen et al. (2008).
#' @return An object of class \code{aalen.model}.
#' @seealso \code{rsaalen}
#' @author Damjan Manevski
#' @keywords survival
#' @examples
#'
#' # Survival:
#' data(rdata)
#' mod <- survaalen(Surv(time, cens)~sex+age, data=rdata)
#' head(mod$coefficients)
#' tail(mod$coefficients)
#'
#' # Multi-state model:
#' data(ebmt1wide)
#' mod <- survaalen(Surv(Tstart, Tstop, status)~age.1+age.2+age.3+strata(trans), data=ebmt1wide)
#' head(mod$coefficients$trans1)
#' head(mod$coefficients$trans2)
#' head(mod$coefficients$trans3)
survaalen <- function(formula, data, variance=FALSE, var_estimator='dN'){

  # Find covariates and strata:
  covs <- attr(terms(formula), 'term.labels')
  strata_obj <- grep("strata\\(", covs, value=TRUE)

  # Prepare objects in case of const():
  formula_new <- formula
  covs_new <- covs
  constTRUE <- grepl('const', deparse(formula[[3]]))
  if(constTRUE){
    covs_const <- grepl('const\\(', covs)
    rnames_gamma <- covs[covs_const]

    covs_wconst <- covs
    covs_wconst[covs_const] <- gsub('const\\(', '', covs_wconst[covs_const])
    covs_wconst[covs_const] <- gsub('\\)', '', covs_wconst[covs_const])

    covs_new <- covs_wconst

    if(length(covs_wconst)>1){
      covs_wconst <- paste0(covs_wconst, collapse = ' + ')
    }
    formula_new <- as.formula(paste0(deparse(formula[[2]]), '~', covs_wconst))
  }

  # Run a multi-state model:
  if(length(strata_obj)>0){

    # Check:
    if(length(strata_obj)>1) stop('You have supplied multiple strata() objects in the formula. Please supply only one.')

    # Save original data:
    data_orig <- data
    data <- as.data.frame(data)

    # Find strata object:
    strata_obj <- gsub("strata\\(|\\)", "", strata_obj)
    strata_levels <- unique(data[,strata_obj])

    coefficients <- list()
    coefficients.var <- list()
    gamma <- vector("list", length = length(strata_levels))
    # gamma.var <- vector("list", length = length(strata_levels))

    all_times <- c()

    # Find max time:
    max_time <- max(data[,as.character(formula_new[[2]][3])])

    # For every strata:
    for(i in 1:length(strata_levels)){
      # Subset data and covs:
      data_tmp <- data[data[,strata_obj]==strata_levels[i], ]
      covs_tmp <- covs[endsWith(covs, paste0(".", strata_levels[i])) | endsWith(covs, paste0(".", strata_levels[i], ')'))] # grep(paste0('.', strata_levels[i]), covs_new, value=TRUE)

      # Prepare formula:
      covs_tmp2 <- paste0(covs_tmp, collapse=' + ')
      formula_tmp <- as.formula(paste0(deparse(formula_new[[2]]), '~', covs_tmp2))

      # Run model:
      mod_tmp <- survaalen(formula_tmp, data_tmp, variance)

      # Remove time 0:
      # if(mod_tmp$coefficients[1,1]==0){
      #   mod_tmp$coefficients <- mod_tmp$coefficients[2:nrow(mod_tmp$coefficients),]
      # }

      # Save coefficients:
      coefficients[[i]] <- mod_tmp$coefficients
      if(variance) coefficients.var[[i]] <- mod_tmp$coefficients.var

      all_times <- c(all_times, mod_tmp$coefficients[,1])

      if(constTRUE){
        if('gamma' %in% names(mod_tmp)){
          gamma[[i]] <- mod_tmp$gamma
          # gamma.var[[i]] <- mod_tmp$gamma.var
        }
      }
    }
    names(coefficients) <- paste0('trans', strata_levels)
    names(gamma) <- paste0('trans', strata_levels)

    if(variance){
      names(coefficients.var) <- paste0('trans', strata_levels)
      # names(gamma.var) <- paste0('trans', strata_levels)
    }

    # Add all times in the mstate model:
    all_times <- sort(unique(c(all_times, max_time)))
    for(i in 1:length(strata_levels)){
      all_times_tmp <- all_times[!(all_times %in% coefficients[[i]][,1])]
      tmp_df <- matrix(NA, nrow=length(all_times_tmp), ncol= ncol(coefficients[[i]]))

      tmp_df[,1] <- all_times_tmp
      colnames(tmp_df) <- colnames(coefficients[[i]])

      coefficients[[i]] <- rbind(coefficients[[i]], tmp_df)
      coefficients[[i]] <- coefficients[[i]][order(coefficients[[i]][,1]),]

      if(variance){
        if(nrow(coefficients.var[[i]]) != 0){
          coefficients.var[[i]] <- rbind(coefficients.var[[i]], tmp_df)
          coefficients.var[[i]] <- coefficients.var[[i]][order(coefficients.var[[i]][,1]),]
        }
      }

      for(j in 2:ncol(coefficients[[i]])){
        coefficients[[i]][,j] <- mstateNAfix(coefficients[[i]][,j], 0)
        if(variance){
          if(nrow(coefficients.var[[i]]) != 0){
            coefficients.var[[i]][,j] <- mstateNAfix(coefficients.var[[i]][,j], 0)
          }
        }
      }
    }

    if(constTRUE){
      if(variance){
        out <- list(coefficients=coefficients, coefficients.var=coefficients.var,
                    gamma=gamma#, gamma.var=gamma.var
                    )
      } else{
        out <- list(coefficients=coefficients, gamma=gamma)
      }
    } else{
      if(variance){
        out <- list(coefficients=coefficients, coefficients.var=coefficients.var)
      } else{
        out <- list(coefficients=coefficients)
      }
    }
    class(out) <- 'aalen.model'

    return(out)

    # Run a usual survival model:
  } else{
    # Save original data:
    data_orig <- data
    sample_size <- nrow(data_orig)

    # Standard relsurv format:
    rform <- rformulate2(formula_new, data)

    data <- rform$data
    xt <- rform$X

    # Times:
    all_times <- unique(c(data$start, data$Y))
    event_times <- unique(data$Y[data$stat==1])

    all_times <- sort(all_times)
    event_times <- sort(event_times)

    # Take only times until last event time:
    all_times <- all_times[all_times <= event_times[length(event_times)]]

    # Find Yt / dNt:
    # dNt <- dNt(data, event_times)
    # Yt <- Yt(data, event_times)

    # xx1 <- prepareX(Yt[[1]], as.matrix(xt))
    # fuu <- fitOLS2(xx1, dNt[[1]], Yt[[1]])

    # browser()

    # Find betas:
    # betas <- calculateBetas(data, as.matrix(xt), event_times, 1)
    if(constTRUE){
      zt <- xt[,covs_const, drop=FALSE]
      xt <- xt[,!covs_const, drop=FALSE]

      all_times_w0 <- all_times[2:length(all_times)]

      betas0 <- calculateBetasGammasR(data, as.matrix(xt), as.matrix(zt), all_times_w0, 1)
    } else{
      betas0 <- calculateBetasR(data, as.matrix(xt), event_times, variance, var_estimator)
    }

    # Variance:
    betas_var <- betas0[[2]]
    betas_var <- lapply(betas_var, function(x) diag(x))
    betas_var2 <- do.call(rbind, betas_var)

    # Point estimates:
    betas <- betas0[[1]]
    # Take care of format:
    betas2 <- t(do.call(cbind, betas))
    # betas2 <- rbind(matrix(0, nrow=1, ncol=(ncol(xt)+1)),
    #                 betas2)

    # Cumulative:
    for(iu in 1:ncol(betas2)){
      betas2[,iu] <- cumsum(betas2[,iu])
      if(ncol(betas_var2)!=0){
        betas_var2[,iu] <- cumsum(betas_var2[,iu])#*sample_size
      }
    }

    # Add zero:
    if(0 %in% all_times){
      event_times <- c(0, event_times)
      betas2 <- rbind(matrix(0, nrow=1, ncol=(ncol(xt)+1)),
                      betas2)
      if(ncol(betas_var2)!=0){
        betas_var2 <- rbind(matrix(0, nrow=1, ncol=(ncol(xt)+1)),
                            betas_var2)
      }
    }

    # Add times:
    if(constTRUE){
      betas2 <- cbind(all_times, betas2)
      betas2 <- betas2[all_times %in% c(0, event_times),]
    } else{
      betas2 <- cbind(event_times, betas2)
    }
    if(ncol(betas_var2)!=0){
      betas_var2 <- cbind(event_times, betas_var2)
    }

    # Add naming:
    colnames(betas2) <- c('time', '(Intercept)', colnames(xt))
    if(ncol(betas_var2)!=0){
      colnames(betas_var2) <- c('time', '(Intercept)', colnames(xt))
    }

    # Prepare final object:
    if(variance){
      var.obj <- betas_var2
      # var.obj[,2:ncol(var.obj)] <- abs(var.obj[,2:ncol(var.obj)])/4

      out <- list(coefficients=betas2, coefficients.var=var.obj) # prej je betas_var2
    } else{
      out <- list(coefficients=betas2)
    }

    if(constTRUE){
      rownames(betas0[[3]]) <- rnames_gamma
      colnames(betas0[[3]]) <- 'gamma'

      out[[length(out)+1]] <- betas0[[3]]
      names(out)[length(out)] <- 'gamma'

      # if(variance){
      #   out[[length(out)+1]] <- betas0[[3]] # tmp
      #   colnames(out[[length(out)]]) <- 'gamma.var'
      #   names(out)[length(out)] <- 'gamma.var'
      # }
    }

    class(out) <- 'aalen.model'

    return(out)
  }
}

# TO DO:
# Scheike vprasanje: zakaj premikas case, ce je ob istem casu vec dogodkov?
# Some plot function for mstate output of surv/rsaalen? Za coefficiente?

#' Compute a Relative Survival Curve from an additive relative survival model
#'
#' Computes the predicted relative survival function for an additive relative
#' survival model fitted with maximum likelihood.
#'
#' Does not work with factor variables - you have to form dummy variables
#' before calling the rsadd function.
#'
#' @param formula a \code{rsadd} object (Implemented only for models fitted
#' with the codemax.lik (default) option.)
#' @param newdata a data frame with the same variable names as those that
#' appear in the \code{rsadd} formula.  a predicted curve for each individual
#' in this data frame shall be calculated
#' @return a \code{survfit} object; see the help on \code{survfit.object} for
#' details.  The \code{survfit} methods are used for \code{print}, \code{plot},
#' \code{lines}, and \code{points}.
#' @seealso \code{survfit}, \code{survexp}
#' @references Package. Pohar M., Stare J. (2006) "Relative survival analysis
#' in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #fit a relative survival model
#' fit <- rsadd(Surv(time,cens)~sex+age+year,rmap=list(age=age*365.241),
#' 	ratetable=slopop,data=rdata,int=c(0:10,15))
#'
#' #calculate the predicted curve for a male individual, aged 65, diagnosed in 1982
#' d <- rs.surv.rsadd(fit,newdata=data.frame(sex=1,age=65,year=as.Date("1982-01-01")))
#' #plot the curve (will result in a step function since the baseline is assumed piecewise constant)
#' plot(d,xscale=365.241)
#'
#' #calculate the predicted survival curves for each individual in the data set
#' d <- rs.surv.rsadd(fit,newdata=rdata)
#' #calculate the average over all predicted survival curves
#' p.surv <- apply(d$surv,1,mean)
#' #plot the relative survival curve
#' plot(d$time/365.241,p.surv,type="b",ylim=c(0,1),xlab="Time",ylab="Relative survival")
#'
rs.surv.rsadd <- function (formula, newdata)
{

    call <- match.call()
    Terms <- terms(formula$formula)  #to rabis, ce je model mal bl smotan - as.factor ali splines ali svasta
    Terms <- delete.response(Terms)
    newdata <- model.frame(Terms,newdata)
    n <- formula$n
    if(formula$method=="max.lik"){
   	 nvar <- length(formula$coef) - length(formula$int)+1
    	 formula$coef <- formula$coef[1:nvar]
    }
    nvar <- length(formula$coef)
    nx <- nrow(newdata)
    nt <- length(formula$times)
    temp <- list(n=formula$n,time=formula$times,call=call,type="right")
    Lambda0 <- formula$Lambda0
    Lambda0 <- matrix(Lambda0,ncol=nt,nrow=nx,byrow=TRUE)
    rate <- attr(Terms, "specials")$ratetable
    R <- as.matrix(newdata[, rate,drop=FALSE])
    rat <- attributes(formula$ratetable)$dimid
    mein <- attributes(newdata[,rate])$dimnames[[2]]
    x <- match(rat,mein)
    R <- R[,x,drop=FALSE]
    newdata <- newdata[,1:nvar,drop=FALSE]
    if(any(formula$mvalue)>0)newdata <- newdata - matrix(formula$mvalue,nrow=nx,byrow=TRUE)
    R <- data.frame(R)
    names(R) <- rat
    ebx <- exp(data.matrix(newdata)%*%as.vector(formula$coef))
    ebx <- matrix(ebx,ncol=nt,nrow=length(ebx))
    Lambdae <- Lambda0*ebx
    temp$surv <- t(exp(-Lambdae))
    temp$n.event <- rep(1,nt)
    temp$n.risk <- n+1 - cumsum(temp$n.event)
    temp$time <- formula$times
    class(temp) <- c("rs.surv.rsadd", "rs.surv","survfit")
    temp
}

my.strata <- function (..., nameslist,   sep = ", ") 
{									#nameslist = lista imen spremenljivk
    words <- as.character((match.call())[-1])				#ime podatkov
    allf <- list(...)							#podatki
    if (length(allf) == 1 && is.list(ttt <- unclass(allf[[1]]))) 	#so samo eni podatki
        allf <- ttt							#ohranim le podatke (ne listo podatkov), v obliki list
    nterms <- length(allf)						#nterms= st. spremenljivk +1 (row.names)
    if (is.null(names(allf)))						#ce ni imen 
        argname <- words[1:nterms]					#jih dam
    else argname <- ifelse(names(allf) == "", words[1:nterms], 		#ce so prazna jih dam
        names(allf))							#imena so v argname
   
  
   varnames <- names(nameslist)
  	
    #1. iteracija
    what <- allf[[1]]							#prva spremenljivka
   
    for(it in 1:length(varnames)){
       if (length(grep(varnames[it],names(allf)[[1]]))) break		#poiscem ji mesto v svojem poimenovanju
       }
   
   
    if (is.null(levels(what)))     what <- factor(what)			#ce se ni, jo prisilimo v faktorsko
    levs <- unclass(what) - 1						#nastavim prvi level = 0
    wlab <- levels(what)						#imena faktorjev
    labs <- paste(argname[1], wlab, sep = "=")				#prvo ime = 0/1
    
    labsnow <- 1
    allab <- NULL
    
    dd <- length(nameslist[[it]])
    if(dd!=2) {
    	mylabs <- rep(argname[1],length(wlab))
    	mylabs[wlab==0] <- ""
    }
    else mylabs <- labs
   
 
     for (i in (1:nterms)[-1]) {
        if(length(grep(varnames[labsnow],names(allf)[[i]]))==0){					#ce je zdaj to nova spremenljivka, moram najprej ustimat prejsnjo
    		mylabs[mylabs==""] <- nameslist[[labsnow]][1] 
    		if(!any(allab!=""))allab <- paste(allab,mylabs,sep="")					#the first time - do not separate by comma
    		else allab <- paste(allab,mylabs,sep=",")
    		mylabs <- rep("",length(mylabs))
    		labsnow <- labsnow+1
        }
        what <- allf[[i]]
        if (is.null(levels(what))) what <- factor(what)
        wlev <- unclass(what) - 1
        wlab <- levels(what)
        labsnew <- format(paste(argname[i], wlab, sep = "="))
	levs <- wlev + levs * (length(wlab))
        a <- rep(labs, rep(length(wlab), length(labs)))
	b <- rep(wlab, length(labs))
	
	mya <- rep(mylabs, rep(length(wlab), length(labs)))
	allab <- rep(allab,rep(length(wlab), length(labs)))
	myb <- rep(argname[i],length(labs)*length(wlab))
	for(it in 1:length(varnames)){								#it se ustavi pri trenutni spremenljivki
	    if (length(grep(varnames[it],names(allf)[[i]]))) break
    	}
	dd <- length(nameslist[[it]])
	if(dd==2)myb <- paste(myb,rep(wlab,length(labs)),sep="=")
	else myb[rep(wlab,length(labs))==0] <- ""
	mylabs <- paste(mya,myb,sep="")
	
        labs <- paste(a,b, sep = sep)
    }
    mylabs[mylabs==""] <- nameslist[[labsnow]][1] 
    if(!any(allab!=""))allab <- paste(allab,mylabs,sep="")
    else allab <- paste(allab,mylabs,sep=",")
    levs <- levs + 1
    ulevs <- sort(unique(levs[!is.na(levs)]))
    levs <- match(levs, ulevs)
    labs <- labs[ulevs]
    allab <- allab[ulevs]
    factor(levs, labels = allab)
}

#' Compute crude probability of death
#'
#' Estimates the crude probability of death due to disease and due to
#' population reasons
#'
#' NOTE: The follow-up time must be specified in days. The \code{ratetable}
#' being used may have different variable names and formats than the user's
#' data set, this is dealt with by the \code{rmap} argument. For example, if
#' age is in years in the data set but in days in the \code{ratetable} object,
#' age=age*365.241 should be used. The calendar year can be in any date format
#' (Date and POSIXt are allowed), the date formats in the
#' \code{ratetable} and in the data may differ.
#'
#' Note that numerical integration is required to calculate the variance
#' estimator. The integration precision is set with argument \code{precision},
#' which defaults to daily intervals, a default that should give enough
#' precision for any practical purpose.
#'
#' The area under the curve is calculated on the interval [0,\code{tau}].
#'
#' Function \code{summary} may be used to get the output at specific points in
#' time.
#'
#' @aliases cmp.rel print.cmp.rel
#' @param formula a formula object, with the response as a \code{Surv} object
#' on the left of a \code{~} operator, and, if desired, terms separated by the
#' \code{+} operator on the right. If no strata are used, \code{~1} should be
#' specified.
#'
#' NOTE: The follow-up time must be in days.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula}.
#' @param ratetable a table of event rates, organized as a \code{ratetable}
#' object, such as \code{slopop}.
#' @param na.action a missing-data filter function, applied to the model.frame,
#' after any subset argument has been used.  Default is
#' \code{options()$na.action}.
#' @param tau the maximum follow-up time of interest, all times larger than
#' \code{tau} shall be censored. Equals maximum observed time by default
#' @param conf.int the level for a two-sided confidence interval on the
#' survival curve(s). Default is 0.95.
#' @param precision the level of precision used in the numerical integration of
#' variance. Default is 1, which means that daily intervals are taken, the
#' value may be decreased to get a higher precision or increased to achieve a
#' faster calculation. The calculation intervals always include at least all
#' times of event and censoring as border points.
#' @param add.times specific times at which the value of estimator and its
#' variance should be evaluated. Default is all the event and censoring times.
#' @param rmap an optional list to be used if the variables are not organized
#' and named in the same way as in the \code{ratetable} object. See details
#' below.
#' @return An object of class \code{cmp.rel}. Objects of this class have
#' methods for the functions \code{print} and \code{plot}. The \code{summary}
#' function can be used for printing output at required time points. An object
#' of class \code{cmp.rel} is composed of several lists, each pertaining the
#' cumulative hazard function for one risk and one strata. Each of the lists
#' contains the following objects: \item{time}{the time-points at which the
#' curves are estimated} \item{est}{the estimate} \item{var}{the variance of
#' the estimate} \item{lower}{the lower limit of the confidence interval}
#' \item{upper}{the upper limit of the confidence interval} \item{area}{the
#' area under the curve calculated on the interval [0,\code{tau}]}
#' \item{index}{indicator of event and censoring times among all the times in
#' the output. The times added via paramater \code{add.times} are also
#' included} \item{add.times}{the times added via parameter \code{add.times}}
#' @seealso \code{rs.surv}, \code{summary.cmp.rel}
#' @references Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric
#' Relative Survival Analysis with the R Package relsurv". Journal of
#' Statistical Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08"
#' @keywords survival
#' @examples
#'
#'
#' data(slopop)
#' data(rdata)
#' #calculate the crude probability of death
#' #note that the variable year must be given in a date format and that
#' #age must be multiplied by 365.241 in order to be expressed in days.
#' fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#' 		ratetable=slopop,data=rdata,tau=3652.41)
#' fit
#' plot(fit,col=c(1,1,2,2),xscale=365.241,xlab="Time (years)")
#' #if no strata are desired:
#' fit <- cmp.rel(Surv(time,cens)~1,rmap=list(age=age*365.241),
#' 		ratetable=slopop,data=rdata,tau=3652.41)
#'
#'
#'
cmp.rel <-  function (formula = formula(data), data = parent.frame(), ratetable = relsurv::slopop,
     na.action,tau,conf.int=0.95,precision=1,add.times,rmap)

    #formula: for example Surv(time,cens)~1 #not implemented for subgroups - DO IT!
    #data: the observed data set
    #ratetable: the population mortality tables
    #conf.type: confidence interval calculation (plain, log or log-log)
    #conf.int: confidence interval
    #tau: max. cas do katerega racuna
{

    call <- match.call()
    if (!missing(rmap)) {
          rmap <- substitute(rmap)
    }
    rform <- rformulate(formula, data, ratetable, na.action,rmap)			#get the data ready
    data <- rform$data								#the data set
    se.fac <- sqrt(qchisq(conf.int, 1))						#factor needed for confidence interval

    if(missing(tau)) tau<-max(rform$Y)

    p <- rform$m								#number of covariates
    if (p > 0) #if covariates
    data$Xs <- strata(rform$X[, ,drop=FALSE ])					#make strata according to covariates
    else data$Xs <- rep(1, nrow(data))						#if no covariates, just put 1

    tab.strata <- table(data$Xs)						#unique strata values
    ntab.strata <- length(tab.strata)						#number of strata

    dtemp <- list(NULL)
    out <- as.list(rep(dtemp,ntab.strata*2))

    for (kt in 1:ntab.strata) {							#for each stratum
        inx <- which(data$Xs == names(tab.strata)[kt])				#individuals within this stratum

	extra <- as.numeric(seq(1,max(rform$Y[inx]),by=precision))
	if(!missing(add.times)) extra <- c(extra,as.numeric(add.times))
	tis <- sort(unique(pmin(tau,union(rform$Y[inx],extra))) )	#1-day long intervals used - to take into the account the continuity of the pop. part

	#if(!all.times)tis <- sort(unique(pmin(rform$Y[inx],tau)))					#unique times
	#else{
	#	tis <- sort(union(rform$Y[inx], as.numeric(1:floor(max(rform$Y[inx]))))) 	#1-day long intervals used - to take into the account the continuity of the pop. part
	#	tis <- unique(pmin(tis,tau))
   	#}
	k <- length(tis)

	out[[2*kt-1]]$time <- out[[2*kt]]$time <- c(0,tis)

	temp <- exp_prep(rform$R[inx,,drop=FALSE],rform$Y[inx],rform$ratetable,rform$status[inx],times=tis,fast=TRUE,cmp=T)		#calculate the values for each interval of time

	areae <- sum(temp$areae)/365.241		# sum(diff(c(0,tis))*temp$cumince)/365.241
	areap <- sum(temp$areap)/365.241		#sum(diff(c(0,tis))*temp$cumincp)/365.241

  	options(warn=-1)
  	out[[2*kt-1]]$est <- c(0,temp$cumince)
  	out[[2*kt-1]]$var <- c(0,temp$ve)
  	out[[2*kt-1]]$lower <- temp$cumince-se.fac*sqrt(temp$ve)
  	out[[2*kt-1]]$upper <- temp$cumince+se.fac*sqrt(temp$ve)
  	out[[2*kt-1]]$area <- areae

  	out[[2*kt]]$est <- c(0,temp$cumincp)
	out[[2*kt]]$var <- c(0,temp$vp)
	out[[2*kt]]$lower <- temp$cumincp-se.fac*sqrt(temp$vp)
	out[[2*kt]]$upper <- temp$cumincp+se.fac*sqrt(temp$vp)
	out[[2*kt]]$area <- areap
	options(warn=0)

	ne <- sum(temp$ve<0)
	if(ne>0) warning(paste(names(tab.strata)[kt],": The estimated variance of crude mortality is negative in ", ne, " out of ", length(temp$ve)," intervals"), call. = FALSE)

	if(!missing(add.times)){
	out[[2*kt-1]]$index <- out[[2*kt]]$index <- unique(c(1,which(tis %in% c(rform$Y[inx],add.times,tau))))
	out[[2*kt-1]]$add.times <- out[[2*kt]]$add.times <- add.times
	}
	else out[[2*kt-1]]$index <- out[[2*kt]]$index <- unique(c(1,which(tis %in% c(rform$Y[inx],tau))))

   }
   if(p>0)names(out) <- paste(rep(c("causeSpec","population"),ntab.strata),rep(names(tab.strata),each=2))
   else names(out) <- c("causeSpec","population")
   out$tau <- tau
   class(out) <- "cmp.rel"
   out
}




#' Plot the crude probability of death
#'
#' Plot method for cmp.rel. Plots the cumulative probability of death due to
#' disease and due to population reasons
#'
#' By default, the graph is plotted as a step function for the cause specific
#' mortality and as a piecewise linear function for the population mortality.
#' It is evaluated at all event and censoring times even though it constantly
#' changes also between these time points.
#'
#' If the argument \code{all.times} is set to \code{TRUE}, the plot is
#' evaluated at all times that were used for numerical integration in the
#' \code{cmp.rel} function (there, the default is set to daily intervals). If
#' only specific time points are to be added, this should be done via argument
#' \code{add.times} in \code{cmp.rel}.
#'
#' @param x a list, with each component representing one curve in the plot,
#' output of the function \code{cmp.rel}.
#' @param main the main title for the plot.
#' @param curvlab Curve labels for the plot. Default is \code{names(x)}, or if
#' that is missing, \code{1:nc}, where \code{nc} is the number of curves in
#' \code{x}.
#' @param ylim yaxis limits for plot.
#' @param xlim xaxis limits for plot (default is 0 to the largest time in any
#' of the curves).
#' @param wh if a vector of length 2, then the upper right coordinates of the
#' legend; otherwise the legend is placed in the upper right corner of the
#' plot.
#' @param xlab X axis label.
#' @param ylab y axis label.
#' @param lty vector of line types. Default \code{1:nc} (\code{nc} is the
#' number of curves in \code{x}). For color displays, \code{lty=1},
#' \code{color=1:nc}, might be more appropriate. If \code{length(lty)<nc}, then
#' \code{lty[1]} is used for all.
#' @param xscale Scale of the X axis. Default is in days (1).
#' @param col vector of colors. If \code{length(col)<nc}, then the
#' \code{col[1]} is used for all.
#' @param lwd vector of line widths. If \code{length(lwd)<nc}, then
#' \code{lwd[1]} is used for all.
#' @param curves Vector if integers, specifies which curves should be plotted.
#' May take values \code{1:nc}, where \code{nc} is the number of curves in
#' \code{x}. By default, all of the curves are plotted.
#' @param conf.int Vector if integers, specifies which confidence intervals
#' should be plotted. May take values \code{1:nc}, where \code{nc} is the
#' number of curves in \code{x}. By default, no confidence intervals are
#' plotted.
#' @param all.times By default, the disease specific mortality estimate is
#' plotted as a step function between event or censoring times. If set to
#' \code{TRUE}, the graph is evaluated at all estimated times.
#' @param ... additional arguments passed to the initial call of the plot
#' function.
#' @return No value is returned.
#' @seealso \code{rs.surv}
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365.241),
#' 		ratetable=slopop,data=rdata,tau=3652.41)
#' plot(fit,col=c(1,1,2,2),xscale=365.241,conf.int=c(1,3))
#'
plot.cmp.rel <- function (x, main = " ", curvlab, ylim = c(0, 1), xlim, wh = 2,
    xlab = "Time (days)", ylab = "Probability", lty = 1:length(x), xscale=1,
    col = 1, lwd = par("lwd"), curves, conf.int, all.times=FALSE,...)
{
#wh= upper left coordinates of the legend, if of length 1, the legend is placed top left.
    tau <- x$tau
    x$tau <- NULL
    nc <- length(x)			#number of curves
    if (length(lty) < nc)		#if not enough different line types
        lty <- rep(lty[1], nc)
    else lty <- lty[1:nc]
    if (length(lwd) < nc)		#if not enough different line widths
        lwd <- rep(lwd[1], nc)
    else lwd <- lwd[1:nc]
    if (length(col) < nc)		#if not enough different colors
        col <- rep(col[1], nc)
    else col <- col[1:nc]
    if (missing(curvlab)) {		#if no curve labels desired
        if (mode(names(x)) == "NULL") { #and no curve labels prespecified
            curvlab <- as.character(1:nc)
        }
        else curvlab <- names(x)[1:nc]	#use prespecified if they exist
    }
    if (missing(xlim)) {		#if no limits desired
        xmax <- 0
        for (i in 1:nc) {
            xmax <- max(c(xmax, x[[i]][[1]]/xscale))	#take max time over all strata
        }
        xlim <- c(0, xmax)
    }
    if(all.times){
    for(it in 1:nc){
       x[[it]]$index <- 1:length(x[[it]][[1]])
    }
    }
    if(missing(curves))curves <- 1:nc
    if(missing(conf.int))conf.int <- NULL
    curves <- unique(curves)
    conf.int <- unique(conf.int)
    if(any((curves %in% 1:nc)==FALSE)) stop(paste("The curves argument should be specified as a vector of integers from 1 to", nc,sep=" "))
    if(any((conf.int %in% 1:nc)==FALSE)) stop(paste("The conf.int argument should be specified as a vector of integers from 1 to", nc,sep=" "))
    if(any((conf.int %in% curves)==FALSE)) stop("Confidence interval may only be plotted if the curve is plotted, see argument curves")

    col_nums <- floor(seq(from=95,to=50,length.out=length(conf.int)+2))
    col.conf.temp <-  sapply(col_nums,function(x)paste("gray",as.character(x),sep=""))
    col.conf.int <- rep("white",nc)
    col.conf.int[conf.int] <-  col.conf.temp[-c(1,length(col.conf.temp))]

    plot((x[[1]][[1]]/xscale)[x[[1]]$index], (x[[1]][[2]])[x[[1]]$index], type = "n", ylim = ylim, xlim = xlim,
        main = main, xlab = xlab, ylab = ylab, bty = "l", ...)		#plot estimates [[1]]=time, [[2]]=est
    if (length(wh) != 2) {
        wh <- c(xlim[1], ylim[2])
    }
    u <- list(...)
    if (length(u) > 0) {
        i <- pmatch(names(u), names(formals(legend)), 0)
        do.call("legend", c(list(x = wh[1], y = wh[2], legend = curvlab[curves],
            col = col[curves], lty = lty[curves], lwd = lwd[curves], bty = "n", bg = -999999),
            u[i > 0]))
    }
    else {
        do.call("legend", list(x = wh[1], y = wh[2], legend = curvlab[curves],
            col = col[curves], lty = lty[curves], lwd = lwd[curves], bty = "n", bg = -999999))
    }
    for(i in conf.int){
    	 if(i%%2==0)with(x[[i]],polygon(c(time[index][!is.na(lower[index])],rev(time[index][!is.na(upper[index])]))/xscale,c(lower[index][!is.na(lower[index])],rev(upper[index][!is.na(upper[index])])),col = col.conf.int[i] , border = FALSE))
    	 else with(x[[i]],my.poly(time[index][!is.na(lower[index])]/xscale,time[index][!is.na(upper[index])]/xscale,lower[index][!is.na(lower[index])],upper[index][!is.na(upper[index])],col = col.conf.int[i] , border = FALSE))
    }
    for (i in curves) {
        tip <- "s"
        if(i%%2==0)tip <- "l"
        lines((x[[i]][[1]]/xscale)[x[[i]]$index], (x[[i]][[2]])[x[[i]]$index], lty = lty[i], col = col[i],
            lwd = lwd[i], type=tip, ...)
    }
}

my.poly <- function(x1,x2,y1,y2,...){
	x1 <- rep(x1,each=2)[-1]
	y1 <- rep(y1,each=2)[-(2*length(y1))]
	x2 <- rep(x2,each=2)[-1]
	y2 <- rep(y2,each=2)[-(2*length(y2))]
	polygon(c(x1,rev(x2)),c(y1,rev(y2)),...)
}


print.cmp.rel <- function (x, ntp = 4, maxtime,scale=365.241, ...)
{
    tau <- x$tau
    x$tau <- NULL
    nc <- length(x)

    if (missing(maxtime)) {
        maxtime <- 0
        for (i in 1:nc) maxtime <- max(maxtime, x[[i]]$time)
    }
    tp <- pretty(c(0, maxtime/scale), ntp + 1)
    tp <- tp[-c(1, length(tp))]

    if(length(x[[1]]$add.times)>0 & length(x[[1]]$add.times)<5){
	tp <- sort(unique(c(tp,round(x[[1]]$add.times/scale,1))))
    }
    cat("Estimates, variances and area under the curves:\n")
    x$tau <- tau
    print(summary(x, tp,scale,area=TRUE), ...)
    invisible()
}



#' Summary of the crude probability of death
#'
#' Returns a list containing the estimated values at required times.
#'
#' The variance is calculated using numerical integration. If the required time
#' is not a time at which the value was estimated, the value at the last time
#' before it is reported. The density of the time points is set by the
#' \code{precision} argument in the \code{cmp.rel} function.
#'
#' @param object output of the function \code{cmp.rel}.
#' @param times the times at which the output is required.
#' @param scale The time scale in which the times are specified. The default
#' value is \code{1}, i.e. days.
#' @param area Should area under the curves at time \code{tau} be printed out?
#' Default is \code{FALSE}.
#' @param ... Additional arguments, currently not implemented
#' @return A list of values is returned.
#' @seealso \code{cmp.rel}
#' @keywords survival
#' @examples
#'
#' data(slopop)
#' data(rdata)
#' #calculate the crude probability of death and summarize it
#' fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365),
#'       ratetable=slopop,data=rdata,tau=3652.41)
#' summary(fit,c(1,3),scale=365.241)
#'
summary.cmp.rel <- function (object, times,scale=365.241,area=FALSE,...)
{
    tau <- object$tau
    object$tau <- NULL
    ng <- length(object)
    times <- sort(unique(times))*scale
    nt <- length(times)
    storage.mode(times) <- "double"
    storage.mode(nt) <- "integer"
    ind <- matrix(0, ncol = nt, nrow = ng)
    oute <- matrix(NA, ncol = nt, nrow = ng)
    outv <- oute
    outa <- matrix(NA,ncol=1,nrow=ng)
    storage.mode(ind) <- "integer"
    slct <- rep(TRUE, ng)
    for (i in 1:ng) {
        if (is.null((object[[i]])$est)) {
            slct[i] <- FALSE
        }
        else {
            z <- rep(NA,nt)
            for(kt in 1:nt)z[kt] <- rev(which(object[[i]][[1]]<=times[kt]))[1]
            ind[i, ] <- z
            oute[i, ind[i, ] > 0] <- object[[i]][[2]][z]
            outa[i,] <- object[[i]][[6]]
            if (length(object[[i]]) > 2)
                outv[i, ind[i, ] > 0] <- object[[i]][[3]][z]
        }
    }
    dimnames(oute) <- list(names(object)[1:ng], as.character(times/scale))
    dimnames(outv) <- dimnames(oute)
    rownames(outa) <- rownames(oute)
    colnames(outa) <- paste("Area at tau =",tau/scale)
    if(area)list(est = oute[slct, , drop = FALSE], var = outv[slct, ,
        drop = FALSE], area=outa[slct,,drop=FALSE])
    else list(est = oute[slct, , drop = FALSE], var = outv[slct, ,
        drop = FALSE])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{invtime}
\alias{invtime}
\title{Inverse transforming of time in Relative Survival}
\usage{
invtime(
  y = 0.1,
  age = 23011,
  sex = "male",
  year = as.Date("1986-01-01"),
  scale = 1,
  ratetable = relsurv::slopop,
  lower,
  upper
)
}
\arguments{
\item{y}{time in Y.}

\item{age}{age of the individual.  Must be in days.}

\item{sex}{sex of the individual. Must be coded in the same way as in the
\code{ratetable}.}

\item{year}{date of diagnosis. Must be in a Date or POSIXt format.}

\item{scale}{numeric value to scale the results. If \code{ratetable} is in
units/day, \code{scale = 365.241} causes the output to be reported in years.}

\item{ratetable}{a table of event rates, such as \code{survexp.us}.}

\item{lower}{the lower bound of interval where the result is expected. This
argument is optional, but, if given, can shorten the time the function needs
to calculate the result.}

\item{upper}{the upper bound of interval where the result is expected. See
\code{lower}.}
}
\value{
A list of values \item{T}{the original time} \item{Y}{the
transformed time}.
}
\description{
This function can be used when predicting in Relative Survival using the
transformed time regression model (using \code{rstrans} function). It
inverses the time from Y to T in relative survival using the given
ratetable. The times Y can be produced with the \code{rstrans} function, in
which case, this is the reverse function. This function does the
transformation for one person at a time.
}
\details{
Works only with ratetables that are split by age, sex and year. Transforming
can be computationally intensive, use lower and/or upper to guess the
interval of the result and thus speed up the function.
}
\examples{

data(slopop)
invtime(y = 0.1, age = 23011, sex = 1, year = as.Date('1986-01-01'), ratetable = slopop)

}
\references{
Package: Pohar M., Stare J. (2006) "Relative survival analysis
in R."  Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741-1749.
}
\seealso{
\code{\link{rstrans}}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cmprel.r
\name{cmp.rel}
\alias{cmp.rel}
\alias{print.cmp.rel}
\title{Compute crude probability of death}
\usage{
cmp.rel(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  na.action,
  tau,
  conf.int = 0.95,
  precision = 1,
  add.times,
  rmap
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right. If no strata are used, \code{~1} should be
specified.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{na.action}{a missing-data filter function, applied to the model.frame,
after any subset argument has been used.  Default is
\code{options()$na.action}.}

\item{tau}{the maximum follow-up time of interest, all times larger than
\code{tau} shall be censored. Equals maximum observed time by default}

\item{conf.int}{the level for a two-sided confidence interval on the
survival curve(s). Default is 0.95.}

\item{precision}{the level of precision used in the numerical integration of
variance. Default is 1, which means that daily intervals are taken, the
value may be decreased to get a higher precision or increased to achieve a
faster calculation. The calculation intervals always include at least all
times of event and censoring as border points.}

\item{add.times}{specific times at which the value of estimator and its
variance should be evaluated. Default is all the event and censoring times.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}
}
\value{
An object of class \code{cmp.rel}. Objects of this class have
methods for the functions \code{print} and \code{plot}. The \code{summary}
function can be used for printing output at required time points. An object
of class \code{cmp.rel} is composed of several lists, each pertaining the
cumulative hazard function for one risk and one strata. Each of the lists
contains the following objects: \item{time}{the time-points at which the
curves are estimated} \item{est}{the estimate} \item{var}{the variance of
the estimate} \item{lower}{the lower limit of the confidence interval}
\item{upper}{the upper limit of the confidence interval} \item{area}{the
area under the curve calculated on the interval [0,\code{tau}]}
\item{index}{indicator of event and censoring times among all the times in
the output. The times added via paramater \code{add.times} are also
included} \item{add.times}{the times added via parameter \code{add.times}}
}
\description{
Estimates the crude probability of death due to disease and due to
population reasons
}
\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with by the \code{rmap} argument. For example, if
age is in years in the data set but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.

Note that numerical integration is required to calculate the variance
estimator. The integration precision is set with argument \code{precision},
which defaults to daily intervals, a default that should give enough
precision for any practical purpose.

The area under the curve is calculated on the interval [0,\code{tau}].

Function \code{summary} may be used to get the output at specific points in
time.
}
\examples{


data(slopop)
data(rdata)
#calculate the crude probability of death
#note that the variable year must be given in a date format and that
#age must be multiplied by 365.241 in order to be expressed in days.
fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,tau=3652.41)
fit
plot(fit,col=c(1,1,2,2),xscale=365.241,xlab="Time (years)")
#if no strata are desired:
fit <- cmp.rel(Surv(time,cens)~1,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,tau=3652.41)



}
\references{
Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric
Relative Survival Analysis with the R Package relsurv". Journal of
Statistical Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08"
}
\seealso{
\code{rs.surv}, \code{summary.cmp.rel}
}
\keyword{survival}

\name{survfit.rsadd}
\alias{survfit.rsadd}



\title{Compute a Predicited Survival Curve}

\description{
Computes a predicted survival curve based on the additive model estimated by rsadd function.
}
\usage{
\method{survfit}{rsadd}(formula, newdata, se.fit = TRUE, conf.int = 0.95, individual = FALSE, 
    conf.type = c("log", "log-log", "plain", "none"),...)
}
\arguments{
\item{formula}{a rsadd object}
\item{newdata}{a data frame with the same variable names as those that appear in the rsadd formula. The curve(s) produced will be representative of a cohort who's covariates correspond to the values in newdata.}
\item{se.fit}{a logical value indicating whether standard errors should be computed. Default is \code{TRUE}.}
\item{conf.int}{the level for a two-sided confidence interval on the survival curve(s). Default is 0.95.}
\item{individual}{a logical value indicating whether the data frame represents different time epochs for only one individual (T), or whether multiple rows indicate multiple individuals (F, the default). If the former only one curve will be produced; if the latter there will be one curve per row in newdata.}
\item{conf.type}{One of \code{none}, \code{plain}, \code{log} (the default), or \code{log-log}. The first option causes confidence intervals not to be generated. The second causes the standard intervals curve +- k *se(curve), where k is determined from conf.int. The log option calculates intervals based on the cumulative hazard or log(survival). The last option bases intervals on the log hazard or log(-log(survival)).  }
\item{...}{Currently not implemented}
}

\details{
When predicting the survival curve, the ratetable values for future years will be equal to those of the last given year. The same ratetables will be used for fitting and predicting. To predict a relative survival curve, use \code{rs.surv.rsadd}.
}

\value{
 a \code{survfit} object; see the help on \code{survfit.object} for details.
 The \code{survfit} methods are used for \code{print},
     \code{plot}, \code{lines}, and \code{points}.

}

\references{
Package: Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine,\bold{81}: 272--278.  

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741--1749.  
}




\examples{
data(slopop)
data(rdata)
#BTW: work on a smaller dataset here to run the example faster
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata[1:500,],method="EM")
survfit.rsadd(fit,newdata=data.frame(sex=1,age=60,year=17000))

}

\seealso{
\code{survfit},
\code{survexp}, \code{\link{rs.surv}}
}

\keyword{survival}

\name{rs.diff}
\alias{rs.diff}
\alias{print.rsdiff}




\title{Test Net Survival Curve Differences}

\description{
     Tests if there is a difference between two or more net survival curves using a log-rank type test.
}
\usage{
     rs.diff(formula, data, ratetable = relsurv::slopop, na.action,precision=1,rmap) 
}
\arguments{
\item{formula}{
A formula expression as for other survival models, of the form \code{Surv(time, status) ~ predictors}.
Each combination of predictor values defines a subgroup. A \code{strata} term may be used to produce a stratified test. 

 NOTE: The follow-up time must be in days. 
}
\item{data}{
a data.frame in which to interpret the variables named in the
          \code{formula}. 

}
\item{ratetable}{
a table of event rates, organized as a \code{ratetable} object,
          such as \code{slopop}.  
}
\item{na.action}{a missing-data filter function, applied to the model.frame,
          after any subset argument has been used.  Default is
          \code{options()$na.action}.} 


\item{precision}{Precision for numerical integration. Default is 1, which means that daily intervals are taken, the value may be decreased to get a higher precision or increased to achieve a faster calculation. The calculation intervals always include at least all times of event and censoring as border points.
}
\item{rmap}{an optional list to be used if the variables are not
          organized and named in the same way as in the \code{ratetable} object. See details below.}

}

\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable} being used may have different variable names and formats than the user's data set, this is dealt with by the \code{rmap} argument. For example, if age is in years in the data set but in days in the \code{ratetable} object, age=age*365.241 should be used. The calendar year can be in any date format (date, Date and POSIXt are allowed), the date formats in the \code{ratetable} and in the data may differ. 
}

\value{
 a \code{rsdiff} object; can be printed with \code{print}.

}

\references{
Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric Relative Survival Analysis with the {R} Package {relsurv}". Journal of Statistical Software. 87(8), 1-27,  doi: "10.18637/jss.v087.i08" 
Theory: Graffeo, N., Castell, F., Belot, A. and Giorgi, R. (2016) "A log-rank-type test to compare net survival distributions. Biometrics. doi: 10.1111/biom.12477"
Theory: Pavlic, K., Pohar Perme, M. (2017) "On comparison of net survival curves. BMC Med Res Meth. doi: 10.1186/s12874-017-0351-3"
}




\examples{
data(slopop)
data(rdata)
#calculate the relative survival curve
#note that the variable year is given in days since 01.01.1960 and that 
#age must be multiplied by 365.241 in order to be expressed in days.
rs.diff(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata)
}


\seealso{
\code{rs.surv},
\code{survdiff}
}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{plot.rs.zph}
\alias{plot.rs.zph}
\title{Graphical Inspection of Proportional Hazards Assumption in Relative Survival
Models}
\usage{
\method{plot}{rs.zph}(
  x,
  resid = TRUE,
  df = 4,
  nsmo = 40,
  var,
  cex = 1,
  add = FALSE,
  col = 1,
  lty = 1,
  xlab,
  ylab,
  xscale = 1,
  ...
)
}
\arguments{
\item{x}{result of the \code{rs.zph} function.}

\item{resid}{a logical value, if \code{TRUE} the residuals are included on
the plot, as well as the smooth fit.}

\item{df}{the degrees of freedom for the fitted natural spline, \code{df=2}
leads to a linear fit.}

\item{nsmo}{number of points used to plot the fitted spline.}

\item{var}{the set of variables for which plots are desired.  By default,
plots are produced in turn for each variable of a model.  Selection of a
single variable allows other features to be added to the plot, e.g., a
horizontal line at zero or a main title.}

\item{cex}{a numerical value giving the amount by which plotting text and
symbols should be scaled relative to the default.}

\item{add}{logical, if \code{TRUE} the plot is added to an existing plot}

\item{col}{a specification for the default plotting color.}

\item{lty}{the line type.}

\item{xlab}{x axis label.}

\item{ylab}{y axis label.}

\item{xscale}{units for x axis, default is 1, i.e. days.}

\item{...}{Additional arguments passed to the \code{plot} function.}
}
\description{
Displays a graph of the scaled partial residuals, along with a smooth curve.
}
\examples{

data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241),
             ratetable=slopop,data=rdata,int=5)
rszph <- rs.zph(fit)
plot(rszph)

}
\references{
Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005)
"Goodness of fit of relative survival models." Statistics in Medicine,
\bold{24}: 3911-3925.

Package: Pohar M., Stare J. (2006) "Relative survival analysis in R."
Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741-1749, 2007.
}
\seealso{
\code{\link{rs.zph}}, \code{\link[survival:plot.cox.zph]{survival::plot.cox.zph}}
}
\keyword{survival}

\name{rdata}
\alias{rdata}
\docType{data}
\title{Survival Data}
\description{
  Survival data.
}
\usage{data(rdata)}
\format{
  A data frame with 1040 observations on the following 6 variables:
 \describe{
    \item{time}{survival time (in days).}
    \item{cens}{censoring indicator (0=censoring, 1=death).}
    \item{age}{age (in years).}
    \item{sex}{sex (1=male, 2=female).}
    \item{year}{date of diagnosis (in date format).}
    \item{agegr}{age group.}
 }
}

\references{
Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.
}


\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{survsplit}
\alias{survsplit}
\title{Split a Survival Data Set at Specified Times}
\usage{
survsplit(
  data,
  cut,
  end,
  event,
  start,
  id = NULL,
  zero = 0,
  episode = NULL,
  interval = NULL
)
}
\arguments{
\item{data}{data frame.}

\item{cut}{vector of timepoints to cut at.}

\item{end}{character string with name of event time variable.}

\item{event}{character string with name of censoring indicator.}

\item{start}{character string with name of start variable (will be created
if it does not exist).}

\item{id}{character string with name of new id variable to create
(optional).}

\item{zero}{If \code{start} doesn't already exist, this is the time that the
original records start. May be a vector or single value.}

\item{episode}{character string with name of new episode variable
(optional).}

\item{interval}{this argument is used by \code{max.lik} function}
}
\value{
New, longer, data frame.
}
\description{
Given a survival data set and a set of specified cut times, the function
splits each record into multiple records at each cut time. The new data set
is be in \code{counting process} format, with a start time, stop time, and
event status for each record. More general than \code{survSplit} as it also
works with the data already in the \code{counting process} format.
}
\seealso{
\code{\link[survival:survSplit]{survival::survSplit}}.
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/popsurv.R
\name{popsurv}
\alias{popsurv}
\title{Calculate the expected (population) survival}
\usage{
popsurv(sex, year = as.Date("1970-01-01"), age = 0, ratetable, times)
}
\arguments{
\item{sex}{Either character ('male'/'female'), or integer (1/2).}

\item{year}{The year from which the individual is followed. Either a Date or POSIXt object. Default is as.Date('1970-01-01').}

\item{age}{The age from which the individual is followed. Must be in days.}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{times}{The times at which the expected (population) survival should be calculated. Must be in days.}
}
\value{
A vector containing the survival estimate at the supplied times.
}
\description{
For a given individual with sex, year, and age, calculate the expected (population) survival at the supplied time points based on the mortality tables.
}
\details{
The follow-up time and age must be specified in days. The calendar year can be in any date format
(Date and POSIXt are allowed)
}
\examples{

library(relsurv)
# Estimate P(T>2000 days) for a newborn:
popsurv(sex='male', year=as.Date('1970-01-01'), age=0, ratetable=slopop, times=2000)
# P(T>300 days) for a 50-year old:
popsurv(sex='male', year=as.Date('1970-01-01'), age=50*365.241, ratetable=slopop, times=300)
}
\seealso{
\code{\link{expprep2}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{rstrans}
\alias{rstrans}
\title{Fit Cox Proportional Hazards Model in Transformed Time}
\usage{
rstrans(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  int,
  na.action,
  init,
  control,
  rmap,
  ...
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, such as \code{slopop}.}

\item{int}{the number of follow-up years used for calculating survival(the
rest is censored). If missing, it is set the the maximum observed follow-up
time.}

\item{na.action}{a missing-data filter function, applied to the model.frame,
after any subset argument has been used.  Default is
\code{options()$na.action}.}

\item{init}{vector of initial values of the iteration.  Default initial
value is zero for all variables.}

\item{control}{a list of parameters for controlling the fitting process.
See the documentation for \code{coxph.control} for details.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}

\item{...}{other arguments will be passed to \code{coxph.control}.}
}
\value{
an object of class \code{coxph}. See \code{coxph.object} and
\code{coxph.detail} for details.  \item{y}{ an object of class \code{Surv}
containing the transformed times (these times do not depend on covariates).
}
}
\description{
The function transforms each person's time to his/her probability of dying
at that time according to the ratetable. It then fits the Cox proportional
hazards model with the transformed times as a response. It can also be used
for calculatin the transformed times (no covariates are needed in the
formula for that purpose).
}
\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with by the \code{rmap} argument. For example, if
age is in years in the data set but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.  A side product of this
function are the transformed times - stored in teh \code{y} object of the
output. To get these times, covariates are of course irrelevant.
}
\examples{

data(slopop)
data(rdata)

#fit a Cox model using the transformed times
#note that the variable year is given in days since 01.01.1960 and that
#age must be multiplied by 365.241 in order to be expressed in days.
fit <- rstrans(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241,
        sex=sex,year=year),ratetable=slopop,data=rdata)


#check the goodness of fit
rs.br(fit)

}
\references{
Method: Stare J., Henderson R., Pohar M. (2005) "An individual
measure for relative survival." Journal of the Royal Statistical Society:
Series C, \bold{54} 115--126.

Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link{rsmul}}, \code{\link{invtime}}, \code{\link{rsadd}},
\code{\link[survival:survexp]{survival::survexp}}.
}
\keyword{survival}

\name{rs.br}
\alias{rs.br}
\alias{plot.rs.br}
\alias{print.rs.br}


\title{Test the Proportional Hazards Assumption for Relative Survival
 Regression Models}
\description{
Test the proportional hazards assumption for relative survival  models  (\code{rsadd}, \code{rsmul} or \code{rstrans})
by forming a Brownian Bridge. }
\usage{
rs.br(fit,sc,rho=0,test="max",global=TRUE)
}

\arguments{
\item{fit}{
the result of fitting a relative survival model, using the
          \code{rsadd}, \code{rsmul} or \code{rstrans} function. 
}
\item{sc}{
partial residuals calculated by the \code{resid} function. This is used to save time if several tests
are to be calculated on these residuals and can otherwise be omitted.
}
\item{rho}{
a number controlling the weigths of residuals. The weights are the number of individuals at
risk at each event time to the power \code{rho}. The default is \code{rho=0}, which sets all
weigths to 1.
}
\item{test}{ 
a character string specifying the test to be performed on Brownian bridge.
Possible values are \code{"max"} (default), which tests the maximum absolute value of the 
bridge, and \code{cvm}, which calculates the Cramer Von Mises statistic.
}
\item{global}{should a global Brownian bridge test be performed, in addition to the
          per-variable tests}

}

\value{
 an object of class \code{rs.br}. This function would usually be followed by both a
print and a plot of the result. The plot gives a Brownian bridge for each of the variables.
The horizontal lines are the 95% and 99% confidence intervals for the maximum absolute value
of the Brownian bridge}




\references{
Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005) "Goodness of fit of relative survival models." Statistics in Medicine, \bold{24}: 3911--3925.

Package. Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741--1749.

}



\examples{
data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,int=5)
rsbr <- rs.br(fit)
rsbr
plot(rsbr)
}

\seealso{\code{\link{rsadd}}, \code{rsmul}, \code{rstrans}, \code{\link{resid}}.}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cmprel.r
\name{plot.cmp.rel}
\alias{plot.cmp.rel}
\title{Plot the crude probability of death}
\usage{
\method{plot}{cmp.rel}(
  x,
  main = " ",
  curvlab,
  ylim = c(0, 1),
  xlim,
  wh = 2,
  xlab = "Time (days)",
  ylab = "Probability",
  lty = 1:length(x),
  xscale = 1,
  col = 1,
  lwd = par("lwd"),
  curves,
  conf.int,
  all.times = FALSE,
  ...
)
}
\arguments{
\item{x}{a list, with each component representing one curve in the plot,
output of the function \code{cmp.rel}.}

\item{main}{the main title for the plot.}

\item{curvlab}{Curve labels for the plot. Default is \code{names(x)}, or if
that is missing, \code{1:nc}, where \code{nc} is the number of curves in
\code{x}.}

\item{ylim}{yaxis limits for plot.}

\item{xlim}{xaxis limits for plot (default is 0 to the largest time in any
of the curves).}

\item{wh}{if a vector of length 2, then the upper right coordinates of the
legend; otherwise the legend is placed in the upper right corner of the
plot.}

\item{xlab}{X axis label.}

\item{ylab}{y axis label.}

\item{lty}{vector of line types. Default \code{1:nc} (\code{nc} is the
number of curves in \code{x}). For color displays, \code{lty=1},
\code{color=1:nc}, might be more appropriate. If \code{length(lty)<nc}, then
\code{lty[1]} is used for all.}

\item{xscale}{Scale of the X axis. Default is in days (1).}

\item{col}{vector of colors. If \code{length(col)<nc}, then the
\code{col[1]} is used for all.}

\item{lwd}{vector of line widths. If \code{length(lwd)<nc}, then
\code{lwd[1]} is used for all.}

\item{curves}{Vector if integers, specifies which curves should be plotted.
May take values \code{1:nc}, where \code{nc} is the number of curves in
\code{x}. By default, all of the curves are plotted.}

\item{conf.int}{Vector if integers, specifies which confidence intervals
should be plotted. May take values \code{1:nc}, where \code{nc} is the
number of curves in \code{x}. By default, no confidence intervals are
plotted.}

\item{all.times}{By default, the disease specific mortality estimate is
plotted as a step function between event or censoring times. If set to
\code{TRUE}, the graph is evaluated at all estimated times.}

\item{...}{additional arguments passed to the initial call of the plot
function.}
}
\value{
No value is returned.
}
\description{
Plot method for cmp.rel. Plots the cumulative probability of death due to
disease and due to population reasons
}
\details{
By default, the graph is plotted as a step function for the cause specific
mortality and as a piecewise linear function for the population mortality.
It is evaluated at all event and censoring times even though it constantly
changes also between these time points.

If the argument \code{all.times} is set to \code{TRUE}, the plot is
evaluated at all times that were used for numerical integration in the
\code{cmp.rel} function (there, the default is set to daily intervals). If
only specific time points are to be added, this should be done via argument
\code{add.times} in \code{cmp.rel}.
}
\examples{

data(slopop)
data(rdata)
fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,tau=3652.41)
plot(fit,col=c(1,1,2,2),xscale=365.241,conf.int=c(1,3))

}
\seealso{
\code{rs.surv}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsdiff.r
\name{rs.diff}
\alias{rs.diff}
\alias{print.rsdiff}
\title{Test Net Survival Curve Differences}
\usage{
rs.diff(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  na.action,
  precision = 1,
  rmap
)
}
\arguments{
\item{formula}{A formula expression as for other survival models, of the
form \code{Surv(time, status) ~ predictors}. Each combination of predictor
values defines a subgroup. A \code{strata} term may be used to produce a
stratified test.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{na.action}{a missing-data filter function, applied to the model.frame,
after any subset argument has been used.  Default is
\code{options()$na.action}.}

\item{precision}{Precision for numerical integration. Default is 1, which
means that daily intervals are taken, the value may be decreased to get a
higher precision or increased to achieve a faster calculation. The
calculation intervals always include at least all times of event and
censoring as border points.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}
}
\value{
a \code{rsdiff} object; can be printed with \code{print}.
}
\description{
Tests if there is a difference between two or more net survival curves using
a log-rank type test.
}
\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with by the \code{rmap} argument. For example, if
age is in years in the data set but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.
}
\examples{

data(slopop)
data(rdata)
#calculate the relative survival curve
#note that the variable year is given in days since 01.01.1960 and that
#age must be multiplied by 365.241 in order to be expressed in days.
rs.diff(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata)

}
\references{
Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric
Relative Survival Analysis with the R Package relsurv". Journal of
Statistical Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08" Theory:
Graffeo, N., Castell, F., Belot, A. and Giorgi, R. (2016) "A log-rank-type
test to compare net survival distributions. Biometrics. doi:
10.1111/biom.12477" Theory: Pavlic, K., Pohar Perme, M. (2017) "On
comparison of net survival curves. BMC Med Res Meth. doi:
10.1186/s12874-017-0351-3"
}
\seealso{
\code{rs.surv}, \code{survdiff}
}
\keyword{survival}

\name{residuals.rsadd}
\alias{residuals.rsadd}



\title{Calculate Residuals for a "rsadd" Fit}
\description{
Calculates partial residuals for an additive relative survival model.
}
\usage{
      \method{residuals}{rsadd}(object,type="schoenfeld",...)
}

\arguments{
\item{object}{
an object inheriting from class \code{rsadd}, representing a
          fitted additive relative survival model. Typically this is the output
          from the \code{rsadd} function. 

}
\item{type}{ character string indicating the type of residual desired. 
Currently only Schoenfeld residuals are implemented. }
\item{...}{other arguments.}
}

\value{
A list of the following values is returned:
\item{res}{a matrix containing the residuals for each variable.}
\item{varr}{the variance for each residual}
\item{varr1}{the sum of \code{varr}.}
\item{kvarr}{the derivative of each residual, to be used in \code{rs.zph} function.}
\item{kvarr1}{the sum of \code{kvarr}.}
}

\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741--1749.

Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005) "Goodness of fit of relative survival models." Statistics in Medicine, \bold{24}: 3911--3925.
}


\examples{
data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
       ratetable=slopop,data=rdata,int=5)
sresid <- residuals.rsadd(fit)
}

\seealso{\code{\link{rsadd}}.}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{rsmul}
\alias{rsmul}
\title{Fit Andersen et al Multiplicative Regression Model for Relative Survival}
\usage{
rsmul(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  int,
  na.action,
  init,
  method = "mul",
  control,
  rmap,
  ...
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, such as \code{slopop}.}

\item{int}{the number of follow-up years used for calculating survival(the
data are censored after this time-point). If missing, it is set the the
maximum observed follow-up time.}

\item{na.action}{a missing-data filter function, applied to the model.frame,
after any subset argument has been used.  Default is
\code{options()$na.action}.}

\item{init}{vector of initial values of the iteration.  Default initial
value is zero for all variables.}

\item{method}{the default method \code{mul} assumes hazard to be constant on
yearly intervals. Method \code{mul1} uses the ratetable to determine the
time points when hazard changes. The \code{mul1} method is therefore more
accurate, but at the same time can be more computationally intensive.}

\item{control}{a list of parameters for controlling the fitting process.
See the documentation for \code{coxph.control} for details.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}

\item{...}{Other arguments will be passed to \code{coxph.control}.}
}
\value{
an object of class \code{coxph} with an additional item:
\item{basehaz}{Cumulative baseline hazard (population values are seen as
offset) at centered values of covariates.}
}
\description{
Fits the Andersen et al multiplicative regression model in relative
survival. An extension of the coxph function using relative survival.
}
\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with by the \code{rmap} argument. For example, if
age is in years in the data set but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.
}
\examples{

data(slopop)
data(rdata)
#fit a multiplicative model
#note that the variable year is given in days since 01.01.1960 and that
#age must be multiplied by 365.241 in order to be expressed in days.
fit <- rsmul(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241),
            ratetable=slopop,data=rdata)


#check the goodness of fit
rs.br(fit)


}
\references{
Method: Andersen, P.K., Borch-Johnsen, K., Deckert, T., Green,
A., Hougaard, P., Keiding, N. and Kreiner, S. (1985) "A Cox regression model
for relative mortality and its application to diabetes mellitus survival
data.", Biometrics, \bold{41}: 921--932.

Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link{rsadd}}, \code{\link{rstrans}}.
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survfitrsadd.r
\name{survfit.rsadd}
\alias{survfit.rsadd}
\title{Compute a Predicited Survival Curve}
\usage{
\method{survfit}{rsadd}(
  formula,
  newdata,
  se.fit = TRUE,
  conf.int = 0.95,
  individual = FALSE,
  conf.type = c("log", "log-log", "plain", "none"),
  ...
)
}
\arguments{
\item{formula}{a rsadd object}

\item{newdata}{a data frame with the same variable names as those that
appear in the rsadd formula. The curve(s) produced will be representative of
a cohort who's covariates correspond to the values in newdata.}

\item{se.fit}{a logical value indicating whether standard errors should be
computed. Default is \code{TRUE}.}

\item{conf.int}{the level for a two-sided confidence interval on the
survival curve(s). Default is 0.95.}

\item{individual}{a logical value indicating whether the data frame
represents different time epochs for only one individual (T), or whether
multiple rows indicate multiple individuals (F, the default). If the former
only one curve will be produced; if the latter there will be one curve per
row in newdata.}

\item{conf.type}{One of \code{none}, \code{plain}, \code{log} (the default),
or \code{log-log}. The first option causes confidence intervals not to be
generated. The second causes the standard intervals curve +- k *se(curve),
where k is determined from conf.int. The log option calculates intervals
based on the cumulative hazard or log(survival). The last option bases
intervals on the log hazard or log(-log(survival)).}

\item{...}{Currently not implemented}
}
\value{
a \code{survfit} object; see the help on \code{survfit.object} for
details.  The \code{survfit} methods are used for \code{print}, \code{plot},
\code{lines}, and \code{points}.
}
\description{
Computes a predicted survival curve based on the additive model estimated by
rsadd function.
}
\details{
When predicting the survival curve, the ratetable values for future years
will be equal to those of the last given year. The same ratetables will be
used for fitting and predicting. To predict a relative survival curve, use
\code{rs.surv.rsadd}.
}
\examples{

data(slopop)
data(rdata)
#BTW: work on a smaller dataset here to run the example faster
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata[1:500,],method="EM")
survfit.rsadd(fit,newdata=data.frame(sex=1,age=60,year=17000))


}
\references{
Package: Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine,\bold{81}: 272--278.

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{survfit}, \code{survexp}, \code{\link{rs.surv}}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{rsadd}
\alias{rsadd}
\title{Fit an Additive model for Relative Survival}
\usage{
rsadd(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  int,
  na.action,
  method = "max.lik",
  init,
  bwin,
  centered = FALSE,
  cause,
  control,
  rmap,
  ...
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right. \code{Surv(start,stop,event)} outcomes
are also possible for time-dependent covariates and left-truncation for
\code{method='EM'}.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{int}{either a single value denoting the number of follow-up years or a
vector specifying the intervals (in years) in which the hazard is constant
(the times that are bigger than \code{max(int)} are censored. If missing,
only one interval (from time 0 to maximum observation time) is assumed.  The
EM method does not need the intervals, only the maximum time can be
specified (all times are censored after this time point).}

\item{na.action}{a missing-data filter function, applied to the model.frame,
after any subset argument has been used.  Default is
\code{options()$na.action}.}

\item{method}{\code{glm.bin} or \code{glm.poi} for a glm model, \code{EM}
for the EM algorithm and \code{max.lik} for the maximum likelihood model
(default).}

\item{init}{vector of initial values of the iteration.  Default initial
value is zero for all variables.}

\item{bwin}{controls the bandwidth used for smoothing in the EM algorithm.
The follow-up time is divided into quartiles and \code{bwin} specifies a
factor by which the maximum between events time length on each interval is
multiplied. The default \code{bwin=-1} lets the function find an appropriate
value. If \code{bwin=0}, no smoothing is applied.}

\item{centered}{if \code{TRUE}, all the variables are centered before
fitting and the baseline excess hazard is calculated accordingly. Default is
\code{FALSE}.}

\item{cause}{A vector of the same length as the number of cases. \code{0}
for population deaths, \code{1} for disease specific deaths, \code{2}
(default) for unknown. Can only be used with the \code{EM} method.}

\item{control}{a list of parameters for controlling the fitting process.
See the documentation for \code{glm.control} for details.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}

\item{...}{other arguments will be passed to \code{glm.control}.}
}
\value{
An object of class \code{rsadd}. In the case of
\code{method="glm.bin"} and \code{method="glm.poi"} the class also inherits
from \code{glm} which inherits from the class \code{lm}. Objects of this
class have methods for the functions \code{print} and \code{summary}. An
object of class \code{rsadd} is a list containing at least the following
components: \item{data}{the data as used in the model, along with the
variables defined in the rate table} \item{ratetable}{the ratetable used.}
\item{int}{the maximum time (in years) used. All the events at and after
this value are censored.} \item{method}{the fitting method that was used.}
\item{linear.predictors}{the vector of linear predictors, one per subject.}
}
\description{
The function fits an additive model to the data. The methods implemented are
the maximum likelihood method, the semiparametric method, a glm model with a
\code{binomial} error and a glm model with a \code{poisson} error.
}
\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with by the \code{rmap} argument. For example, if
age is in years in the data set but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.

The maximum likelihood method and both glm methods assume a fully parametric
model with a piecewise constant baseline excess hazard function. The
intervals on which the baseline is assumed constant should be passed via
argument \code{int}. The EM method is semiparametric, i.e. no assumptions
are made for the baseline hazard and therefore no intervals need to be
specified.

The methods using glm are methods for grouped data. The groups are formed
according to the covariate values. This should be taken into account when
fitting a model. The glm method returns life tables for groups specified by
the covariates in \code{groups}.

The EM method output includes the smoothed baseline excess hazard
\code{lambda0}, the cumulative baseline excess hazard \code{Lambda0} and
\code{times} at which they are estimated. The individual probabilites of
dying due to the excess risk are returned as \code{Nie}.  The EM method
fitting procedure requires some local smoothing of the baseline excess
hazard. The default \code{bwin=-1} value lets the function find an
appropriate value for the smoothing band width. While this ensures an
unbiased estimate, the procedure time is much longer. As the value found by
the function is independent of the covariates in the model, the value can be
read from the output (\code{bwinfac}) and used for refitting different
models to the same data to save time.
}
\examples{

data(slopop)
data(rdata)
#fit an additive model
#note that the variable year is given in days since 01.01.1960 and that
#age must be multiplied by 365.241 in order to be expressed in days.
fit <- rsadd(Surv(time,cens)~sex+as.factor(agegr)+ratetable(age=age*365.241),
	    ratetable=slopop,data=rdata,int=5)

#check the goodness of fit
rs.br(fit)

#use the EM method and plot the smoothed baseline excess hazard
fit <- rsadd(Surv(time,cens)~sex+age,rmap=list(age=age*365.241),
	 ratetable=slopop,data=rdata,int=5,method="EM")
sm <- epa(fit)
plot(sm$times,sm$lambda,type="l")

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.

EM algorithm: Pohar Perme M., Henderson R., Stare, J. (2009) "An approach to
estimation in relative survival regression." Biostatistics, \bold{10}:
136--146.
}
\seealso{
\code{\link{rstrans}}, \code{\link{rsmul}}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/years.R
\name{years}
\alias{years}
\title{Compute one of the life years measures}
\usage{
years(
  formula = formula(data),
  data,
  measure = c("yd", "yl2017", "yl2013"),
  ratetable = relsurv::slopop,
  rmap,
  var.estimator = c("none", "bootstrap", "greenwood"),
  B = 100,
  precision = 30,
  add.times,
  na.action = stats::na.omit,
  conf.int = 0.95,
  timefix = FALSE,
  is.boot = FALSE,
  first.boot = FALSE
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, \code{~1} specified on the right.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{measure}{choose which measure is used: 'yd' (life years difference; Manevski, Ruzic Gorenjec, Andersen, Pohar Perme, 2022), 'yl2017' (years lost/saved; Andersen 2017),
'yl2013' (years lost/saved; Andersen 2013).}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}

\item{var.estimator}{Choose the estimator for the variance ('none', 'bootstrap', 'greenwood'). Default is 'none'.
The 'greenwood' option is possible only for \code{measure='yd'}.}

\item{B}{if \code{var.estimator} is 'bootstrap'. The number of bootstrap replications. Default is 100.}

\item{precision}{precision for numerical integration of the population curve. Default is 30 (days).
The value may be decreased to get a
higher precision or increased to achieve a faster calculation.}

\item{add.times}{specific times at which the curves should be reported.}

\item{na.action}{a missing-data filter function. Default is \code{na.omit}.}

\item{conf.int}{the confidence level for a two-sided confidence interval. Default is 0.95.}

\item{timefix}{the timefix argument in survival::survfit.formula. Default is FALSE.}

\item{is.boot}{if TRUE, the function \code{years} has been called during a bootstrap replication.}

\item{first.boot}{if TRUE, this is the first bootstrap replication.}
}
\value{
A list containing the years measure, the observed and population curves (or the excess curve for Andersen 2013).
The values are given as separate data.frames through time. Times are given in days, all areas are given in years.
For \code{measure='yl2017'} values are reported only at the last time point.
Functions \code{plot_f} and \code{plot_years} can be then used for plotting.
}
\description{
Provides an estimate for one of the following measures: years lost (Andersen, 2013), years lost/saved (Andersen, 2017), or
life years difference (Manevski, Ruzic Gorenjec, Andersen, Pohar Perme, 2022).
}
\details{
The life years difference (\code{measure='yd'}) is taken by default. If other
measures are of interest, use the \code{measure} argument.

The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with the \code{rmap} argument. For example, if
age is in years in the data but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.

Numerical integration is performed, argument
precision is set with argument \code{precision}, which defaults to 30-day
intervals for intergration. For higher accuracy take a smaller value (e.g. precision=1 makes
the integration on a daily basis).

The observed curves are reported at event and censoring times. The
population curves are reported at all times used for the numerical integration.
Note that for the years lost (Andersen, 2013) measure, only the excess absolute risk is reported.
}
\examples{

library(relsurv)
# Estimate the life years difference for the rdata dataset.
mod <- years(Surv(time, cens)~1, data=rdata, measure='yd', ratetable=slopop,
             rmap=list(age=age*365.241), var.estimator = 'none')
# Plot the absolute risk (observed and population curve):
plot_f(mod)
# Plot the life years difference estimate:
plot_years(mod, conf.int=FALSE)
}
\seealso{
\code{\link{plot_f}}, \code{\link{plot_years}}
}

\name{cmp.rel}
\alias{cmp.rel}
\alias{print.cmp.rel}



\title{Compute crude probability of death}

\description{
     Estimates the crude probability of death due to disease and due to population reasons
}
\usage{
     cmp.rel(formula, data, ratetable = relsurv::slopop, na.action,tau,
	     conf.int=0.95,precision=1,add.times,rmap) 
}
\arguments{
\item{formula}{
a formula object, with the response as a \code{Surv} object on the
          left of a \code{~} operator, and, if desired, terms separated  by
          the \code{+} operator on the right. If no strata are used, \code{~1} should be specified.

 NOTE: The follow-up time must be in days.
}
\item{data}{
a data.frame in which to interpret the variables named in the
          \code{formula}. 

}
\item{ratetable}{
a table of event rates, organized as a \code{ratetable} object,
          such as \code{slopop}.  
}
\item{na.action}{a missing-data filter function, applied to the model.frame,
          after any subset argument has been used.  Default is
          \code{options()$na.action}.} 


\item{tau}{the maximum follow-up time of interest, all times larger than \code{tau} shall be censored. Equals maximum observed time by default 
}
\item{conf.int}{the level for a two-sided confidence interval on the survival curve(s). Default is 0.95.}
\item{precision}{the level of precision used in the numerical integration of variance. Default is 1, which means that daily intervals are taken, the value may be decreased to get a higher precision or increased to achieve a faster calculation. The calculation intervals always include at least all times of event and censoring as border points.}
\item{add.times}{specific times at which the value of estimator and its variance should be evaluated. Default is all the event and censoring times.}
\item{rmap}{an optional list to be used if the variables are not
          organized and named in the same way as in the \code{ratetable} object. See details below.}
}

\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable} being used may have different variable names and formats than the user's data set, this is dealt with by the \code{rmap} argument. For example, if age is in years in the data set but in days in the \code{ratetable} object, age=age*365.241 should be used. The calendar year can be in any date format (date, Date and POSIXt are allowed), the date formats in the \code{ratetable} and in the data may differ. 

Note that numerical integration is required to calculate the variance estimator. The integration precision is set with argument \code{precision}, which defaults to daily intervals, a default that should give enough precision for any practical purpose.

The area under the curve is calculated on the interval [0,\code{tau}]. 

Function \code{summary} may be used to get the output at specific points in time.

}

\value{
An object of class \code{cmp.rel}.
Objects of this class have methods for the functions \code{print} and \code{plot}. The \code{summary} function can be used for printing output at required time points.
An object of class \code{cmp.rel} is composed of several lists, each pertaining the cumulative hazard function for one risk and one strata. Each of the lists contains the following objects:
\item{time}{the time-points at which the curves are estimated}
\item{est}{the estimate}
\item{var}{the variance of the estimate}
\item{lower}{the lower limit of the confidence interval}
\item{upper}{the upper limit of the confidence interval}
\item{area}{the area under the curve calculated on the interval [0,\code{tau}]}
\item{index}{indicator of event and censoring times among all the times in the output. The times added via paramater \code{add.times} are also included}
\item{add.times}{the times added via parameter \code{add.times}}
}


\references{
Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric Relative Survival Analysis with the {R} Package {relsurv}". Journal of Statistical Software. 87(8), 1-27,  doi: "10.18637/jss.v087.i08" 
}


\examples{

data(slopop)
data(rdata)
#calculate the crude probability of death
#note that the variable year must be given in a date format and that 
#age must be multiplied by 365.241 in order to be expressed in days.
fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,tau=3652.41)
fit
plot(fit,col=c(1,1,2,2),xscale=365.241,xlab="Time (years)")
#if no strata are desired:
fit <- cmp.rel(Surv(time,cens)~1,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,tau=3652.41)


}
\seealso{
\code{rs.surv}, \code{summary.cmp.rel}
}

\keyword{survival}

\name{colrec}
\alias{colrec}
\docType{data}
\title{Relative Survival Data}
\description{
  Survival of patients with colon and rectal cancer diagnosed in 1994-2000. 
}
\usage{data(colrec)}
\format{
  A data frame with 5971 observations on the following 7 variables:
 \describe{
\item{sex}{sex (1=male, 2=female).}    
\item{age}{age (in days).}
    \item{diag}{date of diagnosis (in date format).}
\item{time}{survival time (in days).}
    \item{stat}{censoring indicator (0=censoring, 1=death).}
    \item{stage}{cancer stage. Values 1-3, code \code{99} stands for unknown.}
\item{site}{cancer site. }
 }
}

\references{
Provided by Slovene Cancer Registry. The \code{age}, \code{time} and \code{diag} variables are randomly perturbed to make the identification of patients impossible.
}


\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsaalen.R
\name{rsaalen}
\alias{rsaalen}
\title{Fit an extended additive hazards model using relative survival}
\usage{
rsaalen(
  formula,
  data,
  variance = FALSE,
  var_estimator = "dN",
  ratetable = relsurv::slopop,
  rmap,
  split.transitions
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{variance}{a logical value indicating whether the variances of the hazards should be computed.
Default is FALSE.}

\item{var_estimator}{Choose variance estimator, in the same way as in survaalen. The default option 'dN' uses dN(t)-dLambda_P(t) in the variance estimator, equivalent to formula 4.63 in Aalen et al. (2008). Option 'XdB' uses X*dB(t), see formula 4.64 in Aalen et al. (2008).}

\item{ratetable}{a table of event rates, organized as a \code{ratetable} object, such as \code{slopop}.}

\item{rmap}{an optional list to be used if the variables are not organized and named in the same way as in the \code{ratetable} object.}

\item{split.transitions}{only relevant if a multi-state model is fitted. An integer vector containing the numbered transitions that should be split. Use same numbering as in the given transition matrix.}
}
\value{
An object of class \code{aalen.model}.
}
\description{
Fits the Aalen additive hazard model using relative survival. The function can be used for multi-state model
data (as in the package mstate; class msdata) by supplying the start and stop times in the
Surv object and adding a strata(trans) object in the formula (where trans denotes the
transition in the multi-state model).
}
\examples{

# Survival:
data(rdata)
mod <- rsaalen(Surv(time, cens)~sex+age, data=rdata, ratetable=slopop,
     rmap=list(age=age*365.241))
head(mod$coefficients)
tail(mod$coefficients)

# Multi-state model:
data(ebmt1wide)
# Generate sex and year data (for illustrative purposes since it is not given in the data):
ebmt1wide$sex <- sample(1:2, size = nrow(ebmt1wide), replace = TRUE)
ebmt1wide$year <- as.Date('2010-01-01')

mod <- rsaalen(Surv(Tstart, Tstop, status)~age.1+age.2+age.3+strata(trans), data=ebmt1wide,
                 ratetable = slopop, rmap = list(age=age*365.241), split.transitions = 2:3)
head(mod$coefficients$trans1)
head(mod$coefficients$trans2)
head(mod$coefficients$trans3)
}
\seealso{
\code{survaalen}
}
\author{
Damjan Manevski
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{transrate}
\alias{transrate}
\title{Reorganize Data into a Ratetable Object}
\usage{
transrate(men, women, yearlim, int.length = 1)
}
\arguments{
\item{men}{a matrix containing the yearly (conditional) probabilities of one
year survival for men. Rows represent age (increasing 1 year per
line,starting with 0), the columns represent cohort years (the limits are in
\code{yearlim}, the increase is in \code{int.length}.}

\item{women}{a matrix containing the yearly (conditional) probabilities of
one year survival for women.}

\item{yearlim}{the first and last cohort year given in the tables.}

\item{int.length}{the length of intervals in which cohort years are given.}
}
\value{
An object of class \code{ratetable}.
}
\description{
The function assists in reorganizing certain types of data into a ratetable
object.
}
\details{
This function only applies for ratetables that are organized by age, sex and
year.
}
\examples{

men <- cbind(exp(-365.241*exp(-14.5+.08*(0:100))),exp(-365*exp(-14.7+.085*(0:100))))
women <- cbind(exp(-365.241*exp(-15.5+.085*(0:100))),exp(-365*exp(-15.7+.09*(0:100))))
table <- transrate(men,women,yearlim=c(1980,1990),int.length=10)

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link[survival:ratetable]{survival::ratetable}}.
}
\keyword{survival}

\name{plot.cmp.rel}
\alias{plot.cmp.rel}


\title{Plot the crude probability of death}

\description{
     Plot method for cmp.rel. Plots the cumulative probability of death due to disease and due to population reasons
}
\usage{
	\method{plot}{cmp.rel}(x, main, curvlab, ylim=c(0, 1), xlim, wh=2, xlab="Time (days)",
	 ylab="Probability", lty=1:length(x), xscale=1,col=1, lwd=par('lwd'),
	 curves, conf.int, all.times=FALSE,...) 
}
\arguments{
\item{x}{a list, with each component representing one curve in the plot, output of the function \code{cmp.rel}.}
\item{main}{the main title for the plot.}
\item{curvlab}{Curve labels for the plot. Default is \code{names(x)}, or if that is missing, \code{1:nc}, where \code{nc} is the number of curves in \code{x}.}
\item{ylim}{yaxis limits for plot.}
\item{xlim}{xaxis limits for plot (default is 0 to the largest time in any of the curves).}
\item{wh}{if a vector of length 2, then the upper right coordinates of the legend; otherwise the legend is placed in the upper right corner of the plot.}
\item{xlab}{X axis label.}
\item{ylab}{y axis label.}
\item{lty}{vector of line types. Default \code{1:nc} (\code{nc} is the number of curves in \code{x}). For color displays, \code{lty=1}, \code{color=1:nc}, might be more appropriate. If \code{length(lty)<nc}, then \code{lty[1]} is used for all.}
\item{xscale}{Scale of the X axis. Default is in days (1).}
\item{col}{vector of colors. If \code{length(col)<nc}, then the \code{col[1]} is used for all.}
\item{lwd}{vector of line widths. If \code{length(lwd)<nc}, then \code{lwd[1]} is used for all.}
\item{curves}{Vector if integers, specifies which curves should be plotted. May take values \code{1:nc}, where \code{nc} is the number of curves in \code{x}. By default, all of the curves are plotted. }
\item{conf.int}{Vector if integers, specifies which confidence intervals should be plotted. May take values \code{1:nc}, where \code{nc} is the number of curves in \code{x}. By default, no confidence intervals are plotted. }
\item{all.times}{By default, the disease specific mortality estimate is plotted as a step function between event or censoring times. If set to \code{TRUE}, the graph is evaluated at all estimated times.}
\item{...}{additional arguments passed to the initial call of the plot function.}
}

\details{
By default, the graph is plotted as a step function for the cause specific mortality and as a piecewise linear function for the population mortality. It is evaluated at all event and censoring times even though it constantly changes also between these time points. 

If the argument \code{all.times} is set to \code{TRUE},  the plot is evaluated at all times that were used for numerical integration in the \code{cmp.rel} function (there, the default is set to daily intervals). If only specific time points are to be added, this should be done via argument \code{add.times} in \code{cmp.rel}.
}

\value{
No value is returned.
}


\examples{
data(slopop)
data(rdata)
fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,tau=3652.41)
plot(fit,col=c(1,1,2,2),xscale=365.241,conf.int=c(1,3))
}


\seealso{
\code{rs.surv}
}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.rsadd.R
\name{predict.rsadd}
\alias{predict.rsadd}
\title{Subject-specific prediction for an excess hazards model}
\usage{
\method{predict}{rsadd}(object, newdata, type = "Haz", add.times, ...)
}
\arguments{
\item{object}{An rsadd object}

\item{newdata}{A data.frame with one row (add covariate values in columns)}

\item{type}{By default ('Haz') returns cumulative hazards. If 'prob' it calculates CIFs. If 'both' a list is returned with Haz and prob.}

\item{add.times}{specific times at which predictions should be obtained.}

\item{...}{Not used for now}
}
\value{
A data.frame containing the times, excess and population components.
}
\description{
A function that returns subject-specific predictions from an rsadd object.
}
\examples{
data(rdata)
fit <- rsadd(Surv(time,cens)~sex+age,rmap=list(age=age*365.241),
             ratetable=slopop,data=rdata,int=5,method="EM")
predict(fit, data.frame(sex=2,age=68, year=as.Date('1982-06-24')))
}
\author{
Damjan Manevski \email{damjan.manevski@mf.uni-lj.si}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{residuals.rsadd}
\alias{residuals.rsadd}
\title{Calculate Residuals for a "rsadd" Fit}
\usage{
\method{residuals}{rsadd}(object, type = "schoenfeld", ...)
}
\arguments{
\item{object}{an object inheriting from class \code{rsadd}, representing a
fitted additive relative survival model. Typically this is the output from
the \code{rsadd} function.}

\item{type}{character string indicating the type of residual desired.
Currently only Schoenfeld residuals are implemented.}

\item{...}{other arguments.}
}
\value{
A list of the following values is returned: \item{res}{a matrix
containing the residuals for each variable.} \item{varr}{the variance for
each residual} \item{varr1}{the sum of \code{varr}.} \item{kvarr}{the
derivative of each residual, to be used in \code{rs.zph} function.}
\item{kvarr1}{the sum of \code{kvarr}.}
}
\description{
Calculates partial residuals for an additive relative survival model.
}
\examples{

data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
       ratetable=slopop,data=rdata,int=5)
sresid <- residuals.rsadd(fit)

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.

Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005) "Goodness of
fit of relative survival models." Statistics in Medicine, \bold{24}:
3911--3925.
}
\seealso{
\code{\link{rsadd}}.
}
\keyword{survival}

\name{rs.zph}
\alias{rs.zph}


\title{Behaviour of Covariates in Time for Relative Survival Regression Models}
\description{
Calculates the scaled partial residuals of a relative survival model (\code{rsadd}, \code{rsmul} or \code{rstrans})}
\usage{
rs.zph(fit,sc,transform="identity",var.type="sum")
}

\arguments{
\item{fit}{
the result of fitting an additive relative survival model, using the
          \code{rsadd}, \code{rsmul} or \code{rstrans}  function. 

	In the case of multiplicative and transformation models the output is 
	identical to \code{cox.zph} function, except no test is performed.
}
\item{sc}{
partial residuals calculated by the \code{resid} function. This is used to save time if several tests
are to be calculated on these residuals and can otherwise be omitted.
}
\item{transform}{
a character string specifying how the survival times should
be transformed. Possible values are \code{"km"}, \code{"rank"}, \code{"identity"} and \code{log}.
The default is \code{"identity"}.

}
\item{var.type}{ 
a character string specifying the variance used to scale the residuals. Possible values are
\code{"each"}, which estimates the variance for each residual separately, and \code{sum}(default), 
which assumes the same variance for all the residuals.
}

}

\value{
 an object of class \code{rs.zph}. This function would usually be followed by a
          plot of the result. The plot gives an estimate of the time-dependent
          coefficient \code{beta(t)}. If the proportional
          hazards assumption is true, \code{beta(t)} will be a horizontal
          line.  
}


\references{
Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005) "Goodness of fit of relative survival models." Statistics in Medicine, \bold{24}: 3911--3925.

Package. Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741--1749.

}


\examples{
data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata,int=5)
rszph <- rs.zph(fit)
plot(rszph)
}

\seealso{\code{\link{rsadd}}, \code{rsmul}, \code{rstrans}, \code{\link{resid}}, \code{\link{cox.zph}}.}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{joinrate}
\alias{joinrate}
\title{Join ratetables}
\usage{
joinrate(tables, dim.name = "country", merge = FALSE)
}
\arguments{
\item{tables}{a list of ratetables. If names are given, they are included as
\code{dimnames}.}

\item{dim.name}{the name of the added dimension.}

\item{merge}{if FALSE (default) only the intersect of all years/ages is taken.
If TRUE all possible years/ages are included (NOTE: in this case hazards are extrapolated
from earlier or later years/ages). This option only works for ratetables with dimensions}
}
\value{
An object of class \code{ratetable}.
}
\description{
The function joins two or more objects organized as \code{ratetable} by
adding a new dimension.
}
\details{
This function joins two or more \code{ratetable} objects by adding a new
dimension. The cutpoints of all the rate tables are compared; if merge=FALSE
(default) only the common intervals are kept, otherwise if merge=TRUE all
intervals are added (and hazards are extrapolated).
If the intervals defined by the cutpoints are not of
the same length, a warning message is displayed.  Each rate table must have
the same dimensions.
}
\examples{

#newpop <- joinrate(list(Arizona=survexp.az,Florida=survexp.fl,
#                   Minnesota=survexp.mn),dim.name="state")

}
\references{
Package: Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741-1749.
}
\seealso{
\code{\link[survival:ratetable]{survival::ratetable}}, \code{\link{transrate.hld}},
\code{\link{transrate.hmd}}, \code{\link{transrate}}.
}
\keyword{survival}

\name{transrate.hmd}
\alias{transrate.hmd}
\title{Reorganize Data obtained from Human Mortality Database into a Ratetable Object}
\description{
 The function assists in reorganizing the .txt files obtained from Human Mortality Database 
(http://www.mortality.org)  into a ratetable object.
}
\usage{
transrate.hmd(male,female)
}

\arguments{
  \item{male}{
	a .txt file, containing the data on males. 
  }
  \item{female}{
	a .txt file, containing the data on females.   }
}
\details{ 
This function works automatically with tables organised in the format provided by the Human Mortality 
Database. Download Life Tables for Males and Females separately from the column named 1x1 (period life tables,
 organized by date of death, yearly cutpoints for age as well as calendar year).


If you wish to provide the data in the required format by yourself, note that the only two columns needed
are calendar year (Year) and probability of death (qx). Death probabilities must be calculated up to
age 110 (in yearly intervals).
}
\value{An object of class \code{ratetable}.}

\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741--1749.
}



\examples{
\dontrun{
auspop <- transrate.hmd("mltper_1x1.txt","fltper_1x1.txt")
}
}
\seealso{\code{\link{ratetable}}, \code{\link{transrate.hld}}, \code{\link{joinrate}}, \code{\link{transrate}}.}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survaalen.R
\name{survaalen}
\alias{survaalen}
\title{Fit an additive hazards model}
\usage{
survaalen(formula, data, variance = FALSE, var_estimator = "dN")
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{variance}{a logical value indicating whether the variances of the hazards should be computed.
Default is FALSE.}

\item{var_estimator}{Choose variance estimator. The default option 'dN' uses dN(t) in the variance formula, see formula 4.63 in Aalen et al. (2008). Option 'XdB' uses X*dB(t), see formula 4.64 in Aalen et al. (2008).}
}
\value{
An object of class \code{aalen.model}.
}
\description{
Fits the Aalen additive hazard model. The function can be used for multi-state model
data (as in the package mstate; class msdata) by supplying the start and stop times in the
Surv object and adding a strata(trans) object in the formula (where trans denotes the
transition in the multi-state model).
}
\examples{

# Survival:
data(rdata)
mod <- survaalen(Surv(time, cens)~sex+age, data=rdata)
head(mod$coefficients)
tail(mod$coefficients)

# Multi-state model:
data(ebmt1wide)
mod <- survaalen(Surv(Tstart, Tstop, status)~age.1+age.2+age.3+strata(trans), data=ebmt1wide)
head(mod$coefficients$trans1)
head(mod$coefficients$trans2)
head(mod$coefficients$trans3)
}
\seealso{
\code{rsaalen}
}
\author{
Damjan Manevski
}
\keyword{survival}

\name{rs.surv.rsadd}
\alias{rs.surv.rsadd}


\title{Compute a Relative Survival Curve from an additive relative survival model}

\description{
     Computes the predicted relative survival function for an additive relative survival model fitted
     with maximum likelihood.
}
\usage{
     rs.surv.rsadd(formula, newdata) 

}
\arguments{
\item{formula}{
a \code{rsadd} object (Implemented only for models fitted with the code{max.lik} (default) option.)
}
\item{newdata}{
a data frame with the same variable names as those that appear in the \code{rsadd} formula. 
a predicted curve for each individual in this data frame shall be calculated
}
}

\details{Does not work with factor variables - you have to form dummy variables before calling the rsadd function.}

\value{
 a \code{survfit} object; see the help on \code{survfit.object} for details.
 The \code{survfit} methods are used for \code{print},
     \code{plot}, \code{lines}, and \code{points}.

}

\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
}




\examples{
data(slopop)
data(rdata)
#fit a relative survival model
fit <- rsadd(Surv(time,cens)~sex+age+year,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata,int=c(0:10,15))

#calculate the predicted curve for a male individual, aged 65, diagnosed in 1982
d <- rs.surv.rsadd(fit,newdata=data.frame(sex=1,age=65,year=as.date("1Jul1982")))
#plot the curve (will result in a step function since the baseline is assumed piecewise constant)
plot(d,xscale=365.241)

#calculate the predicted survival curves for each individual in the data set
d <- rs.surv.rsadd(fit,newdata=rdata)
#calculate the average over all predicted survival curves
p.surv <- apply(d$surv,1,mean)
#plot the relative survival curve
plot(d$time/365.241,p.surv,type="b",ylim=c(0,1),xlab="Time",ylab="Relative survival")
}


\seealso{
\code{survfit},
\code{survexp}
}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/years.R
\name{plot_f}
\alias{plot_f}
\title{Plot the absolute risk (observed and population curve)}
\usage{
plot_f(
  years,
  xlab = "Time interval",
  ylab = "Absolute risk",
  xbreak,
  ybreak,
  xlimits,
  ylimits,
  show.legend = TRUE
)
}
\arguments{
\item{years}{the object obtained using function \code{years}.}

\item{xlab}{a title for the x axis.}

\item{ylab}{a title for the y axis.}

\item{xbreak}{the breaks on the x axis (this is supplied to \code{scale_x_continuous}).}

\item{ybreak}{the breaks on the y axis (this is supplied to \code{scale_y_continuous}).}

\item{xlimits}{define the limits on the x axis (this is supplied to \code{scale_x_continuous}).}

\item{ylimits}{define the limits on the y axis (this is supplied to \code{scale_y_continuous}).}

\item{show.legend}{if TRUE, the legend is shown on the graph.}
}
\value{
A ggplot object
}
\description{
Plots the estimated observed and population curve for the
life years difference (Manevski, Ruzic Gorenjec, Andersen, Pohar Perme, 2022).
}
\details{
A ggplot2 implementation for plotting the observed and population curves. The type of curves is
dependent upon the measure calculated using \code{years} function (argument \code{measure}).
}
\seealso{
\code{\link{years}}, \code{\link{plot_years}}
}

\name{transrate.hld}
\alias{transrate.hld}
\title{Reorganize Data obtained from Human Life-Table Database into a Ratetable Object}
\description{
 The function assists in reorganizing the .txt files obtained from Human Life-Table Database 
(http://www.lifetable.de -> Data by Country)  into a ratetable object.
}
\usage{
transrate.hld(file,cut.year,race)
}

\arguments{
  \item{file}{
	a vector of file names which the data are to be read from. Must be in
	.tex format and in the same format as the files in Human Life-Table Database.
  }
  \item{cut.year}{
	a vector of cutpoints for years. Must be specified when	the year spans in the
	files are not consecutive.
  }
  \item{race}{a vector of race names for the input files.}
}
\details{ 
This function works with any table organised in the format provided by the Human Life-Table
Database, but currently only works with TypeLT 1 (i.e. age intervals of length 1). The age must always 
start with value 0, but can end at different values (when that happens, the last value is carried forward).
 The rates between the cutpoints are taken to be constant.
}
\value{An object of class \code{ratetable}.}
\examples{
\dontrun{
finpop <- transrate.hld(c("FIN_1981-85.txt","FIN_1986-90.txt","FIN_1991-95.txt"))
}
\dontrun{
nzpop <- transrate.hld(c("NZL_1980-82_Non-maori.txt","NZL_1985-87_Non-maori.txt",
	 "NZL_1980-82_Maori.txt","NZL_1985-87_Maori.txt"),
	 cut.year=c(1980,1985),race=rep(c("nonmaori","maori"),each=2))
}
}

\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741--1749.
}



\seealso{\code{\link{ratetable}}, \code{\link{transrate.hmd}}, \code{\link{joinrate}}, \code{\link{transrate}}.}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cmprel.r
\name{summary.cmp.rel}
\alias{summary.cmp.rel}
\title{Summary of the crude probability of death}
\usage{
\method{summary}{cmp.rel}(object, times, scale = 365.241, area = FALSE, ...)
}
\arguments{
\item{object}{output of the function \code{cmp.rel}.}

\item{times}{the times at which the output is required.}

\item{scale}{The time scale in which the times are specified. The default
value is \code{1}, i.e. days.}

\item{area}{Should area under the curves at time \code{tau} be printed out?
Default is \code{FALSE}.}

\item{...}{Additional arguments, currently not implemented}
}
\value{
A list of values is returned.
}
\description{
Returns a list containing the estimated values at required times.
}
\details{
The variance is calculated using numerical integration. If the required time
is not a time at which the value was estimated, the value at the last time
before it is reported. The density of the time points is set by the
\code{precision} argument in the \code{cmp.rel} function.
}
\examples{

data(slopop)
data(rdata)
#calculate the crude probability of death and summarize it
fit <- cmp.rel(Surv(time,cens)~sex,rmap=list(age=age*365),
      ratetable=slopop,data=rdata,tau=3652.41)
summary(fit,c(1,3),scale=365.241)

}
\seealso{
\code{cmp.rel}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/years.R
\name{plot_years}
\alias{plot_years}
\title{Plot the years measure}
\usage{
plot_years(
  years,
  xlab = "Time interval",
  ylab = "Years",
  xbreak,
  ybreak,
  xlimits,
  ylimits,
  conf.int = FALSE,
  ymirror = FALSE,
  yminus = FALSE
)
}
\arguments{
\item{years}{the object obtained using function \code{years}.}

\item{xlab}{a title for the x axis.}

\item{ylab}{a title for the y axis.}

\item{xbreak}{the breaks on the x axis (this is supplied to \code{scale_x_continuous}).}

\item{ybreak}{the breaks on the y axis (this is supplied to \code{scale_y_continuous}).}

\item{xlimits}{define the limits on the x axis (this is supplied to \code{scale_x_continuous}).}

\item{ylimits}{define the limits on the y axis (this is supplied to \code{scale_y_continuous}).}

\item{conf.int}{if TRUE, the confidence interval is plotted.}

\item{ymirror}{mirror the y values (w.r.t. the x axis).}

\item{yminus}{use function y -> -y when plotting.}
}
\value{
A ggplot object
}
\description{
Plot the years measure obtained from the \code{years} function.
}
\details{
A ggplot2 implementation for plotting the years measure. The type of curve is
dependent upon the measure calculated using the \code{years} function (argument \code{measure}).
}
\seealso{
\code{\link{years}}, \code{\link{plot_f}}
}

\name{plot.rs.zph}
\alias{plot.rs.zph}


\title{Graphical Inspection of Proportional Hazards Assumption in Relative Survival Models}
\description{
Displays a graph of the scaled partial residuals, along with a smooth curve.
}
\usage{
\method{plot}{rs.zph}(x, resid=TRUE, df = 4, nsmo = 40,var,cex=1,add=FALSE,col=1,
    lty=1,xlab,ylab,xscale=1,...)
}

\arguments{
\item{x}{
result of the \code{rs.zph} function.
}
\item{resid}{a logical value, if \code{TRUE} the residuals are included on the
          plot, as well as the smooth fit.}
\item{df}{
the degrees of freedom for the fitted natural spline, \code{df=2}
          leads to a linear fit. 
}
\item{nsmo}{
number of points used to plot the fitted spline. 
}
\item{var}{ 
the set of variables for which plots are desired.  By
          default, plots are produced in turn for each variable of a
          model.  Selection of a single variable allows other features
          to be added to the plot, e.g., a horizontal line at zero or a
          main title.
}
\item{cex}{a numerical value giving the amount by which plotting text
          and symbols should be scaled relative to the default.
}
\item{add}{logical, if \code{TRUE} the plot is added to an existing plot}
\item{col}{a specification for the default plotting color.}
\item{lty}{the line type.}
\item{xlab}{x axis label.}
\item{ylab}{y axis label.}
\item{xscale}{units for x axis, default is 1, i.e. days.}
\item{...}{Additional arguments passed to the \code{plot} function. }
}


\references{
Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005) "Goodness of fit of relative survival models." Statistics in Medicine, \bold{24}: 3911-3925.

Package: Pohar M., Stare J. (2006) "Relative survival analysis in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272-278.  

Relative survival:  Pohar, M., Stare, J. (2007) "Making relative survival analysis relatively easy."
Computers in biology and medicine, \bold{37}: 1741-1749, 2007.
}




\examples{
data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex+as.factor(agegr),rmap=list(age=age*365.241),
             ratetable=slopop,data=rdata,int=5)
rszph <- rs.zph(fit)
plot(rszph)
}

\seealso{\code{\link{rs.zph}}, \code{\link{plot.cox.zph}}.}

\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{expprep2}
\alias{expprep2}
\title{expprep2 function}
\usage{
expprep2(
  x,
  y,
  ratetable,
  status,
  times,
  fast = FALSE,
  ys,
  prec,
  cmp = F,
  netweiDM = FALSE
)
}
\arguments{
\item{x}{matrix of demographic covariates - each individual has one line}

\item{y}{follow-up time for each individual (same length as nrow(x))}

\item{ratetable}{rate table used for calculation}

\item{status}{status for each individual (same length as nrow(x)!), not
needed if we only need Spi, status needed for rs.surv}

\item{times}{times at which we wish to evaluate the quantities, not needed
if we only need Spi, times needed for rs.surv}

\item{fast}{for mpp method only}

\item{ys}{entry times (if empty, individuals are followed from time 0)}

\item{prec}{deprecated}

\item{cmp}{should cmpfast.C be used}

\item{netweiDM}{should new netwei script be used}
}
\value{
List containing the calculated hazards and probabilities using the
population mortality tables.
}
\description{
Helper calculation function using C code. Saved also as exp_prep (unexported
function).
}
\details{
Helper function used in rs.surv and other relsurv functions.
}
\seealso{
rs.surv
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rssurvrsadd.r
\name{rs.surv.rsadd}
\alias{rs.surv.rsadd}
\title{Compute a Relative Survival Curve from an additive relative survival model}
\usage{
rs.surv.rsadd(formula, newdata)
}
\arguments{
\item{formula}{a \code{rsadd} object (Implemented only for models fitted
with the codemax.lik (default) option.)}

\item{newdata}{a data frame with the same variable names as those that
appear in the \code{rsadd} formula.  a predicted curve for each individual
in this data frame shall be calculated}
}
\value{
a \code{survfit} object; see the help on \code{survfit.object} for
details.  The \code{survfit} methods are used for \code{print}, \code{plot},
\code{lines}, and \code{points}.
}
\description{
Computes the predicted relative survival function for an additive relative
survival model fitted with maximum likelihood.
}
\details{
Does not work with factor variables - you have to form dummy variables
before calling the rsadd function.
}
\examples{

data(slopop)
data(rdata)
#fit a relative survival model
fit <- rsadd(Surv(time,cens)~sex+age+year,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata,int=c(0:10,15))

#calculate the predicted curve for a male individual, aged 65, diagnosed in 1982
d <- rs.surv.rsadd(fit,newdata=data.frame(sex=1,age=65,year=as.Date("1982-01-01")))
#plot the curve (will result in a step function since the baseline is assumed piecewise constant)
plot(d,xscale=365.241)

#calculate the predicted survival curves for each individual in the data set
d <- rs.surv.rsadd(fit,newdata=rdata)
#calculate the average over all predicted survival curves
p.surv <- apply(d$surv,1,mean)
#plot the relative survival curve
plot(d$time/365.241,p.surv,type="b",ylim=c(0,1),xlab="Time",ylab="Relative survival")

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278
}
\seealso{
\code{survfit}, \code{survexp}
}
\keyword{survival}

\name{slopop}
\alias{slopop}
\docType{data}
\title{Census Data Set for the Slovene Population}
\description{
  Census data set for the Slovene population.
}
\usage{data(slopop)}

\examples{
data(slopop)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{epa}
\alias{epa}
\title{Excess hazard function smoothing}
\usage{
epa(fit, bwin, times, n.bwin = 16, left = FALSE)
}
\arguments{
\item{fit}{Fit from the additive relative survival model using the \code{EM}
method.}

\item{bwin}{The relative width of the smoothing window (default is 1).}

\item{times}{The times at which the smoother is to be evaluated. If missing,
it is evaluated at all event times.}

\item{n.bwin}{Number of times that the window width may change.}

\item{left}{If \code{FALSE} (default) smoothing is performed symmetrically,
if \code{TRUE} only leftside neighbours are considered.}
}
\value{
A list with two components: \item{lambda}{the smoothed excess
baseline hazard function} \item{times}{the times at which the smoothed
excess baseline hazard is evaluated.}
}
\description{
An Epanechnikov kernel function based smoother for smoothing the baseline
excess hazard calculated by the \code{rsadd} function with the \code{EM}
method.
}
\details{
The function performs Epanechnikov kernel smoothing. The follow up time is
divided (according to percentiles of event times) into several intervals
(number of intervals defined by \code{n.bwin}) in which the width is
calculated as a factor of the maximum span between event times.  Boundary
effects are also taken into account on both sides.
}
\examples{

data(slopop)
data(rdata)
#fit an additive model with the EM method
fit <- rsadd(Surv(time,cens)~sex+age,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,int=5,method="EM")
sm <- epa(fit)
plot(sm$times,sm$lambda)

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.

EM algorithm: Pohar Perme M., Henderson R., Stare, J. (2009) "An approach to
estimation in relative survival regression." Biostatistics, \bold{10}:
136--146.
}
\seealso{
\code{\link{rsadd}},
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{rs.surv}
\alias{rs.surv}
\title{Compute a Relative Survival Curve}
\usage{
rs.surv(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  na.action,
  fin.date,
  method = "pohar-perme",
  conf.type = "log",
  conf.int = 0.95,
  type = "kaplan-meier",
  add.times,
  precision = 1,
  rmap,
  weight.table = NULL,
  weight.names = NULL
)
}
\arguments{
\item{formula}{a formula object, with the response as a \code{Surv} object
on the left of a \code{~} operator, and, if desired, terms separated by the
\code{+} operator on the right. If no strata are used, \code{~1} should be
specified.

NOTE: The follow-up time must be in days.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{na.action}{a missing-data filter function, applied to the model.frame,
after any subset argument has been used.  Default is
\code{options()$na.action}.}

\item{fin.date}{the date of the study ending, used for calculating the
potential follow-up times in the Hakulinen method. If missing, it is
calculated as \code{max(year+time)}.}

\item{method}{the method for calculating the relative survival. The options
are \code{pohar-perme}(default), \code{ederer1}, \code{ederer2} and
\code{hakulinen}.}

\item{conf.type}{one of \code{plain}, \code{log} (the default), or
\code{log-log}. The first option causes the standard intervals curve +- k
*se(curve), where k is determined from conf.int. The log option calculates
intervals based on the cumulative hazard or log(survival). The last option
bases intervals on the log hazard or log(-log(survival)).}

\item{conf.int}{the level for a two-sided confidence interval on the
survival curve(s). Default is 0.95.}

\item{type}{defines how survival estimates are to be calculated given the
hazards.  The default (\code{kaplan-meier}) calculates the product integral,
whereas the option \code{fleming-harrington} exponentiates the negative
cumulative hazard. Analogous to the usage in \code{survfit}.}

\item{add.times}{specific times at which the curve should be evaluated.}

\item{precision}{Precision for numerical integration. Default is 1, which
means that daily intervals are taken, the value may be decreased to get a
higher precision or increased to achieve a faster calculation. The
calculation intervals always include at least all times of event and
censoring as border points.}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details
below.}

\item{weight.table}{Default is NULL. If defined it is used for calculating
standardized net survival. Supply a data.frame
that contains the weights and group names for calculating the standardized net survival.
The group column also has to be included in the data argument.
For the theory see the details below.}

\item{weight.names}{Default is NULL. If defined it is used for calculating
standardized net survival. Supply a character vector of length two with the
names of the group and weight columns in \code{weight.table}.}
}
\value{
a \code{survfit} object; see the help on \code{survfit.object} for
details.  The \code{survfit} methods are used for \code{print},
\code{summary}, \code{plot}, \code{lines}, and \code{points}.
}
\description{
Computes an estimate of the relative survival curve using the Ederer I,
Ederer II method, Pohar-Perme method or the Hakulinen method
}
\details{
NOTE: The follow-up time must be specified in days. The \code{ratetable}
being used may have different variable names and formats than the user's
data set, this is dealt with by the \code{rmap} argument. For example, if
age is in years in the data set but in days in the \code{ratetable} object,
age=age*365.241 should be used. The calendar year can be in any date format
(Date and POSIXt are allowed), the date formats in the
\code{ratetable} and in the data may differ.

The potential censoring times needed for the calculation of the expected
survival by the Hakulinen method are calculated automatically. The times of
censoring are left as they are, the times of events are replaced with
\code{fin.date - year}.

The calculation of the Pohar-Perme estimate is more time consuming since
more data are needed from the population tables.  The old version of the
function, now named \code{rs.survo} can be used as a faster version for the
Hakulinen and Ederer II estimate.

Numerical integration is required for Pohar-Perme estimate. The integration
precision is set with argument \code{precision}, which defaults to daily
intervals, a default that should give enough precision for any practical
purpose.

Note that even though the estimate is always calculated using numerical
integration, only the values at event and censoring times are reported.
Hence, the function \code{plot} draws a step function in between and the
function \code{summary} reports the value at the last event or censoring
time before the specified time. If the output of the estimated values at
other points is required, this should be specified with argument
\code{add.times}.

Standardized net survival can be also calculated:

\eqn{SNS (t) = \sum_{j} w_j {NS}_j (t)}

where \eqn{NS_j} is the net survival in the j-th group, \eqn{w_j}
is the weight for the j-th group and \eqn{SNS} is the standardized net survival.
\eqn{SNS} can be calculated by using the \code{weight.table} and \code{weight.names} arguments. The function
also returns the corresponding standard error and confidence interval.
}
\examples{

data(slopop)
data(rdata)
#calculate the relative survival curve
#note that the variable year must be given in a date format and that
#age must be multiplied by 365.241 in order to be expressed in days.
rs.surv(Surv(time,cens)~sex,rmap=list(age=age*365.241), ratetable=slopop,data=rdata)

# SNS:
wei <- data.frame(agegr=c('<54', '54-61', '62-70', '71-95'),
                  weight=c(0.2, 0.2, 0.3, 0.3))
rs.surv(Surv(time,cens)~1, rmap=list(age=age*365.241), ratetable=slopop, data=rdata,
   weight.table=wei, weight.names = c('agegr', 'weight'))

}
\references{
Package: Pohar Perme, M., Pavlic, K. (2018) "Nonparametric
Relative Survival Analysis with the R Package relsurv". Journal of
Statistical Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08" Theory:
Pohar Perme, M., Esteve, J., Rachet, B. (2016) "Analysing Population-Based
Cancer Survival - Settling the Controversies." BMC Cancer, 16 (933), 1-8.
doi:10.1186/s12885-016-2967-9. Theory: Pohar Perme, M., Stare, J., Esteve,
J. (2012) "On Estimation in Relative Survival", Biometrics, 68(1), 113-120.
doi:10.1111/j.1541-0420.2011.01640.x.
}
\seealso{
\code{survfit}, \code{survexp}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{nessie}
\alias{nessie}
\title{Net Expected Sample Size Is Estimated}
\usage{
nessie(
  formula = formula(data),
  data = parent.frame(),
  ratetable = relsurv::slopop,
  times,
  rmap
)
}
\arguments{
\item{formula}{a formula object, same as in \code{rs.surv}. The right-hand
side of the formula object includes the variable that defines the subgroups
(a variable of type \code{factor}) by which the expected sample size is to
be calculated.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula}.}

\item{ratetable}{a table of event rates, organized as a \code{ratetable}
object, such as \code{slopop}.}

\item{times}{Times at which the calculation should be evaluated - in years!}

\item{rmap}{an optional list to be used if the variables are not organized
and named in the same way as in the \code{ratetable} object. See details of
the \code{rs.surv} function.}
}
\value{
A list of values.
}
\description{
Calculates how the sample size decreases in time due to population mortality
}
\details{
The function calculates the sample size we can expect at a certain time
point if the patients die only due to population causes (population survival
* initial sample size in a certain category), i.e. the number of individuals
that remains at risk at given timepoints after the individuals who die due
to population causes are removed. The result should be used as a guideline
for the sensible length of follow-up interval when calculating the net
survival.

The first column of the output reports the number of individuals at time 0.
The last column of the output reports the conditional expected (population)
survival time for each subgroup.
}
\examples{

data(slopop)
data(rdata)
rdata$agegr <-cut(rdata$age,seq(40,95,by=5))
nessie(Surv(time,cens)~agegr,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata,times=c(1,3,5,10,15))

}
\references{
Pohar Perme, M., Pavlic, K. (2018) "Nonparametric Relative
Survival Analysis with the R Package relsurv". Journal of Statistical
Software. 87(8), 1-27, doi: "10.18637/jss.v087.i08"
}
\seealso{
\code{rs.surv}
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{rs.zph}
\alias{rs.zph}
\title{Behaviour of Covariates in Time for Relative Survival Regression Models}
\usage{
rs.zph(fit, sc, transform = "identity", var.type = "sum")
}
\arguments{
\item{fit}{the result of fitting an additive relative survival model, using
the \code{rsadd}, \code{rsmul} or \code{rstrans} function.

In the case of multiplicative and transformation models the output is
identical to \code{cox.zph} function, except no test is performed.}

\item{sc}{partial residuals calculated by the \code{resid} function. This is
used to save time if several tests are to be calculated on these residuals
and can otherwise be omitted.}

\item{transform}{a character string specifying how the survival times should
be transformed. Possible values are \code{"km"}, \code{"rank"},
\code{"identity"} and \code{log}. The default is \code{"identity"}.}

\item{var.type}{a character string specifying the variance used to scale the
residuals. Possible values are \code{"each"}, which estimates the variance
for each residual separately, and \code{sum}(default), which assumes the
same variance for all the residuals.}
}
\value{
an object of class \code{rs.zph}. This function would usually be
followed by a plot of the result. The plot gives an estimate of the
time-dependent coefficient \code{beta(t)}. If the proportional hazards
assumption is true, \code{beta(t)} will be a horizontal line.
}
\description{
Calculates the scaled partial residuals of a relative survival model
(\code{rsadd}, \code{rsmul} or \code{rstrans})
}
\examples{

data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
	ratetable=slopop,data=rdata,int=5)
rszph <- rs.zph(fit)
plot(rszph)

}
\references{
Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005)
"Goodness of fit of relative survival models." Statistics in Medicine,
\bold{24}: 3911--3925.

Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link{rsadd}}, \code{rsmul}, \code{rstrans},
\code{\link{resid}}, \code{\link[survival:cox.zph]{survival::cox.zph}}.
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.aalen.model.R
\name{predict.aalen.model}
\alias{predict.aalen.model}
\title{Subject-specific prediction for an Aalen model}
\usage{
\method{predict}{aalen.model}(object, newdata, type = "Haz", ...)
}
\arguments{
\item{object}{A survaalen or rsaalen object}

\item{newdata}{A data.frame with one row (add covariate values in columns)}

\item{type}{By default ('Haz') returns cumulative hazards. If 'prob' it calculates CDF/CIFs. If 'both' a list is returned with Haz and prob.}

\item{...}{Not used for now}
}
\value{
A data.frame containing the times and survival (or excess/population) estimates.
}
\description{
A function that returns subject-specific predictions from a survaalen or rsaalen object.
}
\examples{
data(rdata)
mod <- survaalen(Surv(time, cens)~sex+age, data=rdata)
predict(mod, data.frame(age=68, sex=2))

mod <- rsaalen(Surv(time, cens)~sex+age, data=rdata, ratetable=slopop,
                rmap=list(age=age*365.241))
predict(mod, data.frame(age=68, sex=2, year=as.Date('1982-06-24')))
}
\author{
Damjan Manevski \email{damjan.manevski@mf.uni-lj.si}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{rs.br}
\alias{rs.br}
\alias{plot.rs.br}
\alias{print.rs.br}
\title{Test the Proportional Hazards Assumption for Relative Survival Regression
Models}
\usage{
rs.br(fit, sc, rho = 0, test = "max", global = TRUE)
}
\arguments{
\item{fit}{the result of fitting a relative survival model, using the
\code{rsadd}, \code{rsmul} or \code{rstrans} function.}

\item{sc}{partial residuals calculated by the \code{resid} function. This is
used to save time if several tests are to be calculated on these residuals
and can otherwise be omitted.}

\item{rho}{a number controlling the weigths of residuals. The weights are
the number of individuals at risk at each event time to the power
\code{rho}. The default is \code{rho=0}, which sets all weigths to 1.}

\item{test}{a character string specifying the test to be performed on
Brownian bridge. Possible values are \code{"max"} (default), which tests the
maximum absolute value of the bridge, and \code{cvm}, which calculates the
Cramer Von Mises statistic.}

\item{global}{should a global Brownian bridge test be performed, in addition
to the per-variable tests}
}
\value{
an object of class \code{rs.br}. This function would usually be
followed by both a print and a plot of the result. The plot gives a Brownian
bridge for each of the variables. The horizontal lines are the 95% and 99%
confidence intervals for the maximum absolute value of the Brownian bridge
}
\description{
Test the proportional hazards assumption for relative survival models
(\code{rsadd}, \code{rsmul} or \code{rstrans}) by forming a Brownian Bridge.
}
\examples{

data(slopop)
data(rdata)
fit <- rsadd(Surv(time,cens)~sex,rmap=list(age=age*365.241),
		ratetable=slopop,data=rdata,int=5)
rsbr <- rs.br(fit)
rsbr
plot(rsbr)

}
\references{
Goodness of fit: Stare J.,Pohar Perme M., Henderson R. (2005)
"Goodness of fit of relative survival models." Statistics in Medicine,
\bold{24}: 3911--3925.

Package. Pohar M., Stare J. (2006) "Relative survival analysis in R."
Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link{rsadd}}, \code{rsmul}, \code{rstrans},
\code{\link{resid}}.
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{transrate.hmd}
\alias{transrate.hmd}
\title{Reorganize Data obtained from Human Mortality Database into a Ratetable
Object}
\usage{
transrate.hmd(male, female)
}
\arguments{
\item{male}{a .txt file, containing the data on males.}

\item{female}{a .txt file, containing the data on females.}
}
\value{
An object of class \code{ratetable}.
}
\description{
The function assists in reorganizing the .txt files obtained from Human
Mortality Database (http://www.mortality.org) into a ratetable object.
}
\details{
This function works automatically with tables organised in the format
provided by the Human Mortality Database. Download Life Tables for Males and
Females separately from the column named 1x1 (period life tables, organized
by date of death, yearly cutpoints for age as well as calendar year).

If you wish to provide the data in the required format by yourself, note
that the only two columns needed are calendar year (Year) and probability of
death (qx). Death probabilities must be calculated up to age 110 (in yearly
intervals).
}
\examples{

\dontrun{
auspop <- transrate.hmd("mltper_1x1.txt","fltper_1x1.txt")
}

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link[survival:ratetable]{survival::ratetable}}, \code{\link{transrate.hld}},
\code{\link{joinrate}}, \code{\link{transrate}}.
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rcode.r
\name{transrate.hld}
\alias{transrate.hld}
\title{Reorganize Data obtained from Human Life-Table Database into a Ratetable
Object}
\usage{
transrate.hld(file, cut.year, race)
}
\arguments{
\item{file}{a vector of file names which the data are to be read from. Must
be in .tex format and in the same format as the files in Human Life-Table
Database.}

\item{cut.year}{a vector of cutpoints for years. Must be specified when the
year spans in the files are not consecutive.}

\item{race}{a vector of race names for the input files.}
}
\value{
An object of class \code{ratetable}.
}
\description{
The function assists in reorganizing the .txt files obtained from Human
Life-Table Database (http://www.lifetable.de -> Data by Country) into a
ratetable object.
}
\details{
This function works with any table organised in the format provided by the
Human Life-Table Database, but currently only works with TypeLT 1 (i.e. age
intervals of length 1). The age must always start with value 0, but can end
at different values (when that happens, the last value is carried forward).
The rates between the cutpoints are taken to be constant.
}
\examples{

\dontrun{
finpop <- transrate.hld(c("FIN_1981-85.txt","FIN_1986-90.txt","FIN_1991-95.txt"))
}
\dontrun{
nzpop <- transrate.hld(c("NZL_1980-82_Non-maori.txt","NZL_1985-87_Non-maori.txt",
	 "NZL_1980-82_Maori.txt","NZL_1985-87_Maori.txt"),
	 cut.year=c(1980,1985),race=rep(c("nonmaori","maori"),each=2))
}

}
\references{
Package. Pohar M., Stare J. (2006) "Relative survival analysis
in R." Computer Methods and Programs in Biomedicine, \bold{81}: 272--278

Relative survival: Pohar, M., Stare, J. (2007) "Making relative survival
analysis relatively easy." Computers in biology and medicine, \bold{37}:
1741--1749.
}
\seealso{
\code{\link[survival:ratetable]{survival::ratetable}}, \code{\link{transrate.hmd}},
\code{\link{joinrate}}, \code{\link{transrate}}.
}
\keyword{survival}

\name{ebmt1wide}
\alias{ebmt1wide}
\docType{data}
\title{EBMT Data Taken From Package mstate}
\description{
  The ebmt1 dataset taken from package mstate transformed in long format.
}
\usage{data(ebmt1wide)}


\keyword{datasets}
